#include "NjoyUtils/GroupStructureNeutron.hpp"

using namespace frendy;

GroupStructureNeutron::GroupStructureNeutron(void)
{
  clear();
}

GroupStructureNeutron::~GroupStructureNeutron(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void GroupStructureNeutron::clear()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_group_structure(int ign)
{
  vector<Real> ng_vec;
  ng_vec.clear();

  if( ign == 1 ) //arbitrary structure (read in)
  {
    ng_vec.resize(0);
  }
  else if( ign == 2 ) //CSEWG 239 group structure
  {
    ng_vec = get_gl2();
  }
  else if( ign == 3 ) //LANL 30 group structure
  {
    ng_vec = get_eg3();
  }
  else if( ign == 4 ) //ANL 27 group structure
  {
    ng_vec = get_gl4();
  }
  else if( ign == 5 ) //RRD 50 group structure
  {
    ng_vec = get_gl5();
  }
  else if( ign == 6 ) //GAM-I 68 group structure
  {
    int i_max = 69;
    ng_vec.resize(i_max);
    ng_vec[0] = 0.0;
    for(int i=1; i<i_max; i++)
    {
      ng_vec[i] = ng_vec[i-1] + 0.25;
    }

    for(int i=0; i<i_max; i++)
    {
      ng_vec[i] = 1.0E+7 * exp(-1.0*ng_vec[i]);
    }
  }
  else if( ign == 7 ) //GAM-II 100 group structure
  {
    int i_max = 101;
    ng_vec.resize(i_max);
    ng_vec[0]         = -0.53062825; //17MeV
    ng_vec[i_max - 1] = 27.631021;   //1.0E-5eV

    ng_vec[1]         = -0.3;
    for(int i=2; i<i_max-1; i++)
    {
      if( i < 50 )
      {
        ng_vec[i] = ng_vec[i-1] + 0.1;
      }
      else
      {
        ng_vec[i] = ng_vec[i-1] + 0.25;
      }
    }

    for(int i=0; i<i_max; i++)
    {
      ng_vec[i] = 1.0E+7 * exp(-1.0*ng_vec[i]);
    }
  }
  else if( ign == 8 ) //LASER-THERMOS 35 group
  {
    ng_vec = get_eg6();
  }
  else if( ign == 9 ) //EPRI-CPM/WIMS 69 group structure
  {
    ng_vec = get_eg9();
  }
  else if( ign == 10 ) //LANL 187-group structure
  {
    int i_max = 188;
    ng_vec.resize(i_max);

    vector<Real> eg10a = get_eg10a();
    for(int i=0; i<static_cast<int>(eg10a.size()); i++)
    {
      ng_vec[i] = eg10a[i];
    }
    eg10a.clear();

    Real u_val = -15.5;
    Real ub    = -14.125;
    Real uc    = -5.875;
    for(int i=48; i<175; i++)
    {
      if( i == 59 )
      {
        ng_vec[i] = 6.868e0;
      }
      else if( i==126 )
      {
        ng_vec[126] = 2.6058e4;
      }
      else
      {
        if( i == 60 )
        {
          u_val = ub;
        }
        else if( i == 127 )
        {
          u_val = uc;
        }

        ng_vec[i] = 1.0E+7 * exp(u_val);
        u_val += 0.125;
      }
    }

    vector<Real> eg10b = get_eg10b();
    for(int i=0; i<static_cast<int>(eg10b.size()); i++)
    {
      ng_vec[175+i] = eg10b[i];
    }
    eg10b.clear();
  }
  else if( ign == 11 ) //LANL 70-group structure
  {
    ng_vec = get_eg11();
  }
  else if( ign == 12 ) //SAND-II 620-group structure
  {
    ng_vec = set_sandii_structure(621);
  }
  else if( ign == 13 ) //LANL 80-group structure
  {
    int i_max = 81;
    ng_vec.resize(i_max);

    vector<Real> u80 = get_u80();

    Real u_val = 0.6931472;
    for(int i=0; i<i_max; i++)
    {
      ng_vec[i] = 1.0E+7 * exp(u_val);

      u_val    -= u80[i];
    }

    ng_vec[0] = 2.0E+7;
    u80.clear();
  }
  else if( ign == 14 ) //EURLIB 100-group structure
  {
    int i_max = 101;
    ng_vec.resize(i_max);
    vector<int>  ig14 = get_ig14();
    vector<Real> gl14 = get_gl14();

    ng_vec[0] = -0.4;

    int ele_no = 0;
    for(int i=1; i<i_max; i++)
    {
      if( i+1 == ig14[ele_no] )
      {
        ele_no++;
      }
      ng_vec[i] = ng_vec[i-1] + gl14[ele_no-1];
    }

    for(int i=0; i<i_max; i++)
    {
      ng_vec[i] = 1.0E+7 * exp(-1.0*ng_vec[i]);
    }

    ig14.clear();
    gl14.clear();
  }
  else if( ign == 15 ) //SAND-IIA 640-group structure
  {
    ng_vec = set_sandii_structure(641);
  }
  else if( ign == 16 ) //VITAMIN-E 174-group structure
  {
    ng_vec = get_eg16();
  }
  else if( ign == 17 ) //VITAMIN-J 175-group structure
  {
    ng_vec = get_eg17();
  }
  else if( ign == 18 ) //XMAS 172-group structure
  {
    ng_vec = get_eg18();
  }
  else if( ign == 19 ) //ECCO  33-group structure
  {
    ng_vec = get_eg19();
  }
  else if( ign == 20 ) //ECCO 1968-group structure
  {
    ng_vec = get_eg20();
  }
  else if( ign == 21 ) //TRIPOLI 315-group structure
  {
    ng_vec = get_eg21();
  }
  else if( ign == 22 ) //XMASLWC 172-group structure
  {
    ng_vec = get_eg22();
  }
  else if( ign == 23 ) //VIT-J lwpc 175-group structure
  {
    ng_vec = get_eg23();
  }
  else if( ign == 24 ) //SHEM CEA 281-group structure
  {
    ng_vec = get_eg24();
  }
  else if( ign == 25 ) //SHEM EPM 295-group structure
  {
    ng_vec = get_eg25();
  }
  else if( ign == 26 ) //SHEM CEA/EPM 361-group structure
  {
    ng_vec = get_eg26();
  }
  else if( ign == 27 ) //SHEM EPM 315-group structure
  {
    ng_vec = get_eg27();
  }
  else if( ign == 28 ) //RAHAB AECL 89-group structure
  {
    ng_vec = get_eg28();
  }
  else if( ign == 29 ) //CCFE   660-group structure
  {
    ng_vec = get_eg29();
  }
  else if( ign == 30 ) //UKAEA 1025-group structure
  {
    ng_vec = get_eg30();
  }
  else if( ign == 31 ) //UKAEA 1067-group structure
  {
    ng_vec = get_eg31();
  }
  else if( ign == 32 ) //UKAEA 1102-group structure
  {
    ng_vec = get_eg32();
  }
  else if( ign == 33 ) //UKAEA  142-group structure
  {
    ng_vec = get_eg33();
  }
  else if( ign == 34 ) //LANL 618 group structure
  {
    ng_vec = get_eg618();
  }
  else if( ign == 101 ) //Vitamin-B6 199 group structure
  {
    ng_vec = get_vit199();
  }
  else if( ign == 102 ) //Vitamin-B6 200 group structure
  {
    ng_vec = get_vit200();
  }
  else
  {
    string class_name = "GroupStructureNeutron";
    string func_name  = "get_group_structure(int ign)";

    ostringstream oss;
    oss << ign;
    string str_data01 = "Energy group structure type (ign) : " + oss.str();

    vector<string> err_com;
    err_com.push_back(str_data01);
    err_com.push_back("This energy group structure type is not available in this program.");
    err_com.push_back("Available energy group structure type is as follows:");
    err_com.push_back("   1  arbitrary structure (read in)");
    err_com.push_back("   2  CSEWG 239 group structure");
    err_com.push_back("   3  LANL 30 group structure");
    err_com.push_back("   4  ANL 27 group structure");
    err_com.push_back("   5  RRD 50 group structure");
    err_com.push_back("   6  GAM-I 68 group structure");
    err_com.push_back("   7  GAM-II 100 group structure");
    err_com.push_back("   8  LASER-THERMOS 35 group");
    err_com.push_back("   9  EPRI-CPM/WIMS 69 group structure");
    err_com.push_back("  10  LANL 187-group structure");
    err_com.push_back("  11  LANL 70-group structure");
    err_com.push_back("  12  SAND-II 620-group structure");
    err_com.push_back("  13  LANL 80-group structure");
    err_com.push_back("  14  EURLIB 100-group structure");
    err_com.push_back("  15  SAND-IIA 640-group structure");
    err_com.push_back("  16  VITAMIN-E 174-group structure");
    err_com.push_back("  17  VITAMIN-J 175-group structure");
    err_com.push_back("  18  XMAS 172-group structure");
    err_com.push_back("  19  ECCO  33-group structure");
    err_com.push_back("  20  ECCO 1968-group structure");
    err_com.push_back("  21  TRIPOLI 315-group structure");
    err_com.push_back("  22  XMASLWC 172-group structure");
    err_com.push_back("  23  VIT-J lwpc 175-group structure");
    err_com.push_back("  24  SHEM CEA 281-group structure");
    err_com.push_back("  25  SHEM EPM 295-group structure");
    err_com.push_back("  26  SHEM CEA/EPM 361-group structure");
    err_com.push_back("  27  SHEM EPM 315-group structure");
    err_com.push_back("  28  RAHAB AECL 89-group structure");
    err_com.push_back("  29  CCFE   660-group structure");
    err_com.push_back("  30  UKAEA 1025-group structure");
    err_com.push_back("  31  UKAEA 1067-group structure");
    err_com.push_back("  32  UKAEA 1102-group structure");
    err_com.push_back("  33  UKAEA  142-group structure");
    err_com.push_back("  34  LANL 618 group structure");
    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  sort(ng_vec.begin(), ng_vec.end());

  return ng_vec;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::set_sandii_structure(int ng)
{
  vector<Real> ng_vec;
  ng_vec.resize(ng);

  ng_vec[0] = 1.0e-4;

  vector<Real> deltl  = get_deltl();
  vector<int>  ndelta = get_ndelta();
  for(int i=0; i<8; i++)
  {
    Real delta = deltl[i] * 1.0e-6;
    for(int j=ndelta[i] - 1; j<ndelta[i+1] - 1; j++)
    {
      ng_vec[j] = ng_vec[j-1] + delta;
    }
  }
  deltl.clear();
  ndelta.clear();

  ng_vec[20] = 2.8e-4;

  for(int i=45; i<450; i++)
  {
    ng_vec[i] = ng_vec[i-45] * 10.0;
  }

  ng_vec[450] = 1.0e6;

  for(int i=451; i<ng; i++)
  {
    ng_vec[i] = ng_vec[i-1] + 1.0E+5;
  }

  return ng_vec;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_gl2()
{
  vector<Real> gl2;
  gl2.resize(241);
  gl2[  0] = 1.000021E-05;
  gl2[  1] = 4.139938E-01;
  gl2[  2] = 5.315785E-01;
  gl2[  3] = 6.250621E-01;
  gl2[  4] = 6.825603E-01;
  gl2[  5] = 8.336811E-01;
  gl2[  6] = 8.764248E-01;
  gl2[  7] = 1.125352E+00;
  gl2[  8] = 1.444980E+00;
  gl2[  9] = 1.855391E+00;
  gl2[ 10] = 2.382370E+00;
  gl2[ 11] = 3.059023E+00;
  gl2[ 12] = 3.927864E+00;
  gl2[ 13] = 5.043477E+00;
  gl2[ 14] = 6.475952E+00;
  gl2[ 15] = 8.315287E+00;
  gl2[ 16] = 1.067704E+01;
  gl2[ 17] = 1.370959E+01;
  gl2[ 18] = 1.760346E+01;
  gl2[ 19] = 2.260329E+01;
  gl2[ 20] = 2.902320E+01;
  gl2[ 21] = 3.726653E+01;
  gl2[ 22] = 4.785117E+01;
  gl2[ 23] = 6.144212E+01;
  gl2[ 24] = 7.889325E+01;
  gl2[ 25] = 1.013009E+02;
  gl2[ 26] = 1.300730E+02;
  gl2[ 27] = 1.670170E+02;
  gl2[ 28] = 2.144541E+02;
  gl2[ 29] = 2.753645E+02;
  gl2[ 30] = 3.535750E+02;
  gl2[ 31] = 4.539993E+02;
  gl2[ 32] = 5.829466E+02;
  gl2[ 33] = 7.485183E+02;
  gl2[ 34] = 9.611165E+02;
  gl2[ 35] = 1.234098E+03;
  gl2[ 36] = 1.363889E+03;
  gl2[ 37] = 1.507331E+03;
  gl2[ 38] = 1.584613E+03;
  gl2[ 39] = 1.665858E+03;
  gl2[ 40] = 1.841058E+03;
  gl2[ 41] = 2.034684E+03;
  gl2[ 42] = 2.248673E+03;
  gl2[ 43] = 2.485168E+03;
  gl2[ 44] = 2.612586E+03;
  gl2[ 45] = 2.746536E+03;
  gl2[ 46] = 2.863488E+03;
  gl2[ 47] = 3.035391E+03;
  gl2[ 48] = 3.354626E+03;
  gl2[ 49] = 3.707435E+03;
  gl2[ 50] = 4.097350E+03;
  gl2[ 51] = 4.307425E+03;
  gl2[ 52] = 4.528272E+03;
  gl2[ 53] = 5.004514E+03;
  gl2[ 54] = 5.530844E+03;
  gl2[ 55] = 6.267267E+03;
  gl2[ 56] = 7.101744E+03;
  gl2[ 57] = 8.047330E+03;
  gl2[ 58] = 9.118820E+03;
  gl2[ 59] = 1.033298E+04;
  gl2[ 60] = 1.170880E+04;
  gl2[ 61] = 1.326780E+04;
  gl2[ 62] = 1.503439E+04;
  gl2[ 63] = 1.703620E+04;
  gl2[ 64] = 1.930454E+04;
  gl2[ 65] = 2.133482E+04;
  gl2[ 66] = 2.187491E+04;
  gl2[ 67] = 2.357862E+04;
  gl2[ 68] = 2.417552E+04;
  gl2[ 69] = 2.478752E+04;
  gl2[ 70] = 2.605841E+04;
  gl2[ 71] = 2.808794E+04;
  gl2[ 72] = 3.182781E+04;
  gl2[ 73] = 3.430669E+04;
  gl2[ 74] = 3.517517E+04;
  gl2[ 75] = 3.606563E+04;
  gl2[ 76] = 4.086771E+04;
  gl2[ 77] = 4.630919E+04;
  gl2[ 78] = 5.247518E+04;
  gl2[ 79] = 5.656217E+04;
  gl2[ 80] = 5.946217E+04;
  gl2[ 81] = 6.251086E+04;
  gl2[ 82] = 6.737947E+04;
  gl2[ 83] = 7.635094E+04;
  gl2[ 84] = 8.651695E+04;
  gl2[ 85] = 9.803655E+04;
  gl2[ 86] = 1.110900E+05;
  gl2[ 87] = 1.167857E+05;
  gl2[ 88] = 1.227734E+05;
  gl2[ 89] = 1.290681E+05;
  gl2[ 90] = 1.356856E+05;
  gl2[ 91] = 1.426423E+05;
  gl2[ 92] = 1.499558E+05;
  gl2[ 93] = 1.576442E+05;
  gl2[ 94] = 1.616349E+05;
  gl2[ 95] = 1.657268E+05;
  gl2[ 96] = 1.699221E+05;
  gl2[ 97] = 1.742237E+05;
  gl2[ 98] = 1.831564E+05;
  gl2[ 99] = 1.925470E+05;
  gl2[100] = 2.024191E+05;
  gl2[101] = 2.127974E+05;
  gl2[102] = 2.237077E+05;
  gl2[103] = 2.351775E+05;
  gl2[104] = 2.472353E+05;
  gl2[105] = 2.599113E+05;
  gl2[106] = 2.732372E+05;
  gl2[107] = 2.801543E+05;
  gl2[108] = 2.872464E+05;
  gl2[109] = 2.945181E+05;
  gl2[110] = 3.019738E+05;
  gl2[111] = 3.096183E+05;
  gl2[112] = 3.174564E+05;
  gl2[113] = 3.337327E+05;
  gl2[114] = 3.508435E+05;
  gl2[115] = 3.688317E+05;
  gl2[116] = 3.877421E+05;
  gl2[117] = 4.076220E+05;
  gl2[118] = 4.285213E+05;
  gl2[119] = 4.504920E+05;
  gl2[120] = 4.735892E+05;
  gl2[121] = 4.978707E+05;
  gl2[122] = 5.104743E+05;
  gl2[123] = 5.233971E+05;
  gl2[124] = 5.366469E+05;
  gl2[125] = 5.502322E+05;
  gl2[126] = 5.784432E+05;
  gl2[127] = 6.081006E+05;
  gl2[128] = 6.392786E+05;
  gl2[129] = 6.720551E+05;
  gl2[130] = 7.065121E+05;
  gl2[131] = 7.427358E+05;
  gl2[132] = 7.808167E+05;
  gl2[133] = 8.208500E+05;
  gl2[134] = 8.629359E+05;
  gl2[135] = 9.071795E+05;
  gl2[136] = 9.536916E+05;
  gl2[137] = 9.616402E+05;
  gl2[138] = 9.778344E+05;
  gl2[139] = 1.002588E+06;
  gl2[140] = 1.053992E+06;
  gl2[141] = 1.108032E+06;
  gl2[142] = 1.164842E+06;
  gl2[143] = 1.194330E+06;
  gl2[144] = 1.224564E+06;
  gl2[145] = 1.287349E+06;
  gl2[146] = 1.353353E+06;
  gl2[147] = 1.422741E+06;
  gl2[148] = 1.495686E+06;
  gl2[149] = 1.533550E+06;
  gl2[150] = 1.572372E+06;
  gl2[151] = 1.612176E+06;
  gl2[152] = 1.652989E+06;
  gl2[153] = 1.737739E+06;
  gl2[154] = 1.826835E+06;
  gl2[155] = 1.873082E+06;
  gl2[156] = 1.920499E+06;
  gl2[157] = 1.969117E+06;
  gl2[158] = 2.018965E+06;
  gl2[159] = 2.122480E+06;
  gl2[160] = 2.231302E+06;
  gl2[161] = 2.268877E+06;
  gl2[162] = 2.306855E+06;
  gl2[163] = 2.345703E+06;
  gl2[164] = 2.365253E+06;
  gl2[165] = 2.385205E+06;
  gl2[166] = 2.425130E+06;
  gl2[167] = 2.465970E+06;
  gl2[168] = 2.592403E+06;
  gl2[169] = 2.725318E+06;
  gl2[170] = 2.865048E+06;
  gl2[171] = 3.011942E+06;
  gl2[172] = 3.088190E+06;
  gl2[173] = 3.166368E+06;
  gl2[174] = 3.246525E+06;
  gl2[175] = 3.328711E+06;
  gl2[176] = 3.499377E+06;
  gl2[177] = 3.678794E+06;
  gl2[178] = 3.867410E+06;
  gl2[179] = 4.065697E+06;
  gl2[180] = 4.274149E+06;
  gl2[181] = 4.493290E+06;
  gl2[182] = 4.607038E+06;
  gl2[183] = 4.723666E+06;
  gl2[184] = 4.843246E+06;
  gl2[185] = 4.965853E+06;
  gl2[186] = 5.220458E+06;
  gl2[187] = 5.488116E+06;
  gl2[188] = 5.769498E+06;
  gl2[189] = 5.915554E+06;
  gl2[190] = 6.065307E+06;
  gl2[191] = 6.218851E+06;
  gl2[192] = 6.376282E+06;
  gl2[193] = 6.537698E+06;
  gl2[194] = 6.592384E+06;
  gl2[195] = 6.647595E+06;
  gl2[196] = 6.703200E+06;
  gl2[197] = 6.872893E+06;
  gl2[198] = 7.046881E+06;
  gl2[199] = 7.225274E+06;
  gl2[200] = 7.408182E+06;
  gl2[201] = 7.595721E+06;
  gl2[202] = 7.788008E+06;
  gl2[203] = 7.985162E+06;
  gl2[204] = 8.187308E+06;
  gl2[205] = 8.394570E+06;
  gl2[206] = 8.607080E+06;
  gl2[207] = 8.824969E+06;
  gl2[208] = 9.048374E+06;
  gl2[209] = 9.277435E+06;
  gl2[210] = 9.512294E+06;
  gl2[211] = 9.753099E+06;
  gl2[212] = 1.000000E+07;
  gl2[213] = 1.025315E+07;
  gl2[214] = 1.051271E+07;
  gl2[215] = 1.077884E+07;
  gl2[216] = 1.105171E+07;
  gl2[217] = 1.133148E+07;
  gl2[218] = 1.161834E+07;
  gl2[219] = 1.191246E+07;
  gl2[220] = 1.221403E+07;
  gl2[221] = 1.252323E+07;
  gl2[222] = 1.284025E+07;
  gl2[223] = 1.316531E+07;
  gl2[224] = 1.349859E+07;
  gl2[225] = 1.384031E+07;
  gl2[226] = 1.419068E+07;
  gl2[227] = 1.454991E+07;
  gl2[228] = 1.491825E+07;
  gl2[229] = 1.529590E+07;
  gl2[230] = 1.568312E+07;
  gl2[231] = 1.608014E+07;
  gl2[232] = 1.648721E+07;
  gl2[233] = 1.690459E+07;
  gl2[234] = 1.733253E+07;
  gl2[235] = 1.777131E+07;
  gl2[236] = 1.822119E+07;
  gl2[237] = 1.868246E+07;
  gl2[238] = 1.915541E+07;
  gl2[239] = 1.964033E+07;
  gl2[240] = 1.997048E+07;

  return gl2;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg3()
{
  vector<Real> eg3;
  eg3.resize(31);
  eg3[ 0] = 1.390000E-04;
  eg3[ 1] = 1.520000E-01;
  eg3[ 2] = 4.140000E-01;
  eg3[ 3] = 1.130000E+00;
  eg3[ 4] = 3.060000E+00;
  eg3[ 5] = 8.320000E+00;
  eg3[ 6] = 2.260000E+01;
  eg3[ 7] = 6.140000E+01;
  eg3[ 8] = 1.670000E+02;
  eg3[ 9] = 4.540000E+02;
  eg3[10] = 1.235000E+03;
  eg3[11] = 3.350000E+03;
  eg3[12] = 9.120000E+03;
  eg3[13] = 2.480000E+04;
  eg3[14] = 6.760000E+04;
  eg3[15] = 1.840000E+05;
  eg3[16] = 3.030000E+05;
  eg3[17] = 5.000000E+05;
  eg3[18] = 8.230000E+05;
  eg3[19] = 1.353000E+06;
  eg3[20] = 1.738000E+06;
  eg3[21] = 2.232000E+06;
  eg3[22] = 2.865000E+06;
  eg3[23] = 3.680000E+06;
  eg3[24] = 6.070000E+06;
  eg3[25] = 7.790000E+06;
  eg3[26] = 1.000000E+07;
  eg3[27] = 1.200000E+07;
  eg3[28] = 1.350000E+07;
  eg3[29] = 1.500000E+07;
  eg3[30] = 1.700000E+07;

  return eg3;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_gl4()
{
  vector<Real> gl4;
  gl4.resize(28);
  gl4[ 0] = 5.043477E+00;
  gl4[ 1] = 2.260329E+01;
  gl4[ 2] = 3.726653E+01;
  gl4[ 3] = 6.144212E+01;
  gl4[ 4] = 1.013009E+02;
  gl4[ 5] = 1.670170E+02;
  gl4[ 6] = 2.753645E+02;
  gl4[ 7] = 4.539993E+02;
  gl4[ 8] = 7.485183E+02;
  gl4[ 9] = 1.234098E+03;
  gl4[10] = 2.034684E+03;
  gl4[11] = 3.354626E+03;
  gl4[12] = 5.530844E+03;
  gl4[13] = 9.118820E+03;
  gl4[14] = 1.503439E+04;
  gl4[15] = 2.478752E+04;
  gl4[16] = 4.086771E+04;
  gl4[17] = 6.737947E+04;
  gl4[18] = 1.110900E+05;
  gl4[19] = 1.831564E+05;
  gl4[20] = 3.019738E+05;
  gl4[21] = 4.978707E+05;
  gl4[22] = 8.208500E+05;
  gl4[23] = 1.353353E+06;
  gl4[24] = 2.231302E+06;
  gl4[25] = 3.678794E+06;
  gl4[26] = 6.065307E+06;
  gl4[27] = 1.000000E+07;

  return gl4;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_gl5()
{
  vector<Real> gl5;
  gl5.resize(51);
  gl5[  0] =1.000021E-05;
  gl5[  1] =6.825603E-01;
  gl5[  2] =1.125352E+00;
  gl5[  3] =1.855391E+00;
  gl5[  4] =3.059023E+00;
  gl5[  5] =5.043477E+00;
  gl5[  6] =8.315287E+00;
  gl5[  7] =1.370959E+01;
  gl5[  8] =2.260329E+01;
  gl5[  9] =3.726653E+01;
  gl5[ 10] =6.144212E+01;
  gl5[ 11] =1.013009E+02;
  gl5[ 12] =1.670170E+02;
  gl5[ 13] =2.753645E+02;
  gl5[ 14] =3.535750E+02;
  gl5[ 15] =4.539993E+02;
  gl5[ 16] =5.829466E+02;
  gl5[ 17] =7.485183E+02;
  gl5[ 18] =9.611165E+02;
  gl5[ 19] =1.234098E+03;
  gl5[ 20] =1.584613E+03;
  gl5[ 21] =2.034684E+03;
  gl5[ 22] =2.612586E+03;
  gl5[ 23] =3.354626E+03;
  gl5[ 24] =4.307425E+03;
  gl5[ 25] =5.530844E+03;
  gl5[ 26] =7.101744E+03;
  gl5[ 27] =9.118820E+03;
  gl5[ 28] =1.170880E+04;
  gl5[ 29] =1.503439E+04;
  gl5[ 30] =1.930454E+04;
  gl5[ 31] =2.478752E+04;
  gl5[ 32] =3.182781E+04;
  gl5[ 33] =4.086771E+04;
  gl5[ 34] =5.247518E+04;
  gl5[ 35] =6.737947E+04;
  gl5[ 36] =8.651695E+04;
  gl5[ 37] =1.110900E+05;
  gl5[ 38] =1.426423E+05;
  gl5[ 39] =1.831564E+05;
  gl5[ 40] =2.351775E+05;
  gl5[ 41] =3.019738E+05;
  gl5[ 42] =3.877421E+05;
  gl5[ 43] =4.978707E+05;
  gl5[ 44] =8.208500E+05;
  gl5[ 45] =1.353353E+06;
  gl5[ 46] =2.231302E+06;
  gl5[ 47] =3.678794E+06;
  gl5[ 48] =6.065307E+06;
  gl5[ 49] =1.000000E+07;
  gl5[ 50] =1.997108E+07;

  return gl5;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg6()
{
  vector<Real> eg6;
  eg6.resize(36);
  eg6[  0] = 2.530000E-04;
  eg6[  1] = 2.277000E-03;
  eg6[  2] = 6.325000E-03;
  eg6[  3] = 1.239700E-02;
  eg6[  4] = 2.049300E-02;
  eg6[  5] = 3.061300E-02;
  eg6[  6] = 4.275700E-02;
  eg6[  7] = 5.692500E-02;
  eg6[  8] = 8.197200E-02;
  eg6[  9] = 1.115900E-01;
  eg6[ 10] = 1.457300E-01;
  eg6[ 11] = 1.844400E-01;
  eg6[ 12] = 2.277000E-01;
  eg6[ 13] = 2.510400E-01;
  eg6[ 14] = 2.705300E-01;
  eg6[ 15] = 2.907500E-01;
  eg6[ 16] = 3.011300E-01;
  eg6[ 17] = 3.206400E-01;
  eg6[ 18] = 3.576800E-01;
  eg6[ 19] = 4.170400E-01;
  eg6[ 20] = 5.032600E-01;
  eg6[ 21] = 6.249300E-01;
  eg6[ 22] = 7.821100E-01;
  eg6[ 23] = 9.507000E-01;
  eg6[ 24] = 1.013700E+00;
  eg6[ 25] = 1.042800E+00;
  eg6[ 26] = 1.052500E+00;
  eg6[ 27] = 1.062400E+00;
  eg6[ 28] = 1.072200E+00;
  eg6[ 29] = 1.098700E+00;
  eg6[ 30] = 1.166400E+00;
  eg6[ 31] = 1.307900E+00;
  eg6[ 32] = 1.457500E+00;
  eg6[ 33] = 1.595000E+00;
  eg6[ 34] = 1.726200E+00;
  eg6[ 35] = 1.855000E+00;

  return eg6;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg9()
{
  vector<Real> eg9;
  eg9.resize(70);
  eg9[  0] = 1.000000E-05;
  eg9[  1] = 5.000000E-03;
  eg9[  2] = 1.000000E-02;
  eg9[  3] = 1.500000E-02;
  eg9[  4] = 2.000000E-02;
  eg9[  5] = 2.500000E-02;
  eg9[  6] = 3.000000E-02;
  eg9[  7] = 3.500000E-02;
  eg9[  8] = 4.200000E-02;
  eg9[  9] = 5.000000E-02;
  eg9[ 10] = 5.800000E-02;
  eg9[ 11] = 6.700000E-02;
  eg9[ 12] = 8.000000E-02;
  eg9[ 13] = 1.000000E-01;
  eg9[ 14] = 1.400000E-01;
  eg9[ 15] = 1.800000E-01;
  eg9[ 16] = 2.200000E-01;
  eg9[ 17] = 2.500000E-01;
  eg9[ 18] = 2.800000E-01;
  eg9[ 19] = 3.000000E-01;
  eg9[ 20] = 3.200000E-01;
  eg9[ 21] = 3.500000E-01;
  eg9[ 22] = 4.000000E-01;
  eg9[ 23] = 5.000000E-01;
  eg9[ 24] = 6.250000E-01;
  eg9[ 25] = 7.800000E-01;
  eg9[ 26] = 8.500000E-01;
  eg9[ 27] = 9.100000E-01;
  eg9[ 28] = 9.500000E-01;
  eg9[ 29] = 9.720000E-01;
  eg9[ 30] = 9.960000E-01;
  eg9[ 31] = 1.020000E+00;
  eg9[ 32] = 1.045000E+00;
  eg9[ 33] = 1.071000E+00;
  eg9[ 34] = 1.097000E+00;
  eg9[ 35] = 1.123000E+00;
  eg9[ 36] = 1.150000E+00;
  eg9[ 37] = 1.300000E+00;
  eg9[ 38] = 1.500000E+00;
  eg9[ 39] = 2.100000E+00;
  eg9[ 40] = 2.600000E+00;
  eg9[ 41] = 3.300000E+00;
  eg9[ 42] = 4.000000E+00;
  eg9[ 43] = 9.877000E+00;
  eg9[ 44] = 1.596800E+01;
  eg9[ 45] = 2.770000E+01;
  eg9[ 46] = 4.805200E+01;
  eg9[ 47] = 7.550100E+01;
  eg9[ 48] = 1.487280E+02;
  eg9[ 49] = 3.672620E+02;
  eg9[ 50] = 9.068980E+02;
  eg9[ 51] = 1.425100E+03;
  eg9[ 52] = 2.239450E+03;
  eg9[ 53] = 3.519100E+03;
  eg9[ 54] = 5.530000E+03;
  eg9[ 55] = 9.118000E+03;
  eg9[ 56] = 1.503000E+04;
  eg9[ 57] = 2.478000E+04;
  eg9[ 58] = 4.085000E+04;
  eg9[ 59] = 6.734000E+04;
  eg9[ 60] = 1.110000E+05;
  eg9[ 61] = 1.830000E+05;
  eg9[ 62] = 3.025000E+05;
  eg9[ 63] = 5.000000E+05;
  eg9[ 64] = 8.210000E+05;
  eg9[ 65] = 1.353000E+06;
  eg9[ 66] = 2.231000E+06;
  eg9[ 67] = 3.679000E+06;
  eg9[ 68] = 6.065500E+06;
  eg9[ 69] = 1.000000E+07;

  return eg9;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg10a()
{
  vector<Real> eg10a;
  eg10a.resize(48);
  eg10a[  0] = 1.000000E-05;
  eg10a[  1] = 2.539900E-04;
  eg10a[  2] = 7.602200E-04;
  eg10a[  3] = 2.276900E-03;
  eg10a[  4] = 6.324700E-03;
  eg10a[  5] = 1.239600E-02;
  eg10a[  6] = 2.049200E-02;
  eg10a[  7] = 2.550000E-02;
  eg10a[  8] = 3.061200E-02;
  eg10a[  9] = 3.550000E-02;
  eg10a[ 10] = 4.275500E-02;
  eg10a[ 11] = 5.000000E-02;
  eg10a[ 12] = 5.692200E-02;
  eg10a[ 13] = 6.700000E-02;
  eg10a[ 14] = 8.196800E-02;
  eg10a[ 15] = 1.115700E-01;
  eg10a[ 16] = 1.457200E-01;
  eg10a[ 17] = 1.523000E-01;
  eg10a[ 18] = 1.844300E-01;
  eg10a[ 19] = 2.276900E-01;
  eg10a[ 20] = 2.510300E-01;
  eg10a[ 21] = 2.705200E-01;
  eg10a[ 22] = 2.907400E-01;
  eg10a[ 23] = 3.011200E-01;
  eg10a[ 24] = 3.206300E-01;
  eg10a[ 25] = 3.576700E-01;
  eg10a[ 26] = 4.149900E-01;
  eg10a[ 27] = 5.032300E-01;
  eg10a[ 28] = 6.250600E-01;
  eg10a[ 29] = 7.820800E-01;
  eg10a[ 30] = 8.336800E-01;
  eg10a[ 31] = 8.764200E-01;
  eg10a[ 32] = 9.100000E-01;
  eg10a[ 33] = 9.506500E-01;
  eg10a[ 34] = 9.710000E-01;
  eg10a[ 35] = 9.920000E-01;
  eg10a[ 36] = 1.013700E+00;
  eg10a[ 37] = 1.042700E+00;
  eg10a[ 38] = 1.052500E+00;
  eg10a[ 39] = 1.062300E+00;
  eg10a[ 40] = 1.072200E+00;
  eg10a[ 41] = 1.098700E+00;
  eg10a[ 42] = 1.125400E+00;
  eg10a[ 43] = 1.166400E+00;
  eg10a[ 44] = 1.307900E+00;
  eg10a[ 45] = 1.457400E+00;
  eg10a[ 46] = 1.594900E+00;
  eg10a[ 47] = 1.726100E+00;

  return eg10a;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg10b()
{
  vector<Real> eg10b;
  eg10b.resize(13);
  eg10b[  0] = 1.100000E+07;
  eg10b[  1] = 1.200000E+07;
  eg10b[  2] = 1.300000E+07;
  eg10b[  3] = 1.350000E+07;
  eg10b[  4] = 1.375000E+07;
  eg10b[  5] = 1.394000E+07;
  eg10b[  6] = 1.420000E+07;
  eg10b[  7] = 1.442000E+07;
  eg10b[  8] = 1.464000E+07;
  eg10b[  9] = 1.500000E+07;
  eg10b[ 10] = 1.600000E+07;
  eg10b[ 11] = 1.700000E+07;
  eg10b[ 12] = 2.000000E+07;

  return eg10b;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg11()
{
  vector<Real> eg11;
  eg11.resize(71);
  eg11[  0] = 1.067700E+01;
  eg11[  1] = 6.144210E+01;
  eg11[  2] = 1.013010E+02;
  eg11[  3] = 1.300730E+02;
  eg11[  4] = 1.670170E+02;
  eg11[  5] = 2.144540E+02;
  eg11[  6] = 2.753650E+02;
  eg11[  7] = 3.535750E+02;
  eg11[  8] = 4.539990E+02;
  eg11[  9] = 5.829470E+02;
  eg11[ 10] = 7.485180E+02;
  eg11[ 11] = 9.611170E+02;
  eg11[ 12] = 1.089090E+03;
  eg11[ 13] = 1.234100E+03;
  eg11[ 14] = 1.398420E+03;
  eg11[ 15] = 1.584610E+03;
  eg11[ 16] = 1.795600E+03;
  eg11[ 17] = 2.034680E+03;
  eg11[ 18] = 2.305600E+03;
  eg11[ 19] = 2.612590E+03;
  eg11[ 20] = 2.960450E+03;
  eg11[ 21] = 3.354630E+03;
  eg11[ 22] = 3.801290E+03;
  eg11[ 23] = 4.307430E+03;
  eg11[ 24] = 4.880950E+03;
  eg11[ 25] = 5.530840E+03;
  eg11[ 26] = 6.267270E+03;
  eg11[ 27] = 7.101740E+03;
  eg11[ 28] = 8.047330E+03;
  eg11[ 29] = 9.118820E+03;
  eg11[ 30] = 1.033330E+04;
  eg11[ 31] = 1.170880E+04;
  eg11[ 32] = 1.326780E+04;
  eg11[ 33] = 1.503440E+04;
  eg11[ 34] = 1.703620E+04;
  eg11[ 35] = 1.930450E+04;
  eg11[ 36] = 2.187490E+04;
  eg11[ 37] = 2.478750E+04;
  eg11[ 38] = 2.808790E+04;
  eg11[ 39] = 3.182780E+04;
  eg11[ 40] = 4.086770E+04;
  eg11[ 41] = 5.247520E+04;
  eg11[ 42] = 6.737950E+04;
  eg11[ 43] = 8.651700E+04;
  eg11[ 44] = 1.110900E+05;
  eg11[ 45] = 1.426420E+05;
  eg11[ 46] = 1.831560E+05;
  eg11[ 47] = 2.351780E+05;
  eg11[ 48] = 3.019740E+05;
  eg11[ 49] = 3.877420E+05;
  eg11[ 50] = 4.393690E+05;
  eg11[ 51] = 4.978710E+05;
  eg11[ 52] = 5.641610E+05;
  eg11[ 53] = 6.392790E+05;
  eg11[ 54] = 7.243980E+05;
  eg11[ 55] = 8.208500E+05;
  eg11[ 56] = 9.301450E+05;
  eg11[ 57] = 1.053990E+06;
  eg11[ 58] = 1.194330E+06;
  eg11[ 59] = 1.353350E+06;
  eg11[ 60] = 1.737740E+06;
  eg11[ 61] = 2.231300E+06;
  eg11[ 62] = 2.865050E+06;
  eg11[ 63] = 3.678790E+06;
  eg11[ 64] = 4.723670E+06;
  eg11[ 65] = 6.065310E+06;
  eg11[ 66] = 7.788010E+06;
  eg11[ 67] = 1.000000E+07;
  eg11[ 68] = 1.284030E+07;
  eg11[ 69] = 1.648720E+07;
  eg11[ 70] = 2.000000E+07;

  return eg11;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_deltl()
{
  vector<Real> deltl;
  deltl.resize(8);
  deltl[0] = 5.000000E+00;
  deltl[1] = 7.500000E+00;
  deltl[2] = 1.000000E+01;
  deltl[3] = 1.500000E+01;
  deltl[4] = 2.000000E+01;
  deltl[5] = 2.500000E+01;
  deltl[6] = 3.000000E+01;
  deltl[7] = 4.000000E+01;

  return deltl;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<int> GroupStructureNeutron::get_ndelta()
{
  vector<int> ndelta;
  ndelta.resize(9);
  ndelta[0] =  2;
  ndelta[1] =  6;
  ndelta[2] = 10;
  ndelta[3] = 19;
  ndelta[4] = 23;
  ndelta[5] = 28;
  ndelta[6] = 36;
  ndelta[7] = 40;
  ndelta[8] = 46;

  return ndelta;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_u80()
{
  vector<Real> u80;
  u80.resize(81);
  u80[  0] = 1.681472E-01;
  u80[  1] = 1.250000E-01;
  u80[  2] = 1.000000E-01;
  u80[  3] = 1.250000E-01;
  u80[  4] = 1.750000E-01;
  u80[  5] = 2.500000E-01;
  u80[  6] = 2.500000E-01;
  u80[  7] = 2.500000E-01;
  u80[  8] = 2.500000E-01;
  u80[  9] = 2.500000E-01;
  u80[ 10] = 2.500000E-01;
  u80[ 11] = 2.500000E-01;
  u80[ 12] = 2.500000E-01;
  u80[ 13] = 1.250000E-01;
  u80[ 14] = 1.250000E-01;
  u80[ 15] = 1.250000E-01;
  u80[ 16] = 1.250000E-01;
  u80[ 17] = 1.250000E-01;
  u80[ 18] = 1.250000E-01;
  u80[ 19] = 1.250000E-01;
  u80[ 20] = 1.250000E-01;
  u80[ 21] = 1.250000E-01;
  u80[ 22] = 1.250000E-01;
  u80[ 23] = 2.500000E-01;
  u80[ 24] = 2.500000E-01;
  u80[ 25] = 2.500000E-01;
  u80[ 26] = 2.500000E-01;
  u80[ 27] = 2.500000E-01;
  u80[ 28] = 2.500000E-01;
  u80[ 29] = 2.500000E-01;
  u80[ 30] = 2.500000E-01;
  u80[ 31] = 2.500000E-01;
  u80[ 32] = 2.500000E-01;
  u80[ 33] = 1.250000E-01;
  u80[ 34] = 7.500000E-02;
  u80[ 35] = 5.000000E-02;
  u80[ 36] = 1.250000E-01;
  u80[ 37] = 1.250000E-01;
  u80[ 38] = 1.250000E-01;
  u80[ 39] = 1.250000E-01;
  u80[ 40] = 1.250000E-01;
  u80[ 41] = 1.250000E-01;
  u80[ 42] = 1.250000E-01;
  u80[ 43] = 1.250000E-01;
  u80[ 44] = 1.250000E-01;
  u80[ 45] = 1.250000E-01;
  u80[ 46] = 1.250000E-01;
  u80[ 47] = 1.250000E-01;
  u80[ 48] = 1.250000E-01;
  u80[ 49] = 1.250000E-01;
  u80[ 50] = 1.250000E-01;
  u80[ 51] = 1.250000E-01;
  u80[ 52] = 1.250000E-01;
  u80[ 53] = 1.250000E-01;
  u80[ 54] = 1.250000E-01;
  u80[ 55] = 1.250000E-01;
  u80[ 56] = 1.250000E-01;
  u80[ 57] = 1.250000E-01;
  u80[ 58] = 1.250000E-01;
  u80[ 59] = 1.250000E-01;
  u80[ 60] = 1.250000E-01;
  u80[ 61] = 1.250000E-01;
  u80[ 62] = 2.500000E-01;
  u80[ 63] = 2.500000E-01;
  u80[ 64] = 2.500000E-01;
  u80[ 65] = 2.500000E-01;
  u80[ 66] = 2.500000E-01;
  u80[ 67] = 5.000000E-01;
  u80[ 68] = 5.000000E-01;
  u80[ 69] = 5.000000E-01;
  u80[ 70] = 5.000000E-01;
  u80[ 71] = 5.000000E-01;
  u80[ 72] = 5.000000E-01;
  u80[ 73] = 5.000000E-01;
  u80[ 74] = 5.000000E-01;
  u80[ 75] = 5.000000E-01;
  u80[ 76] = 1.000000E+00;
  u80[ 77] = 1.000000E+00;
  u80[ 78] = 1.000000E+00;
  u80[ 79] = 7.000000E+00;
  u80[ 80] = 0.000000E+00;

  return u80;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<int> GroupStructureNeutron::get_ig14()
{
  vector<int> ig14;
  ig14.resize(19);
  ig14[  0] =   2;
  ig14[  1] =   9;
  ig14[  2] =  13;
  ig14[  3] =  15;
  ig14[  4] =  17;
  ig14[  5] =  23;
  ig14[  6] =  25;
  ig14[  7] =  55;
  ig14[  8] =  60;
  ig14[  9] =  61;
  ig14[ 10] =  63;
  ig14[ 11] =  64;
  ig14[ 12] =  65;
  ig14[ 13] =  93;
  ig14[ 14] =  94;
  ig14[ 15] =  95;
  ig14[ 16] =  99;
  ig14[ 17] = 100;
  ig14[ 18] = 101;

  return ig14;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_gl14()
{
  vector<Real> gl14;
  gl14.resize(19);
  gl14[  0] = 1.000000E-01;
  gl14[  1] = 5.000000E-02;
  gl14[  2] = 1.000000E-01;
  gl14[  3] = 5.000000E-02;
  gl14[  4] = 1.000000E-01;
  gl14[  5] = 5.000000E-02;
  gl14[  6] = 1.000000E-01;
  gl14[  7] = 2.500000E-01;
  gl14[  8] = 2.000000E-01;
  gl14[  9] = 5.000000E-02;
  gl14[ 10] = 7.500000E-02;
  gl14[ 11] = 1.250000E-01;
  gl14[ 12] = 2.500000E-01;
  gl14[ 13] = 5.000000E-01;
  gl14[ 14] = 2.500000E-01;
  gl14[ 15] = 5.000000E-01;
  gl14[ 16] = 5.880000E-01;
  gl14[ 17] = 4.120000E-01;
  gl14[ 18] = 1.063100E+01;

  return gl14;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg16()
{
  vector<Real> eg16;
  eg16.resize(175);
  eg16[  0] = 1.000000E-05;
  eg16[  1] = 1.000000E-01;
  eg16[  2] = 4.139900E-01;
  eg16[  3] = 5.315800E-01;
  eg16[  4] = 6.825600E-01;
  eg16[  5] = 8.764200E-01;
  eg16[  6] = 1.125400E+00;
  eg16[  7] = 1.445000E+00;
  eg16[  8] = 1.855400E+00;
  eg16[  9] = 2.382400E+00;
  eg16[ 10] = 3.059000E+00;
  eg16[ 11] = 3.927900E+00;
  eg16[ 12] = 5.043500E+00;
  eg16[ 13] = 6.476000E+00;
  eg16[ 14] = 8.315300E+00;
  eg16[ 15] = 1.067700E+01;
  eg16[ 16] = 1.371000E+01;
  eg16[ 17] = 1.760300E+01;
  eg16[ 18] = 2.260300E+01;
  eg16[ 19] = 2.902300E+01;
  eg16[ 20] = 3.726700E+01;
  eg16[ 21] = 4.785100E+01;
  eg16[ 22] = 6.144200E+01;
  eg16[ 23] = 7.889300E+01;
  eg16[ 24] = 1.013000E+02;
  eg16[ 25] = 1.300700E+02;
  eg16[ 26] = 1.670200E+02;
  eg16[ 27] = 2.144500E+02;
  eg16[ 28] = 2.753600E+02;
  eg16[ 29] = 3.535800E+02;
  eg16[ 30] = 4.540000E+02;
  eg16[ 31] = 5.829500E+02;
  eg16[ 32] = 7.485200E+02;
  eg16[ 33] = 9.611200E+02;
  eg16[ 34] = 1.234100E+03;
  eg16[ 35] = 1.584600E+03;
  eg16[ 36] = 2.034700E+03;
  eg16[ 37] = 2.248700E+03;
  eg16[ 38] = 2.485200E+03;
  eg16[ 39] = 2.612600E+03;
  eg16[ 40] = 2.746500E+03;
  eg16[ 41] = 3.035400E+03;
  eg16[ 42] = 3.354600E+03;
  eg16[ 43] = 3.707400E+03;
  eg16[ 44] = 4.307400E+03;
  eg16[ 45] = 5.530800E+03;
  eg16[ 46] = 7.101700E+03;
  eg16[ 47] = 9.118800E+03;
  eg16[ 48] = 1.059500E+04;
  eg16[ 49] = 1.170900E+04;
  eg16[ 50] = 1.503400E+04;
  eg16[ 51] = 1.930500E+04;
  eg16[ 52] = 2.187500E+04;
  eg16[ 53] = 2.357900E+04;
  eg16[ 54] = 2.417600E+04;
  eg16[ 55] = 2.478800E+04;
  eg16[ 56] = 2.605800E+04;
  eg16[ 57] = 2.700000E+04;
  eg16[ 58] = 2.850000E+04;
  eg16[ 59] = 3.182800E+04;
  eg16[ 60] = 3.430700E+04;
  eg16[ 61] = 4.086800E+04;
  eg16[ 62] = 4.630900E+04;
  eg16[ 63] = 5.247500E+04;
  eg16[ 64] = 5.656200E+04;
  eg16[ 65] = 6.737900E+04;
  eg16[ 66] = 7.200000E+04;
  eg16[ 67] = 7.950000E+04;
  eg16[ 68] = 8.250000E+04;
  eg16[ 69] = 8.651700E+04;
  eg16[ 70] = 9.803700E+04;
  eg16[ 71] = 1.110900E+05;
  eg16[ 72] = 1.167900E+05;
  eg16[ 73] = 1.227700E+05;
  eg16[ 74] = 1.290700E+05;
  eg16[ 75] = 1.356900E+05;
  eg16[ 76] = 1.426400E+05;
  eg16[ 77] = 1.499600E+05;
  eg16[ 78] = 1.576400E+05;
  eg16[ 79] = 1.657300E+05;
  eg16[ 80] = 1.742200E+05;
  eg16[ 81] = 1.831600E+05;
  eg16[ 82] = 1.925500E+05;
  eg16[ 83] = 2.024200E+05;
  eg16[ 84] = 2.128000E+05;
  eg16[ 85] = 2.237100E+05;
  eg16[ 86] = 2.351800E+05;
  eg16[ 87] = 2.472400E+05;
  eg16[ 88] = 2.732400E+05;
  eg16[ 89] = 2.872500E+05;
  eg16[ 90] = 2.945200E+05;
  eg16[ 91] = 2.972000E+05;
  eg16[ 92] = 2.985000E+05;
  eg16[ 93] = 3.019700E+05;
  eg16[ 94] = 3.337300E+05;
  eg16[ 95] = 3.688300E+05;
  eg16[ 96] = 3.877400E+05;
  eg16[ 97] = 4.076200E+05;
  eg16[ 98] = 4.504900E+05;
  eg16[ 99] = 4.978700E+05;
  eg16[100] = 5.234000E+05;
  eg16[101] = 5.502300E+05;
  eg16[102] = 5.784400E+05;
  eg16[103] = 6.081000E+05;
  eg16[104] = 6.392800E+05;
  eg16[105] = 6.720600E+05;
  eg16[106] = 7.065100E+05;
  eg16[107] = 7.427400E+05;
  eg16[108] = 7.808200E+05;
  eg16[109] = 8.208500E+05;
  eg16[110] = 8.629400E+05;
  eg16[111] = 9.071800E+05;
  eg16[112] = 9.616400E+05;
  eg16[113] = 1.002600E+06;
  eg16[114] = 1.108000E+06;
  eg16[115] = 1.164800E+06;
  eg16[116] = 1.224600E+06;
  eg16[117] = 1.287300E+06;
  eg16[118] = 1.353400E+06;
  eg16[119] = 1.422700E+06;
  eg16[120] = 1.495700E+06;
  eg16[121] = 1.572400E+06;
  eg16[122] = 1.653000E+06;
  eg16[123] = 1.737700E+06;
  eg16[124] = 1.826800E+06;
  eg16[125] = 1.920500E+06;
  eg16[126] = 2.019000E+06;
  eg16[127] = 2.122500E+06;
  eg16[128] = 2.231300E+06;
  eg16[129] = 2.306900E+06;
  eg16[130] = 2.345700E+06;
  eg16[131] = 2.365300E+06;
  eg16[132] = 2.385200E+06;
  eg16[133] = 2.466000E+06;
  eg16[134] = 2.592400E+06;
  eg16[135] = 2.725300E+06;
  eg16[136] = 2.865000E+06;
  eg16[137] = 3.011900E+06;
  eg16[138] = 3.166400E+06;
  eg16[139] = 3.328700E+06;
  eg16[140] = 3.678800E+06;
  eg16[141] = 4.065700E+06;
  eg16[142] = 4.493300E+06;
  eg16[143] = 4.723700E+06;
  eg16[144] = 4.965900E+06;
  eg16[145] = 5.220500E+06;
  eg16[146] = 5.488100E+06;
  eg16[147] = 5.769500E+06;
  eg16[148] = 6.065300E+06;
  eg16[149] = 6.376300E+06;
  eg16[150] = 6.592400E+06;
  eg16[151] = 6.703200E+06;
  eg16[152] = 7.046900E+06;
  eg16[153] = 7.408200E+06;
  eg16[154] = 7.788000E+06;
  eg16[155] = 8.187300E+06;
  eg16[156] = 8.607100E+06;
  eg16[157] = 9.048400E+06;
  eg16[158] = 9.512300E+06;
  eg16[159] = 1.000000E+07;
  eg16[160] = 1.051300E+07;
  eg16[161] = 1.105200E+07;
  eg16[162] = 1.161800E+07;
  eg16[163] = 1.221400E+07;
  eg16[164] = 1.252300E+07;
  eg16[165] = 1.349900E+07;
  eg16[166] = 1.384000E+07;
  eg16[167] = 1.419100E+07;
  eg16[168] = 1.455000E+07;
  eg16[169] = 1.491800E+07;
  eg16[170] = 1.568300E+07;
  eg16[171] = 1.648700E+07;
  eg16[172] = 1.690500E+07;
  eg16[173] = 1.733300E+07;
  eg16[174] = 1.964000E+07;

  return eg16;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg17()
{
  vector<Real> eg17 = get_eg16();
  eg17.resize(176);
  eg17[165] = 1.284000E+07;
  eg17[166] = 1.349900E+07;
  eg17[167] = 1.384000E+07;
  eg17[168] = 1.419100E+07;
  eg17[169] = 1.455000E+07;
  eg17[170] = 1.491800E+07;
  eg17[171] = 1.568300E+07;
  eg17[172] = 1.648700E+07;
  eg17[173] = 1.690500E+07;
  eg17[174] = 1.733300E+07;
  eg17[175] = 1.964000E+07;

  return eg17;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg18()
{
  vector<Real> eg18;
  eg18.resize(173);
  eg18[  0] = 1.964030E+07;
  eg18[  1] = 1.733250E+07;
  eg18[  2] = 1.491820E+07;
  eg18[  3] = 1.384030E+07;
  eg18[  4] = 1.161830E+07;
  eg18[  5] = 1.000000E+07;
  eg18[  6] = 8.187310E+06;
  eg18[  7] = 6.703200E+06;
  eg18[  8] = 6.065310E+06;
  eg18[  9] = 5.488120E+06;
  eg18[ 10] = 4.493290E+06;
  eg18[ 11] = 3.678790E+06;
  eg18[ 12] = 3.011940E+06;
  eg18[ 13] = 2.465970E+06;
  eg18[ 14] = 2.231300E+06;
  eg18[ 15] = 2.018970E+06;
  eg18[ 16] = 1.652990E+06;
  eg18[ 17] = 1.353350E+06;
  eg18[ 18] = 1.224560E+06;
  eg18[ 19] = 1.108030E+06;
  eg18[ 20] = 1.002590E+06;
  eg18[ 21] = 9.071800E+05;
  eg18[ 22] = 8.208500E+05;
  eg18[ 23] = 6.081010E+05;
  eg18[ 24] = 5.502320E+05;
  eg18[ 25] = 4.978710E+05;
  eg18[ 26] = 4.504920E+05;
  eg18[ 27] = 4.076220E+05;
  eg18[ 28] = 3.019740E+05;
  eg18[ 29] = 2.732370E+05;
  eg18[ 30] = 2.472350E+05;
  eg18[ 31] = 1.831560E+05;
  eg18[ 32] = 1.227730E+05;
  eg18[ 33] = 1.110900E+05;
  eg18[ 34] = 8.229750E+04;
  eg18[ 35] = 6.737950E+04;
  eg18[ 36] = 5.516560E+04;
  eg18[ 37] = 4.086770E+04;
  eg18[ 38] = 3.697860E+04;
  eg18[ 39] = 2.928300E+04;
  eg18[ 40] = 2.739440E+04;
  eg18[ 41] = 2.478750E+04;
  eg18[ 42] = 1.661560E+04;
  eg18[ 43] = 1.503440E+04;
  eg18[ 44] = 1.113780E+04;
  eg18[ 45] = 9.118820E+03;
  eg18[ 46] = 7.465860E+03;
  eg18[ 47] = 5.530840E+03;
  eg18[ 48] = 5.004510E+03;
  eg18[ 49] = 3.526620E+03;
  eg18[ 50] = 3.354630E+03;
  eg18[ 51] = 2.248670E+03;
  eg18[ 52] = 2.034680E+03;
  eg18[ 53] = 1.507330E+03;
  eg18[ 54] = 1.433820E+03;
  eg18[ 55] = 1.234100E+03;
  eg18[ 56] = 1.010390E+03;
  eg18[ 57] = 9.142420E+02;
  eg18[ 58] = 7.485180E+02;
  eg18[ 59] = 6.772870E+02;
  eg18[ 60] = 4.539990E+02;
  eg18[ 61] = 3.717030E+02;
  eg18[ 62] = 3.043250E+02;
  eg18[ 63] = 2.039950E+02;
  eg18[ 64] = 1.486250E+02;
  eg18[ 65] = 1.367420E+02;
  eg18[ 66] = 9.166090E+01;
  eg18[ 67] = 7.567360E+01;
  eg18[ 68] = 6.790410E+01;
  eg18[ 69] = 5.559510E+01;
  eg18[ 70] = 5.157800E+01;
  eg18[ 71] = 4.825160E+01;
  eg18[ 72] = 4.551740E+01;
  eg18[ 73] = 4.016900E+01;
  eg18[ 74] = 3.726650E+01;
  eg18[ 75] = 3.372010E+01;
  eg18[ 76] = 3.051130E+01;
  eg18[ 77] = 2.760770E+01;
  eg18[ 78] = 2.498050E+01;
  eg18[ 79] = 2.260330E+01;
  eg18[ 80] = 1.945480E+01;
  eg18[ 81] = 1.592830E+01;
  eg18[ 82] = 1.370960E+01;
  eg18[ 83] = 1.122450E+01;
  eg18[ 84] = 9.905550E+00;
  eg18[ 85] = 9.189810E+00;
  eg18[ 86] = 8.315290E+00;
  eg18[ 87] = 7.523980E+00;
  eg18[ 88] = 6.160120E+00;
  eg18[ 89] = 5.346430E+00;
  eg18[ 90] = 5.043480E+00;
  eg18[ 91] = 4.129250E+00;
  eg18[ 92] = 4.000000E+00;
  eg18[ 93] = 3.380750E+00;
  eg18[ 94] = 3.300000E+00;
  eg18[ 95] = 2.767920E+00;
  eg18[ 96] = 2.720000E+00;
  eg18[ 97] = 2.600000E+00;
  eg18[ 98] = 2.550000E+00;
  eg18[ 99] = 2.360000E+00;
  eg18[100] = 2.130000E+00;
  eg18[101] = 2.100000E+00;
  eg18[102] = 2.020000E+00;
  eg18[103] = 1.930000E+00;
  eg18[104] = 1.840000E+00;
  eg18[105] = 1.755000E+00;
  eg18[106] = 1.670000E+00;
  eg18[107] = 1.590000E+00;
  eg18[108] = 1.500000E+00;
  eg18[109] = 1.475000E+00;
  eg18[110] = 1.444980E+00;
  eg18[111] = 1.370000E+00;
  eg18[112] = 1.337500E+00;
  eg18[113] = 1.300000E+00;
  eg18[114] = 1.235000E+00;
  eg18[115] = 1.170000E+00;
  eg18[116] = 1.150000E+00;
  eg18[117] = 1.125350E+00;
  eg18[118] = 1.110000E+00;
  eg18[119] = 1.097000E+00;
  eg18[120] = 1.071000E+00;
  eg18[121] = 1.045000E+00;
  eg18[122] = 1.035000E+00;
  eg18[123] = 1.020000E+00;
  eg18[124] = 9.960000E-01;
  eg18[125] = 9.860000E-01;
  eg18[126] = 9.720000E-01;
  eg18[127] = 9.500000E-01;
  eg18[128] = 9.300000E-01;
  eg18[129] = 9.100000E-01;
  eg18[130] = 8.600000E-01;
  eg18[131] = 8.500000E-01;
  eg18[132] = 7.900000E-01;
  eg18[133] = 7.800000E-01;
  eg18[134] = 7.050000E-01;
  eg18[135] = 6.250000E-01;
  eg18[136] = 5.400000E-01;
  eg18[137] = 5.000000E-01;
  eg18[138] = 4.850000E-01;
  eg18[139] = 4.330000E-01;
  eg18[140] = 4.000000E-01;
  eg18[141] = 3.910000E-01;
  eg18[142] = 3.500000E-01;
  eg18[143] = 3.200000E-01;
  eg18[144] = 3.145000E-01;
  eg18[145] = 3.000000E-01;
  eg18[146] = 2.800000E-01;
  eg18[147] = 2.480000E-01;
  eg18[148] = 2.200000E-01;
  eg18[149] = 1.890000E-01;
  eg18[150] = 1.800000E-01;
  eg18[151] = 1.600000E-01;
  eg18[152] = 1.400000E-01;
  eg18[153] = 1.340000E-01;
  eg18[154] = 1.150000E-01;
  eg18[155] = 1.000010E-01;
  eg18[156] = 9.500000E-02;
  eg18[157] = 8.000000E-02;
  eg18[158] = 7.700000E-02;
  eg18[159] = 6.700000E-02;
  eg18[160] = 5.800000E-02;
  eg18[161] = 5.000000E-02;
  eg18[162] = 4.200000E-02;
  eg18[163] = 3.500000E-02;
  eg18[164] = 3.000000E-02;
  eg18[165] = 2.500000E-02;
  eg18[166] = 2.000000E-02;
  eg18[167] = 1.500000E-02;
  eg18[168] = 1.000000E-02;
  eg18[169] = 6.900000E-03;
  eg18[170] = 5.000000E-03;
  eg18[171] = 3.000000E-03;
  eg18[172] = 1.000010E-05;

  return eg18;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg19()
{
  vector<Real> eg19;
  eg19.resize(34);
  eg19[  0] = 1.000010E-05;
  eg19[  1] = 1.000000E-01;
  eg19[  2] = 5.400000E-01;
  eg19[  3] = 4.000000E+00;
  eg19[  4] = 8.315287E+00;
  eg19[  5] = 1.370959E+01;
  eg19[  6] = 2.260329E+01;
  eg19[  7] = 4.016900E+01;
  eg19[  8] = 6.790405E+01;
  eg19[  9] = 9.166088E+01;
  eg19[ 10] = 1.486254E+02;
  eg19[ 11] = 3.043248E+02;
  eg19[ 12] = 4.539993E+02;
  eg19[ 13] = 7.485183E+02;
  eg19[ 14] = 1.234098E+03;
  eg19[ 15] = 2.034684E+03;
  eg19[ 16] = 3.354626E+03;
  eg19[ 17] = 5.530844E+03;
  eg19[ 18] = 9.118820E+03;
  eg19[ 19] = 1.503439E+04;
  eg19[ 20] = 2.478752E+04;
  eg19[ 21] = 4.086771E+04;
  eg19[ 22] = 6.737947E+04;
  eg19[ 23] = 1.110900E+05;
  eg19[ 24] = 1.831564E+05;
  eg19[ 25] = 3.019738E+05;
  eg19[ 26] = 4.978707E+05;
  eg19[ 27] = 8.208500E+05;
  eg19[ 28] = 1.353353E+06;
  eg19[ 29] = 2.231302E+06;
  eg19[ 30] = 3.678794E+06;
  eg19[ 31] = 6.065307E+06;
  eg19[ 32] = 1.000000E+07;
  eg19[ 33] = 1.964033E+07;

  return eg19;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg20()
{
  vector<Real> eg20;
  eg20.resize(1969);
  eg20[   0] = 1.000010E-05;
  eg20[   1] = 3.000000E-03;
  eg20[   2] = 5.000000E-03;
  eg20[   3] = 6.900000E-03;
  eg20[   4] = 1.000000E-02;
  eg20[   5] = 1.500000E-02;
  eg20[   6] = 2.000000E-02;
  eg20[   7] = 2.500000E-02;
  eg20[   8] = 3.000000E-02;
  eg20[   9] = 3.500000E-02;
  eg20[  10] = 4.200000E-02;
  eg20[  11] = 5.000000E-02;
  eg20[  12] = 5.800000E-02;
  eg20[  13] = 6.700000E-02;
  eg20[  14] = 7.700000E-02;
  eg20[  15] = 8.000000E-02;
  eg20[  16] = 9.500000E-02;
  eg20[  17] = 1.000000E-01;
  eg20[  18] = 1.150000E-01;
  eg20[  19] = 1.340000E-01;
  eg20[  20] = 1.400000E-01;
  eg20[  21] = 1.463700E-01;
  eg20[  22] = 1.530300E-01;
  eg20[  23] = 1.600000E-01;
  eg20[  24] = 1.697100E-01;
  eg20[  25] = 1.800000E-01;
  eg20[  26] = 1.890000E-01;
  eg20[  27] = 1.988100E-01;
  eg20[  28] = 2.091400E-01;
  eg20[  29] = 2.200000E-01;
  eg20[  30] = 2.335800E-01;
  eg20[  31] = 2.480000E-01;
  eg20[  32] = 2.635100E-01;
  eg20[  33] = 2.800000E-01;
  eg20[  34] = 3.000000E-01;
  eg20[  35] = 3.145000E-01;
  eg20[  36] = 3.200000E-01;
  eg20[  37] = 3.346600E-01;
  eg20[  38] = 3.500000E-01;
  eg20[  39] = 3.699300E-01;
  eg20[  40] = 3.910000E-01;
  eg20[  41] = 4.000000E-01;
  eg20[  42] = 4.139900E-01;
  eg20[  43] = 4.330000E-01;
  eg20[  44] = 4.496800E-01;
  eg20[  45] = 4.670100E-01;
  eg20[  46] = 4.850000E-01;
  eg20[  47] = 5.000000E-01;
  eg20[  48] = 5.196200E-01;
  eg20[  49] = 5.315800E-01;
  eg20[  50] = 5.400000E-01;
  eg20[  51] = 5.669600E-01;
  eg20[  52] = 5.952800E-01;
  eg20[  53] = 6.250000E-01;
  eg20[  54] = 6.531500E-01;
  eg20[  55] = 6.825600E-01;
  eg20[  56] = 7.050000E-01;
  eg20[  57] = 7.415500E-01;
  eg20[  58] = 7.800000E-01;
  eg20[  59] = 7.900000E-01;
  eg20[  60] = 8.194500E-01;
  eg20[  61] = 8.500000E-01;
  eg20[  62] = 8.600000E-01;
  eg20[  63] = 8.764250E-01;
  eg20[  64] = 9.100000E-01;
  eg20[  65] = 9.300000E-01;
  eg20[  66] = 9.500000E-01;
  eg20[  67] = 9.720000E-01;
  eg20[  68] = 9.860000E-01;
  eg20[  69] = 9.960000E-01;
  eg20[  70] = 1.020000E+00;
  eg20[  71] = 1.035000E+00;
  eg20[  72] = 1.045000E+00;
  eg20[  73] = 1.071000E+00;
  eg20[  74] = 1.080000E+00;
  eg20[  75] = 1.097000E+00;
  eg20[  76] = 1.110000E+00;
  eg20[  77] = 1.123000E+00;
  eg20[  78] = 1.150000E+00;
  eg20[  79] = 1.170000E+00;
  eg20[  80] = 1.202060E+00;
  eg20[  81] = 1.235000E+00;
  eg20[  82] = 1.267080E+00;
  eg20[  83] = 1.300000E+00;
  eg20[  84] = 1.337500E+00;
  eg20[  85] = 1.370000E+00;
  eg20[  86] = 1.404560E+00;
  eg20[  87] = 1.440000E+00;
  eg20[  88] = 1.475000E+00;
  eg20[  89] = 1.500000E+00;
  eg20[  90] = 1.544340E+00;
  eg20[  91] = 1.590000E+00;
  eg20[  92] = 1.629510E+00;
  eg20[  93] = 1.670000E+00;
  eg20[  94] = 1.711970E+00;
  eg20[  95] = 1.755000E+00;
  eg20[  96] = 1.797000E+00;
  eg20[  97] = 1.840000E+00;
  eg20[  98] = 1.855390E+00;
  eg20[  99] = 1.884460E+00;
  eg20[ 100] = 1.930000E+00;
  eg20[ 101] = 1.974490E+00;
  eg20[ 102] = 2.020000E+00;
  eg20[ 103] = 2.059610E+00;
  eg20[ 104] = 2.100000E+00;
  eg20[ 105] = 2.130000E+00;
  eg20[ 106] = 2.185310E+00;
  eg20[ 107] = 2.242050E+00;
  eg20[ 108] = 2.300270E+00;
  eg20[ 109] = 2.360000E+00;
  eg20[ 110] = 2.382370E+00;
  eg20[ 111] = 2.421710E+00;
  eg20[ 112] = 2.485030E+00;
  eg20[ 113] = 2.550000E+00;
  eg20[ 114] = 2.600000E+00;
  eg20[ 115] = 2.659320E+00;
  eg20[ 116] = 2.720000E+00;
  eg20[ 117] = 2.767920E+00;
  eg20[ 118] = 2.837990E+00;
  eg20[ 119] = 2.909830E+00;
  eg20[ 120] = 2.983490E+00;
  eg20[ 121] = 3.059020E+00;
  eg20[ 122] = 3.137330E+00;
  eg20[ 123] = 3.217630E+00;
  eg20[ 124] = 3.300000E+00;
  eg20[ 125] = 3.380750E+00;
  eg20[ 126] = 3.466330E+00;
  eg20[ 127] = 3.554080E+00;
  eg20[ 128] = 3.644050E+00;
  eg20[ 129] = 3.736300E+00;
  eg20[ 130] = 3.830880E+00;
  eg20[ 131] = 3.927860E+00;
  eg20[ 132] = 4.000000E+00;
  eg20[ 133] = 4.129250E+00;
  eg20[ 134] = 4.233782E+00;
  eg20[ 135] = 4.340961E+00;
  eg20[ 136] = 4.450853E+00;
  eg20[ 137] = 4.563526E+00;
  eg20[ 138] = 4.679053E+00;
  eg20[ 139] = 4.797503E+00;
  eg20[ 140] = 4.918953E+00;
  eg20[ 141] = 5.043477E+00;
  eg20[ 142] = 5.085681E+00;
  eg20[ 143] = 5.128239E+00;
  eg20[ 144] = 5.171153E+00;
  eg20[ 145] = 5.214426E+00;
  eg20[ 146] = 5.258061E+00;
  eg20[ 147] = 5.302061E+00;
  eg20[ 148] = 5.346430E+00;
  eg20[ 149] = 5.391169E+00;
  eg20[ 150] = 5.436284E+00;
  eg20[ 151] = 5.481775E+00;
  eg20[ 152] = 5.527647E+00;
  eg20[ 153] = 5.573904E+00;
  eg20[ 154] = 5.620547E+00;
  eg20[ 155] = 5.667581E+00;
  eg20[ 156] = 5.715008E+00;
  eg20[ 157] = 5.762832E+00;
  eg20[ 158] = 5.811056E+00;
  eg20[ 159] = 5.859684E+00;
  eg20[ 160] = 5.908719E+00;
  eg20[ 161] = 5.958164E+00;
  eg20[ 162] = 6.008022E+00;
  eg20[ 163] = 6.058298E+00;
  eg20[ 164] = 6.108995E+00;
  eg20[ 165] = 6.160116E+00;
  eg20[ 166] = 6.211665E+00;
  eg20[ 167] = 6.263645E+00;
  eg20[ 168] = 6.316060E+00;
  eg20[ 169] = 6.368914E+00;
  eg20[ 170] = 6.422210E+00;
  eg20[ 171] = 6.475952E+00;
  eg20[ 172] = 6.530144E+00;
  eg20[ 173] = 6.584789E+00;
  eg20[ 174] = 6.639892E+00;
  eg20[ 175] = 6.695455E+00;
  eg20[ 176] = 6.751484E+00;
  eg20[ 177] = 6.807981E+00;
  eg20[ 178] = 6.864952E+00;
  eg20[ 179] = 6.922399E+00;
  eg20[ 180] = 6.980326E+00;
  eg20[ 181] = 7.038739E+00;
  eg20[ 182] = 7.097640E+00;
  eg20[ 183] = 7.157034E+00;
  eg20[ 184] = 7.216925E+00;
  eg20[ 185] = 7.277317E+00;
  eg20[ 186] = 7.338215E+00;
  eg20[ 187] = 7.399622E+00;
  eg20[ 188] = 7.461544E+00;
  eg20[ 189] = 7.523983E+00;
  eg20[ 190] = 7.586945E+00;
  eg20[ 191] = 7.650434E+00;
  eg20[ 192] = 7.714454E+00;
  eg20[ 193] = 7.779009E+00;
  eg20[ 194] = 7.844105E+00;
  eg20[ 195] = 7.909746E+00;
  eg20[ 196] = 7.975936E+00;
  eg20[ 197] = 8.042680E+00;
  eg20[ 198] = 8.109982E+00;
  eg20[ 199] = 8.177848E+00;
  eg20[ 200] = 8.246281E+00;
  eg20[ 201] = 8.315287E+00;
  eg20[ 202] = 8.384871E+00;
  eg20[ 203] = 8.455037E+00;
  eg20[ 204] = 8.525790E+00;
  eg20[ 205] = 8.597135E+00;
  eg20[ 206] = 8.669077E+00;
  eg20[ 207] = 8.741621E+00;
  eg20[ 208] = 8.814772E+00;
  eg20[ 209] = 8.888536E+00;
  eg20[ 210] = 8.962916E+00;
  eg20[ 211] = 9.037919E+00;
  eg20[ 212] = 9.113550E+00;
  eg20[ 213] = 9.189814E+00;
  eg20[ 214] = 9.266715E+00;
  eg20[ 215] = 9.344261E+00;
  eg20[ 216] = 9.422455E+00;
  eg20[ 217] = 9.501303E+00;
  eg20[ 218] = 9.580812E+00;
  eg20[ 219] = 9.660985E+00;
  eg20[ 220] = 9.741830E+00;
  eg20[ 221] = 9.823351E+00;
  eg20[ 222] = 9.905554E+00;
  eg20[ 223] = 9.988446E+00;
  eg20[ 224] = 1.007203E+01;
  eg20[ 225] = 1.015631E+01;
  eg20[ 226] = 1.024130E+01;
  eg20[ 227] = 1.032701E+01;
  eg20[ 228] = 1.041342E+01;
  eg20[ 229] = 1.050056E+01;
  eg20[ 230] = 1.058843E+01;
  eg20[ 231] = 1.067704E+01;
  eg20[ 232] = 1.076639E+01;
  eg20[ 233] = 1.085648E+01;
  eg20[ 234] = 1.094733E+01;
  eg20[ 235] = 1.103894E+01;
  eg20[ 236] = 1.113132E+01;
  eg20[ 237] = 1.122446E+01;
  eg20[ 238] = 1.131839E+01;
  eg20[ 239] = 1.141311E+01;
  eg20[ 240] = 1.150861E+01;
  eg20[ 241] = 1.160492E+01;
  eg20[ 242] = 1.170203E+01;
  eg20[ 243] = 1.179995E+01;
  eg20[ 244] = 1.189870E+01;
  eg20[ 245] = 1.199827E+01;
  eg20[ 246] = 1.209867E+01;
  eg20[ 247] = 1.219991E+01;
  eg20[ 248] = 1.230201E+01;
  eg20[ 249] = 1.240495E+01;
  eg20[ 250] = 1.250876E+01;
  eg20[ 251] = 1.261343E+01;
  eg20[ 252] = 1.271898E+01;
  eg20[ 253] = 1.282542E+01;
  eg20[ 254] = 1.293274E+01;
  eg20[ 255] = 1.304097E+01;
  eg20[ 256] = 1.315010E+01;
  eg20[ 257] = 1.326014E+01;
  eg20[ 258] = 1.337110E+01;
  eg20[ 259] = 1.348299E+01;
  eg20[ 260] = 1.359582E+01;
  eg20[ 261] = 1.370959E+01;
  eg20[ 262] = 1.382431E+01;
  eg20[ 263] = 1.394000E+01;
  eg20[ 264] = 1.405665E+01;
  eg20[ 265] = 1.417428E+01;
  eg20[ 266] = 1.429289E+01;
  eg20[ 267] = 1.441250E+01;
  eg20[ 268] = 1.453310E+01;
  eg20[ 269] = 1.465472E+01;
  eg20[ 270] = 1.477735E+01;
  eg20[ 271] = 1.490101E+01;
  eg20[ 272] = 1.502570E+01;
  eg20[ 273] = 1.515144E+01;
  eg20[ 274] = 1.527823E+01;
  eg20[ 275] = 1.540608E+01;
  eg20[ 276] = 1.553500E+01;
  eg20[ 277] = 1.566500E+01;
  eg20[ 278] = 1.579609E+01;
  eg20[ 279] = 1.592827E+01;
  eg20[ 280] = 1.606156E+01;
  eg20[ 281] = 1.619597E+01;
  eg20[ 282] = 1.633150E+01;
  eg20[ 283] = 1.646816E+01;
  eg20[ 284] = 1.660597E+01;
  eg20[ 285] = 1.674493E+01;
  eg20[ 286] = 1.688506E+01;
  eg20[ 287] = 1.702635E+01;
  eg20[ 288] = 1.716883E+01;
  eg20[ 289] = 1.731250E+01;
  eg20[ 290] = 1.745738E+01;
  eg20[ 291] = 1.760346E+01;
  eg20[ 292] = 1.775077E+01;
  eg20[ 293] = 1.789931E+01;
  eg20[ 294] = 1.804910E+01;
  eg20[ 295] = 1.820013E+01;
  eg20[ 296] = 1.835244E+01;
  eg20[ 297] = 1.850601E+01;
  eg20[ 298] = 1.866087E+01;
  eg20[ 299] = 1.881703E+01;
  eg20[ 300] = 1.897449E+01;
  eg20[ 301] = 1.913328E+01;
  eg20[ 302] = 1.929339E+01;
  eg20[ 303] = 1.945484E+01;
  eg20[ 304] = 1.961764E+01;
  eg20[ 305] = 1.978180E+01;
  eg20[ 306] = 1.994734E+01;
  eg20[ 307] = 2.011426E+01;
  eg20[ 308] = 2.028258E+01;
  eg20[ 309] = 2.045231E+01;
  eg20[ 310] = 2.062345E+01;
  eg20[ 311] = 2.079603E+01;
  eg20[ 312] = 2.097006E+01;
  eg20[ 313] = 2.114554E+01;
  eg20[ 314] = 2.132249E+01;
  eg20[ 315] = 2.150092E+01;
  eg20[ 316] = 2.168084E+01;
  eg20[ 317] = 2.186227E+01;
  eg20[ 318] = 2.204522E+01;
  eg20[ 319] = 2.222969E+01;
  eg20[ 320] = 2.241572E+01;
  eg20[ 321] = 2.260329E+01;
  eg20[ 322] = 2.279244E+01;
  eg20[ 323] = 2.298317E+01;
  eg20[ 324] = 2.317550E+01;
  eg20[ 325] = 2.336944E+01;
  eg20[ 326] = 2.356499E+01;
  eg20[ 327] = 2.376219E+01;
  eg20[ 328] = 2.396104E+01;
  eg20[ 329] = 2.416154E+01;
  eg20[ 330] = 2.436373E+01;
  eg20[ 331] = 2.456761E+01;
  eg20[ 332] = 2.477320E+01;
  eg20[ 333] = 2.498050E+01;
  eg20[ 334] = 2.518954E+01;
  eg20[ 335] = 2.540033E+01;
  eg20[ 336] = 2.561289E+01;
  eg20[ 337] = 2.582722E+01;
  eg20[ 338] = 2.604335E+01;
  eg20[ 339] = 2.626128E+01;
  eg20[ 340] = 2.648104E+01;
  eg20[ 341] = 2.670264E+01;
  eg20[ 342] = 2.692609E+01;
  eg20[ 343] = 2.715141E+01;
  eg20[ 344] = 2.737862E+01;
  eg20[ 345] = 2.760773E+01;
  eg20[ 346] = 2.783875E+01;
  eg20[ 347] = 2.807171E+01;
  eg20[ 348] = 2.830662E+01;
  eg20[ 349] = 2.854349E+01;
  eg20[ 350] = 2.878235E+01;
  eg20[ 351] = 2.902320E+01;
  eg20[ 352] = 2.926607E+01;
  eg20[ 353] = 2.951098E+01;
  eg20[ 354] = 2.975793E+01;
  eg20[ 355] = 3.000695E+01;
  eg20[ 356] = 3.025805E+01;
  eg20[ 357] = 3.051126E+01;
  eg20[ 358] = 3.076658E+01;
  eg20[ 359] = 3.102404E+01;
  eg20[ 360] = 3.128365E+01;
  eg20[ 361] = 3.154544E+01;
  eg20[ 362] = 3.180942E+01;
  eg20[ 363] = 3.207560E+01;
  eg20[ 364] = 3.234401E+01;
  eg20[ 365] = 3.261467E+01;
  eg20[ 366] = 3.288760E+01;
  eg20[ 367] = 3.316281E+01;
  eg20[ 368] = 3.344032E+01;
  eg20[ 369] = 3.372015E+01;
  eg20[ 370] = 3.400233E+01;
  eg20[ 371] = 3.428686E+01;
  eg20[ 372] = 3.457378E+01;
  eg20[ 373] = 3.486310E+01;
  eg20[ 374] = 3.515484E+01;
  eg20[ 375] = 3.544902E+01;
  eg20[ 376] = 3.574566E+01;
  eg20[ 377] = 3.604479E+01;
  eg20[ 378] = 3.634642E+01;
  eg20[ 379] = 3.665057E+01;
  eg20[ 380] = 3.695727E+01;
  eg20[ 381] = 3.726653E+01;
  eg20[ 382] = 3.757838E+01;
  eg20[ 383] = 3.789285E+01;
  eg20[ 384] = 3.820994E+01;
  eg20[ 385] = 3.852969E+01;
  eg20[ 386] = 3.885211E+01;
  eg20[ 387] = 3.917723E+01;
  eg20[ 388] = 3.950507E+01;
  eg20[ 389] = 3.983565E+01;
  eg20[ 390] = 4.016900E+01;
  eg20[ 391] = 4.050514E+01;
  eg20[ 392] = 4.084410E+01;
  eg20[ 393] = 4.118589E+01;
  eg20[ 394] = 4.153054E+01;
  eg20[ 395] = 4.187807E+01;
  eg20[ 396] = 4.222851E+01;
  eg20[ 397] = 4.258189E+01;
  eg20[ 398] = 4.293822E+01;
  eg20[ 399] = 4.329753E+01;
  eg20[ 400] = 4.365985E+01;
  eg20[ 401] = 4.402521E+01;
  eg20[ 402] = 4.439361E+01;
  eg20[ 403] = 4.476511E+01;
  eg20[ 404] = 4.513971E+01;
  eg20[ 405] = 4.551744E+01;
  eg20[ 406] = 4.589834E+01;
  eg20[ 407] = 4.628243E+01;
  eg20[ 408] = 4.666972E+01;
  eg20[ 409] = 4.706026E+01;
  eg20[ 410] = 4.745407E+01;
  eg20[ 411] = 4.785117E+01;
  eg20[ 412] = 4.825160E+01;
  eg20[ 413] = 4.865538E+01;
  eg20[ 414] = 4.906253E+01;
  eg20[ 415] = 4.947309E+01;
  eg20[ 416] = 4.988709E+01;
  eg20[ 417] = 5.030456E+01;
  eg20[ 418] = 5.072551E+01;
  eg20[ 419] = 5.114999E+01;
  eg20[ 420] = 5.157802E+01;
  eg20[ 421] = 5.200963E+01;
  eg20[ 422] = 5.244486E+01;
  eg20[ 423] = 5.288373E+01;
  eg20[ 424] = 5.332626E+01;
  eg20[ 425] = 5.377251E+01;
  eg20[ 426] = 5.422248E+01;
  eg20[ 427] = 5.467623E+01;
  eg20[ 428] = 5.513376E+01;
  eg20[ 429] = 5.559513E+01;
  eg20[ 430] = 5.606036E+01;
  eg20[ 431] = 5.652948E+01;
  eg20[ 432] = 5.700253E+01;
  eg20[ 433] = 5.747954E+01;
  eg20[ 434] = 5.796053E+01;
  eg20[ 435] = 5.844556E+01;
  eg20[ 436] = 5.893464E+01;
  eg20[ 437] = 5.942781E+01;
  eg20[ 438] = 5.992511E+01;
  eg20[ 439] = 6.042657E+01;
  eg20[ 440] = 6.093223E+01;
  eg20[ 441] = 6.144212E+01;
  eg20[ 442] = 6.195628E+01;
  eg20[ 443] = 6.247474E+01;
  eg20[ 444] = 6.299754E+01;
  eg20[ 445] = 6.352471E+01;
  eg20[ 446] = 6.405630E+01;
  eg20[ 447] = 6.459233E+01;
  eg20[ 448] = 6.513285E+01;
  eg20[ 449] = 6.567789E+01;
  eg20[ 450] = 6.622749E+01;
  eg20[ 451] = 6.678169E+01;
  eg20[ 452] = 6.734053E+01;
  eg20[ 453] = 6.790405E+01;
  eg20[ 454] = 6.847228E+01;
  eg20[ 455] = 6.904527E+01;
  eg20[ 456] = 6.962305E+01;
  eg20[ 457] = 7.020566E+01;
  eg20[ 458] = 7.079316E+01;
  eg20[ 459] = 7.138556E+01;
  eg20[ 460] = 7.198293E+01;
  eg20[ 461] = 7.258529E+01;
  eg20[ 462] = 7.319270E+01;
  eg20[ 463] = 7.380518E+01;
  eg20[ 464] = 7.442280E+01;
  eg20[ 465] = 7.504558E+01;
  eg20[ 466] = 7.567357E+01;
  eg20[ 467] = 7.630682E+01;
  eg20[ 468] = 7.694537E+01;
  eg20[ 469] = 7.758926E+01;
  eg20[ 470] = 7.823854E+01;
  eg20[ 471] = 7.889325E+01;
  eg20[ 472] = 7.955344E+01;
  eg20[ 473] = 8.021915E+01;
  eg20[ 474] = 8.089044E+01;
  eg20[ 475] = 8.156734E+01;
  eg20[ 476] = 8.224991E+01;
  eg20[ 477] = 8.293819E+01;
  eg20[ 478] = 8.363223E+01;
  eg20[ 479] = 8.433208E+01;
  eg20[ 480] = 8.503778E+01;
  eg20[ 481] = 8.574939E+01;
  eg20[ 482] = 8.646695E+01;
  eg20[ 483] = 8.719052E+01;
  eg20[ 484] = 8.792015E+01;
  eg20[ 485] = 8.865588E+01;
  eg20[ 486] = 8.939776E+01;
  eg20[ 487] = 9.014586E+01;
  eg20[ 488] = 9.090021E+01;
  eg20[ 489] = 9.166088E+01;
  eg20[ 490] = 9.242791E+01;
  eg20[ 491] = 9.320136E+01;
  eg20[ 492] = 9.398128E+01;
  eg20[ 493] = 9.476773E+01;
  eg20[ 494] = 9.556076E+01;
  eg20[ 495] = 9.636043E+01;
  eg20[ 496] = 9.716679E+01;
  eg20[ 497] = 9.797990E+01;
  eg20[ 498] = 9.879981E+01;
  eg20[ 499] = 9.962658E+01;
  eg20[ 500] = 1.004603E+02;
  eg20[ 501] = 1.013009E+02;
  eg20[ 502] = 1.021486E+02;
  eg20[ 503] = 1.030034E+02;
  eg20[ 504] = 1.038654E+02;
  eg20[ 505] = 1.047345E+02;
  eg20[ 506] = 1.056110E+02;
  eg20[ 507] = 1.064947E+02;
  eg20[ 508] = 1.073859E+02;
  eg20[ 509] = 1.082845E+02;
  eg20[ 510] = 1.091907E+02;
  eg20[ 511] = 1.101044E+02;
  eg20[ 512] = 1.110258E+02;
  eg20[ 513] = 1.119548E+02;
  eg20[ 514] = 1.128917E+02;
  eg20[ 515] = 1.138364E+02;
  eg20[ 516] = 1.147890E+02;
  eg20[ 517] = 1.157496E+02;
  eg20[ 518] = 1.167182E+02;
  eg20[ 519] = 1.176949E+02;
  eg20[ 520] = 1.186798E+02;
  eg20[ 521] = 1.196729E+02;
  eg20[ 522] = 1.206744E+02;
  eg20[ 523] = 1.216842E+02;
  eg20[ 524] = 1.227024E+02;
  eg20[ 525] = 1.237292E+02;
  eg20[ 526] = 1.247646E+02;
  eg20[ 527] = 1.258087E+02;
  eg20[ 528] = 1.268615E+02;
  eg20[ 529] = 1.279231E+02;
  eg20[ 530] = 1.289935E+02;
  eg20[ 531] = 1.300730E+02;
  eg20[ 532] = 1.311615E+02;
  eg20[ 533] = 1.322590E+02;
  eg20[ 534] = 1.333658E+02;
  eg20[ 535] = 1.344818E+02;
  eg20[ 536] = 1.356072E+02;
  eg20[ 537] = 1.367420E+02;
  eg20[ 538] = 1.378862E+02;
  eg20[ 539] = 1.390401E+02;
  eg20[ 540] = 1.402036E+02;
  eg20[ 541] = 1.413768E+02;
  eg20[ 542] = 1.425599E+02;
  eg20[ 543] = 1.437529E+02;
  eg20[ 544] = 1.449558E+02;
  eg20[ 545] = 1.461688E+02;
  eg20[ 546] = 1.473920E+02;
  eg20[ 547] = 1.486254E+02;
  eg20[ 548] = 1.498691E+02;
  eg20[ 549] = 1.511232E+02;
  eg20[ 550] = 1.523879E+02;
  eg20[ 551] = 1.536631E+02;
  eg20[ 552] = 1.549489E+02;
  eg20[ 553] = 1.562456E+02;
  eg20[ 554] = 1.575531E+02;
  eg20[ 555] = 1.588715E+02;
  eg20[ 556] = 1.602010E+02;
  eg20[ 557] = 1.615415E+02;
  eg20[ 558] = 1.628933E+02;
  eg20[ 559] = 1.642565E+02;
  eg20[ 560] = 1.656310E+02;
  eg20[ 561] = 1.670170E+02;
  eg20[ 562] = 1.684146E+02;
  eg20[ 563] = 1.698239E+02;
  eg20[ 564] = 1.712451E+02;
  eg20[ 565] = 1.726781E+02;
  eg20[ 566] = 1.741231E+02;
  eg20[ 567] = 1.755802E+02;
  eg20[ 568] = 1.770494E+02;
  eg20[ 569] = 1.785310E+02;
  eg20[ 570] = 1.800250E+02;
  eg20[ 571] = 1.815315E+02;
  eg20[ 572] = 1.830505E+02;
  eg20[ 573] = 1.845823E+02;
  eg20[ 574] = 1.861269E+02;
  eg20[ 575] = 1.876845E+02;
  eg20[ 576] = 1.892551E+02;
  eg20[ 577] = 1.908388E+02;
  eg20[ 578] = 1.924358E+02;
  eg20[ 579] = 1.940461E+02;
  eg20[ 580] = 1.956699E+02;
  eg20[ 581] = 1.973073E+02;
  eg20[ 582] = 1.989584E+02;
  eg20[ 583] = 2.006233E+02;
  eg20[ 584] = 2.023021E+02;
  eg20[ 585] = 2.039950E+02;
  eg20[ 586] = 2.057021E+02;
  eg20[ 587] = 2.074234E+02;
  eg20[ 588] = 2.091592E+02;
  eg20[ 589] = 2.109095E+02;
  eg20[ 590] = 2.126744E+02;
  eg20[ 591] = 2.144541E+02;
  eg20[ 592] = 2.162487E+02;
  eg20[ 593] = 2.180583E+02;
  eg20[ 594] = 2.198830E+02;
  eg20[ 595] = 2.217230E+02;
  eg20[ 596] = 2.235784E+02;
  eg20[ 597] = 2.254494E+02;
  eg20[ 598] = 2.273360E+02;
  eg20[ 599] = 2.292384E+02;
  eg20[ 600] = 2.311567E+02;
  eg20[ 601] = 2.330910E+02;
  eg20[ 602] = 2.350416E+02;
  eg20[ 603] = 2.370084E+02;
  eg20[ 604] = 2.389917E+02;
  eg20[ 605] = 2.409917E+02;
  eg20[ 606] = 2.430083E+02;
  eg20[ 607] = 2.450418E+02;
  eg20[ 608] = 2.470924E+02;
  eg20[ 609] = 2.491601E+02;
  eg20[ 610] = 2.512451E+02;
  eg20[ 611] = 2.533476E+02;
  eg20[ 612] = 2.554676E+02;
  eg20[ 613] = 2.576054E+02;
  eg20[ 614] = 2.597611E+02;
  eg20[ 615] = 2.619348E+02;
  eg20[ 616] = 2.641267E+02;
  eg20[ 617] = 2.663370E+02;
  eg20[ 618] = 2.685657E+02;
  eg20[ 619] = 2.708131E+02;
  eg20[ 620] = 2.730793E+02;
  eg20[ 621] = 2.753645E+02;
  eg20[ 622] = 2.776688E+02;
  eg20[ 623] = 2.799924E+02;
  eg20[ 624] = 2.823354E+02;
  eg20[ 625] = 2.846980E+02;
  eg20[ 626] = 2.870804E+02;
  eg20[ 627] = 2.894827E+02;
  eg20[ 628] = 2.919052E+02;
  eg20[ 629] = 2.943479E+02;
  eg20[ 630] = 2.968110E+02;
  eg20[ 631] = 2.992948E+02;
  eg20[ 632] = 3.017993E+02;
  eg20[ 633] = 3.043248E+02;
  eg20[ 634] = 3.068715E+02;
  eg20[ 635] = 3.094394E+02;
  eg20[ 636] = 3.120288E+02;
  eg20[ 637] = 3.146399E+02;
  eg20[ 638] = 3.172729E+02;
  eg20[ 639] = 3.199279E+02;
  eg20[ 640] = 3.226051E+02;
  eg20[ 641] = 3.253047E+02;
  eg20[ 642] = 3.280269E+02;
  eg20[ 643] = 3.307719E+02;
  eg20[ 644] = 3.335398E+02;
  eg20[ 645] = 3.363309E+02;
  eg20[ 646] = 3.391454E+02;
  eg20[ 647] = 3.419834E+02;
  eg20[ 648] = 3.448452E+02;
  eg20[ 649] = 3.477309E+02;
  eg20[ 650] = 3.506408E+02;
  eg20[ 651] = 3.535750E+02;
  eg20[ 652] = 3.565338E+02;
  eg20[ 653] = 3.595173E+02;
  eg20[ 654] = 3.625258E+02;
  eg20[ 655] = 3.655595E+02;
  eg20[ 656] = 3.686185E+02;
  eg20[ 657] = 3.717032E+02;
  eg20[ 658] = 3.748137E+02;
  eg20[ 659] = 3.779502E+02;
  eg20[ 660] = 3.811129E+02;
  eg20[ 661] = 3.843021E+02;
  eg20[ 662] = 3.875180E+02;
  eg20[ 663] = 3.907608E+02;
  eg20[ 664] = 3.940308E+02;
  eg20[ 665] = 3.973281E+02;
  eg20[ 666] = 4.006530E+02;
  eg20[ 667] = 4.040057E+02;
  eg20[ 668] = 4.073865E+02;
  eg20[ 669] = 4.107955E+02;
  eg20[ 670] = 4.142332E+02;
  eg20[ 671] = 4.176995E+02;
  eg20[ 672] = 4.211949E+02;
  eg20[ 673] = 4.247195E+02;
  eg20[ 674] = 4.282736E+02;
  eg20[ 675] = 4.318575E+02;
  eg20[ 676] = 4.354713E+02;
  eg20[ 677] = 4.391154E+02;
  eg20[ 678] = 4.427900E+02;
  eg20[ 679] = 4.464953E+02;
  eg20[ 680] = 4.502317E+02;
  eg20[ 681] = 4.539993E+02;
  eg20[ 682] = 4.577984E+02;
  eg20[ 683] = 4.616294E+02;
  eg20[ 684] = 4.654923E+02;
  eg20[ 685] = 4.693877E+02;
  eg20[ 686] = 4.733156E+02;
  eg20[ 687] = 4.772763E+02;
  eg20[ 688] = 4.812703E+02;
  eg20[ 689] = 4.852976E+02;
  eg20[ 690] = 4.893587E+02;
  eg20[ 691] = 4.934537E+02;
  eg20[ 692] = 4.975830E+02;
  eg20[ 693] = 5.017468E+02;
  eg20[ 694] = 5.059455E+02;
  eg20[ 695] = 5.101793E+02;
  eg20[ 696] = 5.144486E+02;
  eg20[ 697] = 5.187536E+02;
  eg20[ 698] = 5.230946E+02;
  eg20[ 699] = 5.274719E+02;
  eg20[ 700] = 5.318859E+02;
  eg20[ 701] = 5.363368E+02;
  eg20[ 702] = 5.408249E+02;
  eg20[ 703] = 5.453506E+02;
  eg20[ 704] = 5.499142E+02;
  eg20[ 705] = 5.545160E+02;
  eg20[ 706] = 5.591563E+02;
  eg20[ 707] = 5.638354E+02;
  eg20[ 708] = 5.685536E+02;
  eg20[ 709] = 5.733114E+02;
  eg20[ 710] = 5.781089E+02;
  eg20[ 711] = 5.829466E+02;
  eg20[ 712] = 5.878248E+02;
  eg20[ 713] = 5.927438E+02;
  eg20[ 714] = 5.977040E+02;
  eg20[ 715] = 6.027057E+02;
  eg20[ 716] = 6.077492E+02;
  eg20[ 717] = 6.128350E+02;
  eg20[ 718] = 6.179633E+02;
  eg20[ 719] = 6.231345E+02;
  eg20[ 720] = 6.283489E+02;
  eg20[ 721] = 6.336071E+02;
  eg20[ 722] = 6.389092E+02;
  eg20[ 723] = 6.442557E+02;
  eg20[ 724] = 6.496469E+02;
  eg20[ 725] = 6.550832E+02;
  eg20[ 726] = 6.605651E+02;
  eg20[ 727] = 6.660928E+02;
  eg20[ 728] = 6.716668E+02;
  eg20[ 729] = 6.772874E+02;
  eg20[ 730] = 6.829550E+02;
  eg20[ 731] = 6.886701E+02;
  eg20[ 732] = 6.944330E+02;
  eg20[ 733] = 7.002441E+02;
  eg20[ 734] = 7.061038E+02;
  eg20[ 735] = 7.120126E+02;
  eg20[ 736] = 7.179709E+02;
  eg20[ 737] = 7.239790E+02;
  eg20[ 738] = 7.300373E+02;
  eg20[ 739] = 7.361464E+02;
  eg20[ 740] = 7.423066E+02;
  eg20[ 741] = 7.485183E+02;
  eg20[ 742] = 7.547820E+02;
  eg20[ 743] = 7.610981E+02;
  eg20[ 744] = 7.674671E+02;
  eg20[ 745] = 7.738894E+02;
  eg20[ 746] = 7.803654E+02;
  eg20[ 747] = 7.868957E+02;
  eg20[ 748] = 7.934805E+02;
  eg20[ 749] = 8.001205E+02;
  eg20[ 750] = 8.068160E+02;
  eg20[ 751] = 8.135676E+02;
  eg20[ 752] = 8.203756E+02;
  eg20[ 753] = 8.272407E+02;
  eg20[ 754] = 8.341631E+02;
  eg20[ 755] = 8.411435E+02;
  eg20[ 756] = 8.481824E+02;
  eg20[ 757] = 8.552801E+02;
  eg20[ 758] = 8.624372E+02;
  eg20[ 759] = 8.696542E+02;
  eg20[ 760] = 8.769316E+02;
  eg20[ 761] = 8.842699E+02;
  eg20[ 762] = 8.916696E+02;
  eg20[ 763] = 8.991312E+02;
  eg20[ 764] = 9.066553E+02;
  eg20[ 765] = 9.142423E+02;
  eg20[ 766] = 9.218928E+02;
  eg20[ 767] = 9.296074E+02;
  eg20[ 768] = 9.373865E+02;
  eg20[ 769] = 9.452307E+02;
  eg20[ 770] = 9.531405E+02;
  eg20[ 771] = 9.611165E+02;
  eg20[ 772] = 9.691593E+02;
  eg20[ 773] = 9.772694E+02;
  eg20[ 774] = 9.854473E+02;
  eg20[ 775] = 9.936937E+02;
  eg20[ 776] = 1.002009E+03;
  eg20[ 777] = 1.010394E+03;
  eg20[ 778] = 1.018849E+03;
  eg20[ 779] = 1.027375E+03;
  eg20[ 780] = 1.035972E+03;
  eg20[ 781] = 1.044641E+03;
  eg20[ 782] = 1.053383E+03;
  eg20[ 783] = 1.062198E+03;
  eg20[ 784] = 1.071087E+03;
  eg20[ 785] = 1.080050E+03;
  eg20[ 786] = 1.089088E+03;
  eg20[ 787] = 1.098201E+03;
  eg20[ 788] = 1.107391E+03;
  eg20[ 789] = 1.116658E+03;
  eg20[ 790] = 1.126002E+03;
  eg20[ 791] = 1.135425E+03;
  eg20[ 792] = 1.144926E+03;
  eg20[ 793] = 1.154507E+03;
  eg20[ 794] = 1.164168E+03;
  eg20[ 795] = 1.173910E+03;
  eg20[ 796] = 1.183734E+03;
  eg20[ 797] = 1.193639E+03;
  eg20[ 798] = 1.203628E+03;
  eg20[ 799] = 1.213700E+03;
  eg20[ 800] = 1.223857E+03;
  eg20[ 801] = 1.234098E+03;
  eg20[ 802] = 1.244425E+03;
  eg20[ 803] = 1.254839E+03;
  eg20[ 804] = 1.265339E+03;
  eg20[ 805] = 1.275928E+03;
  eg20[ 806] = 1.286605E+03;
  eg20[ 807] = 1.297372E+03;
  eg20[ 808] = 1.308228E+03;
  eg20[ 809] = 1.319176E+03;
  eg20[ 810] = 1.330215E+03;
  eg20[ 811] = 1.341346E+03;
  eg20[ 812] = 1.352571E+03;
  eg20[ 813] = 1.363889E+03;
  eg20[ 814] = 1.375303E+03;
  eg20[ 815] = 1.386811E+03;
  eg20[ 816] = 1.398416E+03;
  eg20[ 817] = 1.410118E+03;
  eg20[ 818] = 1.421919E+03;
  eg20[ 819] = 1.433817E+03;
  eg20[ 820] = 1.445816E+03;
  eg20[ 821] = 1.457915E+03;
  eg20[ 822] = 1.470115E+03;
  eg20[ 823] = 1.482417E+03;
  eg20[ 824] = 1.494822E+03;
  eg20[ 825] = 1.507331E+03;
  eg20[ 826] = 1.519944E+03;
  eg20[ 827] = 1.532663E+03;
  eg20[ 828] = 1.545489E+03;
  eg20[ 829] = 1.558422E+03;
  eg20[ 830] = 1.571463E+03;
  eg20[ 831] = 1.584613E+03;
  eg20[ 832] = 1.597874E+03;
  eg20[ 833] = 1.611245E+03;
  eg20[ 834] = 1.624728E+03;
  eg20[ 835] = 1.638324E+03;
  eg20[ 836] = 1.652034E+03;
  eg20[ 837] = 1.665858E+03;
  eg20[ 838] = 1.679798E+03;
  eg20[ 839] = 1.693855E+03;
  eg20[ 840] = 1.708030E+03;
  eg20[ 841] = 1.722323E+03;
  eg20[ 842] = 1.736735E+03;
  eg20[ 843] = 1.751268E+03;
  eg20[ 844] = 1.765923E+03;
  eg20[ 845] = 1.780701E+03;
  eg20[ 846] = 1.795602E+03;
  eg20[ 847] = 1.810628E+03;
  eg20[ 848] = 1.825780E+03;
  eg20[ 849] = 1.841058E+03;
  eg20[ 850] = 1.856464E+03;
  eg20[ 851] = 1.871999E+03;
  eg20[ 852] = 1.887665E+03;
  eg20[ 853] = 1.903461E+03;
  eg20[ 854] = 1.919389E+03;
  eg20[ 855] = 1.935451E+03;
  eg20[ 856] = 1.951647E+03;
  eg20[ 857] = 1.967979E+03;
  eg20[ 858] = 1.984447E+03;
  eg20[ 859] = 2.001053E+03;
  eg20[ 860] = 2.017798E+03;
  eg20[ 861] = 2.034684E+03;
  eg20[ 862] = 2.051710E+03;
  eg20[ 863] = 2.068879E+03;
  eg20[ 864] = 2.086192E+03;
  eg20[ 865] = 2.103650E+03;
  eg20[ 866] = 2.121253E+03;
  eg20[ 867] = 2.139004E+03;
  eg20[ 868] = 2.156904E+03;
  eg20[ 869] = 2.174953E+03;
  eg20[ 870] = 2.193153E+03;
  eg20[ 871] = 2.211506E+03;
  eg20[ 872] = 2.230012E+03;
  eg20[ 873] = 2.248673E+03;
  eg20[ 874] = 2.267490E+03;
  eg20[ 875] = 2.286465E+03;
  eg20[ 876] = 2.305599E+03;
  eg20[ 877] = 2.324892E+03;
  eg20[ 878] = 2.344347E+03;
  eg20[ 879] = 2.363965E+03;
  eg20[ 880] = 2.383747E+03;
  eg20[ 881] = 2.403695E+03;
  eg20[ 882] = 2.423809E+03;
  eg20[ 883] = 2.444092E+03;
  eg20[ 884] = 2.464545E+03;
  eg20[ 885] = 2.485168E+03;
  eg20[ 886] = 2.505965E+03;
  eg20[ 887] = 2.526935E+03;
  eg20[ 888] = 2.548081E+03;
  eg20[ 889] = 2.569403E+03;
  eg20[ 890] = 2.590904E+03;
  eg20[ 891] = 2.612586E+03;
  eg20[ 892] = 2.634448E+03;
  eg20[ 893] = 2.656494E+03;
  eg20[ 894] = 2.678723E+03;
  eg20[ 895] = 2.701139E+03;
  eg20[ 896] = 2.723743E+03;
  eg20[ 897] = 2.746536E+03;
  eg20[ 898] = 2.769519E+03;
  eg20[ 899] = 2.792695E+03;
  eg20[ 900] = 2.816065E+03;
  eg20[ 901] = 2.839630E+03;
  eg20[ 902] = 2.863392E+03;
  eg20[ 903] = 2.887354E+03;
  eg20[ 904] = 2.911515E+03;
  eg20[ 905] = 2.935879E+03;
  eg20[ 906] = 2.960447E+03;
  eg20[ 907] = 2.985221E+03;
  eg20[ 908] = 3.010202E+03;
  eg20[ 909] = 3.035391E+03;
  eg20[ 910] = 3.060792E+03;
  eg20[ 911] = 3.086405E+03;
  eg20[ 912] = 3.112233E+03;
  eg20[ 913] = 3.138276E+03;
  eg20[ 914] = 3.164538E+03;
  eg20[ 915] = 3.191019E+03;
  eg20[ 916] = 3.217722E+03;
  eg20[ 917] = 3.244649E+03;
  eg20[ 918] = 3.271800E+03;
  eg20[ 919] = 3.299179E+03;
  eg20[ 920] = 3.326787E+03;
  eg20[ 921] = 3.354626E+03;
  eg20[ 922] = 3.382698E+03;
  eg20[ 923] = 3.411005E+03;
  eg20[ 924] = 3.439549E+03;
  eg20[ 925] = 3.468332E+03;
  eg20[ 926] = 3.497355E+03;
  eg20[ 927] = 3.526622E+03;
  eg20[ 928] = 3.556133E+03;
  eg20[ 929] = 3.585891E+03;
  eg20[ 930] = 3.615898E+03;
  eg20[ 931] = 3.646157E+03;
  eg20[ 932] = 3.676668E+03;
  eg20[ 933] = 3.707435E+03;
  eg20[ 934] = 3.738460E+03;
  eg20[ 935] = 3.769744E+03;
  eg20[ 936] = 3.801290E+03;
  eg20[ 937] = 3.833099E+03;
  eg20[ 938] = 3.865175E+03;
  eg20[ 939] = 3.897520E+03;
  eg20[ 940] = 3.930135E+03;
  eg20[ 941] = 3.963023E+03;
  eg20[ 942] = 3.996186E+03;
  eg20[ 943] = 4.029627E+03;
  eg20[ 944] = 4.063347E+03;
  eg20[ 945] = 4.097350E+03;
  eg20[ 946] = 4.131637E+03;
  eg20[ 947] = 4.166211E+03;
  eg20[ 948] = 4.201075E+03;
  eg20[ 949] = 4.236230E+03;
  eg20[ 950] = 4.271679E+03;
  eg20[ 951] = 4.307425E+03;
  eg20[ 952] = 4.343471E+03;
  eg20[ 953] = 4.379817E+03;
  eg20[ 954] = 4.416468E+03;
  eg20[ 955] = 4.453426E+03;
  eg20[ 956] = 4.490693E+03;
  eg20[ 957] = 4.528272E+03;
  eg20[ 958] = 4.566165E+03;
  eg20[ 959] = 4.604375E+03;
  eg20[ 960] = 4.642906E+03;
  eg20[ 961] = 4.681758E+03;
  eg20[ 962] = 4.720936E+03;
  eg20[ 963] = 4.760441E+03;
  eg20[ 964] = 4.800277E+03;
  eg20[ 965] = 4.840447E+03;
  eg20[ 966] = 4.880952E+03;
  eg20[ 967] = 4.921797E+03;
  eg20[ 968] = 4.962983E+03;
  eg20[ 969] = 5.004514E+03;
  eg20[ 970] = 5.046393E+03;
  eg20[ 971] = 5.088622E+03;
  eg20[ 972] = 5.131204E+03;
  eg20[ 973] = 5.174143E+03;
  eg20[ 974] = 5.217441E+03;
  eg20[ 975] = 5.261101E+03;
  eg20[ 976] = 5.305127E+03;
  eg20[ 977] = 5.349521E+03;
  eg20[ 978] = 5.394287E+03;
  eg20[ 979] = 5.439427E+03;
  eg20[ 980] = 5.484945E+03;
  eg20[ 981] = 5.530844E+03;
  eg20[ 982] = 5.577127E+03;
  eg20[ 983] = 5.623797E+03;
  eg20[ 984] = 5.670858E+03;
  eg20[ 985] = 5.718312E+03;
  eg20[ 986] = 5.766164E+03;
  eg20[ 987] = 5.814416E+03;
  eg20[ 988] = 5.863072E+03;
  eg20[ 989] = 5.912135E+03;
  eg20[ 990] = 5.961609E+03;
  eg20[ 991] = 6.011496E+03;
  eg20[ 992] = 6.061802E+03;
  eg20[ 993] = 6.112528E+03;
  eg20[ 994] = 6.163678E+03;
  eg20[ 995] = 6.215257E+03;
  eg20[ 996] = 6.267267E+03;
  eg20[ 997] = 6.319712E+03;
  eg20[ 998] = 6.372597E+03;
  eg20[ 999] = 6.425924E+03;
  eg20[1000] = 6.479697E+03;
  eg20[1001] = 6.533920E+03;
  eg20[1002] = 6.588597E+03;
  eg20[1003] = 6.643731E+03;
  eg20[1004] = 6.699327E+03;
  eg20[1005] = 6.755388E+03;
  eg20[1006] = 6.811918E+03;
  eg20[1007] = 6.868921E+03;
  eg20[1008] = 6.926401E+03;
  eg20[1009] = 6.984362E+03;
  eg20[1010] = 7.042809E+03;
  eg20[1011] = 7.101744E+03;
  eg20[1012] = 7.161172E+03;
  eg20[1013] = 7.221098E+03;
  eg20[1014] = 7.281525E+03;
  eg20[1015] = 7.342458E+03;
  eg20[1016] = 7.403901E+03;
  eg20[1017] = 7.465858E+03;
  eg20[1018] = 7.528334E+03;
  eg20[1019] = 7.591332E+03;
  eg20[1020] = 7.654857E+03;
  eg20[1021] = 7.718914E+03;
  eg20[1022] = 7.783507E+03;
  eg20[1023] = 7.848641E+03;
  eg20[1024] = 7.914319E+03;
  eg20[1025] = 7.980548E+03;
  eg20[1026] = 8.047330E+03;
  eg20[1027] = 8.114671E+03;
  eg20[1028] = 8.182576E+03;
  eg20[1029] = 8.251049E+03;
  eg20[1030] = 8.320095E+03;
  eg20[1031] = 8.389719E+03;
  eg20[1032] = 8.459926E+03;
  eg20[1033] = 8.530719E+03;
  eg20[1034] = 8.602106E+03;
  eg20[1035] = 8.674090E+03;
  eg20[1036] = 8.746676E+03;
  eg20[1037] = 8.819869E+03;
  eg20[1038] = 8.893675E+03;
  eg20[1039] = 8.968099E+03;
  eg20[1040] = 9.043145E+03;
  eg20[1041] = 9.118820E+03;
  eg20[1042] = 9.195127E+03;
  eg20[1043] = 9.272074E+03;
  eg20[1044] = 9.349664E+03;
  eg20[1045] = 9.427903E+03;
  eg20[1046] = 9.506797E+03;
  eg20[1047] = 9.586352E+03;
  eg20[1048] = 9.666572E+03;
  eg20[1049] = 9.747463E+03;
  eg20[1050] = 9.829031E+03;
  eg20[1051] = 9.911282E+03;
  eg20[1052] = 9.994221E+03;
  eg20[1053] = 1.007785E+04;
  eg20[1054] = 1.016219E+04;
  eg20[1055] = 1.024723E+04;
  eg20[1056] = 1.033298E+04;
  eg20[1057] = 1.041944E+04;
  eg20[1058] = 1.050664E+04;
  eg20[1059] = 1.059456E+04;
  eg20[1060] = 1.068321E+04;
  eg20[1061] = 1.077261E+04;
  eg20[1062] = 1.086276E+04;
  eg20[1063] = 1.095366E+04;
  eg20[1064] = 1.104532E+04;
  eg20[1065] = 1.113775E+04;
  eg20[1066] = 1.123095E+04;
  eg20[1067] = 1.132494E+04;
  eg20[1068] = 1.141970E+04;
  eg20[1069] = 1.151527E+04;
  eg20[1070] = 1.161163E+04;
  eg20[1071] = 1.170880E+04;
  eg20[1072] = 1.180678E+04;
  eg20[1073] = 1.190558E+04;
  eg20[1074] = 1.200521E+04;
  eg20[1075] = 1.210567E+04;
  eg20[1076] = 1.220697E+04;
  eg20[1077] = 1.230912E+04;
  eg20[1078] = 1.241212E+04;
  eg20[1079] = 1.251599E+04;
  eg20[1080] = 1.262073E+04;
  eg20[1081] = 1.272634E+04;
  eg20[1082] = 1.283283E+04;
  eg20[1083] = 1.294022E+04;
  eg20[1084] = 1.304851E+04;
  eg20[1085] = 1.315770E+04;
  eg20[1086] = 1.326780E+04;
  eg20[1087] = 1.337883E+04;
  eg20[1088] = 1.349079E+04;
  eg20[1089] = 1.360368E+04;
  eg20[1090] = 1.371752E+04;
  eg20[1091] = 1.383231E+04;
  eg20[1092] = 1.394806E+04;
  eg20[1093] = 1.406478E+04;
  eg20[1094] = 1.418247E+04;
  eg20[1095] = 1.430116E+04;
  eg20[1096] = 1.442083E+04;
  eg20[1097] = 1.454151E+04;
  eg20[1098] = 1.466319E+04;
  eg20[1099] = 1.478590E+04;
  eg20[1100] = 1.490963E+04;
  eg20[1101] = 1.503439E+04;
  eg20[1102] = 1.516020E+04;
  eg20[1103] = 1.528706E+04;
  eg20[1104] = 1.541499E+04;
  eg20[1105] = 1.554398E+04;
  eg20[1106] = 1.567406E+04;
  eg20[1107] = 1.580522E+04;
  eg20[1108] = 1.593748E+04;
  eg20[1109] = 1.607085E+04;
  eg20[1110] = 1.620533E+04;
  eg20[1111] = 1.634094E+04;
  eg20[1112] = 1.647768E+04;
  eg20[1113] = 1.661557E+04;
  eg20[1114] = 1.675461E+04;
  eg20[1115] = 1.689482E+04;
  eg20[1116] = 1.703620E+04;
  eg20[1117] = 1.717876E+04;
  eg20[1118] = 1.732251E+04;
  eg20[1119] = 1.746747E+04;
  eg20[1120] = 1.761364E+04;
  eg20[1121] = 1.776104E+04;
  eg20[1122] = 1.790966E+04;
  eg20[1123] = 1.805953E+04;
  eg20[1124] = 1.821066E+04;
  eg20[1125] = 1.836305E+04;
  eg20[1126] = 1.851671E+04;
  eg20[1127] = 1.867166E+04;
  eg20[1128] = 1.882791E+04;
  eg20[1129] = 1.898547E+04;
  eg20[1130] = 1.914434E+04;
  eg20[1131] = 1.930454E+04;
  eg20[1132] = 1.946608E+04;
  eg20[1133] = 1.962898E+04;
  eg20[1134] = 1.979324E+04;
  eg20[1135] = 1.995887E+04;
  eg20[1136] = 2.012589E+04;
  eg20[1137] = 2.029431E+04;
  eg20[1138] = 2.046413E+04;
  eg20[1139] = 2.063538E+04;
  eg20[1140] = 2.080806E+04;
  eg20[1141] = 2.098218E+04;
  eg20[1142] = 2.115777E+04;
  eg20[1143] = 2.133482E+04;
  eg20[1144] = 2.151335E+04;
  eg20[1145] = 2.169338E+04;
  eg20[1146] = 2.187491E+04;
  eg20[1147] = 2.205796E+04;
  eg20[1148] = 2.224255E+04;
  eg20[1149] = 2.242868E+04;
  eg20[1150] = 2.261636E+04;
  eg20[1151] = 2.280562E+04;
  eg20[1152] = 2.299646E+04;
  eg20[1153] = 2.318890E+04;
  eg20[1154] = 2.338295E+04;
  eg20[1155] = 2.357862E+04;
  eg20[1156] = 2.377593E+04;
  eg20[1157] = 2.397489E+04;
  eg20[1158] = 2.417552E+04;
  eg20[1159] = 2.437782E+04;
  eg20[1160] = 2.458182E+04;
  eg20[1161] = 2.478752E+04;
  eg20[1162] = 2.499495E+04;
  eg20[1163] = 2.520411E+04;
  eg20[1164] = 2.541502E+04;
  eg20[1165] = 2.562770E+04;
  eg20[1166] = 2.584215E+04;
  eg20[1167] = 2.605841E+04;
  eg20[1168] = 2.627647E+04;
  eg20[1169] = 2.649635E+04;
  eg20[1170] = 2.671808E+04;
  eg20[1171] = 2.694166E+04;
  eg20[1172] = 2.700000E+04;
  eg20[1173] = 2.716711E+04;
  eg20[1174] = 2.739445E+04;
  eg20[1175] = 2.762369E+04;
  eg20[1176] = 2.785485E+04;
  eg20[1177] = 2.808794E+04;
  eg20[1178] = 2.832299E+04;
  eg20[1179] = 2.850000E+04;
  eg20[1180] = 2.856000E+04;
  eg20[1181] = 2.879899E+04;
  eg20[1182] = 2.903999E+04;
  eg20[1183] = 2.928300E+04;
  eg20[1184] = 2.952804E+04;
  eg20[1185] = 2.977514E+04;
  eg20[1186] = 3.002430E+04;
  eg20[1187] = 3.027555E+04;
  eg20[1188] = 3.052890E+04;
  eg20[1189] = 3.078437E+04;
  eg20[1190] = 3.104198E+04;
  eg20[1191] = 3.130174E+04;
  eg20[1192] = 3.156368E+04;
  eg20[1193] = 3.182781E+04;
  eg20[1194] = 3.209415E+04;
  eg20[1195] = 3.236272E+04;
  eg20[1196] = 3.263353E+04;
  eg20[1197] = 3.290662E+04;
  eg20[1198] = 3.318198E+04;
  eg20[1199] = 3.345965E+04;
  eg20[1200] = 3.373965E+04;
  eg20[1201] = 3.402199E+04;
  eg20[1202] = 3.430669E+04;
  eg20[1203] = 3.459377E+04;
  eg20[1204] = 3.488326E+04;
  eg20[1205] = 3.517517E+04;
  eg20[1206] = 3.546952E+04;
  eg20[1207] = 3.576633E+04;
  eg20[1208] = 3.606563E+04;
  eg20[1209] = 3.636743E+04;
  eg20[1210] = 3.667176E+04;
  eg20[1211] = 3.697864E+04;
  eg20[1212] = 3.728808E+04;
  eg20[1213] = 3.760011E+04;
  eg20[1214] = 3.791476E+04;
  eg20[1215] = 3.823203E+04;
  eg20[1216] = 3.855196E+04;
  eg20[1217] = 3.887457E+04;
  eg20[1218] = 3.919988E+04;
  eg20[1219] = 3.952791E+04;
  eg20[1220] = 3.985869E+04;
  eg20[1221] = 4.019223E+04;
  eg20[1222] = 4.052857E+04;
  eg20[1223] = 4.086771E+04;
  eg20[1224] = 4.120970E+04;
  eg20[1225] = 4.155455E+04;
  eg20[1226] = 4.190229E+04;
  eg20[1227] = 4.225293E+04;
  eg20[1228] = 4.260651E+04;
  eg20[1229] = 4.296305E+04;
  eg20[1230] = 4.332257E+04;
  eg20[1231] = 4.368510E+04;
  eg20[1232] = 4.405066E+04;
  eg20[1233] = 4.441928E+04;
  eg20[1234] = 4.479099E+04;
  eg20[1235] = 4.516581E+04;
  eg20[1236] = 4.554376E+04;
  eg20[1237] = 4.592488E+04;
  eg20[1238] = 4.630919E+04;
  eg20[1239] = 4.669671E+04;
  eg20[1240] = 4.708747E+04;
  eg20[1241] = 4.748151E+04;
  eg20[1242] = 4.787884E+04;
  eg20[1243] = 4.827950E+04;
  eg20[1244] = 4.868351E+04;
  eg20[1245] = 4.909090E+04;
  eg20[1246] = 4.950170E+04;
  eg20[1247] = 4.991594E+04;
  eg20[1248] = 5.033364E+04;
  eg20[1249] = 5.075484E+04;
  eg20[1250] = 5.117957E+04;
  eg20[1251] = 5.160785E+04;
  eg20[1252] = 5.203971E+04;
  eg20[1253] = 5.247518E+04;
  eg20[1254] = 5.291430E+04;
  eg20[1255] = 5.335710E+04;
  eg20[1256] = 5.380360E+04;
  eg20[1257] = 5.425384E+04;
  eg20[1258] = 5.470784E+04;
  eg20[1259] = 5.516564E+04;
  eg20[1260] = 5.562728E+04;
  eg20[1261] = 5.609278E+04;
  eg20[1262] = 5.656217E+04;
  eg20[1263] = 5.703549E+04;
  eg20[1264] = 5.751277E+04;
  eg20[1265] = 5.799405E+04;
  eg20[1266] = 5.847935E+04;
  eg20[1267] = 5.896871E+04;
  eg20[1268] = 5.946217E+04;
  eg20[1269] = 5.995976E+04;
  eg20[1270] = 6.046151E+04;
  eg20[1271] = 6.096747E+04;
  eg20[1272] = 6.147765E+04;
  eg20[1273] = 6.199211E+04;
  eg20[1274] = 6.251086E+04;
  eg20[1275] = 6.303396E+04;
  eg20[1276] = 6.356144E+04;
  eg20[1277] = 6.409333E+04;
  eg20[1278] = 6.462968E+04;
  eg20[1279] = 6.517051E+04;
  eg20[1280] = 6.571586E+04;
  eg20[1281] = 6.626579E+04;
  eg20[1282] = 6.682031E+04;
  eg20[1283] = 6.737947E+04;
  eg20[1284] = 6.794331E+04;
  eg20[1285] = 6.851187E+04;
  eg20[1286] = 6.908519E+04;
  eg20[1287] = 6.966330E+04;
  eg20[1288] = 7.024626E+04;
  eg20[1289] = 7.083409E+04;
  eg20[1290] = 7.142684E+04;
  eg20[1291] = 7.202455E+04;
  eg20[1292] = 7.262726E+04;
  eg20[1293] = 7.323502E+04;
  eg20[1294] = 7.384786E+04;
  eg20[1295] = 7.446583E+04;
  eg20[1296] = 7.508897E+04;
  eg20[1297] = 7.571733E+04;
  eg20[1298] = 7.635094E+04;
  eg20[1299] = 7.698986E+04;
  eg20[1300] = 7.763412E+04;
  eg20[1301] = 7.828378E+04;
  eg20[1302] = 7.893887E+04;
  eg20[1303] = 7.950000E+04;
  eg20[1304] = 7.959944E+04;
  eg20[1305] = 8.026554E+04;
  eg20[1306] = 8.093721E+04;
  eg20[1307] = 8.161451E+04;
  eg20[1308] = 8.229747E+04;
  eg20[1309] = 8.250000E+04;
  eg20[1310] = 8.298615E+04;
  eg20[1311] = 8.368059E+04;
  eg20[1312] = 8.438084E+04;
  eg20[1313] = 8.508695E+04;
  eg20[1314] = 8.579897E+04;
  eg20[1315] = 8.651695E+04;
  eg20[1316] = 8.724094E+04;
  eg20[1317] = 8.797098E+04;
  eg20[1318] = 8.870714E+04;
  eg20[1319] = 8.944945E+04;
  eg20[1320] = 9.019798E+04;
  eg20[1321] = 9.095277E+04;
  eg20[1322] = 9.171388E+04;
  eg20[1323] = 9.248135E+04;
  eg20[1324] = 9.325525E+04;
  eg20[1325] = 9.403563E+04;
  eg20[1326] = 9.482253E+04;
  eg20[1327] = 9.561602E+04;
  eg20[1328] = 9.641615E+04;
  eg20[1329] = 9.722297E+04;
  eg20[1330] = 9.803655E+04;
  eg20[1331] = 9.885694E+04;
  eg20[1332] = 9.968419E+04;
  eg20[1333] = 1.005184E+05;
  eg20[1334] = 1.013595E+05;
  eg20[1335] = 1.022077E+05;
  eg20[1336] = 1.030630E+05;
  eg20[1337] = 1.039254E+05;
  eg20[1338] = 1.047951E+05;
  eg20[1339] = 1.056720E+05;
  eg20[1340] = 1.065563E+05;
  eg20[1341] = 1.074480E+05;
  eg20[1342] = 1.083471E+05;
  eg20[1343] = 1.092538E+05;
  eg20[1344] = 1.101681E+05;
  eg20[1345] = 1.110900E+05;
  eg20[1346] = 1.120196E+05;
  eg20[1347] = 1.129570E+05;
  eg20[1348] = 1.139022E+05;
  eg20[1349] = 1.148554E+05;
  eg20[1350] = 1.158165E+05;
  eg20[1351] = 1.167857E+05;
  eg20[1352] = 1.177629E+05;
  eg20[1353] = 1.187484E+05;
  eg20[1354] = 1.197421E+05;
  eg20[1355] = 1.207441E+05;
  eg20[1356] = 1.217545E+05;
  eg20[1357] = 1.227734E+05;
  eg20[1358] = 1.238008E+05;
  eg20[1359] = 1.248368E+05;
  eg20[1360] = 1.258814E+05;
  eg20[1361] = 1.269348E+05;
  eg20[1362] = 1.279970E+05;
  eg20[1363] = 1.290681E+05;
  eg20[1364] = 1.301482E+05;
  eg20[1365] = 1.312373E+05;
  eg20[1366] = 1.323355E+05;
  eg20[1367] = 1.334429E+05;
  eg20[1368] = 1.345596E+05;
  eg20[1369] = 1.356856E+05;
  eg20[1370] = 1.368210E+05;
  eg20[1371] = 1.379660E+05;
  eg20[1372] = 1.391205E+05;
  eg20[1373] = 1.402847E+05;
  eg20[1374] = 1.414586E+05;
  eg20[1375] = 1.426423E+05;
  eg20[1376] = 1.438360E+05;
  eg20[1377] = 1.450396E+05;
  eg20[1378] = 1.462533E+05;
  eg20[1379] = 1.474772E+05;
  eg20[1380] = 1.487113E+05;
  eg20[1381] = 1.499558E+05;
  eg20[1382] = 1.512106E+05;
  eg20[1383] = 1.524760E+05;
  eg20[1384] = 1.537519E+05;
  eg20[1385] = 1.550385E+05;
  eg20[1386] = 1.563359E+05;
  eg20[1387] = 1.576442E+05;
  eg20[1388] = 1.589634E+05;
  eg20[1389] = 1.602936E+05;
  eg20[1390] = 1.616349E+05;
  eg20[1391] = 1.629875E+05;
  eg20[1392] = 1.643514E+05;
  eg20[1393] = 1.657268E+05;
  eg20[1394] = 1.671136E+05;
  eg20[1395] = 1.685120E+05;
  eg20[1396] = 1.699221E+05;
  eg20[1397] = 1.713441E+05;
  eg20[1398] = 1.727779E+05;
  eg20[1399] = 1.742237E+05;
  eg20[1400] = 1.756817E+05;
  eg20[1401] = 1.771518E+05;
  eg20[1402] = 1.786342E+05;
  eg20[1403] = 1.801291E+05;
  eg20[1404] = 1.816364E+05;
  eg20[1405] = 1.831564E+05;
  eg20[1406] = 1.846891E+05;
  eg20[1407] = 1.862346E+05;
  eg20[1408] = 1.877930E+05;
  eg20[1409] = 1.893645E+05;
  eg20[1410] = 1.909491E+05;
  eg20[1411] = 1.925470E+05;
  eg20[1412] = 1.941583E+05;
  eg20[1413] = 1.957830E+05;
  eg20[1414] = 1.974214E+05;
  eg20[1415] = 1.990734E+05;
  eg20[1416] = 2.007393E+05;
  eg20[1417] = 2.024191E+05;
  eg20[1418] = 2.041130E+05;
  eg20[1419] = 2.058210E+05;
  eg20[1420] = 2.075434E+05;
  eg20[1421] = 2.092801E+05;
  eg20[1422] = 2.110314E+05;
  eg20[1423] = 2.127974E+05;
  eg20[1424] = 2.145781E+05;
  eg20[1425] = 2.163737E+05;
  eg20[1426] = 2.181844E+05;
  eg20[1427] = 2.200102E+05;
  eg20[1428] = 2.218512E+05;
  eg20[1429] = 2.237077E+05;
  eg20[1430] = 2.255797E+05;
  eg20[1431] = 2.274674E+05;
  eg20[1432] = 2.293709E+05;
  eg20[1433] = 2.312903E+05;
  eg20[1434] = 2.332258E+05;
  eg20[1435] = 2.351775E+05;
  eg20[1436] = 2.371455E+05;
  eg20[1437] = 2.391299E+05;
  eg20[1438] = 2.411310E+05;
  eg20[1439] = 2.431488E+05;
  eg20[1440] = 2.451835E+05;
  eg20[1441] = 2.472353E+05;
  eg20[1442] = 2.493042E+05;
  eg20[1443] = 2.513904E+05;
  eg20[1444] = 2.534941E+05;
  eg20[1445] = 2.556153E+05;
  eg20[1446] = 2.577544E+05;
  eg20[1447] = 2.599113E+05;
  eg20[1448] = 2.620863E+05;
  eg20[1449] = 2.642794E+05;
  eg20[1450] = 2.664910E+05;
  eg20[1451] = 2.687210E+05;
  eg20[1452] = 2.709697E+05;
  eg20[1453] = 2.732372E+05;
  eg20[1454] = 2.755237E+05;
  eg20[1455] = 2.778293E+05;
  eg20[1456] = 2.801543E+05;
  eg20[1457] = 2.824986E+05;
  eg20[1458] = 2.848626E+05;
  eg20[1459] = 2.872464E+05;
  eg20[1460] = 2.896501E+05;
  eg20[1461] = 2.920740E+05;
  eg20[1462] = 2.945181E+05;
  eg20[1463] = 2.969826E+05;
  eg20[1464] = 2.972000E+05;
  eg20[1465] = 2.985000E+05;
  eg20[1466] = 2.994678E+05;
  eg20[1467] = 3.019738E+05;
  eg20[1468] = 3.045008E+05;
  eg20[1469] = 3.070489E+05;
  eg20[1470] = 3.096183E+05;
  eg20[1471] = 3.122093E+05;
  eg20[1472] = 3.148219E+05;
  eg20[1473] = 3.174564E+05;
  eg20[1474] = 3.201129E+05;
  eg20[1475] = 3.227916E+05;
  eg20[1476] = 3.254928E+05;
  eg20[1477] = 3.282166E+05;
  eg20[1478] = 3.309631E+05;
  eg20[1479] = 3.337327E+05;
  eg20[1480] = 3.365254E+05;
  eg20[1481] = 3.393415E+05;
  eg20[1482] = 3.421812E+05;
  eg20[1483] = 3.450446E+05;
  eg20[1484] = 3.479320E+05;
  eg20[1485] = 3.508435E+05;
  eg20[1486] = 3.537795E+05;
  eg20[1487] = 3.567399E+05;
  eg20[1488] = 3.597252E+05;
  eg20[1489] = 3.627354E+05;
  eg20[1490] = 3.657708E+05;
  eg20[1491] = 3.688317E+05;
  eg20[1492] = 3.719181E+05;
  eg20[1493] = 3.750304E+05;
  eg20[1494] = 3.781687E+05;
  eg20[1495] = 3.813333E+05;
  eg20[1496] = 3.845243E+05;
  eg20[1497] = 3.877421E+05;
  eg20[1498] = 3.909868E+05;
  eg20[1499] = 3.942586E+05;
  eg20[1500] = 3.975578E+05;
  eg20[1501] = 4.008846E+05;
  eg20[1502] = 4.042393E+05;
  eg20[1503] = 4.076220E+05;
  eg20[1504] = 4.110331E+05;
  eg20[1505] = 4.144727E+05;
  eg20[1506] = 4.179410E+05;
  eg20[1507] = 4.214384E+05;
  eg20[1508] = 4.249651E+05;
  eg20[1509] = 4.285213E+05;
  eg20[1510] = 4.321072E+05;
  eg20[1511] = 4.357231E+05;
  eg20[1512] = 4.393693E+05;
  eg20[1513] = 4.430460E+05;
  eg20[1514] = 4.467535E+05;
  eg20[1515] = 4.504920E+05;
  eg20[1516] = 4.542618E+05;
  eg20[1517] = 4.580631E+05;
  eg20[1518] = 4.618963E+05;
  eg20[1519] = 4.657615E+05;
  eg20[1520] = 4.696591E+05;
  eg20[1521] = 4.735892E+05;
  eg20[1522] = 4.775523E+05;
  eg20[1523] = 4.815485E+05;
  eg20[1524] = 4.855782E+05;
  eg20[1525] = 4.896416E+05;
  eg20[1526] = 4.937390E+05;
  eg20[1527] = 4.978707E+05;
  eg20[1528] = 5.020369E+05;
  eg20[1529] = 5.062381E+05;
  eg20[1530] = 5.104743E+05;
  eg20[1531] = 5.147461E+05;
  eg20[1532] = 5.190535E+05;
  eg20[1533] = 5.233971E+05;
  eg20[1534] = 5.277769E+05;
  eg20[1535] = 5.321934E+05;
  eg20[1536] = 5.366469E+05;
  eg20[1537] = 5.411377E+05;
  eg20[1538] = 5.456660E+05;
  eg20[1539] = 5.502322E+05;
  eg20[1540] = 5.548366E+05;
  eg20[1541] = 5.594796E+05;
  eg20[1542] = 5.641614E+05;
  eg20[1543] = 5.688824E+05;
  eg20[1544] = 5.736429E+05;
  eg20[1545] = 5.784432E+05;
  eg20[1546] = 5.832837E+05;
  eg20[1547] = 5.881647E+05;
  eg20[1548] = 5.930866E+05;
  eg20[1549] = 5.980496E+05;
  eg20[1550] = 6.030542E+05;
  eg20[1551] = 6.081006E+05;
  eg20[1552] = 6.131893E+05;
  eg20[1553] = 6.183206E+05;
  eg20[1554] = 6.234948E+05;
  eg20[1555] = 6.287123E+05;
  eg20[1556] = 6.339734E+05;
  eg20[1557] = 6.392786E+05;
  eg20[1558] = 6.446282E+05;
  eg20[1559] = 6.500225E+05;
  eg20[1560] = 6.554620E+05;
  eg20[1561] = 6.609470E+05;
  eg20[1562] = 6.664779E+05;
  eg20[1563] = 6.720551E+05;
  eg20[1564] = 6.776790E+05;
  eg20[1565] = 6.833499E+05;
  eg20[1566] = 6.890683E+05;
  eg20[1567] = 6.948345E+05;
  eg20[1568] = 7.006490E+05;
  eg20[1569] = 7.065121E+05;
  eg20[1570] = 7.124243E+05;
  eg20[1571] = 7.183860E+05;
  eg20[1572] = 7.243976E+05;
  eg20[1573] = 7.304594E+05;
  eg20[1574] = 7.365720E+05;
  eg20[1575] = 7.427358E+05;
  eg20[1576] = 7.489511E+05;
  eg20[1577] = 7.552184E+05;
  eg20[1578] = 7.615382E+05;
  eg20[1579] = 7.679109E+05;
  eg20[1580] = 7.743369E+05;
  eg20[1581] = 7.808167E+05;
  eg20[1582] = 7.873507E+05;
  eg20[1583] = 7.939393E+05;
  eg20[1584] = 8.005831E+05;
  eg20[1585] = 8.072825E+05;
  eg20[1586] = 8.140380E+05;
  eg20[1587] = 8.208500E+05;
  eg20[1588] = 8.277190E+05;
  eg20[1589] = 8.346455E+05;
  eg20[1590] = 8.416299E+05;
  eg20[1591] = 8.486728E+05;
  eg20[1592] = 8.557746E+05;
  eg20[1593] = 8.629359E+05;
  eg20[1594] = 8.701570E+05;
  eg20[1595] = 8.774387E+05;
  eg20[1596] = 8.847812E+05;
  eg20[1597] = 8.921852E+05;
  eg20[1598] = 8.996511E+05;
  eg20[1599] = 9.071795E+05;
  eg20[1600] = 9.147709E+05;
  eg20[1601] = 9.224259E+05;
  eg20[1602] = 9.301449E+05;
  eg20[1603] = 9.379285E+05;
  eg20[1604] = 9.457772E+05;
  eg20[1605] = 9.536916E+05;
  eg20[1606] = 9.616723E+05;
  eg20[1607] = 9.697197E+05;
  eg20[1608] = 9.778344E+05;
  eg20[1609] = 9.860171E+05;
  eg20[1610] = 9.942682E+05;
  eg20[1611] = 1.002588E+06;
  eg20[1612] = 1.010978E+06;
  eg20[1613] = 1.019438E+06;
  eg20[1614] = 1.027969E+06;
  eg20[1615] = 1.036571E+06;
  eg20[1616] = 1.045245E+06;
  eg20[1617] = 1.053992E+06;
  eg20[1618] = 1.062812E+06;
  eg20[1619] = 1.071706E+06;
  eg20[1620] = 1.080674E+06;
  eg20[1621] = 1.089717E+06;
  eg20[1622] = 1.098836E+06;
  eg20[1623] = 1.108032E+06;
  eg20[1624] = 1.117304E+06;
  eg20[1625] = 1.126654E+06;
  eg20[1626] = 1.136082E+06;
  eg20[1627] = 1.145588E+06;
  eg20[1628] = 1.155175E+06;
  eg20[1629] = 1.164842E+06;
  eg20[1630] = 1.174589E+06;
  eg20[1631] = 1.184418E+06;
  eg20[1632] = 1.194330E+06;
  eg20[1633] = 1.204324E+06;
  eg20[1634] = 1.214402E+06;
  eg20[1635] = 1.224564E+06;
  eg20[1636] = 1.234812E+06;
  eg20[1637] = 1.245145E+06;
  eg20[1638] = 1.255564E+06;
  eg20[1639] = 1.266071E+06;
  eg20[1640] = 1.276666E+06;
  eg20[1641] = 1.287349E+06;
  eg20[1642] = 1.298122E+06;
  eg20[1643] = 1.308985E+06;
  eg20[1644] = 1.319938E+06;
  eg20[1645] = 1.330984E+06;
  eg20[1646] = 1.342122E+06;
  eg20[1647] = 1.353353E+06;
  eg20[1648] = 1.364678E+06;
  eg20[1649] = 1.376098E+06;
  eg20[1650] = 1.387613E+06;
  eg20[1651] = 1.399225E+06;
  eg20[1652] = 1.410934E+06;
  eg20[1653] = 1.422741E+06;
  eg20[1654] = 1.434646E+06;
  eg20[1655] = 1.446652E+06;
  eg20[1656] = 1.458758E+06;
  eg20[1657] = 1.470965E+06;
  eg20[1658] = 1.483274E+06;
  eg20[1659] = 1.495686E+06;
  eg20[1660] = 1.508202E+06;
  eg20[1661] = 1.520823E+06;
  eg20[1662] = 1.533550E+06;
  eg20[1663] = 1.546383E+06;
  eg20[1664] = 1.559323E+06;
  eg20[1665] = 1.572372E+06;
  eg20[1666] = 1.585530E+06;
  eg20[1667] = 1.598797E+06;
  eg20[1668] = 1.612176E+06;
  eg20[1669] = 1.625667E+06;
  eg20[1670] = 1.639271E+06;
  eg20[1671] = 1.652989E+06;
  eg20[1672] = 1.666821E+06;
  eg20[1673] = 1.680770E+06;
  eg20[1674] = 1.694834E+06;
  eg20[1675] = 1.709017E+06;
  eg20[1676] = 1.723318E+06;
  eg20[1677] = 1.737739E+06;
  eg20[1678] = 1.752281E+06;
  eg20[1679] = 1.766944E+06;
  eg20[1680] = 1.781731E+06;
  eg20[1681] = 1.796640E+06;
  eg20[1682] = 1.811675E+06;
  eg20[1683] = 1.826835E+06;
  eg20[1684] = 1.842122E+06;
  eg20[1685] = 1.857538E+06;
  eg20[1686] = 1.873082E+06;
  eg20[1687] = 1.888756E+06;
  eg20[1688] = 1.904561E+06;
  eg20[1689] = 1.920499E+06;
  eg20[1690] = 1.936570E+06;
  eg20[1691] = 1.952776E+06;
  eg20[1692] = 1.969117E+06;
  eg20[1693] = 1.985595E+06;
  eg20[1694] = 2.002210E+06;
  eg20[1695] = 2.018965E+06;
  eg20[1696] = 2.035860E+06;
  eg20[1697] = 2.052897E+06;
  eg20[1698] = 2.070076E+06;
  eg20[1699] = 2.087398E+06;
  eg20[1700] = 2.104866E+06;
  eg20[1701] = 2.122480E+06;
  eg20[1702] = 2.140241E+06;
  eg20[1703] = 2.158151E+06;
  eg20[1704] = 2.176211E+06;
  eg20[1705] = 2.194421E+06;
  eg20[1706] = 2.212785E+06;
  eg20[1707] = 2.231302E+06;
  eg20[1708] = 2.249973E+06;
  eg20[1709] = 2.268802E+06;
  eg20[1710] = 2.287787E+06;
  eg20[1711] = 2.306932E+06;
  eg20[1712] = 2.326237E+06;
  eg20[1713] = 2.345703E+06;
  eg20[1714] = 2.365332E+06;
  eg20[1715] = 2.385126E+06;
  eg20[1716] = 2.405085E+06;
  eg20[1717] = 2.425211E+06;
  eg20[1718] = 2.445505E+06;
  eg20[1719] = 2.465970E+06;
  eg20[1720] = 2.486605E+06;
  eg20[1721] = 2.507414E+06;
  eg20[1722] = 2.528396E+06;
  eg20[1723] = 2.549554E+06;
  eg20[1724] = 2.570889E+06;
  eg20[1725] = 2.592403E+06;
  eg20[1726] = 2.614096E+06;
  eg20[1727] = 2.635971E+06;
  eg20[1728] = 2.658030E+06;
  eg20[1729] = 2.680272E+06;
  eg20[1730] = 2.702701E+06;
  eg20[1731] = 2.725318E+06;
  eg20[1732] = 2.748124E+06;
  eg20[1733] = 2.771121E+06;
  eg20[1734] = 2.794310E+06;
  eg20[1735] = 2.817693E+06;
  eg20[1736] = 2.841272E+06;
  eg20[1737] = 2.865048E+06;
  eg20[1738] = 2.889023E+06;
  eg20[1739] = 2.913199E+06;
  eg20[1740] = 2.937577E+06;
  eg20[1741] = 2.962159E+06;
  eg20[1742] = 2.986947E+06;
  eg20[1743] = 3.011942E+06;
  eg20[1744] = 3.037147E+06;
  eg20[1745] = 3.062562E+06;
  eg20[1746] = 3.088190E+06;
  eg20[1747] = 3.114032E+06;
  eg20[1748] = 3.140091E+06;
  eg20[1749] = 3.166368E+06;
  eg20[1750] = 3.192864E+06;
  eg20[1751] = 3.219583E+06;
  eg20[1752] = 3.246525E+06;
  eg20[1753] = 3.273692E+06;
  eg20[1754] = 3.301087E+06;
  eg20[1755] = 3.328711E+06;
  eg20[1756] = 3.356566E+06;
  eg20[1757] = 3.384654E+06;
  eg20[1758] = 3.412978E+06;
  eg20[1759] = 3.441538E+06;
  eg20[1760] = 3.470337E+06;
  eg20[1761] = 3.499377E+06;
  eg20[1762] = 3.528661E+06;
  eg20[1763] = 3.558189E+06;
  eg20[1764] = 3.587965E+06;
  eg20[1765] = 3.617989E+06;
  eg20[1766] = 3.648265E+06;
  eg20[1767] = 3.678794E+06;
  eg20[1768] = 3.709579E+06;
  eg20[1769] = 3.740621E+06;
  eg20[1770] = 3.771924E+06;
  eg20[1771] = 3.803488E+06;
  eg20[1772] = 3.835316E+06;
  eg20[1773] = 3.867410E+06;
  eg20[1774] = 3.899773E+06;
  eg20[1775] = 3.932407E+06;
  eg20[1776] = 3.965314E+06;
  eg20[1777] = 3.998497E+06;
  eg20[1778] = 4.031957E+06;
  eg20[1779] = 4.065697E+06;
  eg20[1780] = 4.099719E+06;
  eg20[1781] = 4.134026E+06;
  eg20[1782] = 4.168620E+06;
  eg20[1783] = 4.203504E+06;
  eg20[1784] = 4.238679E+06;
  eg20[1785] = 4.274149E+06;
  eg20[1786] = 4.309916E+06;
  eg20[1787] = 4.345982E+06;
  eg20[1788] = 4.382350E+06;
  eg20[1789] = 4.419022E+06;
  eg20[1790] = 4.456001E+06;
  eg20[1791] = 4.493290E+06;
  eg20[1792] = 4.530890E+06;
  eg20[1793] = 4.568805E+06;
  eg20[1794] = 4.607038E+06;
  eg20[1795] = 4.645590E+06;
  eg20[1796] = 4.684465E+06;
  eg20[1797] = 4.723666E+06;
  eg20[1798] = 4.763194E+06;
  eg20[1799] = 4.803053E+06;
  eg20[1800] = 4.843246E+06;
  eg20[1801] = 4.883775E+06;
  eg20[1802] = 4.924643E+06;
  eg20[1803] = 4.965853E+06;
  eg20[1804] = 5.007408E+06;
  eg20[1805] = 5.049311E+06;
  eg20[1806] = 5.091564E+06;
  eg20[1807] = 5.134171E+06;
  eg20[1808] = 5.177135E+06;
  eg20[1809] = 5.220458E+06;
  eg20[1810] = 5.264143E+06;
  eg20[1811] = 5.308195E+06;
  eg20[1812] = 5.352614E+06;
  eg20[1813] = 5.397406E+06;
  eg20[1814] = 5.442572E+06;
  eg20[1815] = 5.488116E+06;
  eg20[1816] = 5.534042E+06;
  eg20[1817] = 5.580351E+06;
  eg20[1818] = 5.627049E+06;
  eg20[1819] = 5.674137E+06;
  eg20[1820] = 5.721619E+06;
  eg20[1821] = 5.769498E+06;
  eg20[1822] = 5.817778E+06;
  eg20[1823] = 5.866462E+06;
  eg20[1824] = 5.915554E+06;
  eg20[1825] = 5.965056E+06;
  eg20[1826] = 6.014972E+06;
  eg20[1827] = 6.065307E+06;
  eg20[1828] = 6.116062E+06;
  eg20[1829] = 6.167242E+06;
  eg20[1830] = 6.218851E+06;
  eg20[1831] = 6.270891E+06;
  eg20[1832] = 6.323367E+06;
  eg20[1833] = 6.376282E+06;
  eg20[1834] = 6.429639E+06;
  eg20[1835] = 6.483443E+06;
  eg20[1836] = 6.537698E+06;
  eg20[1837] = 6.592406E+06;
  eg20[1838] = 6.647573E+06;
  eg20[1839] = 6.703200E+06;
  eg20[1840] = 6.759294E+06;
  eg20[1841] = 6.815857E+06;
  eg20[1842] = 6.872893E+06;
  eg20[1843] = 6.930406E+06;
  eg20[1844] = 6.988401E+06;
  eg20[1845] = 7.046881E+06;
  eg20[1846] = 7.105850E+06;
  eg20[1847] = 7.165313E+06;
  eg20[1848] = 7.225274E+06;
  eg20[1849] = 7.285736E+06;
  eg20[1850] = 7.346704E+06;
  eg20[1851] = 7.408182E+06;
  eg20[1852] = 7.470175E+06;
  eg20[1853] = 7.532687E+06;
  eg20[1854] = 7.595721E+06;
  eg20[1855] = 7.659283E+06;
  eg20[1856] = 7.723377E+06;
  eg20[1857] = 7.788008E+06;
  eg20[1858] = 7.853179E+06;
  eg20[1859] = 7.918896E+06;
  eg20[1860] = 7.985162E+06;
  eg20[1861] = 8.051983E+06;
  eg20[1862] = 8.119363E+06;
  eg20[1863] = 8.187308E+06;
  eg20[1864] = 8.255820E+06;
  eg20[1865] = 8.324906E+06;
  eg20[1866] = 8.394570E+06;
  eg20[1867] = 8.464817E+06;
  eg20[1868] = 8.535652E+06;
  eg20[1869] = 8.607080E+06;
  eg20[1870] = 8.679105E+06;
  eg20[1871] = 8.751733E+06;
  eg20[1872] = 8.824969E+06;
  eg20[1873] = 8.898818E+06;
  eg20[1874] = 8.973284E+06;
  eg20[1875] = 9.048374E+06;
  eg20[1876] = 9.124092E+06;
  eg20[1877] = 9.200444E+06;
  eg20[1878] = 9.277435E+06;
  eg20[1879] = 9.355070E+06;
  eg20[1880] = 9.433354E+06;
  eg20[1881] = 9.512294E+06;
  eg20[1882] = 9.591895E+06;
  eg20[1883] = 9.672161E+06;
  eg20[1884] = 9.753099E+06;
  eg20[1885] = 9.834715E+06;
  eg20[1886] = 9.917013E+06;
  eg20[1887] = 1.000000E+07;
  eg20[1888] = 1.008368E+07;
  eg20[1889] = 1.016806E+07;
  eg20[1890] = 1.025315E+07;
  eg20[1891] = 1.033895E+07;
  eg20[1892] = 1.042547E+07;
  eg20[1893] = 1.051271E+07;
  eg20[1894] = 1.060068E+07;
  eg20[1895] = 1.068939E+07;
  eg20[1896] = 1.077884E+07;
  eg20[1897] = 1.086904E+07;
  eg20[1898] = 1.095999E+07;
  eg20[1899] = 1.105171E+07;
  eg20[1900] = 1.114419E+07;
  eg20[1901] = 1.123745E+07;
  eg20[1902] = 1.133148E+07;
  eg20[1903] = 1.142631E+07;
  eg20[1904] = 1.152193E+07;
  eg20[1905] = 1.161834E+07;
  eg20[1906] = 1.171557E+07;
  eg20[1907] = 1.181360E+07;
  eg20[1908] = 1.191246E+07;
  eg20[1909] = 1.201215E+07;
  eg20[1910] = 1.211267E+07;
  eg20[1911] = 1.221403E+07;
  eg20[1912] = 1.231624E+07;
  eg20[1913] = 1.241930E+07;
  eg20[1914] = 1.252323E+07;
  eg20[1915] = 1.262802E+07;
  eg20[1916] = 1.273370E+07;
  eg20[1917] = 1.284025E+07;
  eg20[1918] = 1.294770E+07;
  eg20[1919] = 1.305605E+07;
  eg20[1920] = 1.316531E+07;
  eg20[1921] = 1.327548E+07;
  eg20[1922] = 1.338657E+07;
  eg20[1923] = 1.349859E+07;
  eg20[1924] = 1.361155E+07;
  eg20[1925] = 1.372545E+07;
  eg20[1926] = 1.384031E+07;
  eg20[1927] = 1.395612E+07;
  eg20[1928] = 1.407291E+07;
  eg20[1929] = 1.419068E+07;
  eg20[1930] = 1.430943E+07;
  eg20[1931] = 1.442917E+07;
  eg20[1932] = 1.454991E+07;
  eg20[1933] = 1.467167E+07;
  eg20[1934] = 1.479444E+07;
  eg20[1935] = 1.491825E+07;
  eg20[1936] = 1.504309E+07;
  eg20[1937] = 1.516897E+07;
  eg20[1938] = 1.529590E+07;
  eg20[1939] = 1.542390E+07;
  eg20[1940] = 1.555297E+07;
  eg20[1941] = 1.568312E+07;
  eg20[1942] = 1.581436E+07;
  eg20[1943] = 1.594670E+07;
  eg20[1944] = 1.608014E+07;
  eg20[1945] = 1.621470E+07;
  eg20[1946] = 1.635039E+07;
  eg20[1947] = 1.648721E+07;
  eg20[1948] = 1.662518E+07;
  eg20[1949] = 1.676430E+07;
  eg20[1950] = 1.690459E+07;
  eg20[1951] = 1.704605E+07;
  eg20[1952] = 1.718869E+07;
  eg20[1953] = 1.733253E+07;
  eg20[1954] = 1.747757E+07;
  eg20[1955] = 1.762383E+07;
  eg20[1956] = 1.777131E+07;
  eg20[1957] = 1.792002E+07;
  eg20[1958] = 1.806998E+07;
  eg20[1959] = 1.822119E+07;
  eg20[1960] = 1.837367E+07;
  eg20[1961] = 1.852742E+07;
  eg20[1962] = 1.868246E+07;
  eg20[1963] = 1.883880E+07;
  eg20[1964] = 1.899644E+07;
  eg20[1965] = 1.915541E+07;
  eg20[1966] = 1.931570E+07;
  eg20[1967] = 1.947734E+07;
  eg20[1968] = 1.964033E+07;

  return eg20;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg21()
{
  vector<Real> eg21;
  eg21.resize(316);
  eg21[  0] = 1.000010E-05;
  eg21[  1] = 1.100000E-04;
  eg21[  2] = 3.000000E-03;
  eg21[  3] = 5.500100E-03;
  eg21[  4] = 1.000000E-02;
  eg21[  5] = 1.500000E-02;
  eg21[  6] = 2.000000E-02;
  eg21[  7] = 3.000000E-02;
  eg21[  8] = 3.200000E-02;
  eg21[  9] = 3.238000E-02;
  eg21[ 10] = 4.300000E-02;
  eg21[ 11] = 5.900100E-02;
  eg21[ 12] = 7.700100E-02;
  eg21[ 13] = 9.500000E-02;
  eg21[ 14] = 1.000000E-01;
  eg21[ 15] = 1.150000E-01;
  eg21[ 16] = 1.340000E-01;
  eg21[ 17] = 1.600000E-01;
  eg21[ 18] = 1.890000E-01;
  eg21[ 19] = 2.200000E-01;
  eg21[ 20] = 2.480000E-01;
  eg21[ 21] = 2.825000E-01;
  eg21[ 22] = 3.145000E-01;
  eg21[ 23] = 3.520000E-01;
  eg21[ 24] = 3.910100E-01;
  eg21[ 25] = 4.139900E-01;
  eg21[ 26] = 4.330000E-01;
  eg21[ 27] = 4.850100E-01;
  eg21[ 28] = 5.315800E-01;
  eg21[ 29] = 5.400100E-01;
  eg21[ 30] = 6.250100E-01;
  eg21[ 31] = 6.825600E-01;
  eg21[ 32] = 7.050000E-01;
  eg21[ 33] = 7.900100E-01;
  eg21[ 34] = 8.600100E-01;
  eg21[ 35] = 8.764200E-01;
  eg21[ 36] = 9.300100E-01;
  eg21[ 37] = 9.860100E-01;
  eg21[ 38] = 1.010000E+00;
  eg21[ 39] = 1.035000E+00;
  eg21[ 40] = 1.070000E+00;
  eg21[ 41] = 1.080000E+00;
  eg21[ 42] = 1.090000E+00;
  eg21[ 43] = 1.110000E+00;
  eg21[ 44] = 1.125400E+00;
  eg21[ 45] = 1.170000E+00;
  eg21[ 46] = 1.235000E+00;
  eg21[ 47] = 1.305000E+00;
  eg21[ 48] = 1.370000E+00;
  eg21[ 49] = 1.440000E+00;
  eg21[ 50] = 1.445000E+00;
  eg21[ 51] = 1.510000E+00;
  eg21[ 52] = 1.590000E+00;
  eg21[ 53] = 1.670000E+00;
  eg21[ 54] = 1.755000E+00;
  eg21[ 55] = 1.840000E+00;
  eg21[ 56] = 1.855400E+00;
  eg21[ 57] = 1.930000E+00;
  eg21[ 58] = 2.020000E+00;
  eg21[ 59] = 2.130000E+00;
  eg21[ 60] = 2.360000E+00;
  eg21[ 61] = 2.372400E+00;
  eg21[ 62] = 2.767900E+00;
  eg21[ 63] = 3.059000E+00;
  eg21[ 64] = 3.380700E+00;
  eg21[ 65] = 3.927900E+00;
  eg21[ 66] = 4.129200E+00;
  eg21[ 67] = 4.470000E+00;
  eg21[ 68] = 4.670000E+00;
  eg21[ 69] = 5.043500E+00;
  eg21[ 70] = 5.623000E+00;
  eg21[ 71] = 6.160100E+00;
  eg21[ 72] = 6.476000E+00;
  eg21[ 73] = 7.079000E+00;
  eg21[ 74] = 7.524000E+00;
  eg21[ 75] = 7.943000E+00;
  eg21[ 76] = 8.315300E+00;
  eg21[ 77] = 8.913000E+00;
  eg21[ 78] = 9.189800E+00;
  eg21[ 79] = 1.000000E+01;
  eg21[ 80] = 1.067700E+01;
  eg21[ 81] = 1.122400E+01;
  eg21[ 82] = 1.259000E+01;
  eg21[ 83] = 1.371000E+01;
  eg21[ 84] = 1.522700E+01;
  eg21[ 85] = 1.674500E+01;
  eg21[ 86] = 1.760300E+01;
  eg21[ 87] = 1.902800E+01;
  eg21[ 88] = 2.045200E+01;
  eg21[ 89] = 2.260300E+01;
  eg21[ 90] = 2.498000E+01;
  eg21[ 91] = 2.791800E+01;
  eg21[ 92] = 2.920300E+01;
  eg21[ 93] = 3.051100E+01;
  eg21[ 94] = 3.388900E+01;
  eg21[ 95] = 3.726700E+01;
  eg21[ 96] = 3.981000E+01;
  eg21[ 97] = 4.551700E+01;
  eg21[ 98] = 4.785100E+01;
  eg21[ 99] = 5.012000E+01;
  eg21[100] = 5.559500E+01;
  eg21[101] = 6.144200E+01;
  eg21[102] = 6.310000E+01;
  eg21[103] = 6.790400E+01;
  eg21[104] = 7.079000E+01;
  eg21[105] = 7.889300E+01;
  eg21[106] = 8.527700E+01;
  eg21[107] = 9.166100E+01;
  eg21[108] = 1.013000E+02;
  eg21[109] = 1.122000E+02;
  eg21[110] = 1.300700E+02;
  eg21[111] = 1.367400E+02;
  eg21[112] = 1.585000E+02;
  eg21[113] = 1.670200E+02;
  eg21[114] = 1.778000E+02;
  eg21[115] = 2.039900E+02;
  eg21[116] = 2.144500E+02;
  eg21[117] = 2.430100E+02;
  eg21[118] = 2.753600E+02;
  eg21[119] = 3.043200E+02;
  eg21[120] = 3.535800E+02;
  eg21[121] = 3.981000E+02;
  eg21[122] = 4.540000E+02;
  eg21[123] = 5.144600E+02;
  eg21[124] = 5.829500E+02;
  eg21[125] = 6.310000E+02;
  eg21[126] = 6.772900E+02;
  eg21[127] = 7.079000E+02;
  eg21[128] = 7.485200E+02;
  eg21[129] = 8.482000E+02;
  eg21[130] = 9.611200E+02;
  eg21[131] = 1.010400E+03;
  eg21[132] = 1.116700E+03;
  eg21[133] = 1.234100E+03;
  eg21[134] = 1.363900E+03;
  eg21[135] = 1.507300E+03;
  eg21[136] = 1.584600E+03;
  eg21[137] = 1.795600E+03;
  eg21[138] = 2.034700E+03;
  eg21[139] = 2.113000E+03;
  eg21[140] = 2.248700E+03;
  eg21[141] = 2.371000E+03;
  eg21[142] = 2.485200E+03;
  eg21[143] = 2.612600E+03;
  eg21[144] = 2.661000E+03;
  eg21[145] = 2.746500E+03;
  eg21[146] = 2.818000E+03;
  eg21[147] = 3.035400E+03;
  eg21[148] = 3.162000E+03;
  eg21[149] = 3.354600E+03;
  eg21[150] = 3.548000E+03;
  eg21[151] = 3.707400E+03;
  eg21[152] = 3.981000E+03;
  eg21[153] = 4.307400E+03;
  eg21[154] = 4.642900E+03;
  eg21[155] = 5.004500E+03;
  eg21[156] = 5.530800E+03;
  eg21[157] = 6.267300E+03;
  eg21[158] = 7.101700E+03;
  eg21[159] = 7.465900E+03;
  eg21[160] = 8.251000E+03;
  eg21[161] = 9.118800E+03;
  eg21[162] = 1.007800E+04;
  eg21[163] = 1.113800E+04;
  eg21[164] = 1.170900E+04;
  eg21[165] = 1.272600E+04;
  eg21[166] = 1.383200E+04;
  eg21[167] = 1.503400E+04;
  eg21[168] = 1.585000E+04;
  eg21[169] = 1.661600E+04;
  eg21[170] = 1.778000E+04;
  eg21[171] = 1.930500E+04;
  eg21[172] = 1.995000E+04;
  eg21[173] = 2.054000E+04;
  eg21[174] = 2.113000E+04;
  eg21[175] = 2.187500E+04;
  eg21[176] = 2.239000E+04;
  eg21[177] = 2.304000E+04;
  eg21[178] = 2.357900E+04;
  eg21[179] = 2.417600E+04;
  eg21[180] = 2.441000E+04;
  eg21[181] = 2.478800E+04;
  eg21[182] = 2.512000E+04;
  eg21[183] = 2.585000E+04;
  eg21[184] = 2.605800E+04;
  eg21[185] = 2.661000E+04;
  eg21[186] = 2.700000E+04;
  eg21[187] = 2.738000E+04;
  eg21[188] = 2.818000E+04;
  eg21[189] = 2.850000E+04;
  eg21[190] = 2.901000E+04;
  eg21[191] = 2.985000E+04;
  eg21[192] = 3.073000E+04;
  eg21[193] = 3.162000E+04;
  eg21[194] = 3.182800E+04;
  eg21[195] = 3.430700E+04;
  eg21[196] = 3.697900E+04;
  eg21[197] = 4.086800E+04;
  eg21[198] = 4.358900E+04;
  eg21[199] = 4.630900E+04;
  eg21[200] = 4.939200E+04;
  eg21[201] = 5.247500E+04;
  eg21[202] = 5.516600E+04;
  eg21[203] = 5.656200E+04;
  eg21[204] = 6.172500E+04;
  eg21[205] = 6.737900E+04;
  eg21[206] = 7.200000E+04;
  eg21[207] = 7.499000E+04;
  eg21[208] = 7.950000E+04;
  eg21[209] = 8.229700E+04;
  eg21[210] = 8.250000E+04;
  eg21[211] = 8.651700E+04;
  eg21[212] = 9.803700E+04;
  eg21[213] = 1.110900E+05;
  eg21[214] = 1.167900E+05;
  eg21[215] = 1.227700E+05;
  eg21[216] = 1.290700E+05;
  eg21[217] = 1.356900E+05;
  eg21[218] = 1.426400E+05;
  eg21[219] = 1.499600E+05;
  eg21[220] = 1.576400E+05;
  eg21[221] = 1.657300E+05;
  eg21[222] = 1.742200E+05;
  eg21[223] = 1.831600E+05;
  eg21[224] = 1.925500E+05;
  eg21[225] = 2.024200E+05;
  eg21[226] = 2.128000E+05;
  eg21[227] = 2.237100E+05;
  eg21[228] = 2.351800E+05;
  eg21[229] = 2.472400E+05;
  eg21[230] = 2.732400E+05;
  eg21[231] = 2.872500E+05;
  eg21[232] = 2.945200E+05;
  eg21[233] = 2.972000E+05;
  eg21[234] = 2.985000E+05;
  eg21[235] = 3.019700E+05;
  eg21[236] = 3.337300E+05;
  eg21[237] = 3.688300E+05;
  eg21[238] = 3.877400E+05;
  eg21[239] = 4.076200E+05;
  eg21[240] = 4.504900E+05;
  eg21[241] = 5.234000E+05;
  eg21[242] = 5.502300E+05;
  eg21[243] = 5.784400E+05;
  eg21[244] = 6.081000E+05;
  eg21[245] = 6.392800E+05;
  eg21[246] = 6.720600E+05;
  eg21[247] = 7.065100E+05;
  eg21[248] = 7.427400E+05;
  eg21[249] = 7.808200E+05;
  eg21[250] = 8.208500E+05;
  eg21[251] = 8.629400E+05;
  eg21[252] = 9.071800E+05;
  eg21[253] = 9.616400E+05;
  eg21[254] = 1.002600E+06;
  eg21[255] = 1.108000E+06;
  eg21[256] = 1.164800E+06;
  eg21[257] = 1.224600E+06;
  eg21[258] = 1.287300E+06;
  eg21[259] = 1.353400E+06;
  eg21[260] = 1.422700E+06;
  eg21[261] = 1.495700E+06;
  eg21[262] = 1.572400E+06;
  eg21[263] = 1.653000E+06;
  eg21[264] = 1.737700E+06;
  eg21[265] = 1.826800E+06;
  eg21[266] = 1.920500E+06;
  eg21[267] = 2.019000E+06;
  eg21[268] = 2.122500E+06;
  eg21[269] = 2.231300E+06;
  eg21[270] = 2.306900E+06;
  eg21[271] = 2.345700E+06;
  eg21[272] = 2.365300E+06;
  eg21[273] = 2.385200E+06;
  eg21[274] = 2.466000E+06;
  eg21[275] = 2.592400E+06;
  eg21[276] = 2.725300E+06;
  eg21[277] = 2.865000E+06;
  eg21[278] = 3.011900E+06;
  eg21[279] = 3.166400E+06;
  eg21[280] = 3.328700E+06;
  eg21[281] = 3.678800E+06;
  eg21[282] = 4.065700E+06;
  eg21[283] = 4.493300E+06;
  eg21[284] = 4.723700E+06;
  eg21[285] = 4.965900E+06;
  eg21[286] = 5.220500E+06;
  eg21[287] = 5.488100E+06;
  eg21[288] = 5.769500E+06;
  eg21[289] = 6.065300E+06;
  eg21[290] = 6.376300E+06;
  eg21[291] = 6.592400E+06;
  eg21[292] = 6.703200E+06;
  eg21[293] = 7.046900E+06;
  eg21[294] = 7.408200E+06;
  eg21[295] = 7.788000E+06;
  eg21[296] = 8.187300E+06;
  eg21[297] = 8.607100E+06;
  eg21[298] = 9.048400E+06;
  eg21[299] = 9.512300E+06;
  eg21[300] = 1.000000E+07;
  eg21[301] = 1.051300E+07;
  eg21[302] = 1.105200E+07;
  eg21[303] = 1.161800E+07;
  eg21[304] = 1.221400E+07;
  eg21[305] = 1.284000E+07;
  eg21[306] = 1.349900E+07;
  eg21[307] = 1.384000E+07;
  eg21[308] = 1.419100E+07;
  eg21[309] = 1.455000E+07;
  eg21[310] = 1.491800E+07;
  eg21[311] = 1.568300E+07;
  eg21[312] = 1.648700E+07;
  eg21[313] = 1.690500E+07;
  eg21[314] = 1.733300E+07;
  eg21[315] = 1.964000E+07;

  return eg21;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg22()
{
  vector<Real> eg22;
  eg22.resize(173);
  eg22[  0] = 1.000010E-05;
  eg22[  1] = 3.000000E-03;
  eg22[  2] = 5.000000E-03;
  eg22[  3] = 6.900000E-03;
  eg22[  4] = 1.000000E-02;
  eg22[  5] = 1.500000E-02;
  eg22[  6] = 2.000000E-02;
  eg22[  7] = 2.500000E-02;
  eg22[  8] = 3.000000E-02;
  eg22[  9] = 3.500000E-02;
  eg22[ 10] = 4.200000E-02;
  eg22[ 11] = 5.000000E-02;
  eg22[ 12] = 5.800000E-02;
  eg22[ 13] = 6.700000E-02;
  eg22[ 14] = 7.700000E-02;
  eg22[ 15] = 8.000000E-02;
  eg22[ 16] = 9.500000E-02;
  eg22[ 17] = 1.000000E-01;
  eg22[ 18] = 1.150000E-01;
  eg22[ 19] = 1.340000E-01;
  eg22[ 20] = 1.400000E-01;
  eg22[ 21] = 1.600000E-01;
  eg22[ 22] = 1.800000E-01;
  eg22[ 23] = 1.890000E-01;
  eg22[ 24] = 2.200000E-01;
  eg22[ 25] = 2.480000E-01;
  eg22[ 26] = 2.800000E-01;
  eg22[ 27] = 3.000000E-01;
  eg22[ 28] = 3.145000E-01;
  eg22[ 29] = 3.200000E-01;
  eg22[ 30] = 3.500000E-01;
  eg22[ 31] = 3.910000E-01;
  eg22[ 32] = 4.000000E-01;
  eg22[ 33] = 4.330000E-01;
  eg22[ 34] = 4.850000E-01;
  eg22[ 35] = 5.000000E-01;
  eg22[ 36] = 5.400000E-01;
  eg22[ 37] = 6.250000E-01;
  eg22[ 38] = 7.050000E-01;
  eg22[ 39] = 7.800000E-01;
  eg22[ 40] = 7.900000E-01;
  eg22[ 41] = 8.500000E-01;
  eg22[ 42] = 8.600000E-01;
  eg22[ 43] = 9.100000E-01;
  eg22[ 44] = 9.300000E-01;
  eg22[ 45] = 9.500000E-01;
  eg22[ 46] = 9.720000E-01;
  eg22[ 47] = 9.860000E-01;
  eg22[ 48] = 9.960000E-01;
  eg22[ 49] = 1.020000E+00;
  eg22[ 50] = 1.035000E+00;
  eg22[ 51] = 1.045000E+00;
  eg22[ 52] = 1.071000E+00;
  eg22[ 53] = 1.097000E+00;
  eg22[ 54] = 1.110000E+00;
  eg22[ 55] = 1.123000E+00;
  eg22[ 56] = 1.150000E+00;
  eg22[ 57] = 1.170000E+00;
  eg22[ 58] = 1.235000E+00;
  eg22[ 59] = 1.300000E+00;
  eg22[ 60] = 1.337500E+00;
  eg22[ 61] = 1.370000E+00;
  eg22[ 62] = 1.440000E+00;
  eg22[ 63] = 1.475000E+00;
  eg22[ 64] = 1.500000E+00;
  eg22[ 65] = 1.590000E+00;
  eg22[ 66] = 1.670000E+00;
  eg22[ 67] = 1.755000E+00;
  eg22[ 68] = 1.840000E+00;
  eg22[ 69] = 1.930000E+00;
  eg22[ 70] = 2.020000E+00;
  eg22[ 71] = 2.100000E+00;
  eg22[ 72] = 2.130000E+00;
  eg22[ 73] = 2.360000E+00;
  eg22[ 74] = 2.550000E+00;
  eg22[ 75] = 2.600000E+00;
  eg22[ 76] = 2.720000E+00;
  eg22[ 77] = 2.767920E+00;
  eg22[ 78] = 3.300000E+00;
  eg22[ 79] = 3.380750E+00;
  eg22[ 80] = 4.000000E+00;
  eg22[ 81] = 4.129250E+00;
  eg22[ 82] = 5.043477E+00;
  eg22[ 83] = 5.346430E+00;
  eg22[ 84] = 6.160116E+00;
  eg22[ 85] = 7.523983E+00;
  eg22[ 86] = 8.315287E+00;
  eg22[ 87] = 9.189814E+00;
  eg22[ 88] = 9.905554E+00;
  eg22[ 89] = 1.122446E+01;
  eg22[ 90] = 1.370959E+01;
  eg22[ 91] = 1.592827E+01;
  eg22[ 92] = 1.945484E+01;
  eg22[ 93] = 2.260329E+01;
  eg22[ 94] = 2.498050E+01;
  eg22[ 95] = 2.760773E+01;
  eg22[ 96] = 3.051126E+01;
  eg22[ 97] = 3.372015E+01;
  eg22[ 98] = 3.726653E+01;
  eg22[ 99] = 4.016900E+01;
  eg22[100] = 4.551744E+01;
  eg22[101] = 4.825160E+01;
  eg22[102] = 5.157802E+01;
  eg22[103] = 5.559513E+01;
  eg22[104] = 6.790405E+01;
  eg22[105] = 7.567357E+01;
  eg22[106] = 9.166088E+01;
  eg22[107] = 1.367420E+02;
  eg22[108] = 1.486254E+02;
  eg22[109] = 2.039950E+02;
  eg22[110] = 3.043248E+02;
  eg22[111] = 3.717032E+02;
  eg22[112] = 4.539993E+02;
  eg22[113] = 6.772874E+02;
  eg22[114] = 7.485183E+02;
  eg22[115] = 9.142423E+02;
  eg22[116] = 1.010394E+03;
  eg22[117] = 1.234098E+03;
  eg22[118] = 1.433817E+03;
  eg22[119] = 1.507331E+03;
  eg22[120] = 2.034684E+03;
  eg22[121] = 2.248673E+03;
  eg22[122] = 3.354626E+03;
  eg22[123] = 3.526622E+03;
  eg22[124] = 5.004514E+03;
  eg22[125] = 5.530844E+03;
  eg22[126] = 7.465858E+03;
  eg22[127] = 9.118820E+03;
  eg22[128] = 1.113775E+04;
  eg22[129] = 1.503439E+04;
  eg22[130] = 1.661557E+04;
  eg22[131] = 2.478752E+04;
  eg22[132] = 2.739445E+04;
  eg22[133] = 2.928300E+04;
  eg22[134] = 3.697864E+04;
  eg22[135] = 4.086771E+04;
  eg22[136] = 5.516564E+04;
  eg22[137] = 6.737947E+04;
  eg22[138] = 8.229747E+04;
  eg22[139] = 1.110900E+05;
  eg22[140] = 1.227734E+05;
  eg22[141] = 1.831564E+05;
  eg22[142] = 2.472353E+05;
  eg22[143] = 2.732372E+05;
  eg22[144] = 3.019738E+05;
  eg22[145] = 4.076220E+05;
  eg22[146] = 4.504920E+05;
  eg22[147] = 4.978707E+05;
  eg22[148] = 5.502322E+05;
  eg22[149] = 6.081006E+05;
  eg22[150] = 8.208500E+05;
  eg22[151] = 9.071795E+05;
  eg22[152] = 1.002588E+06;
  eg22[153] = 1.108032E+06;
  eg22[154] = 1.224564E+06;
  eg22[155] = 1.353353E+06;
  eg22[156] = 1.652989E+06;
  eg22[157] = 2.018965E+06;
  eg22[158] = 2.231302E+06;
  eg22[159] = 2.465970E+06;
  eg22[160] = 3.011942E+06;
  eg22[161] = 3.678794E+06;
  eg22[162] = 4.493290E+06;
  eg22[163] = 5.488116E+06;
  eg22[164] = 6.065307E+06;
  eg22[165] = 6.703200E+06;
  eg22[166] = 8.187308E+06;
  eg22[167] = 1.000000E+07;
  eg22[168] = 1.161834E+07;
  eg22[169] = 1.384031E+07;
  eg22[170] = 1.491825E+07;
  eg22[171] = 1.733253E+07;
  eg22[172] = 1.964033E+07;

  return eg22;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg23()
{
  vector<Real> eg23;
  eg23.resize(176);
  eg23[  0] = 1.000010e-05;
  eg23[  1] = 1.000010e-01;
  eg23[  2] = 4.139940e-01;
  eg23[  3] = 5.315790e-01;
  eg23[  4] = 6.825600e-01;
  eg23[  5] = 8.764250e-01;
  eg23[  6] = 1.123000e+00;
  eg23[  7] = 1.440000e+00;
  eg23[  8] = 1.855390e+00;
  eg23[  9] = 2.382370e+00;
  eg23[ 10] = 3.059020e+00;
  eg23[ 11] = 3.927860e+00;
  eg23[ 12] = 5.043480e+00;
  eg23[ 13] = 6.475950e+00;
  eg23[ 14] = 8.315290e+00;
  eg23[ 15] = 1.067700e+01;
  eg23[ 16] = 1.370960e+01;
  eg23[ 17] = 1.760350e+01;
  eg23[ 18] = 2.260330e+01;
  eg23[ 19] = 2.902320e+01;
  eg23[ 20] = 3.726650e+01;
  eg23[ 21] = 4.785120e+01;
  eg23[ 22] = 6.144210e+01;
  eg23[ 23] = 7.889320e+01;
  eg23[ 24] = 1.013010e+02;
  eg23[ 25] = 1.300730e+02;
  eg23[ 26] = 1.670170e+02;
  eg23[ 27] = 2.144540e+02;
  eg23[ 28] = 2.753640e+02;
  eg23[ 29] = 3.535750e+02;
  eg23[ 30] = 4.539990e+02;
  eg23[ 31] = 5.829470e+02;
  eg23[ 32] = 7.485180e+02;
  eg23[ 33] = 9.611170e+02;
  eg23[ 34] = 1.234100e+03;
  eg23[ 35] = 1.584610e+03;
  eg23[ 36] = 2.034680e+03;
  eg23[ 37] = 2.248670e+03;
  eg23[ 38] = 2.485170e+03;
  eg23[ 39] = 2.612590e+03;
  eg23[ 40] = 2.746540e+03;
  eg23[ 41] = 3.035390e+03;
  eg23[ 42] = 3.354630e+03;
  eg23[ 43] = 3.707440e+03;
  eg23[ 44] = 4.307420e+03;
  eg23[ 45] = 5.530840e+03;
  eg23[ 46] = 7.101740e+03;
  eg23[ 47] = 9.118820e+03;
  eg23[ 48] = 1.059460e+04;
  eg23[ 49] = 1.170880e+04;
  eg23[ 50] = 1.503440e+04;
  eg23[ 51] = 1.930450e+04;
  eg23[ 52] = 2.187490e+04;
  eg23[ 53] = 2.357860e+04;
  eg23[ 54] = 2.417550e+04;
  eg23[ 55] = 2.478750e+04;
  eg23[ 56] = 2.605840e+04;
  eg23[ 57] = 2.700010e+04;
  eg23[ 58] = 2.850110e+04;
  eg23[ 59] = 3.182780e+04;
  eg23[ 60] = 3.430670e+04;
  eg23[ 61] = 4.086770e+04;
  eg23[ 62] = 4.630920e+04;
  eg23[ 63] = 5.247520e+04;
  eg23[ 64] = 5.656220e+04;
  eg23[ 65] = 6.737950e+04;
  eg23[ 66] = 7.202450e+04;
  eg23[ 67] = 7.949870e+04;
  eg23[ 68] = 8.250340e+04;
  eg23[ 69] = 8.651700e+04;
  eg23[ 70] = 9.803650e+04;
  eg23[ 71] = 1.110900e+05;
  eg23[ 72] = 1.167860e+05;
  eg23[ 73] = 1.227730e+05;
  eg23[ 74] = 1.290680e+05;
  eg23[ 75] = 1.356860e+05;
  eg23[ 76] = 1.426420e+05;
  eg23[ 77] = 1.499560e+05;
  eg23[ 78] = 1.576440e+05;
  eg23[ 79] = 1.657270e+05;
  eg23[ 80] = 1.742240e+05;
  eg23[ 81] = 1.831560e+05;
  eg23[ 82] = 1.925470e+05;
  eg23[ 83] = 2.024190e+05;
  eg23[ 84] = 2.127970e+05;
  eg23[ 85] = 2.237080e+05;
  eg23[ 86] = 2.351770e+05;
  eg23[ 87] = 2.472350e+05;
  eg23[ 88] = 2.732370e+05;
  eg23[ 89] = 2.872460e+05;
  eg23[ 90] = 2.945180e+05;
  eg23[ 91] = 2.972110e+05;
  eg23[ 92] = 2.984910e+05;
  eg23[ 93] = 3.019740e+05;
  eg23[ 94] = 3.337330e+05;
  eg23[ 95] = 3.688320e+05;
  eg23[ 96] = 3.877420e+05;
  eg23[ 97] = 4.076220e+05;
  eg23[ 98] = 4.504920e+05;
  eg23[ 99] = 4.978710e+05;
  eg23[100] = 5.233970e+05;
  eg23[101] = 5.502320e+05;
  eg23[102] = 5.784430e+05;
  eg23[103] = 6.081010e+05;
  eg23[104] = 6.392790e+05;
  eg23[105] = 6.720550e+05;
  eg23[106] = 7.065120e+05;
  eg23[107] = 7.427360e+05;
  eg23[108] = 7.808170e+05;
  eg23[109] = 8.208500e+05;
  eg23[110] = 8.629360e+05;
  eg23[111] = 9.071800e+05;
  eg23[112] = 9.616720e+05;
  eg23[113] = 1.002590e+06;
  eg23[114] = 1.108030e+06;
  eg23[115] = 1.164840e+06;
  eg23[116] = 1.224560e+06;
  eg23[117] = 1.287350e+06;
  eg23[118] = 1.353350e+06;
  eg23[119] = 1.422740e+06;
  eg23[120] = 1.495690e+06;
  eg23[121] = 1.572370e+06;
  eg23[122] = 1.652990e+06;
  eg23[123] = 1.737740e+06;
  eg23[124] = 1.826840e+06;
  eg23[125] = 1.920500e+06;
  eg23[126] = 2.018970e+06;
  eg23[127] = 2.122480e+06;
  eg23[128] = 2.231300e+06;
  eg23[129] = 2.306930e+06;
  eg23[130] = 2.345700e+06;
  eg23[131] = 2.365330e+06;
  eg23[132] = 2.385130e+06;
  eg23[133] = 2.465970e+06;
  eg23[134] = 2.592400e+06;
  eg23[135] = 2.725320e+06;
  eg23[136] = 2.865050e+06;
  eg23[137] = 3.011940e+06;
  eg23[138] = 3.166370e+06;
  eg23[139] = 3.328710e+06;
  eg23[140] = 3.678790e+06;
  eg23[141] = 4.065700e+06;
  eg23[142] = 4.493290e+06;
  eg23[143] = 4.723670e+06;
  eg23[144] = 4.965850e+06;
  eg23[145] = 5.220460e+06;
  eg23[146] = 5.488120e+06;
  eg23[147] = 5.769500e+06;
  eg23[148] = 6.065310e+06;
  eg23[149] = 6.376280e+06;
  eg23[150] = 6.592410e+06;
  eg23[151] = 6.703200e+06;
  eg23[152] = 7.046880e+06;
  eg23[153] = 7.408180e+06;
  eg23[154] = 7.788010e+06;
  eg23[155] = 8.187310e+06;
  eg23[156] = 8.607080e+06;
  eg23[157] = 9.048370e+06;
  eg23[158] = 9.512290e+06;
  eg23[159] = 1.000000e+07;
  eg23[160] = 1.051270e+07;
  eg23[161] = 1.105170e+07;
  eg23[162] = 1.161830e+07;
  eg23[163] = 1.221400e+07;
  eg23[164] = 1.252320e+07;
  eg23[165] = 1.284030e+07;
  eg23[166] = 1.349860e+07;
  eg23[167] = 1.384030e+07;
  eg23[168] = 1.419070e+07;
  eg23[169] = 1.454990e+07;
  eg23[170] = 1.491820e+07;
  eg23[171] = 1.568310e+07;
  eg23[172] = 1.648720e+07;
  eg23[173] = 1.690460e+07;
  eg23[174] = 1.733250e+07;
  eg23[175] = 1.964030e+07;

  return eg23;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg24()
{
  vector<Real> eg24;
  eg24.resize(282);
  eg24[  0] = 1.100027E-04;
  eg24[  1] = 2.499897E-03;
  eg24[  2] = 4.556021E-03;
  eg24[  3] = 7.145263E-03;
  eg24[  4] = 1.045050E-02;
  eg24[  5] = 1.482996E-02;
  eg24[  6] = 2.001035E-02;
  eg24[  7] = 2.493942E-02;
  eg24[  8] = 2.929889E-02;
  eg24[  9] = 3.439976E-02;
  eg24[ 10] = 4.029993E-02;
  eg24[ 11] = 4.730186E-02;
  eg24[ 12] = 5.549815E-02;
  eg24[ 13] = 6.519936E-02;
  eg24[ 14] = 7.649686E-02;
  eg24[ 15] = 8.979683E-02;
  eg24[ 16] = 1.042977E-01;
  eg24[ 17] = 1.199949E-01;
  eg24[ 18] = 1.379994E-01;
  eg24[ 19] = 1.618953E-01;
  eg24[ 20] = 1.900049E-01;
  eg24[ 21] = 2.096102E-01;
  eg24[ 22] = 2.311923E-01;
  eg24[ 23] = 2.549965E-01;
  eg24[ 24] = 2.799888E-01;
  eg24[ 25] = 3.050115E-01;
  eg24[ 26] = 3.250079E-01;
  eg24[ 27] = 3.529935E-01;
  eg24[ 28] = 3.900011E-01;
  eg24[ 29] = 4.315786E-01;
  eg24[ 30] = 4.750165E-01;
  eg24[ 31] = 5.200108E-01;
  eg24[ 32] = 5.549897E-01;
  eg24[ 33] = 5.949930E-01;
  eg24[ 34] = 6.249987E-01;
  eg24[ 35] = 7.199989E-01;
  eg24[ 36] = 8.200371E-01;
  eg24[ 37] = 8.800244E-01;
  eg24[ 38] = 9.199779E-01;
  eg24[ 39] = 9.440222E-01;
  eg24[ 40] = 9.639598E-01;
  eg24[ 41] = 9.819591E-01;
  eg24[ 42] = 9.965005E-01;
  eg24[ 43] = 1.009035E+00;
  eg24[ 44] = 1.021012E+00;
  eg24[ 45] = 1.034993E+00;
  eg24[ 46] = 1.077986E+00;
  eg24[ 47] = 1.091982E+00;
  eg24[ 48] = 1.103950E+00;
  eg24[ 49] = 1.116049E+00;
  eg24[ 50] = 1.129974E+00;
  eg24[ 51] = 1.147969E+00;
  eg24[ 52] = 1.169989E+00;
  eg24[ 53] = 1.213968E+00;
  eg24[ 54] = 1.250939E+00;
  eg24[ 55] = 1.293038E+00;
  eg24[ 56] = 1.330952E+00;
  eg24[ 57] = 1.380981E+00;
  eg24[ 58] = 1.410007E+00;
  eg24[ 59] = 1.443967E+00;
  eg24[ 60] = 1.519976E+00;
  eg24[ 61] = 1.588030E+00;
  eg24[ 62] = 1.668949E+00;
  eg24[ 63] = 1.779966E+00;
  eg24[ 64] = 1.900077E+00;
  eg24[ 65] = 1.989920E+00;
  eg24[ 66] = 2.070095E+00;
  eg24[ 67] = 2.156948E+00;
  eg24[ 68] = 2.217087E+00;
  eg24[ 69] = 2.272986E+00;
  eg24[ 70] = 2.330061E+00;
  eg24[ 71] = 2.469941E+00;
  eg24[ 72] = 2.550003E+00;
  eg24[ 73] = 2.590094E+00;
  eg24[ 74] = 2.620053E+00;
  eg24[ 75] = 2.640041E+00;
  eg24[ 76] = 2.700115E+00;
  eg24[ 77] = 2.719898E+00;
  eg24[ 78] = 2.740922E+00;
  eg24[ 79] = 2.775121E+00;
  eg24[ 80] = 2.884047E+00;
  eg24[ 81] = 3.142109E+00;
  eg24[ 82] = 3.543073E+00;
  eg24[ 83] = 3.712087E+00;
  eg24[ 84] = 3.882170E+00;
  eg24[ 85] = 4.000000E+00;
  eg24[ 86] = 4.219828E+00;
  eg24[ 87] = 4.309812E+00;
  eg24[ 88] = 4.419800E+00;
  eg24[ 89] = 4.767845E+00;
  eg24[ 90] = 4.933232E+00;
  eg24[ 91] = 5.109974E+00;
  eg24[ 92] = 5.210076E+00;
  eg24[ 93] = 5.320112E+00;
  eg24[ 94] = 5.380032E+00;
  eg24[ 95] = 5.410245E+00;
  eg24[ 96] = 5.488167E+00;
  eg24[ 97] = 5.530036E+00;
  eg24[ 98] = 5.619790E+00;
  eg24[ 99] = 5.720146E+00;
  eg24[100] = 5.800211E+00;
  eg24[101] = 5.960142E+00;
  eg24[102] = 6.059906E+00;
  eg24[103] = 6.160108E+00;
  eg24[104] = 6.280153E+00;
  eg24[105] = 6.359784E+00;
  eg24[106] = 6.432057E+00;
  eg24[107] = 6.481775E+00;
  eg24[108] = 6.514916E+00;
  eg24[109] = 6.539066E+00;
  eg24[110] = 6.556090E+00;
  eg24[111] = 6.571843E+00;
  eg24[112] = 6.588293E+00;
  eg24[113] = 6.606106E+00;
  eg24[114] = 6.631257E+00;
  eg24[115] = 6.716683E+00;
  eg24[116] = 6.742254E+00;
  eg24[117] = 6.759807E+00;
  eg24[118] = 6.776050E+00;
  eg24[119] = 6.791653E+00;
  eg24[120] = 6.810696E+00;
  eg24[121] = 6.835259E+00;
  eg24[122] = 6.870208E+00;
  eg24[123] = 6.917776E+00;
  eg24[124] = 6.994292E+00;
  eg24[125] = 7.139869E+00;
  eg24[126] = 7.380153E+00;
  eg24[127] = 7.600350E+00;
  eg24[128] = 7.739943E+00;
  eg24[129] = 7.839651E+00;
  eg24[130] = 7.970079E+00;
  eg24[131] = 8.130272E+00;
  eg24[132] = 8.300322E+00;
  eg24[133] = 8.524074E+00;
  eg24[134] = 8.673690E+00;
  eg24[135] = 8.800375E+00;
  eg24[136] = 8.979950E+00;
  eg24[137] = 9.140311E+00;
  eg24[138] = 9.500024E+00;
  eg24[139] = 1.057925E+01;
  eg24[140] = 1.080376E+01;
  eg24[141] = 1.105292E+01;
  eg24[142] = 1.126944E+01;
  eg24[143] = 1.158944E+01;
  eg24[144] = 1.170943E+01;
  eg24[145] = 1.181529E+01;
  eg24[146] = 1.197947E+01;
  eg24[147] = 1.213015E+01;
  eg24[148] = 1.230855E+01;
  eg24[149] = 1.247210E+01;
  eg24[150] = 1.259997E+01;
  eg24[151] = 1.332970E+01;
  eg24[152] = 1.354604E+01;
  eg24[153] = 1.404961E+01;
  eg24[154] = 1.425053E+01;
  eg24[155] = 1.447024E+01;
  eg24[156] = 1.459522E+01;
  eg24[157] = 1.473012E+01;
  eg24[158] = 1.486626E+01;
  eg24[159] = 1.577923E+01;
  eg24[160] = 1.604977E+01;
  eg24[161] = 1.655014E+01;
  eg24[162] = 1.683053E+01;
  eg24[163] = 1.744572E+01;
  eg24[164] = 1.756476E+01;
  eg24[165] = 1.775903E+01;
  eg24[166] = 1.795905E+01;
  eg24[167] = 1.908484E+01;
  eg24[168] = 1.919969E+01;
  eg24[169] = 1.939265E+01;
  eg24[170] = 1.959735E+01;
  eg24[171] = 2.007338E+01;
  eg24[172] = 2.027512E+01;
  eg24[173] = 2.041754E+01;
  eg24[174] = 2.051988E+01;
  eg24[175] = 2.060213E+01;
  eg24[176] = 2.068470E+01;
  eg24[177] = 2.076761E+01;
  eg24[178] = 2.097632E+01;
  eg24[179] = 2.106040E+01;
  eg24[180] = 2.114481E+01;
  eg24[181] = 2.122956E+01;
  eg24[182] = 2.133597E+01;
  eg24[183] = 2.148585E+01;
  eg24[184] = 2.170178E+01;
  eg24[185] = 2.200114E+01;
  eg24[186] = 2.215569E+01;
  eg24[187] = 2.237836E+01;
  eg24[188] = 2.253556E+01;
  eg24[189] = 2.460856E+01;
  eg24[190] = 2.760769E+01;
  eg24[191] = 3.372011E+01;
  eg24[192] = 4.016895E+01;
  eg24[193] = 4.399581E+01;
  eg24[194] = 4.579131E+01;
  eg24[195] = 5.267255E+01;
  eg24[196] = 6.144204E+01;
  eg24[197] = 7.504548E+01;
  eg24[198] = 8.895177E+01;
  eg24[199] = 1.086459E+02;
  eg24[200] = 1.327005E+02;
  eg24[201] = 1.620807E+02;
  eg24[202] = 1.979658E+02;
  eg24[203] = 2.417960E+02;
  eg24[204] = 2.837502E+02;
  eg24[205] = 3.199275E+02;
  eg24[206] = 3.535746E+02;
  eg24[207] = 4.107950E+02;
  eg24[208] = 5.017462E+02;
  eg24[209] = 6.128342E+02;
  eg24[210] = 7.485173E+02;
  eg24[211] = 9.075007E+02;
  eg24[212] = 1.064962E+03;
  eg24[213] = 1.135007E+03;
  eg24[214] = 1.345061E+03;
  eg24[215] = 1.614038E+03;
  eg24[216] = 1.910451E+03;
  eg24[217] = 2.219627E+03;
  eg24[218] = 2.578838E+03;
  eg24[219] = 2.996183E+03;
  eg24[220] = 3.481068E+03;
  eg24[221] = 4.097345E+03;
  eg24[222] = 5.004508E+03;
  eg24[223] = 6.112520E+03;
  eg24[224] = 7.465848E+03;
  eg24[225] = 9.118808E+03;
  eg24[226] = 1.113774E+04;
  eg24[227] = 1.360366E+04;
  eg24[228] = 1.489967E+04;
  eg24[229] = 1.620045E+04;
  eg24[230] = 1.858471E+04;
  eg24[231] = 2.269941E+04;
  eg24[232] = 2.499908E+04;
  eg24[233] = 2.610010E+04;
  eg24[234] = 2.739441E+04;
  eg24[235] = 2.928101E+04;
  eg24[236] = 3.345961E+04;
  eg24[237] = 3.697859E+04;
  eg24[238] = 4.086766E+04;
  eg24[239] = 4.991587E+04;
  eg24[240] = 5.516557E+04;
  eg24[241] = 6.737938E+04;
  eg24[242] = 8.229736E+04;
  eg24[243] = 9.466450E+04;
  eg24[244] = 1.156235E+05;
  eg24[245] = 1.227732E+05;
  eg24[246] = 1.399995E+05;
  eg24[247] = 1.649989E+05;
  eg24[248] = 1.950077E+05;
  eg24[249] = 2.300137E+05;
  eg24[250] = 2.678264E+05;
  eg24[251] = 3.206464E+05;
  eg24[252] = 3.838835E+05;
  eg24[253] = 4.125012E+05;
  eg24[254] = 4.560211E+05;
  eg24[255] = 4.940018E+05;
  eg24[256] = 5.784425E+05;
  eg24[257] = 7.065112E+05;
  eg24[258] = 8.600058E+05;
  eg24[259] = 9.511189E+05;
  eg24[260] = 1.051149E+06;
  eg24[261] = 1.162048E+06;
  eg24[262] = 1.286961E+06;
  eg24[263] = 1.336941E+06;
  eg24[264] = 1.405768E+06;
  eg24[265] = 1.636539E+06;
  eg24[266] = 1.901387E+06;
  eg24[267] = 2.231299E+06;
  eg24[268] = 2.725314E+06;
  eg24[269] = 3.328707E+06;
  eg24[270] = 4.065691E+06;
  eg24[271] = 4.965847E+06;
  eg24[272] = 6.065299E+06;
  eg24[273] = 6.703192E+06;
  eg24[274] = 7.408173E+06;
  eg24[275] = 8.187297E+06;
  eg24[276] = 9.048363E+06;
  eg24[277] = 9.999987E+06;
  eg24[278] = 1.161833E+07;
  eg24[279] = 1.384029E+07;
  eg24[280] = 1.491823E+07;
  eg24[281] = 1.964030E+07;

  return eg24;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg25()
{
  vector<Real> eg25;
  eg25.resize(296);
  eg25[  0] = 1.100027E-04;
  eg25[  1] = 2.499897E-03;
  eg25[  2] = 4.556021E-03;
  eg25[  3] = 7.145263E-03;
  eg25[  4] = 1.045050E-02;
  eg25[  5] = 1.482996E-02;
  eg25[  6] = 2.001035E-02;
  eg25[  7] = 2.493942E-02;
  eg25[  8] = 2.929889E-02;
  eg25[  9] = 3.439976E-02;
  eg25[ 10] = 4.029993E-02;
  eg25[ 11] = 4.730186E-02;
  eg25[ 12] = 5.549815E-02;
  eg25[ 13] = 6.519936E-02;
  eg25[ 14] = 7.649686E-02;
  eg25[ 15] = 8.979683E-02;
  eg25[ 16] = 1.042977E-01;
  eg25[ 17] = 1.199949E-01;
  eg25[ 18] = 1.379994E-01;
  eg25[ 19] = 1.618953E-01;
  eg25[ 20] = 1.900049E-01;
  eg25[ 21] = 2.096102E-01;
  eg25[ 22] = 2.311923E-01;
  eg25[ 23] = 2.549965E-01;
  eg25[ 24] = 2.799888E-01;
  eg25[ 25] = 3.050115E-01;
  eg25[ 26] = 3.250079E-01;
  eg25[ 27] = 3.529935E-01;
  eg25[ 28] = 3.900011E-01;
  eg25[ 29] = 4.315786E-01;
  eg25[ 30] = 4.750165E-01;
  eg25[ 31] = 5.200108E-01;
  eg25[ 32] = 5.549897E-01;
  eg25[ 33] = 5.949930E-01;
  eg25[ 34] = 6.249987E-01;
  eg25[ 35] = 7.199989E-01;
  eg25[ 36] = 8.200371E-01;
  eg25[ 37] = 8.800244E-01;
  eg25[ 38] = 9.199779E-01;
  eg25[ 39] = 9.440222E-01;
  eg25[ 40] = 9.639598E-01;
  eg25[ 41] = 9.819591E-01;
  eg25[ 42] = 9.965005E-01;
  eg25[ 43] = 1.009035E+00;
  eg25[ 44] = 1.021012E+00;
  eg25[ 45] = 1.034993E+00;
  eg25[ 46] = 1.077986E+00;
  eg25[ 47] = 1.091982E+00;
  eg25[ 48] = 1.103950E+00;
  eg25[ 49] = 1.116049E+00;
  eg25[ 50] = 1.129974E+00;
  eg25[ 51] = 1.147969E+00;
  eg25[ 52] = 1.169989E+00;
  eg25[ 53] = 1.213968E+00;
  eg25[ 54] = 1.250939E+00;
  eg25[ 55] = 1.293038E+00;
  eg25[ 56] = 1.330952E+00;
  eg25[ 57] = 1.380981E+00;
  eg25[ 58] = 1.410007E+00;
  eg25[ 59] = 1.443967E+00;
  eg25[ 60] = 1.519976E+00;
  eg25[ 61] = 1.588030E+00;
  eg25[ 62] = 1.668949E+00;
  eg25[ 63] = 1.779966E+00;
  eg25[ 64] = 1.900077E+00;
  eg25[ 65] = 1.989920E+00;
  eg25[ 66] = 2.070095E+00;
  eg25[ 67] = 2.156948E+00;
  eg25[ 68] = 2.217087E+00;
  eg25[ 69] = 2.272986E+00;
  eg25[ 70] = 2.330061E+00;
  eg25[ 71] = 2.469941E+00;
  eg25[ 72] = 2.550003E+00;
  eg25[ 73] = 2.590094E+00;
  eg25[ 74] = 2.620053E+00;
  eg25[ 75] = 2.640041E+00;
  eg25[ 76] = 2.700115E+00;
  eg25[ 77] = 2.719898E+00;
  eg25[ 78] = 2.740922E+00;
  eg25[ 79] = 2.775121E+00;
  eg25[ 80] = 2.884047E+00;
  eg25[ 81] = 3.142109E+00;
  eg25[ 82] = 3.543073E+00;
  eg25[ 83] = 3.712087E+00;
  eg25[ 84] = 3.882170E+00;
  eg25[ 85] = 4.000000E+00;
  eg25[ 86] = 4.219828E+00;
  eg25[ 87] = 4.309812E+00;
  eg25[ 88] = 4.402156E+00;
  eg25[ 89] = 4.632489E+00;
  eg25[ 90] = 4.958456E+00;
  eg25[ 91] = 5.317984E+00;
  eg25[ 92] = 5.618667E+00;
  eg25[ 93] = 5.918567E+00;
  eg25[ 94] = 6.228244E+00;
  eg25[ 95] = 6.508405E+00;
  eg25[ 96] = 6.828427E+00;
  eg25[ 97] = 7.214511E+00;
  eg25[ 98] = 7.622423E+00;
  eg25[ 99] = 7.965298E+00;
  eg25[100] = 8.415661E+00;
  eg25[101] = 8.855993E+00;
  eg25[102] = 9.310049E+00;
  eg25[103] = 9.787385E+00;
  eg25[104] = 1.023788E+01;
  eg25[105] = 1.070911E+01;
  eg25[106] = 1.120202E+01;
  eg25[107] = 1.174109E+01;
  eg25[108] = 1.223248E+01;
  eg25[109] = 1.288539E+01;
  eg25[110] = 1.357316E+01;
  eg25[111] = 1.421211E+01;
  eg25[112] = 1.497069E+01;
  eg25[113] = 1.573825E+01;
  eg25[114] = 1.656173E+01;
  eg25[115] = 1.737608E+01;
  eg25[116] = 1.830354E+01;
  eg25[117] = 1.903148E+01;
  eg25[118] = 2.010753E+01;
  eg25[119] = 2.053425E+01;
  eg25[120] = 2.122319E+01;
  eg25[121] = 2.200114E+01;
  eg25[122] = 2.267118E+01;
  eg25[123] = 2.465783E+01;
  eg25[124] = 2.788515E+01;
  eg25[125] = 3.169295E+01;
  eg25[126] = 3.308547E+01;
  eg25[127] = 3.453918E+01;
  eg25[128] = 3.569799E+01;
  eg25[129] = 3.605676E+01;
  eg25[130] = 3.641914E+01;
  eg25[131] = 3.685880E+01;
  eg25[132] = 3.730377E+01;
  eg25[133] = 3.779188E+01;
  eg25[134] = 3.878736E+01;
  eg25[135] = 3.972951E+01;
  eg25[136] = 4.122704E+01;
  eg25[137] = 4.214409E+01;
  eg25[138] = 4.312463E+01;
  eg25[139] = 4.417214E+01;
  eg25[140] = 4.529037E+01;
  eg25[141] = 4.620529E+01;
  eg25[142] = 4.751732E+01;
  eg25[143] = 4.925911E+01;
  eg25[144] = 5.178468E+01;
  eg25[145] = 5.298953E+01;
  eg25[146] = 5.405999E+01;
  eg25[147] = 5.705949E+01;
  eg25[148] = 5.992503E+01;
  eg25[149] = 6.230828E+01;
  eg25[150] = 6.363059E+01;
  eg25[151] = 6.459225E+01;
  eg25[152] = 6.504598E+01;
  eg25[153] = 6.550290E+01;
  eg25[154] = 6.583123E+01;
  eg25[155] = 6.616121E+01;
  eg25[156] = 6.649285E+01;
  eg25[157] = 6.682614E+01;
  eg25[158] = 6.906820E+01;
  eg25[159] = 7.188692E+01;
  eg25[160] = 7.355948E+01;
  eg25[161] = 7.633216E+01;
  eg25[162] = 7.936793E+01;
  eg25[163] = 8.393934E+01;
  eg25[164] = 8.877405E+01;
  eg25[165] = 9.332559E+01;
  eg25[166] = 9.732874E+01;
  eg25[167] = 1.005942E+02;
  eg25[168] = 1.010984E+02;
  eg25[169] = 1.016052E+02;
  eg25[170] = 1.021145E+02;
  eg25[171] = 1.030376E+02;
  eg25[172] = 1.056461E+02;
  eg25[173] = 1.102879E+02;
  eg25[174] = 1.128539E+02;
  eg25[175] = 1.154797E+02;
  eg25[176] = 1.165237E+02;
  eg25[177] = 1.175771E+02;
  eg25[178] = 1.205536E+02;
  eg25[179] = 1.262286E+02;
  eg25[180] = 1.327005E+02;
  eg25[181] = 1.395042E+02;
  eg25[182] = 1.466567E+02;
  eg25[183] = 1.541759E+02;
  eg25[184] = 1.630561E+02;
  eg25[185] = 1.675186E+02;
  eg25[186] = 1.752291E+02;
  eg25[187] = 1.832945E+02;
  eg25[188] = 1.849516E+02;
  eg25[189] = 1.862508E+02;
  eg25[190] = 1.875592E+02;
  eg25[191] = 1.888767E+02;
  eg25[192] = 1.902035E+02;
  eg25[193] = 1.930780E+02;
  eg25[194] = 1.959960E+02;
  eg25[195] = 2.009577E+02;
  eg25[196] = 2.121077E+02;
  eg25[197] = 2.243247E+02;
  eg25[198] = 2.355903E+02;
  eg25[199] = 2.417960E+02;
  eg25[200] = 2.567478E+02;
  eg25[201] = 2.682969E+02;
  eg25[202] = 2.764678E+02;
  eg25[203] = 2.848875E+02;
  eg25[204] = 2.883267E+02;
  eg25[205] = 2.959215E+02;
  eg25[206] = 3.199275E+02;
  eg25[207] = 3.353230E+02;
  eg25[208] = 3.535746E+02;
  eg25[209] = 3.717027E+02;
  eg25[210] = 3.907603E+02;
  eg25[211] = 4.190936E+02;
  eg25[212] = 4.539987E+02;
  eg25[213] = 5.017462E+02;
  eg25[214] = 5.392042E+02;
  eg25[215] = 5.771455E+02;
  eg25[216] = 5.929407E+02;
  eg25[217] = 6.000988E+02;
  eg25[218] = 6.128342E+02;
  eg25[219] = 6.468370E+02;
  eg25[220] = 6.772865E+02;
  eg25[221] = 7.485173E+02;
  eg25[222] = 8.322179E+02;
  eg25[223] = 9.096813E+02;
  eg25[224] = 9.824941E+02;
  eg25[225] = 1.064323E+03;
  eg25[226] = 1.134667E+03;
  eg25[227] = 1.343582E+03;
  eg25[228] = 1.586197E+03;
  eg25[229] = 1.811833E+03;
  eg25[230] = 2.084104E+03;
  eg25[231] = 2.397290E+03;
  eg25[232] = 2.700236E+03;
  eg25[233] = 2.996183E+03;
  eg25[234] = 3.481068E+03;
  eg25[235] = 4.097345E+03;
  eg25[236] = 5.004508E+03;
  eg25[237] = 6.112520E+03;
  eg25[238] = 7.465848E+03;
  eg25[239] = 9.118808E+03;
  eg25[240] = 1.113774E+04;
  eg25[241] = 1.360366E+04;
  eg25[242] = 1.489967E+04;
  eg25[243] = 1.620045E+04;
  eg25[244] = 1.858471E+04;
  eg25[245] = 2.269941E+04;
  eg25[246] = 2.499908E+04;
  eg25[247] = 2.610010E+04;
  eg25[248] = 2.739441E+04;
  eg25[249] = 2.928101E+04;
  eg25[250] = 3.345961E+04;
  eg25[251] = 3.697859E+04;
  eg25[252] = 4.086766E+04;
  eg25[253] = 4.991587E+04;
  eg25[254] = 5.516557E+04;
  eg25[255] = 6.737938E+04;
  eg25[256] = 8.229736E+04;
  eg25[257] = 9.466450E+04;
  eg25[258] = 1.156235E+05;
  eg25[259] = 1.227732E+05;
  eg25[260] = 1.400976E+05;
  eg25[261] = 1.650650E+05;
  eg25[262] = 1.950662E+05;
  eg25[263] = 2.300598E+05;
  eg25[264] = 2.678264E+05;
  eg25[265] = 3.206464E+05;
  eg25[266] = 3.838835E+05;
  eg25[267] = 4.125012E+05;
  eg25[268] = 4.560211E+05;
  eg25[269] = 4.940018E+05;
  eg25[270] = 5.784425E+05;
  eg25[271] = 7.065112E+05;
  eg25[272] = 8.600058E+05;
  eg25[273] = 9.511189E+05;
  eg25[274] = 1.051149E+06;
  eg25[275] = 1.162048E+06;
  eg25[276] = 1.286961E+06;
  eg25[277] = 1.336941E+06;
  eg25[278] = 1.405768E+06;
  eg25[279] = 1.636539E+06;
  eg25[280] = 1.901387E+06;
  eg25[281] = 2.231299E+06;
  eg25[282] = 2.725314E+06;
  eg25[283] = 3.328707E+06;
  eg25[284] = 4.065691E+06;
  eg25[285] = 4.965847E+06;
  eg25[286] = 6.065299E+06;
  eg25[287] = 6.703192E+06;
  eg25[288] = 7.408173E+06;
  eg25[289] = 8.187297E+06;
  eg25[290] = 9.048363E+06;
  eg25[291] = 9.999987E+06;
  eg25[292] = 1.161833E+07;
  eg25[293] = 1.384029E+07;
  eg25[294] = 1.491823E+07;
  eg25[295] = 1.964030E+07;

  return eg25;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg26()
{
  vector<Real> eg26;
  eg26.resize(362);
  eg26[  0] = 1.100027E-04;
  eg26[  1] = 2.499897E-03;
  eg26[  2] = 4.556021E-03;
  eg26[  3] = 7.145263E-03;
  eg26[  4] = 1.045050E-02;
  eg26[  5] = 1.482996E-02;
  eg26[  6] = 2.001035E-02;
  eg26[  7] = 2.493942E-02;
  eg26[  8] = 2.929889E-02;
  eg26[  9] = 3.439976E-02;
  eg26[ 10] = 4.029993E-02;
  eg26[ 11] = 4.730186E-02;
  eg26[ 12] = 5.549815E-02;
  eg26[ 13] = 6.519936E-02;
  eg26[ 14] = 7.649686E-02;
  eg26[ 15] = 8.979683E-02;
  eg26[ 16] = 1.042977E-01;
  eg26[ 17] = 1.199949E-01;
  eg26[ 18] = 1.379994E-01;
  eg26[ 19] = 1.618953E-01;
  eg26[ 20] = 1.900049E-01;
  eg26[ 21] = 2.096102E-01;
  eg26[ 22] = 2.311923E-01;
  eg26[ 23] = 2.549965E-01;
  eg26[ 24] = 2.799888E-01;
  eg26[ 25] = 3.050115E-01;
  eg26[ 26] = 3.250079E-01;
  eg26[ 27] = 3.529935E-01;
  eg26[ 28] = 3.900011E-01;
  eg26[ 29] = 4.315786E-01;
  eg26[ 30] = 4.750165E-01;
  eg26[ 31] = 5.200108E-01;
  eg26[ 32] = 5.549897E-01;
  eg26[ 33] = 5.949930E-01;
  eg26[ 34] = 6.249987E-01;
  eg26[ 35] = 7.199989E-01;
  eg26[ 36] = 8.200371E-01;
  eg26[ 37] = 8.800244E-01;
  eg26[ 38] = 9.199779E-01;
  eg26[ 39] = 9.440222E-01;
  eg26[ 40] = 9.639598E-01;
  eg26[ 41] = 9.819591E-01;
  eg26[ 42] = 9.965005E-01;
  eg26[ 43] = 1.009035E+00;
  eg26[ 44] = 1.021012E+00;
  eg26[ 45] = 1.034993E+00;
  eg26[ 46] = 1.077986E+00;
  eg26[ 47] = 1.091982E+00;
  eg26[ 48] = 1.103950E+00;
  eg26[ 49] = 1.116049E+00;
  eg26[ 50] = 1.129974E+00;
  eg26[ 51] = 1.147969E+00;
  eg26[ 52] = 1.169989E+00;
  eg26[ 53] = 1.213968E+00;
  eg26[ 54] = 1.250939E+00;
  eg26[ 55] = 1.293038E+00;
  eg26[ 56] = 1.330952E+00;
  eg26[ 57] = 1.380981E+00;
  eg26[ 58] = 1.410007E+00;
  eg26[ 59] = 1.443967E+00;
  eg26[ 60] = 1.519976E+00;
  eg26[ 61] = 1.588030E+00;
  eg26[ 62] = 1.668949E+00;
  eg26[ 63] = 1.779966E+00;
  eg26[ 64] = 1.900077E+00;
  eg26[ 65] = 1.989920E+00;
  eg26[ 66] = 2.070095E+00;
  eg26[ 67] = 2.156948E+00;
  eg26[ 68] = 2.217087E+00;
  eg26[ 69] = 2.272986E+00;
  eg26[ 70] = 2.330061E+00;
  eg26[ 71] = 2.469941E+00;
  eg26[ 72] = 2.550003E+00;
  eg26[ 73] = 2.590094E+00;
  eg26[ 74] = 2.620053E+00;
  eg26[ 75] = 2.640041E+00;
  eg26[ 76] = 2.700115E+00;
  eg26[ 77] = 2.719898E+00;
  eg26[ 78] = 2.740922E+00;
  eg26[ 79] = 2.775121E+00;
  eg26[ 80] = 2.884047E+00;
  eg26[ 81] = 3.142109E+00;
  eg26[ 82] = 3.543073E+00;
  eg26[ 83] = 3.712087E+00;
  eg26[ 84] = 3.882170E+00;
  eg26[ 85] = 4.000000E+00;
  eg26[ 86] = 4.219828E+00;
  eg26[ 87] = 4.309812E+00;
  eg26[ 88] = 4.419800E+00;
  eg26[ 89] = 4.767845E+00;
  eg26[ 90] = 4.933232E+00;
  eg26[ 91] = 5.109974E+00;
  eg26[ 92] = 5.210076E+00;
  eg26[ 93] = 5.320112E+00;
  eg26[ 94] = 5.380032E+00;
  eg26[ 95] = 5.410245E+00;
  eg26[ 96] = 5.488167E+00;
  eg26[ 97] = 5.530036E+00;
  eg26[ 98] = 5.619790E+00;
  eg26[ 99] = 5.720146E+00;
  eg26[100] = 5.800211E+00;
  eg26[101] = 5.960142E+00;
  eg26[102] = 6.059906E+00;
  eg26[103] = 6.160108E+00;
  eg26[104] = 6.280153E+00;
  eg26[105] = 6.359784E+00;
  eg26[106] = 6.432057E+00;
  eg26[107] = 6.481775E+00;
  eg26[108] = 6.514916E+00;
  eg26[109] = 6.539066E+00;
  eg26[110] = 6.556090E+00;
  eg26[111] = 6.571843E+00;
  eg26[112] = 6.588293E+00;
  eg26[113] = 6.606106E+00;
  eg26[114] = 6.631257E+00;
  eg26[115] = 6.716683E+00;
  eg26[116] = 6.742254E+00;
  eg26[117] = 6.759807E+00;
  eg26[118] = 6.776050E+00;
  eg26[119] = 6.791653E+00;
  eg26[120] = 6.810696E+00;
  eg26[121] = 6.835259E+00;
  eg26[122] = 6.870208E+00;
  eg26[123] = 6.917776E+00;
  eg26[124] = 6.994292E+00;
  eg26[125] = 7.139869E+00;
  eg26[126] = 7.380153E+00;
  eg26[127] = 7.600350E+00;
  eg26[128] = 7.739943E+00;
  eg26[129] = 7.839651E+00;
  eg26[130] = 7.970079E+00;
  eg26[131] = 8.130272E+00;
  eg26[132] = 8.300322E+00;
  eg26[133] = 8.524074E+00;
  eg26[134] = 8.673690E+00;
  eg26[135] = 8.800375E+00;
  eg26[136] = 8.979950E+00;
  eg26[137] = 9.140311E+00;
  eg26[138] = 9.500024E+00;
  eg26[139] = 1.057925E+01;
  eg26[140] = 1.080376E+01;
  eg26[141] = 1.105292E+01;
  eg26[142] = 1.126944E+01;
  eg26[143] = 1.158944E+01;
  eg26[144] = 1.170943E+01;
  eg26[145] = 1.181529E+01;
  eg26[146] = 1.197947E+01;
  eg26[147] = 1.213015E+01;
  eg26[148] = 1.230855E+01;
  eg26[149] = 1.247210E+01;
  eg26[150] = 1.259997E+01;
  eg26[151] = 1.332970E+01;
  eg26[152] = 1.354604E+01;
  eg26[153] = 1.404961E+01;
  eg26[154] = 1.425053E+01;
  eg26[155] = 1.447024E+01;
  eg26[156] = 1.459522E+01;
  eg26[157] = 1.473012E+01;
  eg26[158] = 1.486626E+01;
  eg26[159] = 1.577923E+01;
  eg26[160] = 1.604977E+01;
  eg26[161] = 1.655014E+01;
  eg26[162] = 1.683053E+01;
  eg26[163] = 1.744572E+01;
  eg26[164] = 1.756476E+01;
  eg26[165] = 1.775903E+01;
  eg26[166] = 1.795905E+01;
  eg26[167] = 1.908484E+01;
  eg26[168] = 1.919969E+01;
  eg26[169] = 1.939265E+01;
  eg26[170] = 1.959735E+01;
  eg26[171] = 2.007338E+01;
  eg26[172] = 2.027512E+01;
  eg26[173] = 2.041754E+01;
  eg26[174] = 2.051988E+01;
  eg26[175] = 2.060213E+01;
  eg26[176] = 2.068470E+01;
  eg26[177] = 2.076761E+01;
  eg26[178] = 2.097632E+01;
  eg26[179] = 2.106040E+01;
  eg26[180] = 2.114481E+01;
  eg26[181] = 2.122956E+01;
  eg26[182] = 2.133597E+01;
  eg26[183] = 2.148585E+01;
  eg26[184] = 2.170178E+01;
  eg26[185] = 2.200114E+01;
  eg26[186] = 2.215569E+01;
  eg26[187] = 2.237836E+01;
  eg26[188] = 2.253556E+01;
  eg26[189] = 2.465783E+01;
  eg26[190] = 2.788515E+01;
  eg26[191] = 3.169295E+01;
  eg26[192] = 3.308547E+01;
  eg26[193] = 3.453918E+01;
  eg26[194] = 3.569799E+01;
  eg26[195] = 3.605676E+01;
  eg26[196] = 3.641914E+01;
  eg26[197] = 3.685880E+01;
  eg26[198] = 3.730377E+01;
  eg26[199] = 3.779188E+01;
  eg26[200] = 3.878736E+01;
  eg26[201] = 3.972951E+01;
  eg26[202] = 4.122704E+01;
  eg26[203] = 4.214409E+01;
  eg26[204] = 4.312463E+01;
  eg26[205] = 4.417214E+01;
  eg26[206] = 4.529037E+01;
  eg26[207] = 4.620529E+01;
  eg26[208] = 4.751732E+01;
  eg26[209] = 4.925911E+01;
  eg26[210] = 5.178468E+01;
  eg26[211] = 5.298953E+01;
  eg26[212] = 5.405999E+01;
  eg26[213] = 5.705949E+01;
  eg26[214] = 5.992503E+01;
  eg26[215] = 6.230828E+01;
  eg26[216] = 6.363059E+01;
  eg26[217] = 6.459225E+01;
  eg26[218] = 6.504598E+01;
  eg26[219] = 6.550290E+01;
  eg26[220] = 6.583123E+01;
  eg26[221] = 6.616121E+01;
  eg26[222] = 6.649285E+01;
  eg26[223] = 6.682614E+01;
  eg26[224] = 6.906820E+01;
  eg26[225] = 7.188692E+01;
  eg26[226] = 7.355948E+01;
  eg26[227] = 7.633216E+01;
  eg26[228] = 7.936793E+01;
  eg26[229] = 8.393934E+01;
  eg26[230] = 8.877405E+01;
  eg26[231] = 9.332559E+01;
  eg26[232] = 9.732874E+01;
  eg26[233] = 1.005942E+02;
  eg26[234] = 1.010984E+02;
  eg26[235] = 1.016052E+02;
  eg26[236] = 1.021145E+02;
  eg26[237] = 1.030376E+02;
  eg26[238] = 1.056461E+02;
  eg26[239] = 1.102879E+02;
  eg26[240] = 1.128539E+02;
  eg26[241] = 1.154797E+02;
  eg26[242] = 1.165237E+02;
  eg26[243] = 1.175771E+02;
  eg26[244] = 1.205536E+02;
  eg26[245] = 1.262286E+02;
  eg26[246] = 1.327005E+02;
  eg26[247] = 1.395042E+02;
  eg26[248] = 1.466567E+02;
  eg26[249] = 1.541759E+02;
  eg26[250] = 1.630561E+02;
  eg26[251] = 1.675186E+02;
  eg26[252] = 1.752291E+02;
  eg26[253] = 1.832945E+02;
  eg26[254] = 1.849516E+02;
  eg26[255] = 1.862508E+02;
  eg26[256] = 1.875592E+02;
  eg26[257] = 1.888767E+02;
  eg26[258] = 1.902035E+02;
  eg26[259] = 1.930780E+02;
  eg26[260] = 1.959960E+02;
  eg26[261] = 2.009577E+02;
  eg26[262] = 2.121077E+02;
  eg26[263] = 2.243247E+02;
  eg26[264] = 2.355903E+02;
  eg26[265] = 2.417960E+02;
  eg26[266] = 2.567478E+02;
  eg26[267] = 2.682969E+02;
  eg26[268] = 2.764678E+02;
  eg26[269] = 2.848875E+02;
  eg26[270] = 2.883267E+02;
  eg26[271] = 2.959215E+02;
  eg26[272] = 3.199275E+02;
  eg26[273] = 3.353230E+02;
  eg26[274] = 3.535746E+02;
  eg26[275] = 3.717027E+02;
  eg26[276] = 3.907603E+02;
  eg26[277] = 4.190936E+02;
  eg26[278] = 4.539987E+02;
  eg26[279] = 5.017462E+02;
  eg26[280] = 5.392042E+02;
  eg26[281] = 5.771455E+02;
  eg26[282] = 5.929407E+02;
  eg26[283] = 6.000988E+02;
  eg26[284] = 6.128342E+02;
  eg26[285] = 6.468370E+02;
  eg26[286] = 6.772865E+02;
  eg26[287] = 7.485173E+02;
  eg26[288] = 8.322179E+02;
  eg26[289] = 9.096813E+02;
  eg26[290] = 9.824941E+02;
  eg26[291] = 1.064323E+03;
  eg26[292] = 1.134667E+03;
  eg26[293] = 1.343582E+03;
  eg26[294] = 1.586197E+03;
  eg26[295] = 1.811833E+03;
  eg26[296] = 2.084104E+03;
  eg26[297] = 2.397290E+03;
  eg26[298] = 2.700236E+03;
  eg26[299] = 2.996183E+03;
  eg26[300] = 3.481068E+03;
  eg26[301] = 4.097345E+03;
  eg26[302] = 5.004508E+03;
  eg26[303] = 6.112520E+03;
  eg26[304] = 7.465848E+03;
  eg26[305] = 9.118808E+03;
  eg26[306] = 1.113774E+04;
  eg26[307] = 1.360366E+04;
  eg26[308] = 1.489967E+04;
  eg26[309] = 1.620045E+04;
  eg26[310] = 1.858471E+04;
  eg26[311] = 2.269941E+04;
  eg26[312] = 2.499908E+04;
  eg26[313] = 2.610010E+04;
  eg26[314] = 2.739441E+04;
  eg26[315] = 2.928101E+04;
  eg26[316] = 3.345961E+04;
  eg26[317] = 3.697859E+04;
  eg26[318] = 4.086766E+04;
  eg26[319] = 4.991587E+04;
  eg26[320] = 5.516557E+04;
  eg26[321] = 6.737938E+04;
  eg26[322] = 8.229736E+04;
  eg26[323] = 9.466450E+04;
  eg26[324] = 1.156235E+05;
  eg26[325] = 1.227732E+05;
  eg26[326] = 1.400976E+05;
  eg26[327] = 1.650650E+05;
  eg26[328] = 1.950662E+05;
  eg26[329] = 2.300598E+05;
  eg26[330] = 2.678264E+05;
  eg26[331] = 3.206464E+05;
  eg26[332] = 3.838835E+05;
  eg26[333] = 4.125012E+05;
  eg26[334] = 4.560211E+05;
  eg26[335] = 4.940018E+05;
  eg26[336] = 5.784425E+05;
  eg26[337] = 7.065112E+05;
  eg26[338] = 8.600058E+05;
  eg26[339] = 9.511189E+05;
  eg26[340] = 1.051149E+06;
  eg26[341] = 1.162048E+06;
  eg26[342] = 1.286961E+06;
  eg26[343] = 1.336941E+06;
  eg26[344] = 1.405768E+06;
  eg26[345] = 1.636539E+06;
  eg26[346] = 1.901387E+06;
  eg26[347] = 2.231299E+06;
  eg26[348] = 2.725314E+06;
  eg26[349] = 3.328707E+06;
  eg26[350] = 4.065691E+06;
  eg26[351] = 4.965847E+06;
  eg26[352] = 6.065299E+06;
  eg26[353] = 6.703192E+06;
  eg26[354] = 7.408173E+06;
  eg26[355] = 8.187297E+06;
  eg26[356] = 9.048363E+06;
  eg26[357] = 9.999987E+06;
  eg26[358] = 1.161833E+07;
  eg26[359] = 1.384029E+07;
  eg26[360] = 1.491823E+07;
  eg26[361] = 1.964030E+07;

  return eg26;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg27()
{
  vector<Real> eg27;
  eg27.resize(316);
  eg27[  0] = 1.100028E-04;
  eg27[  1] = 2.499900E-03;
  eg27[  2] = 4.556027E-03;
  eg27[  3] = 7.145272E-03;
  eg27[  4] = 1.045051E-02;
  eg27[  5] = 1.482998E-02;
  eg27[  6] = 2.001038E-02;
  eg27[  7] = 2.493945E-02;
  eg27[  8] = 2.929893E-02;
  eg27[  9] = 3.439981E-02;
  eg27[ 10] = 4.029998E-02;
  eg27[ 11] = 4.730192E-02;
  eg27[ 12] = 5.549822E-02;
  eg27[ 13] = 6.519944E-02;
  eg27[ 14] = 7.649695E-02;
  eg27[ 15] = 8.979694E-02;
  eg27[ 16] = 1.000001E-01;
  eg27[ 17] = 1.199951E-01;
  eg27[ 18] = 1.379996E-01;
  eg27[ 19] = 1.618955E-01;
  eg27[ 20] = 1.900051E-01;
  eg27[ 21] = 2.096105E-01;
  eg27[ 22] = 2.311926E-01;
  eg27[ 23] = 2.549968E-01;
  eg27[ 24] = 2.799892E-01;
  eg27[ 25] = 3.050119E-01;
  eg27[ 26] = 3.250084E-01;
  eg27[ 27] = 3.529940E-01;
  eg27[ 28] = 3.900016E-01;
  eg27[ 29] = 4.315792E-01;
  eg27[ 30] = 4.750172E-01;
  eg27[ 31] = 5.200115E-01;
  eg27[ 32] = 5.400010E-01;
  eg27[ 33] = 5.949938E-01;
  eg27[ 34] = 6.249996E-01;
  eg27[ 35] = 7.199998E-01;
  eg27[ 36] = 8.200382E-01;
  eg27[ 37] = 8.800255E-01;
  eg27[ 38] = 9.199791E-01;
  eg27[ 39] = 9.440234E-01;
  eg27[ 40] = 9.639611E-01;
  eg27[ 41] = 9.819603E-01;
  eg27[ 42] = 9.965018E-01;
  eg27[ 43] = 1.009036E+00;
  eg27[ 44] = 1.021013E+00;
  eg27[ 45] = 1.034994E+00;
  eg27[ 46] = 1.077987E+00;
  eg27[ 47] = 1.091983E+00;
  eg27[ 48] = 1.103951E+00;
  eg27[ 49] = 1.116050E+00;
  eg27[ 50] = 1.129975E+00;
  eg27[ 51] = 1.147971E+00;
  eg27[ 52] = 1.169991E+00;
  eg27[ 53] = 1.213970E+00;
  eg27[ 54] = 1.250940E+00;
  eg27[ 55] = 1.293040E+00;
  eg27[ 56] = 1.330954E+00;
  eg27[ 57] = 1.380983E+00;
  eg27[ 58] = 1.410008E+00;
  eg27[ 59] = 1.443969E+00;
  eg27[ 60] = 1.519978E+00;
  eg27[ 61] = 1.588032E+00;
  eg27[ 62] = 1.668952E+00;
  eg27[ 63] = 1.779968E+00;
  eg27[ 64] = 1.900079E+00;
  eg27[ 65] = 1.989922E+00;
  eg27[ 66] = 2.070097E+00;
  eg27[ 67] = 2.156951E+00;
  eg27[ 68] = 2.217090E+00;
  eg27[ 69] = 2.272989E+00;
  eg27[ 70] = 2.330064E+00;
  eg27[ 71] = 2.469944E+00;
  eg27[ 72] = 2.550006E+00;
  eg27[ 73] = 2.590098E+00;
  eg27[ 74] = 2.620056E+00;
  eg27[ 75] = 2.640044E+00;
  eg27[ 76] = 2.700118E+00;
  eg27[ 77] = 2.719901E+00;
  eg27[ 78] = 2.740926E+00;
  eg27[ 79] = 2.775125E+00;
  eg27[ 80] = 2.884050E+00;
  eg27[ 81] = 3.142113E+00;
  eg27[ 82] = 3.543077E+00;
  eg27[ 83] = 3.712092E+00;
  eg27[ 84] = 3.882175E+00;
  eg27[ 85] = 4.000000E+00;
  eg27[ 86] = 4.219833E+00;
  eg27[ 87] = 4.309818E+00;
  eg27[ 88] = 4.402161E+00;
  eg27[ 89] = 4.632495E+00;
  eg27[ 90] = 4.958462E+00;
  eg27[ 91] = 5.317991E+00;
  eg27[ 92] = 5.618674E+00;
  eg27[ 93] = 5.918575E+00;
  eg27[ 94] = 6.228252E+00;
  eg27[ 95] = 6.508413E+00;
  eg27[ 96] = 6.828436E+00;
  eg27[ 97] = 7.214520E+00;
  eg27[ 98] = 7.622433E+00;
  eg27[ 99] = 7.965308E+00;
  eg27[100] = 8.315287E+00;
  eg27[101] = 8.856004E+00;
  eg27[102] = 9.310061E+00;
  eg27[103] = 9.787398E+00;
  eg27[104] = 1.023789E+01;
  eg27[105] = 1.070912E+01;
  eg27[106] = 1.120204E+01;
  eg27[107] = 1.174110E+01;
  eg27[108] = 1.223249E+01;
  eg27[109] = 1.288541E+01;
  eg27[110] = 1.370959E+01;
  eg27[111] = 1.421213E+01;
  eg27[112] = 1.497071E+01;
  eg27[113] = 1.573828E+01;
  eg27[114] = 1.656175E+01;
  eg27[115] = 1.737610E+01;
  eg27[116] = 1.830356E+01;
  eg27[117] = 1.903150E+01;
  eg27[118] = 2.010756E+01;
  eg27[119] = 2.053428E+01;
  eg27[120] = 2.122322E+01;
  eg27[121] = 2.200117E+01;
  eg27[122] = 2.260329E+01;
  eg27[123] = 2.465786E+01;
  eg27[124] = 2.788519E+01;
  eg27[125] = 3.169299E+01;
  eg27[126] = 3.308552E+01;
  eg27[127] = 3.453923E+01;
  eg27[128] = 3.569804E+01;
  eg27[129] = 3.605681E+01;
  eg27[130] = 3.641918E+01;
  eg27[131] = 3.685885E+01;
  eg27[132] = 3.730382E+01;
  eg27[133] = 3.779193E+01;
  eg27[134] = 3.878741E+01;
  eg27[135] = 4.016900E+01;
  eg27[136] = 4.122709E+01;
  eg27[137] = 4.214414E+01;
  eg27[138] = 4.312469E+01;
  eg27[139] = 4.417220E+01;
  eg27[140] = 4.529043E+01;
  eg27[141] = 4.620535E+01;
  eg27[142] = 4.751739E+01;
  eg27[143] = 4.925918E+01;
  eg27[144] = 5.178475E+01;
  eg27[145] = 5.298960E+01;
  eg27[146] = 5.406006E+01;
  eg27[147] = 5.705956E+01;
  eg27[148] = 5.992511E+01;
  eg27[149] = 6.230836E+01;
  eg27[150] = 6.363067E+01;
  eg27[151] = 6.459233E+01;
  eg27[152] = 6.504606E+01;
  eg27[153] = 6.550298E+01;
  eg27[154] = 6.583132E+01;
  eg27[155] = 6.616130E+01;
  eg27[156] = 6.649293E+01;
  eg27[157] = 6.682623E+01;
  eg27[158] = 6.790405E+01;
  eg27[159] = 6.906828E+01;
  eg27[160] = 7.188702E+01;
  eg27[161] = 7.355958E+01;
  eg27[162] = 7.633226E+01;
  eg27[163] = 7.936803E+01;
  eg27[164] = 8.393945E+01;
  eg27[165] = 8.877416E+01;
  eg27[166] = 9.166088E+01;
  eg27[167] = 9.332571E+01;
  eg27[168] = 9.732887E+01;
  eg27[169] = 1.005943E+02;
  eg27[170] = 1.010985E+02;
  eg27[171] = 1.016053E+02;
  eg27[172] = 1.021146E+02;
  eg27[173] = 1.030378E+02;
  eg27[174] = 1.056462E+02;
  eg27[175] = 1.102881E+02;
  eg27[176] = 1.128541E+02;
  eg27[177] = 1.154798E+02;
  eg27[178] = 1.165238E+02;
  eg27[179] = 1.175773E+02;
  eg27[180] = 1.205537E+02;
  eg27[181] = 1.262287E+02;
  eg27[182] = 1.327006E+02;
  eg27[183] = 1.395043E+02;
  eg27[184] = 1.485759E+02;
  eg27[185] = 1.541761E+02;
  eg27[186] = 1.630563E+02;
  eg27[187] = 1.675188E+02;
  eg27[188] = 1.752293E+02;
  eg27[189] = 1.832948E+02;
  eg27[190] = 1.849519E+02;
  eg27[191] = 1.862511E+02;
  eg27[192] = 1.875594E+02;
  eg27[193] = 1.888769E+02;
  eg27[194] = 1.902037E+02;
  eg27[195] = 1.930783E+02;
  eg27[196] = 1.959963E+02;
  eg27[197] = 2.009579E+02;
  eg27[198] = 2.121080E+02;
  eg27[199] = 2.243249E+02;
  eg27[200] = 2.355906E+02;
  eg27[201] = 2.417963E+02;
  eg27[202] = 2.567482E+02;
  eg27[203] = 2.682973E+02;
  eg27[204] = 2.723521E+02;
  eg27[205] = 2.764682E+02;
  eg27[206] = 2.848879E+02;
  eg27[207] = 2.883271E+02;
  eg27[208] = 2.959219E+02;
  eg27[209] = 3.043248E+02;
  eg27[210] = 3.199279E+02;
  eg27[211] = 3.353235E+02;
  eg27[212] = 3.535750E+02;
  eg27[213] = 3.717032E+02;
  eg27[214] = 3.907608E+02;
  eg27[215] = 4.190942E+02;
  eg27[216] = 4.539993E+02;
  eg27[217] = 5.017468E+02;
  eg27[218] = 5.392049E+02;
  eg27[219] = 5.771462E+02;
  eg27[220] = 5.929414E+02;
  eg27[221] = 6.000996E+02;
  eg27[222] = 6.128350E+02;
  eg27[223] = 6.468379E+02;
  eg27[224] = 6.772874E+02;
  eg27[225] = 7.485183E+02;
  eg27[226] = 8.322190E+02;
  eg27[227] = 9.096825E+02;
  eg27[228] = 9.824954E+02;
  eg27[229] = 1.064325E+03;
  eg27[230] = 1.134668E+03;
  eg27[231] = 1.234098E+03;
  eg27[232] = 1.343584E+03;
  eg27[233] = 1.586199E+03;
  eg27[234] = 1.811835E+03;
  eg27[235] = 2.034684E+03;
  eg27[236] = 2.284941E+03;
  eg27[237] = 2.423809E+03;
  eg27[238] = 2.571117E+03;
  eg27[239] = 2.768596E+03;
  eg27[240] = 2.951579E+03;
  eg27[241] = 3.146656E+03;
  eg27[242] = 3.354626E+03;
  eg27[243] = 3.707435E+03;
  eg27[244] = 4.097350E+03;
  eg27[245] = 4.528272E+03;
  eg27[246] = 5.004514E+03;
  eg27[247] = 5.530844E+03;
  eg27[248] = 6.112528E+03;
  eg27[249] = 6.755388E+03;
  eg27[250] = 7.465858E+03;
  eg27[251] = 8.251049E+03;
  eg27[252] = 9.118820E+03;
  eg27[253] = 1.007785E+04;
  eg27[254] = 1.113775E+04;
  eg27[255] = 1.360368E+04;
  eg27[256] = 1.503439E+04;
  eg27[257] = 1.620047E+04;
  eg27[258] = 1.858473E+04;
  eg27[259] = 2.269944E+04;
  eg27[260] = 2.478752E+04;
  eg27[261] = 2.610013E+04;
  eg27[262] = 2.739445E+04;
  eg27[263] = 2.928104E+04;
  eg27[264] = 3.345965E+04;
  eg27[265] = 3.697864E+04;
  eg27[266] = 4.086771E+04;
  eg27[267] = 4.991594E+04;
  eg27[268] = 5.516564E+04;
  eg27[269] = 6.737947E+04;
  eg27[270] = 8.229747E+04;
  eg27[271] = 9.466462E+04;
  eg27[272] = 1.110900E+05;
  eg27[273] = 1.227734E+05;
  eg27[274] = 1.400977E+05;
  eg27[275] = 1.650652E+05;
  eg27[276] = 1.831564E+05;
  eg27[277] = 1.950665E+05;
  eg27[278] = 2.300601E+05;
  eg27[279] = 2.678268E+05;
  eg27[280] = 3.019738E+05;
  eg27[281] = 3.206469E+05;
  eg27[282] = 3.838840E+05;
  eg27[283] = 4.125017E+05;
  eg27[284] = 4.560217E+05;
  eg27[285] = 4.978707E+05;
  eg27[286] = 5.784432E+05;
  eg27[287] = 7.065121E+05;
  eg27[288] = 7.808167E+05;
  eg27[289] = 8.208500E+05;
  eg27[290] = 9.508348E+05;
  eg27[291] = 1.051150E+06;
  eg27[292] = 1.162049E+06;
  eg27[293] = 1.286963E+06;
  eg27[294] = 1.353353E+06;
  eg27[295] = 1.408584E+06;
  eg27[296] = 1.636541E+06;
  eg27[297] = 1.901390E+06;
  eg27[298] = 2.231302E+06;
  eg27[299] = 2.528396E+06;
  eg27[300] = 2.865048E+06;
  eg27[301] = 3.246525E+06;
  eg27[302] = 3.678794E+06;
  eg27[303] = 4.168620E+06;
  eg27[304] = 4.723666E+06;
  eg27[305] = 5.352614E+06;
  eg27[306] = 6.065307E+06;
  eg27[307] = 6.703200E+06;
  eg27[308] = 7.408182E+06;
  eg27[309] = 8.187308E+06;
  eg27[310] = 9.048374E+06;
  eg27[311] = 1.000000E+07;
  eg27[312] = 1.161834E+07;
  eg27[313] = 1.384031E+07;
  eg27[314] = 1.491825E+07;
  eg27[315] = 1.964033E+07;

  return eg27;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg28()
{
  vector<Real> eg28;
  eg28.resize(90);
  eg28[  0] = 2.000000E-04;
  eg28[  1] = 5.000000E-03;
  eg28[  2] = 1.000000E-02;
  eg28[  3] = 1.500000E-02;
  eg28[  4] = 2.000000E-02;
  eg28[  5] = 2.500000E-02;
  eg28[  6] = 3.000000E-02;
  eg28[  7] = 3.500000E-02;
  eg28[  8] = 4.200000E-02;
  eg28[  9] = 5.000000E-02;
  eg28[ 10] = 5.800000E-02;
  eg28[ 11] = 6.700000E-02;
  eg28[ 12] = 8.000000E-02;
  eg28[ 13] = 1.000000E-01;
  eg28[ 14] = 1.400000E-01;
  eg28[ 15] = 1.800000E-01;
  eg28[ 16] = 2.200000E-01;
  eg28[ 17] = 2.500000E-01;
  eg28[ 18] = 2.800000E-01;
  eg28[ 19] = 3.000000E-01;
  eg28[ 20] = 3.200000E-01;
  eg28[ 21] = 3.500000E-01;
  eg28[ 22] = 4.000000E-01;
  eg28[ 23] = 5.000000E-01;
  eg28[ 24] = 6.250000E-01;
  eg28[ 25] = 7.800000E-01;
  eg28[ 26] = 8.500000E-01;
  eg28[ 27] = 9.100000E-01;
  eg28[ 28] = 9.500000E-01;
  eg28[ 29] = 9.720000E-01;
  eg28[ 30] = 9.960000E-01;
  eg28[ 31] = 1.020000E+00;
  eg28[ 32] = 1.045000E+00;
  eg28[ 33] = 1.071000E+00;
  eg28[ 34] = 1.097000E+00;
  eg28[ 35] = 1.123000E+00;
  eg28[ 36] = 1.150000E+00;
  eg28[ 37] = 1.300000E+00;
  eg28[ 38] = 1.500000E+00;
  eg28[ 39] = 2.100000E+00;
  eg28[ 40] = 2.600000E+00;
  eg28[ 41] = 3.300000E+00;
  eg28[ 42] = 4.000000E+00;
  eg28[ 43] = 5.043500E+00;
  eg28[ 44] = 6.476000E+00;
  eg28[ 45] = 8.315300E+00;
  eg28[ 46] = 1.067700E+01;
  eg28[ 47] = 1.371000E+01;
  eg28[ 48] = 1.760300E+01;
  eg28[ 49] = 2.260300E+01;
  eg28[ 50] = 2.902300E+01;
  eg28[ 51] = 3.726700E+01;
  eg28[ 52] = 4.785100E+01;
  eg28[ 53] = 6.144200E+01;
  eg28[ 54] = 7.889300E+01;
  eg28[ 55] = 1.013000E+02;
  eg28[ 56] = 1.300700E+02;
  eg28[ 57] = 1.670200E+02;
  eg28[ 58] = 2.753600E+02;
  eg28[ 59] = 4.540000E+02;
  eg28[ 60] = 7.485200E+02;
  eg28[ 61] = 1.234100E+03;
  eg28[ 62] = 2.034700E+03;
  eg28[ 63] = 3.354600E+03;
  eg28[ 64] = 5.530800E+03;
  eg28[ 65] = 9.118800E+03;
  eg28[ 66] = 1.503400E+04;
  eg28[ 67] = 2.478800E+04;
  eg28[ 68] = 4.086800E+04;
  eg28[ 69] = 6.737900E+04;
  eg28[ 70] = 8.651700E+04;
  eg28[ 71] = 1.110900E+05;
  eg28[ 72] = 1.426400E+05;
  eg28[ 73] = 1.831600E+05;
  eg28[ 74] = 2.351800E+05;
  eg28[ 75] = 3.019700E+05;
  eg28[ 76] = 3.877300E+05;
  eg28[ 77] = 4.978700E+05;
  eg28[ 78] = 6.392800E+05;
  eg28[ 79] = 8.208500E+05;
  eg28[ 80] = 1.054000E+06;
  eg28[ 81] = 1.353400E+06;
  eg28[ 82] = 1.737700E+06;
  eg28[ 83] = 2.231300E+06;
  eg28[ 84] = 2.865000E+06;
  eg28[ 85] = 3.678800E+06;
  eg28[ 86] = 4.723700E+06;
  eg28[ 87] = 6.065300E+06;
  eg28[ 88] = 7.788000E+06;
  eg28[ 89] = 1.000000E+07;

  return eg28;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg29()
{
  vector<Real> eg29;
  eg29.resize(661);
  eg29[  0] = 1.000000E-05;
  eg29[  1] = 1.047129E-05;
  eg29[  2] = 1.096478E-05;
  eg29[  3] = 1.148154E-05;
  eg29[  4] = 1.202264E-05;
  eg29[  5] = 1.258925E-05;
  eg29[  6] = 1.318257E-05;
  eg29[  7] = 1.380384E-05;
  eg29[  8] = 1.445440E-05;
  eg29[  9] = 1.513561E-05;
  eg29[ 10] = 1.584893E-05;
  eg29[ 11] = 1.659587E-05;
  eg29[ 12] = 1.737801E-05;
  eg29[ 13] = 1.819701E-05;
  eg29[ 14] = 1.905461E-05;
  eg29[ 15] = 1.995262E-05;
  eg29[ 16] = 2.089296E-05;
  eg29[ 17] = 2.187762E-05;
  eg29[ 18] = 2.290868E-05;
  eg29[ 19] = 2.398833E-05;
  eg29[ 20] = 2.511886E-05;
  eg29[ 21] = 2.630268E-05;
  eg29[ 22] = 2.754229E-05;
  eg29[ 23] = 2.884032E-05;
  eg29[ 24] = 3.019952E-05;
  eg29[ 25] = 3.162278E-05;
  eg29[ 26] = 3.311311E-05;
  eg29[ 27] = 3.467369E-05;
  eg29[ 28] = 3.630781E-05;
  eg29[ 29] = 3.801894E-05;
  eg29[ 30] = 3.981072E-05;
  eg29[ 31] = 4.168694E-05;
  eg29[ 32] = 4.365158E-05;
  eg29[ 33] = 4.570882E-05;
  eg29[ 34] = 4.786301E-05;
  eg29[ 35] = 5.011872E-05;
  eg29[ 36] = 5.248075E-05;
  eg29[ 37] = 5.495409E-05;
  eg29[ 38] = 5.754399E-05;
  eg29[ 39] = 6.025596E-05;
  eg29[ 40] = 6.309573E-05;
  eg29[ 41] = 6.606934E-05;
  eg29[ 42] = 6.918310E-05;
  eg29[ 43] = 7.244360E-05;
  eg29[ 44] = 7.585776E-05;
  eg29[ 45] = 7.943282E-05;
  eg29[ 46] = 8.317638E-05;
  eg29[ 47] = 8.709636E-05;
  eg29[ 48] = 9.120108E-05;
  eg29[ 49] = 9.549926E-05;
  eg29[ 50] = 1.000000E-04;
  eg29[ 51] = 1.047129E-04;
  eg29[ 52] = 1.096478E-04;
  eg29[ 53] = 1.148154E-04;
  eg29[ 54] = 1.202264E-04;
  eg29[ 55] = 1.258925E-04;
  eg29[ 56] = 1.318257E-04;
  eg29[ 57] = 1.380384E-04;
  eg29[ 58] = 1.445440E-04;
  eg29[ 59] = 1.513561E-04;
  eg29[ 60] = 1.584893E-04;
  eg29[ 61] = 1.659587E-04;
  eg29[ 62] = 1.737801E-04;
  eg29[ 63] = 1.819701E-04;
  eg29[ 64] = 1.905461E-04;
  eg29[ 65] = 1.995262E-04;
  eg29[ 66] = 2.089296E-04;
  eg29[ 67] = 2.187762E-04;
  eg29[ 68] = 2.290868E-04;
  eg29[ 69] = 2.398833E-04;
  eg29[ 70] = 2.511886E-04;
  eg29[ 71] = 2.630268E-04;
  eg29[ 72] = 2.754229E-04;
  eg29[ 73] = 2.884032E-04;
  eg29[ 74] = 3.019952E-04;
  eg29[ 75] = 3.162278E-04;
  eg29[ 76] = 3.311311E-04;
  eg29[ 77] = 3.467369E-04;
  eg29[ 78] = 3.630781E-04;
  eg29[ 79] = 3.801894E-04;
  eg29[ 80] = 3.981072E-04;
  eg29[ 81] = 4.168694E-04;
  eg29[ 82] = 4.365158E-04;
  eg29[ 83] = 4.570882E-04;
  eg29[ 84] = 4.786301E-04;
  eg29[ 85] = 5.011872E-04;
  eg29[ 86] = 5.248075E-04;
  eg29[ 87] = 5.495409E-04;
  eg29[ 88] = 5.754399E-04;
  eg29[ 89] = 6.025596E-04;
  eg29[ 90] = 6.309573E-04;
  eg29[ 91] = 6.606934E-04;
  eg29[ 92] = 6.918310E-04;
  eg29[ 93] = 7.244360E-04;
  eg29[ 94] = 7.585776E-04;
  eg29[ 95] = 7.943282E-04;
  eg29[ 96] = 8.317638E-04;
  eg29[ 97] = 8.709636E-04;
  eg29[ 98] = 9.120108E-04;
  eg29[ 99] = 9.549926E-04;
  eg29[100] = 1.000000E-03;
  eg29[101] = 1.047129E-03;
  eg29[102] = 1.096478E-03;
  eg29[103] = 1.148154E-03;
  eg29[104] = 1.202264E-03;
  eg29[105] = 1.258925E-03;
  eg29[106] = 1.318257E-03;
  eg29[107] = 1.380384E-03;
  eg29[108] = 1.445440E-03;
  eg29[109] = 1.513561E-03;
  eg29[110] = 1.584893E-03;
  eg29[111] = 1.659587E-03;
  eg29[112] = 1.737801E-03;
  eg29[113] = 1.819701E-03;
  eg29[114] = 1.905461E-03;
  eg29[115] = 1.995262E-03;
  eg29[116] = 2.089296E-03;
  eg29[117] = 2.187762E-03;
  eg29[118] = 2.290868E-03;
  eg29[119] = 2.398833E-03;
  eg29[120] = 2.511886E-03;
  eg29[121] = 2.630268E-03;
  eg29[122] = 2.754229E-03;
  eg29[123] = 2.884032E-03;
  eg29[124] = 3.019952E-03;
  eg29[125] = 3.162278E-03;
  eg29[126] = 3.311311E-03;
  eg29[127] = 3.467369E-03;
  eg29[128] = 3.630781E-03;
  eg29[129] = 3.801894E-03;
  eg29[130] = 3.981072E-03;
  eg29[131] = 4.168694E-03;
  eg29[132] = 4.365158E-03;
  eg29[133] = 4.570882E-03;
  eg29[134] = 4.786301E-03;
  eg29[135] = 5.011872E-03;
  eg29[136] = 5.248075E-03;
  eg29[137] = 5.495409E-03;
  eg29[138] = 5.754399E-03;
  eg29[139] = 6.025596E-03;
  eg29[140] = 6.309573E-03;
  eg29[141] = 6.606934E-03;
  eg29[142] = 6.918310E-03;
  eg29[143] = 7.244360E-03;
  eg29[144] = 7.585776E-03;
  eg29[145] = 7.943282E-03;
  eg29[146] = 8.317638E-03;
  eg29[147] = 8.709636E-03;
  eg29[148] = 9.120108E-03;
  eg29[149] = 9.549926E-03;
  eg29[150] = 1.000000E-02;
  eg29[151] = 1.047129E-02;
  eg29[152] = 1.096478E-02;
  eg29[153] = 1.148154E-02;
  eg29[154] = 1.202264E-02;
  eg29[155] = 1.258925E-02;
  eg29[156] = 1.318257E-02;
  eg29[157] = 1.380384E-02;
  eg29[158] = 1.445440E-02;
  eg29[159] = 1.513561E-02;
  eg29[160] = 1.584893E-02;
  eg29[161] = 1.659587E-02;
  eg29[162] = 1.737801E-02;
  eg29[163] = 1.819701E-02;
  eg29[164] = 1.905461E-02;
  eg29[165] = 1.995262E-02;
  eg29[166] = 2.089296E-02;
  eg29[167] = 2.187762E-02;
  eg29[168] = 2.290868E-02;
  eg29[169] = 2.398833E-02;
  eg29[170] = 2.511886E-02;
  eg29[171] = 2.630268E-02;
  eg29[172] = 2.754229E-02;
  eg29[173] = 2.884032E-02;
  eg29[174] = 3.019952E-02;
  eg29[175] = 3.162278E-02;
  eg29[176] = 3.311311E-02;
  eg29[177] = 3.467369E-02;
  eg29[178] = 3.630781E-02;
  eg29[179] = 3.801894E-02;
  eg29[180] = 3.981072E-02;
  eg29[181] = 4.168694E-02;
  eg29[182] = 4.365158E-02;
  eg29[183] = 4.570882E-02;
  eg29[184] = 4.786301E-02;
  eg29[185] = 5.011872E-02;
  eg29[186] = 5.248075E-02;
  eg29[187] = 5.495409E-02;
  eg29[188] = 5.754399E-02;
  eg29[189] = 6.025596E-02;
  eg29[190] = 6.309573E-02;
  eg29[191] = 6.606934E-02;
  eg29[192] = 6.918310E-02;
  eg29[193] = 7.244360E-02;
  eg29[194] = 7.585776E-02;
  eg29[195] = 7.943282E-02;
  eg29[196] = 8.317638E-02;
  eg29[197] = 8.709636E-02;
  eg29[198] = 9.120108E-02;
  eg29[199] = 9.549926E-02;
  eg29[200] = 1.000000E-01;
  eg29[201] = 1.047129E-01;
  eg29[202] = 1.096478E-01;
  eg29[203] = 1.148154E-01;
  eg29[204] = 1.202264E-01;
  eg29[205] = 1.258925E-01;
  eg29[206] = 1.318257E-01;
  eg29[207] = 1.380384E-01;
  eg29[208] = 1.445440E-01;
  eg29[209] = 1.513561E-01;
  eg29[210] = 1.584893E-01;
  eg29[211] = 1.659587E-01;
  eg29[212] = 1.737801E-01;
  eg29[213] = 1.819701E-01;
  eg29[214] = 1.905461E-01;
  eg29[215] = 1.995262E-01;
  eg29[216] = 2.089296E-01;
  eg29[217] = 2.187762E-01;
  eg29[218] = 2.290868E-01;
  eg29[219] = 2.398833E-01;
  eg29[220] = 2.511886E-01;
  eg29[221] = 2.630268E-01;
  eg29[222] = 2.754229E-01;
  eg29[223] = 2.884032E-01;
  eg29[224] = 3.019952E-01;
  eg29[225] = 3.162278E-01;
  eg29[226] = 3.311311E-01;
  eg29[227] = 3.467369E-01;
  eg29[228] = 3.630781E-01;
  eg29[229] = 3.801894E-01;
  eg29[230] = 3.981072E-01;
  eg29[231] = 4.168694E-01;
  eg29[232] = 4.365158E-01;
  eg29[233] = 4.570882E-01;
  eg29[234] = 4.786301E-01;
  eg29[235] = 5.011872E-01;
  eg29[236] = 5.248075E-01;
  eg29[237] = 5.495409E-01;
  eg29[238] = 5.754399E-01;
  eg29[239] = 6.025596E-01;
  eg29[240] = 6.309573E-01;
  eg29[241] = 6.606934E-01;
  eg29[242] = 6.918310E-01;
  eg29[243] = 7.244360E-01;
  eg29[244] = 7.585776E-01;
  eg29[245] = 7.943282E-01;
  eg29[246] = 8.317638E-01;
  eg29[247] = 8.709636E-01;
  eg29[248] = 9.120108E-01;
  eg29[249] = 9.549926E-01;
  eg29[250] = 1.000000E+00;
  eg29[251] = 1.047129E+00;
  eg29[252] = 1.096478E+00;
  eg29[253] = 1.148154E+00;
  eg29[254] = 1.202264E+00;
  eg29[255] = 1.258925E+00;
  eg29[256] = 1.318257E+00;
  eg29[257] = 1.380384E+00;
  eg29[258] = 1.445440E+00;
  eg29[259] = 1.513561E+00;
  eg29[260] = 1.584893E+00;
  eg29[261] = 1.659587E+00;
  eg29[262] = 1.737801E+00;
  eg29[263] = 1.819701E+00;
  eg29[264] = 1.905461E+00;
  eg29[265] = 1.995262E+00;
  eg29[266] = 2.089296E+00;
  eg29[267] = 2.187762E+00;
  eg29[268] = 2.290868E+00;
  eg29[269] = 2.398833E+00;
  eg29[270] = 2.511886E+00;
  eg29[271] = 2.630268E+00;
  eg29[272] = 2.754229E+00;
  eg29[273] = 2.884032E+00;
  eg29[274] = 3.019952E+00;
  eg29[275] = 3.162278E+00;
  eg29[276] = 3.311311E+00;
  eg29[277] = 3.467369E+00;
  eg29[278] = 3.630781E+00;
  eg29[279] = 3.801894E+00;
  eg29[280] = 3.981072E+00;
  eg29[281] = 4.168694E+00;
  eg29[282] = 4.365158E+00;
  eg29[283] = 4.570882E+00;
  eg29[284] = 4.786301E+00;
  eg29[285] = 5.011872E+00;
  eg29[286] = 5.248075E+00;
  eg29[287] = 5.495409E+00;
  eg29[288] = 5.754399E+00;
  eg29[289] = 6.025596E+00;
  eg29[290] = 6.309573E+00;
  eg29[291] = 6.606934E+00;
  eg29[292] = 6.918310E+00;
  eg29[293] = 7.244360E+00;
  eg29[294] = 7.585776E+00;
  eg29[295] = 7.943282E+00;
  eg29[296] = 8.317638E+00;
  eg29[297] = 8.709636E+00;
  eg29[298] = 9.120108E+00;
  eg29[299] = 9.549926E+00;
  eg29[300] = 1.000000E+01;
  eg29[301] = 1.047129E+01;
  eg29[302] = 1.096478E+01;
  eg29[303] = 1.148154E+01;
  eg29[304] = 1.202264E+01;
  eg29[305] = 1.258925E+01;
  eg29[306] = 1.318257E+01;
  eg29[307] = 1.380384E+01;
  eg29[308] = 1.445440E+01;
  eg29[309] = 1.513561E+01;
  eg29[310] = 1.584893E+01;
  eg29[311] = 1.659587E+01;
  eg29[312] = 1.737801E+01;
  eg29[313] = 1.819701E+01;
  eg29[314] = 1.905461E+01;
  eg29[315] = 1.995262E+01;
  eg29[316] = 2.089296E+01;
  eg29[317] = 2.187762E+01;
  eg29[318] = 2.290868E+01;
  eg29[319] = 2.398833E+01;
  eg29[320] = 2.511886E+01;
  eg29[321] = 2.630268E+01;
  eg29[322] = 2.754229E+01;
  eg29[323] = 2.884032E+01;
  eg29[324] = 3.019952E+01;
  eg29[325] = 3.162278E+01;
  eg29[326] = 3.311311E+01;
  eg29[327] = 3.467369E+01;
  eg29[328] = 3.630781E+01;
  eg29[329] = 3.801894E+01;
  eg29[330] = 3.981072E+01;
  eg29[331] = 4.168694E+01;
  eg29[332] = 4.365158E+01;
  eg29[333] = 4.570882E+01;
  eg29[334] = 4.786301E+01;
  eg29[335] = 5.011872E+01;
  eg29[336] = 5.248075E+01;
  eg29[337] = 5.495409E+01;
  eg29[338] = 5.754399E+01;
  eg29[339] = 6.025596E+01;
  eg29[340] = 6.309573E+01;
  eg29[341] = 6.606934E+01;
  eg29[342] = 6.918310E+01;
  eg29[343] = 7.244360E+01;
  eg29[344] = 7.585776E+01;
  eg29[345] = 7.943282E+01;
  eg29[346] = 8.317638E+01;
  eg29[347] = 8.709636E+01;
  eg29[348] = 9.120108E+01;
  eg29[349] = 9.549926E+01;
  eg29[350] = 1.000000E+02;
  eg29[351] = 1.047129E+02;
  eg29[352] = 1.096478E+02;
  eg29[353] = 1.148154E+02;
  eg29[354] = 1.202264E+02;
  eg29[355] = 1.258925E+02;
  eg29[356] = 1.318257E+02;
  eg29[357] = 1.380384E+02;
  eg29[358] = 1.445440E+02;
  eg29[359] = 1.513561E+02;
  eg29[360] = 1.584893E+02;
  eg29[361] = 1.659587E+02;
  eg29[362] = 1.737801E+02;
  eg29[363] = 1.819701E+02;
  eg29[364] = 1.905461E+02;
  eg29[365] = 1.995262E+02;
  eg29[366] = 2.089296E+02;
  eg29[367] = 2.187762E+02;
  eg29[368] = 2.290868E+02;
  eg29[369] = 2.398833E+02;
  eg29[370] = 2.511886E+02;
  eg29[371] = 2.630268E+02;
  eg29[372] = 2.754229E+02;
  eg29[373] = 2.884032E+02;
  eg29[374] = 3.019952E+02;
  eg29[375] = 3.162278E+02;
  eg29[376] = 3.311311E+02;
  eg29[377] = 3.467369E+02;
  eg29[378] = 3.630781E+02;
  eg29[379] = 3.801894E+02;
  eg29[380] = 3.981072E+02;
  eg29[381] = 4.168694E+02;
  eg29[382] = 4.365158E+02;
  eg29[383] = 4.570882E+02;
  eg29[384] = 4.786301E+02;
  eg29[385] = 5.011872E+02;
  eg29[386] = 5.248075E+02;
  eg29[387] = 5.495409E+02;
  eg29[388] = 5.754399E+02;
  eg29[389] = 6.025596E+02;
  eg29[390] = 6.309573E+02;
  eg29[391] = 6.606934E+02;
  eg29[392] = 6.918310E+02;
  eg29[393] = 7.244360E+02;
  eg29[394] = 7.585776E+02;
  eg29[395] = 7.943282E+02;
  eg29[396] = 8.317638E+02;
  eg29[397] = 8.709636E+02;
  eg29[398] = 9.120108E+02;
  eg29[399] = 9.549926E+02;
  eg29[400] = 1.000000E+03;
  eg29[401] = 1.047129E+03;
  eg29[402] = 1.096478E+03;
  eg29[403] = 1.148154E+03;
  eg29[404] = 1.202264E+03;
  eg29[405] = 1.258925E+03;
  eg29[406] = 1.318257E+03;
  eg29[407] = 1.380384E+03;
  eg29[408] = 1.445440E+03;
  eg29[409] = 1.513561E+03;
  eg29[410] = 1.584893E+03;
  eg29[411] = 1.659587E+03;
  eg29[412] = 1.737801E+03;
  eg29[413] = 1.819701E+03;
  eg29[414] = 1.905461E+03;
  eg29[415] = 1.995262E+03;
  eg29[416] = 2.089296E+03;
  eg29[417] = 2.187762E+03;
  eg29[418] = 2.290868E+03;
  eg29[419] = 2.398833E+03;
  eg29[420] = 2.511886E+03;
  eg29[421] = 2.630268E+03;
  eg29[422] = 2.754229E+03;
  eg29[423] = 2.884032E+03;
  eg29[424] = 3.019952E+03;
  eg29[425] = 3.162278E+03;
  eg29[426] = 3.311311E+03;
  eg29[427] = 3.467369E+03;
  eg29[428] = 3.630781E+03;
  eg29[429] = 3.801894E+03;
  eg29[430] = 3.981072E+03;
  eg29[431] = 4.168694E+03;
  eg29[432] = 4.365158E+03;
  eg29[433] = 4.570882E+03;
  eg29[434] = 4.786301E+03;
  eg29[435] = 5.011872E+03;
  eg29[436] = 5.248075E+03;
  eg29[437] = 5.495409E+03;
  eg29[438] = 5.754399E+03;
  eg29[439] = 6.025596E+03;
  eg29[440] = 6.309573E+03;
  eg29[441] = 6.606934E+03;
  eg29[442] = 6.918310E+03;
  eg29[443] = 7.244360E+03;
  eg29[444] = 7.585776E+03;
  eg29[445] = 7.943282E+03;
  eg29[446] = 8.317638E+03;
  eg29[447] = 8.709636E+03;
  eg29[448] = 9.120108E+03;
  eg29[449] = 9.549926E+03;
  eg29[450] = 1.000000E+04;
  eg29[451] = 1.047129E+04;
  eg29[452] = 1.096478E+04;
  eg29[453] = 1.148154E+04;
  eg29[454] = 1.202264E+04;
  eg29[455] = 1.258925E+04;
  eg29[456] = 1.318257E+04;
  eg29[457] = 1.380384E+04;
  eg29[458] = 1.445440E+04;
  eg29[459] = 1.513561E+04;
  eg29[460] = 1.584893E+04;
  eg29[461] = 1.659587E+04;
  eg29[462] = 1.737801E+04;
  eg29[463] = 1.819701E+04;
  eg29[464] = 1.905461E+04;
  eg29[465] = 1.995262E+04;
  eg29[466] = 2.089296E+04;
  eg29[467] = 2.187762E+04;
  eg29[468] = 2.290868E+04;
  eg29[469] = 2.398833E+04;
  eg29[470] = 2.511886E+04;
  eg29[471] = 2.630268E+04;
  eg29[472] = 2.754229E+04;
  eg29[473] = 2.884032E+04;
  eg29[474] = 3.019952E+04;
  eg29[475] = 3.162278E+04;
  eg29[476] = 3.311311E+04;
  eg29[477] = 3.467369E+04;
  eg29[478] = 3.630781E+04;
  eg29[479] = 3.801894E+04;
  eg29[480] = 3.981072E+04;
  eg29[481] = 4.168694E+04;
  eg29[482] = 4.365158E+04;
  eg29[483] = 4.570882E+04;
  eg29[484] = 4.786301E+04;
  eg29[485] = 5.011872E+04;
  eg29[486] = 5.248075E+04;
  eg29[487] = 5.495409E+04;
  eg29[488] = 5.754399E+04;
  eg29[489] = 6.025596E+04;
  eg29[490] = 6.309573E+04;
  eg29[491] = 6.606934E+04;
  eg29[492] = 6.918310E+04;
  eg29[493] = 7.244360E+04;
  eg29[494] = 7.585776E+04;
  eg29[495] = 7.943282E+04;
  eg29[496] = 8.317638E+04;
  eg29[497] = 8.709636E+04;
  eg29[498] = 9.120108E+04;
  eg29[499] = 9.549926E+04;
  eg29[500] = 1.000000E+05;
  eg29[501] = 1.047129E+05;
  eg29[502] = 1.096478E+05;
  eg29[503] = 1.148154E+05;
  eg29[504] = 1.202264E+05;
  eg29[505] = 1.258925E+05;
  eg29[506] = 1.318257E+05;
  eg29[507] = 1.380384E+05;
  eg29[508] = 1.445440E+05;
  eg29[509] = 1.513561E+05;
  eg29[510] = 1.584893E+05;
  eg29[511] = 1.659587E+05;
  eg29[512] = 1.737801E+05;
  eg29[513] = 1.819701E+05;
  eg29[514] = 1.905461E+05;
  eg29[515] = 1.995262E+05;
  eg29[516] = 2.089296E+05;
  eg29[517] = 2.187762E+05;
  eg29[518] = 2.290868E+05;
  eg29[519] = 2.398833E+05;
  eg29[520] = 2.511886E+05;
  eg29[521] = 2.630268E+05;
  eg29[522] = 2.754229E+05;
  eg29[523] = 2.884032E+05;
  eg29[524] = 3.019952E+05;
  eg29[525] = 3.162278E+05;
  eg29[526] = 3.311311E+05;
  eg29[527] = 3.467369E+05;
  eg29[528] = 3.630781E+05;
  eg29[529] = 3.801894E+05;
  eg29[530] = 3.981072E+05;
  eg29[531] = 4.168694E+05;
  eg29[532] = 4.365158E+05;
  eg29[533] = 4.570882E+05;
  eg29[534] = 4.786301E+05;
  eg29[535] = 5.011872E+05;
  eg29[536] = 5.248075E+05;
  eg29[537] = 5.495409E+05;
  eg29[538] = 5.754399E+05;
  eg29[539] = 6.025596E+05;
  eg29[540] = 6.309573E+05;
  eg29[541] = 6.606934E+05;
  eg29[542] = 6.918310E+05;
  eg29[543] = 7.244360E+05;
  eg29[544] = 7.585776E+05;
  eg29[545] = 7.943282E+05;
  eg29[546] = 8.317638E+05;
  eg29[547] = 8.709636E+05;
  eg29[548] = 9.120108E+05;
  eg29[549] = 9.549926E+05;
  eg29[550] = 1.000000E+06;
  eg29[551] = 1.047129E+06;
  eg29[552] = 1.096478E+06;
  eg29[553] = 1.148154E+06;
  eg29[554] = 1.202264E+06;
  eg29[555] = 1.258925E+06;
  eg29[556] = 1.318257E+06;
  eg29[557] = 1.380384E+06;
  eg29[558] = 1.445440E+06;
  eg29[559] = 1.513561E+06;
  eg29[560] = 1.584893E+06;
  eg29[561] = 1.659587E+06;
  eg29[562] = 1.737801E+06;
  eg29[563] = 1.819701E+06;
  eg29[564] = 1.905461E+06;
  eg29[565] = 1.995262E+06;
  eg29[566] = 2.089296E+06;
  eg29[567] = 2.187762E+06;
  eg29[568] = 2.290868E+06;
  eg29[569] = 2.398833E+06;
  eg29[570] = 2.511886E+06;
  eg29[571] = 2.630268E+06;
  eg29[572] = 2.754229E+06;
  eg29[573] = 2.884032E+06;
  eg29[574] = 3.019952E+06;
  eg29[575] = 3.162278E+06;
  eg29[576] = 3.311311E+06;
  eg29[577] = 3.467369E+06;
  eg29[578] = 3.630781E+06;
  eg29[579] = 3.801894E+06;
  eg29[580] = 3.981072E+06;
  eg29[581] = 4.168694E+06;
  eg29[582] = 4.365158E+06;
  eg29[583] = 4.570882E+06;
  eg29[584] = 4.786301E+06;
  eg29[585] = 5.011872E+06;
  eg29[586] = 5.248075E+06;
  eg29[587] = 5.495409E+06;
  eg29[588] = 5.754399E+06;
  eg29[589] = 6.025596E+06;
  eg29[590] = 6.309573E+06;
  eg29[591] = 6.606934E+06;
  eg29[592] = 6.918310E+06;
  eg29[593] = 7.244360E+06;
  eg29[594] = 7.585776E+06;
  eg29[595] = 7.943282E+06;
  eg29[596] = 8.317638E+06;
  eg29[597] = 8.709636E+06;
  eg29[598] = 9.120108E+06;
  eg29[599] = 9.549926E+06;
  eg29[600] = 1.000000E+07;
  eg29[601] = 1.020000E+07;
  eg29[602] = 1.040000E+07;
  eg29[603] = 1.060000E+07;
  eg29[604] = 1.080000E+07;
  eg29[605] = 1.100000E+07;
  eg29[606] = 1.120000E+07;
  eg29[607] = 1.140000E+07;
  eg29[608] = 1.160000E+07;
  eg29[609] = 1.180000E+07;
  eg29[610] = 1.200000E+07;
  eg29[611] = 1.220000E+07;
  eg29[612] = 1.240000E+07;
  eg29[613] = 1.260000E+07;
  eg29[614] = 1.280000E+07;
  eg29[615] = 1.300000E+07;
  eg29[616] = 1.320000E+07;
  eg29[617] = 1.340000E+07;
  eg29[618] = 1.360000E+07;
  eg29[619] = 1.380000E+07;
  eg29[620] = 1.400000E+07;
  eg29[621] = 1.420000E+07;
  eg29[622] = 1.440000E+07;
  eg29[623] = 1.460000E+07;
  eg29[624] = 1.480000E+07;
  eg29[625] = 1.500000E+07;
  eg29[626] = 1.520000E+07;
  eg29[627] = 1.540000E+07;
  eg29[628] = 1.560000E+07;
  eg29[629] = 1.580000E+07;
  eg29[630] = 1.600000E+07;
  eg29[631] = 1.620000E+07;
  eg29[632] = 1.640000E+07;
  eg29[633] = 1.660000E+07;
  eg29[634] = 1.680000E+07;
  eg29[635] = 1.700000E+07;
  eg29[636] = 1.720000E+07;
  eg29[637] = 1.740000E+07;
  eg29[638] = 1.760000E+07;
  eg29[639] = 1.780000E+07;
  eg29[640] = 1.800000E+07;
  eg29[641] = 1.820000E+07;
  eg29[642] = 1.840000E+07;
  eg29[643] = 1.860000E+07;
  eg29[644] = 1.880000E+07;
  eg29[645] = 1.900000E+07;
  eg29[646] = 1.920000E+07;
  eg29[647] = 1.940000E+07;
  eg29[648] = 1.960000E+07;
  eg29[649] = 1.980000E+07;
  eg29[650] = 2.000000E+07;
  eg29[651] = 2.100000E+07;
  eg29[652] = 2.200000E+07;
  eg29[653] = 2.300000E+07;
  eg29[654] = 2.400000E+07;
  eg29[655] = 2.500000E+07;
  eg29[656] = 2.600000E+07;
  eg29[657] = 2.700000E+07;
  eg29[658] = 2.800000E+07;
  eg29[659] = 2.900000E+07;
  eg29[660] = 3.000000E+07;

  return eg29;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg30()
{
  vector<Real> eg30;
  eg30.resize(1026);
  eg30[   0] = 1.000000E-05;
  eg30[   1] = 1.047129E-05;
  eg30[   2] = 1.096478E-05;
  eg30[   3] = 1.148154E-05;
  eg30[   4] = 1.202264E-05;
  eg30[   5] = 1.258925E-05;
  eg30[   6] = 1.318257E-05;
  eg30[   7] = 1.380384E-05;
  eg30[   8] = 1.445440E-05;
  eg30[   9] = 1.513561E-05;
  eg30[  10] = 1.584893E-05;
  eg30[  11] = 1.659587E-05;
  eg30[  12] = 1.737801E-05;
  eg30[  13] = 1.819701E-05;
  eg30[  14] = 1.905461E-05;
  eg30[  15] = 1.995262E-05;
  eg30[  16] = 2.089296E-05;
  eg30[  17] = 2.187762E-05;
  eg30[  18] = 2.290868E-05;
  eg30[  19] = 2.398833E-05;
  eg30[  20] = 2.511886E-05;
  eg30[  21] = 2.630268E-05;
  eg30[  22] = 2.754229E-05;
  eg30[  23] = 2.884032E-05;
  eg30[  24] = 3.019952E-05;
  eg30[  25] = 3.162278E-05;
  eg30[  26] = 3.311311E-05;
  eg30[  27] = 3.467369E-05;
  eg30[  28] = 3.630781E-05;
  eg30[  29] = 3.801894E-05;
  eg30[  30] = 3.981072E-05;
  eg30[  31] = 4.168694E-05;
  eg30[  32] = 4.365158E-05;
  eg30[  33] = 4.570882E-05;
  eg30[  34] = 4.786301E-05;
  eg30[  35] = 5.011872E-05;
  eg30[  36] = 5.248075E-05;
  eg30[  37] = 5.495409E-05;
  eg30[  38] = 5.754399E-05;
  eg30[  39] = 6.025596E-05;
  eg30[  40] = 6.309573E-05;
  eg30[  41] = 6.606934E-05;
  eg30[  42] = 6.918310E-05;
  eg30[  43] = 7.244360E-05;
  eg30[  44] = 7.585776E-05;
  eg30[  45] = 7.943282E-05;
  eg30[  46] = 8.317638E-05;
  eg30[  47] = 8.709636E-05;
  eg30[  48] = 9.120108E-05;
  eg30[  49] = 9.549926E-05;
  eg30[  50] = 1.000000E-04;
  eg30[  51] = 1.047129E-04;
  eg30[  52] = 1.096478E-04;
  eg30[  53] = 1.148154E-04;
  eg30[  54] = 1.202264E-04;
  eg30[  55] = 1.258925E-04;
  eg30[  56] = 1.318257E-04;
  eg30[  57] = 1.380384E-04;
  eg30[  58] = 1.445440E-04;
  eg30[  59] = 1.513561E-04;
  eg30[  60] = 1.584893E-04;
  eg30[  61] = 1.659587E-04;
  eg30[  62] = 1.737801E-04;
  eg30[  63] = 1.819701E-04;
  eg30[  64] = 1.905461E-04;
  eg30[  65] = 1.995262E-04;
  eg30[  66] = 2.089296E-04;
  eg30[  67] = 2.187762E-04;
  eg30[  68] = 2.290868E-04;
  eg30[  69] = 2.398833E-04;
  eg30[  70] = 2.511886E-04;
  eg30[  71] = 2.630268E-04;
  eg30[  72] = 2.754229E-04;
  eg30[  73] = 2.884032E-04;
  eg30[  74] = 3.019952E-04;
  eg30[  75] = 3.162278E-04;
  eg30[  76] = 3.311311E-04;
  eg30[  77] = 3.467369E-04;
  eg30[  78] = 3.630781E-04;
  eg30[  79] = 3.801894E-04;
  eg30[  80] = 3.981072E-04;
  eg30[  81] = 4.168694E-04;
  eg30[  82] = 4.365158E-04;
  eg30[  83] = 4.570882E-04;
  eg30[  84] = 4.786301E-04;
  eg30[  85] = 5.011872E-04;
  eg30[  86] = 5.248075E-04;
  eg30[  87] = 5.495409E-04;
  eg30[  88] = 5.754399E-04;
  eg30[  89] = 6.025596E-04;
  eg30[  90] = 6.309573E-04;
  eg30[  91] = 6.606934E-04;
  eg30[  92] = 6.918310E-04;
  eg30[  93] = 7.244360E-04;
  eg30[  94] = 7.585776E-04;
  eg30[  95] = 7.943282E-04;
  eg30[  96] = 8.317638E-04;
  eg30[  97] = 8.709636E-04;
  eg30[  98] = 9.120108E-04;
  eg30[  99] = 9.549926E-04;
  eg30[ 100] = 1.000000E-03;
  eg30[ 101] = 1.047129E-03;
  eg30[ 102] = 1.096478E-03;
  eg30[ 103] = 1.148154E-03;
  eg30[ 104] = 1.202264E-03;
  eg30[ 105] = 1.258925E-03;
  eg30[ 106] = 1.318257E-03;
  eg30[ 107] = 1.380384E-03;
  eg30[ 108] = 1.445440E-03;
  eg30[ 109] = 1.513561E-03;
  eg30[ 110] = 1.584893E-03;
  eg30[ 111] = 1.659587E-03;
  eg30[ 112] = 1.737801E-03;
  eg30[ 113] = 1.819701E-03;
  eg30[ 114] = 1.905461E-03;
  eg30[ 115] = 1.995262E-03;
  eg30[ 116] = 2.089296E-03;
  eg30[ 117] = 2.187762E-03;
  eg30[ 118] = 2.290868E-03;
  eg30[ 119] = 2.398833E-03;
  eg30[ 120] = 2.511886E-03;
  eg30[ 121] = 2.630268E-03;
  eg30[ 122] = 2.754229E-03;
  eg30[ 123] = 2.884032E-03;
  eg30[ 124] = 3.019952E-03;
  eg30[ 125] = 3.162278E-03;
  eg30[ 126] = 3.311311E-03;
  eg30[ 127] = 3.467369E-03;
  eg30[ 128] = 3.630781E-03;
  eg30[ 129] = 3.801894E-03;
  eg30[ 130] = 3.981072E-03;
  eg30[ 131] = 4.168694E-03;
  eg30[ 132] = 4.365158E-03;
  eg30[ 133] = 4.570882E-03;
  eg30[ 134] = 4.786301E-03;
  eg30[ 135] = 5.011872E-03;
  eg30[ 136] = 5.248075E-03;
  eg30[ 137] = 5.495409E-03;
  eg30[ 138] = 5.754399E-03;
  eg30[ 139] = 6.025596E-03;
  eg30[ 140] = 6.309573E-03;
  eg30[ 141] = 6.606934E-03;
  eg30[ 142] = 6.918310E-03;
  eg30[ 143] = 7.244360E-03;
  eg30[ 144] = 7.585776E-03;
  eg30[ 145] = 7.943282E-03;
  eg30[ 146] = 8.317638E-03;
  eg30[ 147] = 8.709636E-03;
  eg30[ 148] = 9.120108E-03;
  eg30[ 149] = 9.549926E-03;
  eg30[ 150] = 1.000000E-02;
  eg30[ 151] = 1.047129E-02;
  eg30[ 152] = 1.096478E-02;
  eg30[ 153] = 1.148154E-02;
  eg30[ 154] = 1.202264E-02;
  eg30[ 155] = 1.258925E-02;
  eg30[ 156] = 1.318257E-02;
  eg30[ 157] = 1.380384E-02;
  eg30[ 158] = 1.445440E-02;
  eg30[ 159] = 1.513561E-02;
  eg30[ 160] = 1.584893E-02;
  eg30[ 161] = 1.659587E-02;
  eg30[ 162] = 1.737801E-02;
  eg30[ 163] = 1.819701E-02;
  eg30[ 164] = 1.905461E-02;
  eg30[ 165] = 1.995262E-02;
  eg30[ 166] = 2.089296E-02;
  eg30[ 167] = 2.187762E-02;
  eg30[ 168] = 2.290868E-02;
  eg30[ 169] = 2.398833E-02;
  eg30[ 170] = 2.511886E-02;
  eg30[ 171] = 2.630268E-02;
  eg30[ 172] = 2.754229E-02;
  eg30[ 173] = 2.884032E-02;
  eg30[ 174] = 3.019952E-02;
  eg30[ 175] = 3.162278E-02;
  eg30[ 176] = 3.311311E-02;
  eg30[ 177] = 3.467369E-02;
  eg30[ 178] = 3.630781E-02;
  eg30[ 179] = 3.801894E-02;
  eg30[ 180] = 3.981072E-02;
  eg30[ 181] = 4.168694E-02;
  eg30[ 182] = 4.365158E-02;
  eg30[ 183] = 4.570882E-02;
  eg30[ 184] = 4.786301E-02;
  eg30[ 185] = 5.011872E-02;
  eg30[ 186] = 5.248075E-02;
  eg30[ 187] = 5.495409E-02;
  eg30[ 188] = 5.754399E-02;
  eg30[ 189] = 6.025596E-02;
  eg30[ 190] = 6.309573E-02;
  eg30[ 191] = 6.606934E-02;
  eg30[ 192] = 6.918310E-02;
  eg30[ 193] = 7.244360E-02;
  eg30[ 194] = 7.585776E-02;
  eg30[ 195] = 7.943282E-02;
  eg30[ 196] = 8.317638E-02;
  eg30[ 197] = 8.709636E-02;
  eg30[ 198] = 9.120108E-02;
  eg30[ 199] = 9.549926E-02;
  eg30[ 200] = 1.000000E-01;
  eg30[ 201] = 1.047129E-01;
  eg30[ 202] = 1.096478E-01;
  eg30[ 203] = 1.148154E-01;
  eg30[ 204] = 1.202264E-01;
  eg30[ 205] = 1.258925E-01;
  eg30[ 206] = 1.318257E-01;
  eg30[ 207] = 1.380384E-01;
  eg30[ 208] = 1.445440E-01;
  eg30[ 209] = 1.513561E-01;
  eg30[ 210] = 1.584893E-01;
  eg30[ 211] = 1.659587E-01;
  eg30[ 212] = 1.737801E-01;
  eg30[ 213] = 1.819701E-01;
  eg30[ 214] = 1.905461E-01;
  eg30[ 215] = 1.995262E-01;
  eg30[ 216] = 2.089296E-01;
  eg30[ 217] = 2.187762E-01;
  eg30[ 218] = 2.290868E-01;
  eg30[ 219] = 2.398833E-01;
  eg30[ 220] = 2.511886E-01;
  eg30[ 221] = 2.630268E-01;
  eg30[ 222] = 2.754229E-01;
  eg30[ 223] = 2.884032E-01;
  eg30[ 224] = 3.019952E-01;
  eg30[ 225] = 3.162278E-01;
  eg30[ 226] = 3.311311E-01;
  eg30[ 227] = 3.467369E-01;
  eg30[ 228] = 3.630781E-01;
  eg30[ 229] = 3.801894E-01;
  eg30[ 230] = 3.981072E-01;
  eg30[ 231] = 4.168694E-01;
  eg30[ 232] = 4.365158E-01;
  eg30[ 233] = 4.570882E-01;
  eg30[ 234] = 4.786301E-01;
  eg30[ 235] = 5.011872E-01;
  eg30[ 236] = 5.248075E-01;
  eg30[ 237] = 5.500000E-01;
  eg30[ 238] = 5.750000E-01;
  eg30[ 239] = 6.000000E-01;
  eg30[ 240] = 6.250000E-01;
  eg30[ 241] = 6.500000E-01;
  eg30[ 242] = 6.750000E-01;
  eg30[ 243] = 7.000000E-01;
  eg30[ 244] = 7.250000E-01;
  eg30[ 245] = 7.500000E-01;
  eg30[ 246] = 7.750000E-01;
  eg30[ 247] = 8.000000E-01;
  eg30[ 248] = 8.250000E-01;
  eg30[ 249] = 8.500000E-01;
  eg30[ 250] = 8.750000E-01;
  eg30[ 251] = 9.000000E-01;
  eg30[ 252] = 9.250000E-01;
  eg30[ 253] = 9.500000E-01;
  eg30[ 254] = 9.750000E-01;
  eg30[ 255] = 1.000000E+00;
  eg30[ 256] = 1.025000E+00;
  eg30[ 257] = 1.050000E+00;
  eg30[ 258] = 1.075000E+00;
  eg30[ 259] = 1.100000E+00;
  eg30[ 260] = 1.125000E+00;
  eg30[ 261] = 1.150000E+00;
  eg30[ 262] = 1.175000E+00;
  eg30[ 263] = 1.200000E+00;
  eg30[ 264] = 1.225000E+00;
  eg30[ 265] = 1.250000E+00;
  eg30[ 266] = 1.275000E+00;
  eg30[ 267] = 1.300000E+00;
  eg30[ 268] = 1.325000E+00;
  eg30[ 269] = 1.350000E+00;
  eg30[ 270] = 1.375000E+00;
  eg30[ 271] = 1.400000E+00;
  eg30[ 272] = 1.425000E+00;
  eg30[ 273] = 1.450000E+00;
  eg30[ 274] = 1.475000E+00;
  eg30[ 275] = 1.500000E+00;
  eg30[ 276] = 1.525000E+00;
  eg30[ 277] = 1.550000E+00;
  eg30[ 278] = 1.575000E+00;
  eg30[ 279] = 1.600000E+00;
  eg30[ 280] = 1.625000E+00;
  eg30[ 281] = 1.650000E+00;
  eg30[ 282] = 1.675000E+00;
  eg30[ 283] = 1.700000E+00;
  eg30[ 284] = 1.725000E+00;
  eg30[ 285] = 1.750000E+00;
  eg30[ 286] = 1.775000E+00;
  eg30[ 287] = 1.800000E+00;
  eg30[ 288] = 1.825000E+00;
  eg30[ 289] = 1.850000E+00;
  eg30[ 290] = 1.875000E+00;
  eg30[ 291] = 1.900000E+00;
  eg30[ 292] = 1.925000E+00;
  eg30[ 293] = 1.950000E+00;
  eg30[ 294] = 1.975000E+00;
  eg30[ 295] = 2.000000E+00;
  eg30[ 296] = 2.025000E+00;
  eg30[ 297] = 2.050000E+00;
  eg30[ 298] = 2.075000E+00;
  eg30[ 299] = 2.100000E+00;
  eg30[ 300] = 2.125000E+00;
  eg30[ 301] = 2.150000E+00;
  eg30[ 302] = 2.175000E+00;
  eg30[ 303] = 2.200000E+00;
  eg30[ 304] = 2.225000E+00;
  eg30[ 305] = 2.250000E+00;
  eg30[ 306] = 2.275000E+00;
  eg30[ 307] = 2.300000E+00;
  eg30[ 308] = 2.325000E+00;
  eg30[ 309] = 2.350000E+00;
  eg30[ 310] = 2.375000E+00;
  eg30[ 311] = 2.400000E+00;
  eg30[ 312] = 2.425000E+00;
  eg30[ 313] = 2.450000E+00;
  eg30[ 314] = 2.475000E+00;
  eg30[ 315] = 2.500000E+00;
  eg30[ 316] = 2.525000E+00;
  eg30[ 317] = 2.550000E+00;
  eg30[ 318] = 2.575000E+00;
  eg30[ 319] = 2.600000E+00;
  eg30[ 320] = 2.625000E+00;
  eg30[ 321] = 2.650000E+00;
  eg30[ 322] = 2.675000E+00;
  eg30[ 323] = 2.700000E+00;
  eg30[ 324] = 2.725000E+00;
  eg30[ 325] = 2.750000E+00;
  eg30[ 326] = 2.775000E+00;
  eg30[ 327] = 2.800000E+00;
  eg30[ 328] = 2.825000E+00;
  eg30[ 329] = 2.850000E+00;
  eg30[ 330] = 2.875000E+00;
  eg30[ 331] = 2.900000E+00;
  eg30[ 332] = 2.925000E+00;
  eg30[ 333] = 2.950000E+00;
  eg30[ 334] = 2.975000E+00;
  eg30[ 335] = 3.000000E+00;
  eg30[ 336] = 3.025000E+00;
  eg30[ 337] = 3.050000E+00;
  eg30[ 338] = 3.075000E+00;
  eg30[ 339] = 3.100000E+00;
  eg30[ 340] = 3.125000E+00;
  eg30[ 341] = 3.150000E+00;
  eg30[ 342] = 3.175000E+00;
  eg30[ 343] = 3.200000E+00;
  eg30[ 344] = 3.225000E+00;
  eg30[ 345] = 3.250000E+00;
  eg30[ 346] = 3.275000E+00;
  eg30[ 347] = 3.300000E+00;
  eg30[ 348] = 3.325000E+00;
  eg30[ 349] = 3.350000E+00;
  eg30[ 350] = 3.375000E+00;
  eg30[ 351] = 3.400000E+00;
  eg30[ 352] = 3.425000E+00;
  eg30[ 353] = 3.450000E+00;
  eg30[ 354] = 3.475000E+00;
  eg30[ 355] = 3.500000E+00;
  eg30[ 356] = 3.525000E+00;
  eg30[ 357] = 3.550000E+00;
  eg30[ 358] = 3.575000E+00;
  eg30[ 359] = 3.600000E+00;
  eg30[ 360] = 3.625000E+00;
  eg30[ 361] = 3.650000E+00;
  eg30[ 362] = 3.675000E+00;
  eg30[ 363] = 3.700000E+00;
  eg30[ 364] = 3.725000E+00;
  eg30[ 365] = 3.750000E+00;
  eg30[ 366] = 3.775000E+00;
  eg30[ 367] = 3.800000E+00;
  eg30[ 368] = 3.825000E+00;
  eg30[ 369] = 3.850000E+00;
  eg30[ 370] = 3.875000E+00;
  eg30[ 371] = 3.900000E+00;
  eg30[ 372] = 3.925000E+00;
  eg30[ 373] = 3.950000E+00;
  eg30[ 374] = 3.975000E+00;
  eg30[ 375] = 4.000000E+00;
  eg30[ 376] = 4.025000E+00;
  eg30[ 377] = 4.050000E+00;
  eg30[ 378] = 4.075000E+00;
  eg30[ 379] = 4.100000E+00;
  eg30[ 380] = 4.125000E+00;
  eg30[ 381] = 4.150000E+00;
  eg30[ 382] = 4.175000E+00;
  eg30[ 383] = 4.200000E+00;
  eg30[ 384] = 4.225000E+00;
  eg30[ 385] = 4.250000E+00;
  eg30[ 386] = 4.275000E+00;
  eg30[ 387] = 4.300000E+00;
  eg30[ 388] = 4.325000E+00;
  eg30[ 389] = 4.350000E+00;
  eg30[ 390] = 4.375000E+00;
  eg30[ 391] = 4.400000E+00;
  eg30[ 392] = 4.425000E+00;
  eg30[ 393] = 4.450000E+00;
  eg30[ 394] = 4.475000E+00;
  eg30[ 395] = 4.500000E+00;
  eg30[ 396] = 4.525000E+00;
  eg30[ 397] = 4.550000E+00;
  eg30[ 398] = 4.575000E+00;
  eg30[ 399] = 4.600000E+00;
  eg30[ 400] = 4.625000E+00;
  eg30[ 401] = 4.650000E+00;
  eg30[ 402] = 4.675000E+00;
  eg30[ 403] = 4.700000E+00;
  eg30[ 404] = 4.725000E+00;
  eg30[ 405] = 4.750000E+00;
  eg30[ 406] = 4.775000E+00;
  eg30[ 407] = 4.800000E+00;
  eg30[ 408] = 4.825000E+00;
  eg30[ 409] = 4.850000E+00;
  eg30[ 410] = 4.875000E+00;
  eg30[ 411] = 4.900000E+00;
  eg30[ 412] = 4.925000E+00;
  eg30[ 413] = 4.950000E+00;
  eg30[ 414] = 4.975000E+00;
  eg30[ 415] = 5.000000E+00;
  eg30[ 416] = 5.025000E+00;
  eg30[ 417] = 5.050000E+00;
  eg30[ 418] = 5.075000E+00;
  eg30[ 419] = 5.100000E+00;
  eg30[ 420] = 5.125000E+00;
  eg30[ 421] = 5.150000E+00;
  eg30[ 422] = 5.175000E+00;
  eg30[ 423] = 5.200000E+00;
  eg30[ 424] = 5.225000E+00;
  eg30[ 425] = 5.250000E+00;
  eg30[ 426] = 5.275000E+00;
  eg30[ 427] = 5.300000E+00;
  eg30[ 428] = 5.325000E+00;
  eg30[ 429] = 5.350000E+00;
  eg30[ 430] = 5.375000E+00;
  eg30[ 431] = 5.400000E+00;
  eg30[ 432] = 5.425000E+00;
  eg30[ 433] = 5.450000E+00;
  eg30[ 434] = 5.475000E+00;
  eg30[ 435] = 5.500000E+00;
  eg30[ 436] = 5.525000E+00;
  eg30[ 437] = 5.550000E+00;
  eg30[ 438] = 5.575000E+00;
  eg30[ 439] = 5.600000E+00;
  eg30[ 440] = 5.625000E+00;
  eg30[ 441] = 5.650000E+00;
  eg30[ 442] = 5.675000E+00;
  eg30[ 443] = 5.700000E+00;
  eg30[ 444] = 5.725000E+00;
  eg30[ 445] = 5.750000E+00;
  eg30[ 446] = 5.775000E+00;
  eg30[ 447] = 5.800000E+00;
  eg30[ 448] = 5.825000E+00;
  eg30[ 449] = 5.850000E+00;
  eg30[ 450] = 5.875000E+00;
  eg30[ 451] = 5.900000E+00;
  eg30[ 452] = 5.925000E+00;
  eg30[ 453] = 5.950000E+00;
  eg30[ 454] = 5.975000E+00;
  eg30[ 455] = 6.000000E+00;
  eg30[ 456] = 6.025000E+00;
  eg30[ 457] = 6.050000E+00;
  eg30[ 458] = 6.075000E+00;
  eg30[ 459] = 6.100000E+00;
  eg30[ 460] = 6.125000E+00;
  eg30[ 461] = 6.150000E+00;
  eg30[ 462] = 6.175000E+00;
  eg30[ 463] = 6.200000E+00;
  eg30[ 464] = 6.225000E+00;
  eg30[ 465] = 6.250000E+00;
  eg30[ 466] = 6.275000E+00;
  eg30[ 467] = 6.300000E+00;
  eg30[ 468] = 6.325000E+00;
  eg30[ 469] = 6.350000E+00;
  eg30[ 470] = 6.375000E+00;
  eg30[ 471] = 6.400000E+00;
  eg30[ 472] = 6.425000E+00;
  eg30[ 473] = 6.450000E+00;
  eg30[ 474] = 6.475000E+00;
  eg30[ 475] = 6.500000E+00;
  eg30[ 476] = 6.525000E+00;
  eg30[ 477] = 6.550000E+00;
  eg30[ 478] = 6.575000E+00;
  eg30[ 479] = 6.600000E+00;
  eg30[ 480] = 6.625000E+00;
  eg30[ 481] = 6.650000E+00;
  eg30[ 482] = 6.675000E+00;
  eg30[ 483] = 6.700000E+00;
  eg30[ 484] = 6.725000E+00;
  eg30[ 485] = 6.750000E+00;
  eg30[ 486] = 6.775000E+00;
  eg30[ 487] = 6.800000E+00;
  eg30[ 488] = 6.825000E+00;
  eg30[ 489] = 6.850000E+00;
  eg30[ 490] = 6.875000E+00;
  eg30[ 491] = 6.900000E+00;
  eg30[ 492] = 6.925000E+00;
  eg30[ 493] = 6.950000E+00;
  eg30[ 494] = 6.975000E+00;
  eg30[ 495] = 7.000000E+00;
  eg30[ 496] = 7.025000E+00;
  eg30[ 497] = 7.050000E+00;
  eg30[ 498] = 7.075000E+00;
  eg30[ 499] = 7.100000E+00;
  eg30[ 500] = 7.125000E+00;
  eg30[ 501] = 7.150000E+00;
  eg30[ 502] = 7.175000E+00;
  eg30[ 503] = 7.200000E+00;
  eg30[ 504] = 7.225000E+00;
  eg30[ 505] = 7.250000E+00;
  eg30[ 506] = 7.275000E+00;
  eg30[ 507] = 7.300000E+00;
  eg30[ 508] = 7.325000E+00;
  eg30[ 509] = 7.350000E+00;
  eg30[ 510] = 7.375000E+00;
  eg30[ 511] = 7.400000E+00;
  eg30[ 512] = 7.425000E+00;
  eg30[ 513] = 7.450000E+00;
  eg30[ 514] = 7.475000E+00;
  eg30[ 515] = 7.500000E+00;
  eg30[ 516] = 7.525000E+00;
  eg30[ 517] = 7.550000E+00;
  eg30[ 518] = 7.575000E+00;
  eg30[ 519] = 7.600000E+00;
  eg30[ 520] = 7.625000E+00;
  eg30[ 521] = 7.650000E+00;
  eg30[ 522] = 7.675000E+00;
  eg30[ 523] = 7.700000E+00;
  eg30[ 524] = 7.725000E+00;
  eg30[ 525] = 7.750000E+00;
  eg30[ 526] = 7.775000E+00;
  eg30[ 527] = 7.800000E+00;
  eg30[ 528] = 7.825000E+00;
  eg30[ 529] = 7.850000E+00;
  eg30[ 530] = 7.875000E+00;
  eg30[ 531] = 7.900000E+00;
  eg30[ 532] = 7.925000E+00;
  eg30[ 533] = 7.950000E+00;
  eg30[ 534] = 7.975000E+00;
  eg30[ 535] = 8.000000E+00;
  eg30[ 536] = 8.025000E+00;
  eg30[ 537] = 8.050000E+00;
  eg30[ 538] = 8.075000E+00;
  eg30[ 539] = 8.100000E+00;
  eg30[ 540] = 8.125000E+00;
  eg30[ 541] = 8.150000E+00;
  eg30[ 542] = 8.175000E+00;
  eg30[ 543] = 8.200000E+00;
  eg30[ 544] = 8.225000E+00;
  eg30[ 545] = 8.250000E+00;
  eg30[ 546] = 8.275000E+00;
  eg30[ 547] = 8.300000E+00;
  eg30[ 548] = 8.325000E+00;
  eg30[ 549] = 8.350000E+00;
  eg30[ 550] = 8.375000E+00;
  eg30[ 551] = 8.400000E+00;
  eg30[ 552] = 8.425000E+00;
  eg30[ 553] = 8.450000E+00;
  eg30[ 554] = 8.475000E+00;
  eg30[ 555] = 8.500000E+00;
  eg30[ 556] = 8.525000E+00;
  eg30[ 557] = 8.550000E+00;
  eg30[ 558] = 8.575000E+00;
  eg30[ 559] = 8.600000E+00;
  eg30[ 560] = 8.625000E+00;
  eg30[ 561] = 8.650000E+00;
  eg30[ 562] = 8.675000E+00;
  eg30[ 563] = 8.700000E+00;
  eg30[ 564] = 8.725000E+00;
  eg30[ 565] = 8.750000E+00;
  eg30[ 566] = 8.775000E+00;
  eg30[ 567] = 8.800000E+00;
  eg30[ 568] = 8.825000E+00;
  eg30[ 569] = 8.850000E+00;
  eg30[ 570] = 8.875000E+00;
  eg30[ 571] = 8.900000E+00;
  eg30[ 572] = 8.925000E+00;
  eg30[ 573] = 8.950000E+00;
  eg30[ 574] = 8.975000E+00;
  eg30[ 575] = 9.000000E+00;
  eg30[ 576] = 9.025000E+00;
  eg30[ 577] = 9.050000E+00;
  eg30[ 578] = 9.075000E+00;
  eg30[ 579] = 9.100000E+00;
  eg30[ 580] = 9.125000E+00;
  eg30[ 581] = 9.150000E+00;
  eg30[ 582] = 9.175000E+00;
  eg30[ 583] = 9.200000E+00;
  eg30[ 584] = 9.225000E+00;
  eg30[ 585] = 9.250000E+00;
  eg30[ 586] = 9.275000E+00;
  eg30[ 587] = 9.300000E+00;
  eg30[ 588] = 9.325000E+00;
  eg30[ 589] = 9.350000E+00;
  eg30[ 590] = 9.375000E+00;
  eg30[ 591] = 9.400000E+00;
  eg30[ 592] = 9.425000E+00;
  eg30[ 593] = 9.450000E+00;
  eg30[ 594] = 9.475000E+00;
  eg30[ 595] = 9.500000E+00;
  eg30[ 596] = 9.525000E+00;
  eg30[ 597] = 9.550000E+00;
  eg30[ 598] = 9.575000E+00;
  eg30[ 599] = 9.600000E+00;
  eg30[ 600] = 9.625000E+00;
  eg30[ 601] = 9.650000E+00;
  eg30[ 602] = 9.675000E+00;
  eg30[ 603] = 9.700000E+00;
  eg30[ 604] = 9.725000E+00;
  eg30[ 605] = 9.750000E+00;
  eg30[ 606] = 9.775000E+00;
  eg30[ 607] = 9.800000E+00;
  eg30[ 608] = 9.825000E+00;
  eg30[ 609] = 9.850000E+00;
  eg30[ 610] = 9.875000E+00;
  eg30[ 611] = 9.900000E+00;
  eg30[ 612] = 9.925000E+00;
  eg30[ 613] = 9.950000E+00;
  eg30[ 614] = 9.975000E+00;
  eg30[ 615] = 1.000000E+01;
  eg30[ 616] = 1.047129E+01;
  eg30[ 617] = 1.096478E+01;
  eg30[ 618] = 1.148154E+01;
  eg30[ 619] = 1.202264E+01;
  eg30[ 620] = 1.258925E+01;
  eg30[ 621] = 1.318257E+01;
  eg30[ 622] = 1.380384E+01;
  eg30[ 623] = 1.445440E+01;
  eg30[ 624] = 1.513561E+01;
  eg30[ 625] = 1.584893E+01;
  eg30[ 626] = 1.659587E+01;
  eg30[ 627] = 1.737801E+01;
  eg30[ 628] = 1.819701E+01;
  eg30[ 629] = 1.905461E+01;
  eg30[ 630] = 1.995262E+01;
  eg30[ 631] = 2.089296E+01;
  eg30[ 632] = 2.187762E+01;
  eg30[ 633] = 2.290868E+01;
  eg30[ 634] = 2.398833E+01;
  eg30[ 635] = 2.511886E+01;
  eg30[ 636] = 2.630268E+01;
  eg30[ 637] = 2.754229E+01;
  eg30[ 638] = 2.884032E+01;
  eg30[ 639] = 3.019952E+01;
  eg30[ 640] = 3.162278E+01;
  eg30[ 641] = 3.311311E+01;
  eg30[ 642] = 3.467369E+01;
  eg30[ 643] = 3.630781E+01;
  eg30[ 644] = 3.801894E+01;
  eg30[ 645] = 3.981072E+01;
  eg30[ 646] = 4.168694E+01;
  eg30[ 647] = 4.365158E+01;
  eg30[ 648] = 4.570882E+01;
  eg30[ 649] = 4.786301E+01;
  eg30[ 650] = 5.011872E+01;
  eg30[ 651] = 5.248075E+01;
  eg30[ 652] = 5.495409E+01;
  eg30[ 653] = 5.754399E+01;
  eg30[ 654] = 6.025596E+01;
  eg30[ 655] = 6.309573E+01;
  eg30[ 656] = 6.606934E+01;
  eg30[ 657] = 6.918310E+01;
  eg30[ 658] = 7.244360E+01;
  eg30[ 659] = 7.585776E+01;
  eg30[ 660] = 7.943282E+01;
  eg30[ 661] = 8.317638E+01;
  eg30[ 662] = 8.709636E+01;
  eg30[ 663] = 9.120108E+01;
  eg30[ 664] = 9.549926E+01;
  eg30[ 665] = 1.000000E+02;
  eg30[ 666] = 1.047129E+02;
  eg30[ 667] = 1.096478E+02;
  eg30[ 668] = 1.148154E+02;
  eg30[ 669] = 1.202264E+02;
  eg30[ 670] = 1.258925E+02;
  eg30[ 671] = 1.318257E+02;
  eg30[ 672] = 1.380384E+02;
  eg30[ 673] = 1.445440E+02;
  eg30[ 674] = 1.513561E+02;
  eg30[ 675] = 1.584893E+02;
  eg30[ 676] = 1.659587E+02;
  eg30[ 677] = 1.737801E+02;
  eg30[ 678] = 1.819701E+02;
  eg30[ 679] = 1.905461E+02;
  eg30[ 680] = 1.995262E+02;
  eg30[ 681] = 2.089296E+02;
  eg30[ 682] = 2.187762E+02;
  eg30[ 683] = 2.290868E+02;
  eg30[ 684] = 2.398833E+02;
  eg30[ 685] = 2.511886E+02;
  eg30[ 686] = 2.630268E+02;
  eg30[ 687] = 2.754229E+02;
  eg30[ 688] = 2.884032E+02;
  eg30[ 689] = 3.019952E+02;
  eg30[ 690] = 3.162278E+02;
  eg30[ 691] = 3.311311E+02;
  eg30[ 692] = 3.467369E+02;
  eg30[ 693] = 3.630781E+02;
  eg30[ 694] = 3.801894E+02;
  eg30[ 695] = 3.981072E+02;
  eg30[ 696] = 4.168694E+02;
  eg30[ 697] = 4.365158E+02;
  eg30[ 698] = 4.570882E+02;
  eg30[ 699] = 4.786301E+02;
  eg30[ 700] = 5.011872E+02;
  eg30[ 701] = 5.248075E+02;
  eg30[ 702] = 5.495409E+02;
  eg30[ 703] = 5.754399E+02;
  eg30[ 704] = 6.025596E+02;
  eg30[ 705] = 6.309573E+02;
  eg30[ 706] = 6.606934E+02;
  eg30[ 707] = 6.918310E+02;
  eg30[ 708] = 7.244360E+02;
  eg30[ 709] = 7.585776E+02;
  eg30[ 710] = 7.943282E+02;
  eg30[ 711] = 8.317638E+02;
  eg30[ 712] = 8.709636E+02;
  eg30[ 713] = 9.120108E+02;
  eg30[ 714] = 9.549926E+02;
  eg30[ 715] = 1.000000E+03;
  eg30[ 716] = 1.047129E+03;
  eg30[ 717] = 1.096478E+03;
  eg30[ 718] = 1.148154E+03;
  eg30[ 719] = 1.202264E+03;
  eg30[ 720] = 1.258925E+03;
  eg30[ 721] = 1.318257E+03;
  eg30[ 722] = 1.380384E+03;
  eg30[ 723] = 1.445440E+03;
  eg30[ 724] = 1.513561E+03;
  eg30[ 725] = 1.584893E+03;
  eg30[ 726] = 1.659587E+03;
  eg30[ 727] = 1.737801E+03;
  eg30[ 728] = 1.819701E+03;
  eg30[ 729] = 1.905461E+03;
  eg30[ 730] = 1.995262E+03;
  eg30[ 731] = 2.089296E+03;
  eg30[ 732] = 2.187762E+03;
  eg30[ 733] = 2.290868E+03;
  eg30[ 734] = 2.398833E+03;
  eg30[ 735] = 2.511886E+03;
  eg30[ 736] = 2.630268E+03;
  eg30[ 737] = 2.754229E+03;
  eg30[ 738] = 2.884032E+03;
  eg30[ 739] = 3.019952E+03;
  eg30[ 740] = 3.162278E+03;
  eg30[ 741] = 3.311311E+03;
  eg30[ 742] = 3.467369E+03;
  eg30[ 743] = 3.630781E+03;
  eg30[ 744] = 3.801894E+03;
  eg30[ 745] = 3.981072E+03;
  eg30[ 746] = 4.168694E+03;
  eg30[ 747] = 4.365158E+03;
  eg30[ 748] = 4.570882E+03;
  eg30[ 749] = 4.786301E+03;
  eg30[ 750] = 5.011872E+03;
  eg30[ 751] = 5.248075E+03;
  eg30[ 752] = 5.495409E+03;
  eg30[ 753] = 5.754399E+03;
  eg30[ 754] = 6.025596E+03;
  eg30[ 755] = 6.309573E+03;
  eg30[ 756] = 6.606934E+03;
  eg30[ 757] = 6.918310E+03;
  eg30[ 758] = 7.244360E+03;
  eg30[ 759] = 7.585776E+03;
  eg30[ 760] = 7.943282E+03;
  eg30[ 761] = 8.317638E+03;
  eg30[ 762] = 8.709636E+03;
  eg30[ 763] = 9.120108E+03;
  eg30[ 764] = 9.549926E+03;
  eg30[ 765] = 1.000000E+04;
  eg30[ 766] = 1.047129E+04;
  eg30[ 767] = 1.096478E+04;
  eg30[ 768] = 1.148154E+04;
  eg30[ 769] = 1.202264E+04;
  eg30[ 770] = 1.258925E+04;
  eg30[ 771] = 1.318257E+04;
  eg30[ 772] = 1.380384E+04;
  eg30[ 773] = 1.445440E+04;
  eg30[ 774] = 1.513561E+04;
  eg30[ 775] = 1.584893E+04;
  eg30[ 776] = 1.659587E+04;
  eg30[ 777] = 1.737801E+04;
  eg30[ 778] = 1.819701E+04;
  eg30[ 779] = 1.905461E+04;
  eg30[ 780] = 1.995262E+04;
  eg30[ 781] = 2.089296E+04;
  eg30[ 782] = 2.187762E+04;
  eg30[ 783] = 2.290868E+04;
  eg30[ 784] = 2.398833E+04;
  eg30[ 785] = 2.511886E+04;
  eg30[ 786] = 2.630268E+04;
  eg30[ 787] = 2.754229E+04;
  eg30[ 788] = 2.884032E+04;
  eg30[ 789] = 3.019952E+04;
  eg30[ 790] = 3.162278E+04;
  eg30[ 791] = 3.311311E+04;
  eg30[ 792] = 3.467369E+04;
  eg30[ 793] = 3.630781E+04;
  eg30[ 794] = 3.801894E+04;
  eg30[ 795] = 3.981072E+04;
  eg30[ 796] = 4.168694E+04;
  eg30[ 797] = 4.365158E+04;
  eg30[ 798] = 4.570882E+04;
  eg30[ 799] = 4.786301E+04;
  eg30[ 800] = 5.011872E+04;
  eg30[ 801] = 5.248075E+04;
  eg30[ 802] = 5.495409E+04;
  eg30[ 803] = 5.754399E+04;
  eg30[ 804] = 6.025596E+04;
  eg30[ 805] = 6.309573E+04;
  eg30[ 806] = 6.606934E+04;
  eg30[ 807] = 6.918310E+04;
  eg30[ 808] = 7.244360E+04;
  eg30[ 809] = 7.585776E+04;
  eg30[ 810] = 7.943282E+04;
  eg30[ 811] = 8.317638E+04;
  eg30[ 812] = 8.709636E+04;
  eg30[ 813] = 9.120108E+04;
  eg30[ 814] = 9.549926E+04;
  eg30[ 815] = 1.000000E+05;
  eg30[ 816] = 1.047129E+05;
  eg30[ 817] = 1.096478E+05;
  eg30[ 818] = 1.148154E+05;
  eg30[ 819] = 1.202264E+05;
  eg30[ 820] = 1.258925E+05;
  eg30[ 821] = 1.318257E+05;
  eg30[ 822] = 1.380384E+05;
  eg30[ 823] = 1.445440E+05;
  eg30[ 824] = 1.513561E+05;
  eg30[ 825] = 1.584893E+05;
  eg30[ 826] = 1.659587E+05;
  eg30[ 827] = 1.737801E+05;
  eg30[ 828] = 1.819701E+05;
  eg30[ 829] = 1.905461E+05;
  eg30[ 830] = 1.995262E+05;
  eg30[ 831] = 2.089296E+05;
  eg30[ 832] = 2.187762E+05;
  eg30[ 833] = 2.290868E+05;
  eg30[ 834] = 2.398833E+05;
  eg30[ 835] = 2.511886E+05;
  eg30[ 836] = 2.630268E+05;
  eg30[ 837] = 2.754229E+05;
  eg30[ 838] = 2.884032E+05;
  eg30[ 839] = 3.019952E+05;
  eg30[ 840] = 3.162278E+05;
  eg30[ 841] = 3.311311E+05;
  eg30[ 842] = 3.467369E+05;
  eg30[ 843] = 3.630781E+05;
  eg30[ 844] = 3.801894E+05;
  eg30[ 845] = 3.981072E+05;
  eg30[ 846] = 4.168694E+05;
  eg30[ 847] = 4.365158E+05;
  eg30[ 848] = 4.570882E+05;
  eg30[ 849] = 4.786301E+05;
  eg30[ 850] = 5.011872E+05;
  eg30[ 851] = 5.248075E+05;
  eg30[ 852] = 5.495409E+05;
  eg30[ 853] = 5.754399E+05;
  eg30[ 854] = 6.025596E+05;
  eg30[ 855] = 6.309573E+05;
  eg30[ 856] = 6.606934E+05;
  eg30[ 857] = 6.918310E+05;
  eg30[ 858] = 7.244360E+05;
  eg30[ 859] = 7.585776E+05;
  eg30[ 860] = 7.943282E+05;
  eg30[ 861] = 8.317638E+05;
  eg30[ 862] = 8.709636E+05;
  eg30[ 863] = 9.120108E+05;
  eg30[ 864] = 9.549926E+05;
  eg30[ 865] = 1.000000E+06;
  eg30[ 866] = 1.047129E+06;
  eg30[ 867] = 1.096478E+06;
  eg30[ 868] = 1.148154E+06;
  eg30[ 869] = 1.202264E+06;
  eg30[ 870] = 1.258925E+06;
  eg30[ 871] = 1.318257E+06;
  eg30[ 872] = 1.380384E+06;
  eg30[ 873] = 1.445440E+06;
  eg30[ 874] = 1.513561E+06;
  eg30[ 875] = 1.584893E+06;
  eg30[ 876] = 1.659587E+06;
  eg30[ 877] = 1.737801E+06;
  eg30[ 878] = 1.819701E+06;
  eg30[ 879] = 1.905461E+06;
  eg30[ 880] = 1.995262E+06;
  eg30[ 881] = 2.089296E+06;
  eg30[ 882] = 2.187762E+06;
  eg30[ 883] = 2.290868E+06;
  eg30[ 884] = 2.398833E+06;
  eg30[ 885] = 2.511886E+06;
  eg30[ 886] = 2.630268E+06;
  eg30[ 887] = 2.754229E+06;
  eg30[ 888] = 2.884032E+06;
  eg30[ 889] = 3.019952E+06;
  eg30[ 890] = 3.162278E+06;
  eg30[ 891] = 3.311311E+06;
  eg30[ 892] = 3.467369E+06;
  eg30[ 893] = 3.630781E+06;
  eg30[ 894] = 3.801894E+06;
  eg30[ 895] = 3.981072E+06;
  eg30[ 896] = 4.168694E+06;
  eg30[ 897] = 4.365158E+06;
  eg30[ 898] = 4.570882E+06;
  eg30[ 899] = 4.786301E+06;
  eg30[ 900] = 5.000000E+06;
  eg30[ 901] = 5.200000E+06;
  eg30[ 902] = 5.400000E+06;
  eg30[ 903] = 5.600000E+06;
  eg30[ 904] = 5.800000E+06;
  eg30[ 905] = 6.000000E+06;
  eg30[ 906] = 6.200000E+06;
  eg30[ 907] = 6.400000E+06;
  eg30[ 908] = 6.600000E+06;
  eg30[ 909] = 6.800000E+06;
  eg30[ 910] = 7.000000E+06;
  eg30[ 911] = 7.200000E+06;
  eg30[ 912] = 7.400000E+06;
  eg30[ 913] = 7.600000E+06;
  eg30[ 914] = 7.800000E+06;
  eg30[ 915] = 8.000000E+06;
  eg30[ 916] = 8.200000E+06;
  eg30[ 917] = 8.400000E+06;
  eg30[ 918] = 8.600000E+06;
  eg30[ 919] = 8.800000E+06;
  eg30[ 920] = 9.000000E+06;
  eg30[ 921] = 9.200000E+06;
  eg30[ 922] = 9.400000E+06;
  eg30[ 923] = 9.600000E+06;
  eg30[ 924] = 9.800000E+06;
  eg30[ 925] = 1.000000E+07;
  eg30[ 926] = 1.020000E+07;
  eg30[ 927] = 1.040000E+07;
  eg30[ 928] = 1.060000E+07;
  eg30[ 929] = 1.080000E+07;
  eg30[ 930] = 1.100000E+07;
  eg30[ 931] = 1.120000E+07;
  eg30[ 932] = 1.140000E+07;
  eg30[ 933] = 1.160000E+07;
  eg30[ 934] = 1.180000E+07;
  eg30[ 935] = 1.200000E+07;
  eg30[ 936] = 1.220000E+07;
  eg30[ 937] = 1.240000E+07;
  eg30[ 938] = 1.260000E+07;
  eg30[ 939] = 1.280000E+07;
  eg30[ 940] = 1.300000E+07;
  eg30[ 941] = 1.320000E+07;
  eg30[ 942] = 1.340000E+07;
  eg30[ 943] = 1.360000E+07;
  eg30[ 944] = 1.380000E+07;
  eg30[ 945] = 1.400000E+07;
  eg30[ 946] = 1.420000E+07;
  eg30[ 947] = 1.440000E+07;
  eg30[ 948] = 1.460000E+07;
  eg30[ 949] = 1.480000E+07;
  eg30[ 950] = 1.500000E+07;
  eg30[ 951] = 1.520000E+07;
  eg30[ 952] = 1.540000E+07;
  eg30[ 953] = 1.560000E+07;
  eg30[ 954] = 1.580000E+07;
  eg30[ 955] = 1.600000E+07;
  eg30[ 956] = 1.620000E+07;
  eg30[ 957] = 1.640000E+07;
  eg30[ 958] = 1.660000E+07;
  eg30[ 959] = 1.680000E+07;
  eg30[ 960] = 1.700000E+07;
  eg30[ 961] = 1.720000E+07;
  eg30[ 962] = 1.740000E+07;
  eg30[ 963] = 1.760000E+07;
  eg30[ 964] = 1.780000E+07;
  eg30[ 965] = 1.800000E+07;
  eg30[ 966] = 1.820000E+07;
  eg30[ 967] = 1.840000E+07;
  eg30[ 968] = 1.860000E+07;
  eg30[ 969] = 1.880000E+07;
  eg30[ 970] = 1.900000E+07;
  eg30[ 971] = 1.920000E+07;
  eg30[ 972] = 1.940000E+07;
  eg30[ 973] = 1.960000E+07;
  eg30[ 974] = 1.980000E+07;
  eg30[ 975] = 2.000000E+07;
  eg30[ 976] = 2.020000E+07;
  eg30[ 977] = 2.040000E+07;
  eg30[ 978] = 2.060000E+07;
  eg30[ 979] = 2.080000E+07;
  eg30[ 980] = 2.100000E+07;
  eg30[ 981] = 2.120000E+07;
  eg30[ 982] = 2.140000E+07;
  eg30[ 983] = 2.160000E+07;
  eg30[ 984] = 2.180000E+07;
  eg30[ 985] = 2.200000E+07;
  eg30[ 986] = 2.220000E+07;
  eg30[ 987] = 2.240000E+07;
  eg30[ 988] = 2.260000E+07;
  eg30[ 989] = 2.280000E+07;
  eg30[ 990] = 2.300000E+07;
  eg30[ 991] = 2.320000E+07;
  eg30[ 992] = 2.340000E+07;
  eg30[ 993] = 2.360000E+07;
  eg30[ 994] = 2.380000E+07;
  eg30[ 995] = 2.400000E+07;
  eg30[ 996] = 2.420000E+07;
  eg30[ 997] = 2.440000E+07;
  eg30[ 998] = 2.460000E+07;
  eg30[ 999] = 2.480000E+07;
  eg30[1000] = 2.500000E+07;
  eg30[1001] = 2.520000E+07;
  eg30[1002] = 2.540000E+07;
  eg30[1003] = 2.560000E+07;
  eg30[1004] = 2.580000E+07;
  eg30[1005] = 2.600000E+07;
  eg30[1006] = 2.620000E+07;
  eg30[1007] = 2.640000E+07;
  eg30[1008] = 2.660000E+07;
  eg30[1009] = 2.680000E+07;
  eg30[1010] = 2.700000E+07;
  eg30[1011] = 2.720000E+07;
  eg30[1012] = 2.740000E+07;
  eg30[1013] = 2.760000E+07;
  eg30[1014] = 2.780000E+07;
  eg30[1015] = 2.800000E+07;
  eg30[1016] = 2.820000E+07;
  eg30[1017] = 2.840000E+07;
  eg30[1018] = 2.860000E+07;
  eg30[1019] = 2.880000E+07;
  eg30[1020] = 2.900000E+07;
  eg30[1021] = 2.920000E+07;
  eg30[1022] = 2.940000E+07;
  eg30[1023] = 2.960000E+07;
  eg30[1024] = 2.980000E+07;
  eg30[1025] = 3.000000E+07;

  return eg30;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg31()
{
  vector<Real> eg31;
  eg31.resize(1068);
  eg31[   0] = 1.000000E-05;
  eg31[   1] = 1.047129E-05;
  eg31[   2] = 1.096478E-05;
  eg31[   3] = 1.148154E-05;
  eg31[   4] = 1.202264E-05;
  eg31[   5] = 1.258925E-05;
  eg31[   6] = 1.318257E-05;
  eg31[   7] = 1.380384E-05;
  eg31[   8] = 1.445440E-05;
  eg31[   9] = 1.513561E-05;
  eg31[  10] = 1.584893E-05;
  eg31[  11] = 1.659587E-05;
  eg31[  12] = 1.737801E-05;
  eg31[  13] = 1.819701E-05;
  eg31[  14] = 1.905461E-05;
  eg31[  15] = 1.995262E-05;
  eg31[  16] = 2.089296E-05;
  eg31[  17] = 2.187762E-05;
  eg31[  18] = 2.290868E-05;
  eg31[  19] = 2.398833E-05;
  eg31[  20] = 2.511886E-05;
  eg31[  21] = 2.630268E-05;
  eg31[  22] = 2.754229E-05;
  eg31[  23] = 2.884032E-05;
  eg31[  24] = 3.019952E-05;
  eg31[  25] = 3.162278E-05;
  eg31[  26] = 3.311311E-05;
  eg31[  27] = 3.467369E-05;
  eg31[  28] = 3.630781E-05;
  eg31[  29] = 3.801894E-05;
  eg31[  30] = 3.981072E-05;
  eg31[  31] = 4.168694E-05;
  eg31[  32] = 4.365158E-05;
  eg31[  33] = 4.570882E-05;
  eg31[  34] = 4.786301E-05;
  eg31[  35] = 5.011872E-05;
  eg31[  36] = 5.248075E-05;
  eg31[  37] = 5.495409E-05;
  eg31[  38] = 5.754399E-05;
  eg31[  39] = 6.025596E-05;
  eg31[  40] = 6.309573E-05;
  eg31[  41] = 6.606934E-05;
  eg31[  42] = 6.918310E-05;
  eg31[  43] = 7.244360E-05;
  eg31[  44] = 7.585776E-05;
  eg31[  45] = 7.943282E-05;
  eg31[  46] = 8.317638E-05;
  eg31[  47] = 8.709636E-05;
  eg31[  48] = 9.120108E-05;
  eg31[  49] = 9.549926E-05;
  eg31[  50] = 1.000000E-04;
  eg31[  51] = 1.047129E-04;
  eg31[  52] = 1.096478E-04;
  eg31[  53] = 1.148154E-04;
  eg31[  54] = 1.202264E-04;
  eg31[  55] = 1.258925E-04;
  eg31[  56] = 1.318257E-04;
  eg31[  57] = 1.380384E-04;
  eg31[  58] = 1.445440E-04;
  eg31[  59] = 1.513561E-04;
  eg31[  60] = 1.584893E-04;
  eg31[  61] = 1.659587E-04;
  eg31[  62] = 1.737801E-04;
  eg31[  63] = 1.819701E-04;
  eg31[  64] = 1.905461E-04;
  eg31[  65] = 1.995262E-04;
  eg31[  66] = 2.089296E-04;
  eg31[  67] = 2.187762E-04;
  eg31[  68] = 2.290868E-04;
  eg31[  69] = 2.398833E-04;
  eg31[  70] = 2.511886E-04;
  eg31[  71] = 2.630268E-04;
  eg31[  72] = 2.754229E-04;
  eg31[  73] = 2.884032E-04;
  eg31[  74] = 3.019952E-04;
  eg31[  75] = 3.162278E-04;
  eg31[  76] = 3.311311E-04;
  eg31[  77] = 3.467369E-04;
  eg31[  78] = 3.630781E-04;
  eg31[  79] = 3.801894E-04;
  eg31[  80] = 3.981072E-04;
  eg31[  81] = 4.168694E-04;
  eg31[  82] = 4.365158E-04;
  eg31[  83] = 4.570882E-04;
  eg31[  84] = 4.786301E-04;
  eg31[  85] = 5.011872E-04;
  eg31[  86] = 5.248075E-04;
  eg31[  87] = 5.495409E-04;
  eg31[  88] = 5.754399E-04;
  eg31[  89] = 6.025596E-04;
  eg31[  90] = 6.309573E-04;
  eg31[  91] = 6.606934E-04;
  eg31[  92] = 6.918310E-04;
  eg31[  93] = 7.244360E-04;
  eg31[  94] = 7.585776E-04;
  eg31[  95] = 7.943282E-04;
  eg31[  96] = 8.317638E-04;
  eg31[  97] = 8.709636E-04;
  eg31[  98] = 9.120108E-04;
  eg31[  99] = 9.549926E-04;
  eg31[ 100] = 1.000000E-03;
  eg31[ 101] = 1.047129E-03;
  eg31[ 102] = 1.096478E-03;
  eg31[ 103] = 1.148154E-03;
  eg31[ 104] = 1.202264E-03;
  eg31[ 105] = 1.258925E-03;
  eg31[ 106] = 1.318257E-03;
  eg31[ 107] = 1.380384E-03;
  eg31[ 108] = 1.445440E-03;
  eg31[ 109] = 1.513561E-03;
  eg31[ 110] = 1.584893E-03;
  eg31[ 111] = 1.659587E-03;
  eg31[ 112] = 1.737801E-03;
  eg31[ 113] = 1.819701E-03;
  eg31[ 114] = 1.905461E-03;
  eg31[ 115] = 1.995262E-03;
  eg31[ 116] = 2.089296E-03;
  eg31[ 117] = 2.187762E-03;
  eg31[ 118] = 2.290868E-03;
  eg31[ 119] = 2.398833E-03;
  eg31[ 120] = 2.511886E-03;
  eg31[ 121] = 2.630268E-03;
  eg31[ 122] = 2.754229E-03;
  eg31[ 123] = 2.884032E-03;
  eg31[ 124] = 3.019952E-03;
  eg31[ 125] = 3.162278E-03;
  eg31[ 126] = 3.311311E-03;
  eg31[ 127] = 3.467369E-03;
  eg31[ 128] = 3.630781E-03;
  eg31[ 129] = 3.801894E-03;
  eg31[ 130] = 3.981072E-03;
  eg31[ 131] = 4.168694E-03;
  eg31[ 132] = 4.365158E-03;
  eg31[ 133] = 4.570882E-03;
  eg31[ 134] = 4.786301E-03;
  eg31[ 135] = 5.011872E-03;
  eg31[ 136] = 5.248075E-03;
  eg31[ 137] = 5.495409E-03;
  eg31[ 138] = 5.754399E-03;
  eg31[ 139] = 6.025596E-03;
  eg31[ 140] = 6.309573E-03;
  eg31[ 141] = 6.606934E-03;
  eg31[ 142] = 6.918310E-03;
  eg31[ 143] = 7.244360E-03;
  eg31[ 144] = 7.585776E-03;
  eg31[ 145] = 7.943282E-03;
  eg31[ 146] = 8.317638E-03;
  eg31[ 147] = 8.709636E-03;
  eg31[ 148] = 9.120108E-03;
  eg31[ 149] = 9.549926E-03;
  eg31[ 150] = 1.000000E-02;
  eg31[ 151] = 1.047129E-02;
  eg31[ 152] = 1.096478E-02;
  eg31[ 153] = 1.148154E-02;
  eg31[ 154] = 1.202264E-02;
  eg31[ 155] = 1.258925E-02;
  eg31[ 156] = 1.318257E-02;
  eg31[ 157] = 1.380384E-02;
  eg31[ 158] = 1.445440E-02;
  eg31[ 159] = 1.513561E-02;
  eg31[ 160] = 1.584893E-02;
  eg31[ 161] = 1.659587E-02;
  eg31[ 162] = 1.737801E-02;
  eg31[ 163] = 1.819701E-02;
  eg31[ 164] = 1.905461E-02;
  eg31[ 165] = 1.995262E-02;
  eg31[ 166] = 2.089296E-02;
  eg31[ 167] = 2.187762E-02;
  eg31[ 168] = 2.290868E-02;
  eg31[ 169] = 2.398833E-02;
  eg31[ 170] = 2.511886E-02;
  eg31[ 171] = 2.630268E-02;
  eg31[ 172] = 2.754229E-02;
  eg31[ 173] = 2.884032E-02;
  eg31[ 174] = 3.019952E-02;
  eg31[ 175] = 3.162278E-02;
  eg31[ 176] = 3.311311E-02;
  eg31[ 177] = 3.467369E-02;
  eg31[ 178] = 3.630781E-02;
  eg31[ 179] = 3.801894E-02;
  eg31[ 180] = 3.981072E-02;
  eg31[ 181] = 4.168694E-02;
  eg31[ 182] = 4.365158E-02;
  eg31[ 183] = 4.570882E-02;
  eg31[ 184] = 4.786301E-02;
  eg31[ 185] = 5.011872E-02;
  eg31[ 186] = 5.248075E-02;
  eg31[ 187] = 5.495409E-02;
  eg31[ 188] = 5.754399E-02;
  eg31[ 189] = 6.025596E-02;
  eg31[ 190] = 6.309573E-02;
  eg31[ 191] = 6.606934E-02;
  eg31[ 192] = 6.918310E-02;
  eg31[ 193] = 7.244360E-02;
  eg31[ 194] = 7.585776E-02;
  eg31[ 195] = 7.943282E-02;
  eg31[ 196] = 8.317638E-02;
  eg31[ 197] = 8.709636E-02;
  eg31[ 198] = 9.120108E-02;
  eg31[ 199] = 9.549926E-02;
  eg31[ 200] = 1.000000E-01;
  eg31[ 201] = 1.047129E-01;
  eg31[ 202] = 1.096478E-01;
  eg31[ 203] = 1.148154E-01;
  eg31[ 204] = 1.202264E-01;
  eg31[ 205] = 1.258925E-01;
  eg31[ 206] = 1.318257E-01;
  eg31[ 207] = 1.380384E-01;
  eg31[ 208] = 1.445440E-01;
  eg31[ 209] = 1.513561E-01;
  eg31[ 210] = 1.584893E-01;
  eg31[ 211] = 1.659587E-01;
  eg31[ 212] = 1.737801E-01;
  eg31[ 213] = 1.819701E-01;
  eg31[ 214] = 1.905461E-01;
  eg31[ 215] = 1.995262E-01;
  eg31[ 216] = 2.089296E-01;
  eg31[ 217] = 2.187762E-01;
  eg31[ 218] = 2.290868E-01;
  eg31[ 219] = 2.398833E-01;
  eg31[ 220] = 2.511886E-01;
  eg31[ 221] = 2.630268E-01;
  eg31[ 222] = 2.754229E-01;
  eg31[ 223] = 2.884032E-01;
  eg31[ 224] = 3.019952E-01;
  eg31[ 225] = 3.162278E-01;
  eg31[ 226] = 3.311311E-01;
  eg31[ 227] = 3.467369E-01;
  eg31[ 228] = 3.630781E-01;
  eg31[ 229] = 3.801894E-01;
  eg31[ 230] = 3.981072E-01;
  eg31[ 231] = 4.168694E-01;
  eg31[ 232] = 4.365158E-01;
  eg31[ 233] = 4.570882E-01;
  eg31[ 234] = 4.786301E-01;
  eg31[ 235] = 5.011872E-01;
  eg31[ 236] = 5.248075E-01;
  eg31[ 237] = 5.500000E-01;
  eg31[ 238] = 5.750000E-01;
  eg31[ 239] = 6.000000E-01;
  eg31[ 240] = 6.250000E-01;
  eg31[ 241] = 6.500000E-01;
  eg31[ 242] = 6.750000E-01;
  eg31[ 243] = 7.000000E-01;
  eg31[ 244] = 7.250000E-01;
  eg31[ 245] = 7.500000E-01;
  eg31[ 246] = 7.750000E-01;
  eg31[ 247] = 8.000000E-01;
  eg31[ 248] = 8.250000E-01;
  eg31[ 249] = 8.500000E-01;
  eg31[ 250] = 8.750000E-01;
  eg31[ 251] = 9.000000E-01;
  eg31[ 252] = 9.250000E-01;
  eg31[ 253] = 9.500000E-01;
  eg31[ 254] = 9.750000E-01;
  eg31[ 255] = 1.000000E+00;
  eg31[ 256] = 1.025000E+00;
  eg31[ 257] = 1.050000E+00;
  eg31[ 258] = 1.075000E+00;
  eg31[ 259] = 1.100000E+00;
  eg31[ 260] = 1.125000E+00;
  eg31[ 261] = 1.150000E+00;
  eg31[ 262] = 1.175000E+00;
  eg31[ 263] = 1.200000E+00;
  eg31[ 264] = 1.225000E+00;
  eg31[ 265] = 1.250000E+00;
  eg31[ 266] = 1.275000E+00;
  eg31[ 267] = 1.300000E+00;
  eg31[ 268] = 1.325000E+00;
  eg31[ 269] = 1.350000E+00;
  eg31[ 270] = 1.375000E+00;
  eg31[ 271] = 1.400000E+00;
  eg31[ 272] = 1.425000E+00;
  eg31[ 273] = 1.450000E+00;
  eg31[ 274] = 1.475000E+00;
  eg31[ 275] = 1.500000E+00;
  eg31[ 276] = 1.525000E+00;
  eg31[ 277] = 1.550000E+00;
  eg31[ 278] = 1.575000E+00;
  eg31[ 279] = 1.600000E+00;
  eg31[ 280] = 1.625000E+00;
  eg31[ 281] = 1.650000E+00;
  eg31[ 282] = 1.675000E+00;
  eg31[ 283] = 1.700000E+00;
  eg31[ 284] = 1.725000E+00;
  eg31[ 285] = 1.750000E+00;
  eg31[ 286] = 1.775000E+00;
  eg31[ 287] = 1.800000E+00;
  eg31[ 288] = 1.825000E+00;
  eg31[ 289] = 1.850000E+00;
  eg31[ 290] = 1.875000E+00;
  eg31[ 291] = 1.900000E+00;
  eg31[ 292] = 1.925000E+00;
  eg31[ 293] = 1.950000E+00;
  eg31[ 294] = 1.975000E+00;
  eg31[ 295] = 2.000000E+00;
  eg31[ 296] = 2.025000E+00;
  eg31[ 297] = 2.050000E+00;
  eg31[ 298] = 2.075000E+00;
  eg31[ 299] = 2.100000E+00;
  eg31[ 300] = 2.125000E+00;
  eg31[ 301] = 2.150000E+00;
  eg31[ 302] = 2.175000E+00;
  eg31[ 303] = 2.200000E+00;
  eg31[ 304] = 2.225000E+00;
  eg31[ 305] = 2.250000E+00;
  eg31[ 306] = 2.275000E+00;
  eg31[ 307] = 2.300000E+00;
  eg31[ 308] = 2.325000E+00;
  eg31[ 309] = 2.350000E+00;
  eg31[ 310] = 2.375000E+00;
  eg31[ 311] = 2.400000E+00;
  eg31[ 312] = 2.425000E+00;
  eg31[ 313] = 2.450000E+00;
  eg31[ 314] = 2.475000E+00;
  eg31[ 315] = 2.500000E+00;
  eg31[ 316] = 2.525000E+00;
  eg31[ 317] = 2.550000E+00;
  eg31[ 318] = 2.575000E+00;
  eg31[ 319] = 2.600000E+00;
  eg31[ 320] = 2.625000E+00;
  eg31[ 321] = 2.650000E+00;
  eg31[ 322] = 2.675000E+00;
  eg31[ 323] = 2.700000E+00;
  eg31[ 324] = 2.725000E+00;
  eg31[ 325] = 2.750000E+00;
  eg31[ 326] = 2.775000E+00;
  eg31[ 327] = 2.800000E+00;
  eg31[ 328] = 2.825000E+00;
  eg31[ 329] = 2.850000E+00;
  eg31[ 330] = 2.875000E+00;
  eg31[ 331] = 2.900000E+00;
  eg31[ 332] = 2.925000E+00;
  eg31[ 333] = 2.950000E+00;
  eg31[ 334] = 2.975000E+00;
  eg31[ 335] = 3.000000E+00;
  eg31[ 336] = 3.025000E+00;
  eg31[ 337] = 3.050000E+00;
  eg31[ 338] = 3.075000E+00;
  eg31[ 339] = 3.100000E+00;
  eg31[ 340] = 3.125000E+00;
  eg31[ 341] = 3.150000E+00;
  eg31[ 342] = 3.175000E+00;
  eg31[ 343] = 3.200000E+00;
  eg31[ 344] = 3.225000E+00;
  eg31[ 345] = 3.250000E+00;
  eg31[ 346] = 3.275000E+00;
  eg31[ 347] = 3.300000E+00;
  eg31[ 348] = 3.325000E+00;
  eg31[ 349] = 3.350000E+00;
  eg31[ 350] = 3.375000E+00;
  eg31[ 351] = 3.400000E+00;
  eg31[ 352] = 3.425000E+00;
  eg31[ 353] = 3.450000E+00;
  eg31[ 354] = 3.475000E+00;
  eg31[ 355] = 3.500000E+00;
  eg31[ 356] = 3.525000E+00;
  eg31[ 357] = 3.550000E+00;
  eg31[ 358] = 3.575000E+00;
  eg31[ 359] = 3.600000E+00;
  eg31[ 360] = 3.625000E+00;
  eg31[ 361] = 3.650000E+00;
  eg31[ 362] = 3.675000E+00;
  eg31[ 363] = 3.700000E+00;
  eg31[ 364] = 3.725000E+00;
  eg31[ 365] = 3.750000E+00;
  eg31[ 366] = 3.775000E+00;
  eg31[ 367] = 3.800000E+00;
  eg31[ 368] = 3.825000E+00;
  eg31[ 369] = 3.850000E+00;
  eg31[ 370] = 3.875000E+00;
  eg31[ 371] = 3.900000E+00;
  eg31[ 372] = 3.925000E+00;
  eg31[ 373] = 3.950000E+00;
  eg31[ 374] = 3.975000E+00;
  eg31[ 375] = 4.000000E+00;
  eg31[ 376] = 4.025000E+00;
  eg31[ 377] = 4.050000E+00;
  eg31[ 378] = 4.075000E+00;
  eg31[ 379] = 4.100000E+00;
  eg31[ 380] = 4.125000E+00;
  eg31[ 381] = 4.150000E+00;
  eg31[ 382] = 4.175000E+00;
  eg31[ 383] = 4.200000E+00;
  eg31[ 384] = 4.225000E+00;
  eg31[ 385] = 4.250000E+00;
  eg31[ 386] = 4.275000E+00;
  eg31[ 387] = 4.300000E+00;
  eg31[ 388] = 4.325000E+00;
  eg31[ 389] = 4.350000E+00;
  eg31[ 390] = 4.375000E+00;
  eg31[ 391] = 4.400000E+00;
  eg31[ 392] = 4.425000E+00;
  eg31[ 393] = 4.450000E+00;
  eg31[ 394] = 4.475000E+00;
  eg31[ 395] = 4.500000E+00;
  eg31[ 396] = 4.525000E+00;
  eg31[ 397] = 4.550000E+00;
  eg31[ 398] = 4.575000E+00;
  eg31[ 399] = 4.600000E+00;
  eg31[ 400] = 4.625000E+00;
  eg31[ 401] = 4.650000E+00;
  eg31[ 402] = 4.675000E+00;
  eg31[ 403] = 4.700000E+00;
  eg31[ 404] = 4.725000E+00;
  eg31[ 405] = 4.750000E+00;
  eg31[ 406] = 4.775000E+00;
  eg31[ 407] = 4.800000E+00;
  eg31[ 408] = 4.825000E+00;
  eg31[ 409] = 4.850000E+00;
  eg31[ 410] = 4.875000E+00;
  eg31[ 411] = 4.900000E+00;
  eg31[ 412] = 4.925000E+00;
  eg31[ 413] = 4.950000E+00;
  eg31[ 414] = 4.975000E+00;
  eg31[ 415] = 5.000000E+00;
  eg31[ 416] = 5.025000E+00;
  eg31[ 417] = 5.050000E+00;
  eg31[ 418] = 5.075000E+00;
  eg31[ 419] = 5.100000E+00;
  eg31[ 420] = 5.125000E+00;
  eg31[ 421] = 5.150000E+00;
  eg31[ 422] = 5.175000E+00;
  eg31[ 423] = 5.200000E+00;
  eg31[ 424] = 5.225000E+00;
  eg31[ 425] = 5.250000E+00;
  eg31[ 426] = 5.275000E+00;
  eg31[ 427] = 5.300000E+00;
  eg31[ 428] = 5.325000E+00;
  eg31[ 429] = 5.350000E+00;
  eg31[ 430] = 5.375000E+00;
  eg31[ 431] = 5.400000E+00;
  eg31[ 432] = 5.425000E+00;
  eg31[ 433] = 5.450000E+00;
  eg31[ 434] = 5.475000E+00;
  eg31[ 435] = 5.500000E+00;
  eg31[ 436] = 5.525000E+00;
  eg31[ 437] = 5.550000E+00;
  eg31[ 438] = 5.575000E+00;
  eg31[ 439] = 5.600000E+00;
  eg31[ 440] = 5.625000E+00;
  eg31[ 441] = 5.650000E+00;
  eg31[ 442] = 5.675000E+00;
  eg31[ 443] = 5.700000E+00;
  eg31[ 444] = 5.725000E+00;
  eg31[ 445] = 5.750000E+00;
  eg31[ 446] = 5.775000E+00;
  eg31[ 447] = 5.800000E+00;
  eg31[ 448] = 5.825000E+00;
  eg31[ 449] = 5.850000E+00;
  eg31[ 450] = 5.875000E+00;
  eg31[ 451] = 5.900000E+00;
  eg31[ 452] = 5.925000E+00;
  eg31[ 453] = 5.950000E+00;
  eg31[ 454] = 5.975000E+00;
  eg31[ 455] = 6.000000E+00;
  eg31[ 456] = 6.025000E+00;
  eg31[ 457] = 6.050000E+00;
  eg31[ 458] = 6.075000E+00;
  eg31[ 459] = 6.100000E+00;
  eg31[ 460] = 6.125000E+00;
  eg31[ 461] = 6.150000E+00;
  eg31[ 462] = 6.175000E+00;
  eg31[ 463] = 6.200000E+00;
  eg31[ 464] = 6.225000E+00;
  eg31[ 465] = 6.250000E+00;
  eg31[ 466] = 6.275000E+00;
  eg31[ 467] = 6.300000E+00;
  eg31[ 468] = 6.325000E+00;
  eg31[ 469] = 6.350000E+00;
  eg31[ 470] = 6.375000E+00;
  eg31[ 471] = 6.400000E+00;
  eg31[ 472] = 6.425000E+00;
  eg31[ 473] = 6.450000E+00;
  eg31[ 474] = 6.475000E+00;
  eg31[ 475] = 6.500000E+00;
  eg31[ 476] = 6.525000E+00;
  eg31[ 477] = 6.550000E+00;
  eg31[ 478] = 6.575000E+00;
  eg31[ 479] = 6.600000E+00;
  eg31[ 480] = 6.625000E+00;
  eg31[ 481] = 6.650000E+00;
  eg31[ 482] = 6.675000E+00;
  eg31[ 483] = 6.700000E+00;
  eg31[ 484] = 6.725000E+00;
  eg31[ 485] = 6.750000E+00;
  eg31[ 486] = 6.775000E+00;
  eg31[ 487] = 6.800000E+00;
  eg31[ 488] = 6.825000E+00;
  eg31[ 489] = 6.850000E+00;
  eg31[ 490] = 6.875000E+00;
  eg31[ 491] = 6.900000E+00;
  eg31[ 492] = 6.925000E+00;
  eg31[ 493] = 6.950000E+00;
  eg31[ 494] = 6.975000E+00;
  eg31[ 495] = 7.000000E+00;
  eg31[ 496] = 7.025000E+00;
  eg31[ 497] = 7.050000E+00;
  eg31[ 498] = 7.075000E+00;
  eg31[ 499] = 7.100000E+00;
  eg31[ 500] = 7.125000E+00;
  eg31[ 501] = 7.150000E+00;
  eg31[ 502] = 7.175000E+00;
  eg31[ 503] = 7.200000E+00;
  eg31[ 504] = 7.225000E+00;
  eg31[ 505] = 7.250000E+00;
  eg31[ 506] = 7.275000E+00;
  eg31[ 507] = 7.300000E+00;
  eg31[ 508] = 7.325000E+00;
  eg31[ 509] = 7.350000E+00;
  eg31[ 510] = 7.375000E+00;
  eg31[ 511] = 7.400000E+00;
  eg31[ 512] = 7.425000E+00;
  eg31[ 513] = 7.450000E+00;
  eg31[ 514] = 7.475000E+00;
  eg31[ 515] = 7.500000E+00;
  eg31[ 516] = 7.525000E+00;
  eg31[ 517] = 7.550000E+00;
  eg31[ 518] = 7.575000E+00;
  eg31[ 519] = 7.600000E+00;
  eg31[ 520] = 7.625000E+00;
  eg31[ 521] = 7.650000E+00;
  eg31[ 522] = 7.675000E+00;
  eg31[ 523] = 7.700000E+00;
  eg31[ 524] = 7.725000E+00;
  eg31[ 525] = 7.750000E+00;
  eg31[ 526] = 7.775000E+00;
  eg31[ 527] = 7.800000E+00;
  eg31[ 528] = 7.825000E+00;
  eg31[ 529] = 7.850000E+00;
  eg31[ 530] = 7.875000E+00;
  eg31[ 531] = 7.900000E+00;
  eg31[ 532] = 7.925000E+00;
  eg31[ 533] = 7.950000E+00;
  eg31[ 534] = 7.975000E+00;
  eg31[ 535] = 8.000000E+00;
  eg31[ 536] = 8.025000E+00;
  eg31[ 537] = 8.050000E+00;
  eg31[ 538] = 8.075000E+00;
  eg31[ 539] = 8.100000E+00;
  eg31[ 540] = 8.125000E+00;
  eg31[ 541] = 8.150000E+00;
  eg31[ 542] = 8.175000E+00;
  eg31[ 543] = 8.200000E+00;
  eg31[ 544] = 8.225000E+00;
  eg31[ 545] = 8.250000E+00;
  eg31[ 546] = 8.275000E+00;
  eg31[ 547] = 8.300000E+00;
  eg31[ 548] = 8.325000E+00;
  eg31[ 549] = 8.350000E+00;
  eg31[ 550] = 8.375000E+00;
  eg31[ 551] = 8.400000E+00;
  eg31[ 552] = 8.425000E+00;
  eg31[ 553] = 8.450000E+00;
  eg31[ 554] = 8.475000E+00;
  eg31[ 555] = 8.500000E+00;
  eg31[ 556] = 8.525000E+00;
  eg31[ 557] = 8.550000E+00;
  eg31[ 558] = 8.575000E+00;
  eg31[ 559] = 8.600000E+00;
  eg31[ 560] = 8.625000E+00;
  eg31[ 561] = 8.650000E+00;
  eg31[ 562] = 8.675000E+00;
  eg31[ 563] = 8.700000E+00;
  eg31[ 564] = 8.725000E+00;
  eg31[ 565] = 8.750000E+00;
  eg31[ 566] = 8.775000E+00;
  eg31[ 567] = 8.800000E+00;
  eg31[ 568] = 8.825000E+00;
  eg31[ 569] = 8.850000E+00;
  eg31[ 570] = 8.875000E+00;
  eg31[ 571] = 8.900000E+00;
  eg31[ 572] = 8.925000E+00;
  eg31[ 573] = 8.950000E+00;
  eg31[ 574] = 8.975000E+00;
  eg31[ 575] = 9.000000E+00;
  eg31[ 576] = 9.025000E+00;
  eg31[ 577] = 9.050000E+00;
  eg31[ 578] = 9.075000E+00;
  eg31[ 579] = 9.100000E+00;
  eg31[ 580] = 9.125000E+00;
  eg31[ 581] = 9.150000E+00;
  eg31[ 582] = 9.175000E+00;
  eg31[ 583] = 9.200000E+00;
  eg31[ 584] = 9.225000E+00;
  eg31[ 585] = 9.250000E+00;
  eg31[ 586] = 9.275000E+00;
  eg31[ 587] = 9.300000E+00;
  eg31[ 588] = 9.325000E+00;
  eg31[ 589] = 9.350000E+00;
  eg31[ 590] = 9.375000E+00;
  eg31[ 591] = 9.400000E+00;
  eg31[ 592] = 9.425000E+00;
  eg31[ 593] = 9.450000E+00;
  eg31[ 594] = 9.475000E+00;
  eg31[ 595] = 9.500000E+00;
  eg31[ 596] = 9.525000E+00;
  eg31[ 597] = 9.550000E+00;
  eg31[ 598] = 9.575000E+00;
  eg31[ 599] = 9.600000E+00;
  eg31[ 600] = 9.625000E+00;
  eg31[ 601] = 9.650000E+00;
  eg31[ 602] = 9.675000E+00;
  eg31[ 603] = 9.700000E+00;
  eg31[ 604] = 9.725000E+00;
  eg31[ 605] = 9.750000E+00;
  eg31[ 606] = 9.775000E+00;
  eg31[ 607] = 9.800000E+00;
  eg31[ 608] = 9.825000E+00;
  eg31[ 609] = 9.850000E+00;
  eg31[ 610] = 9.875000E+00;
  eg31[ 611] = 9.900000E+00;
  eg31[ 612] = 9.925000E+00;
  eg31[ 613] = 9.950000E+00;
  eg31[ 614] = 9.975000E+00;
  eg31[ 615] = 1.000000E+01;
  eg31[ 616] = 1.047129E+01;
  eg31[ 617] = 1.096478E+01;
  eg31[ 618] = 1.148154E+01;
  eg31[ 619] = 1.202264E+01;
  eg31[ 620] = 1.258925E+01;
  eg31[ 621] = 1.318257E+01;
  eg31[ 622] = 1.380384E+01;
  eg31[ 623] = 1.445440E+01;
  eg31[ 624] = 1.513561E+01;
  eg31[ 625] = 1.584893E+01;
  eg31[ 626] = 1.659587E+01;
  eg31[ 627] = 1.737801E+01;
  eg31[ 628] = 1.819701E+01;
  eg31[ 629] = 1.905461E+01;
  eg31[ 630] = 1.995262E+01;
  eg31[ 631] = 2.089296E+01;
  eg31[ 632] = 2.187762E+01;
  eg31[ 633] = 2.290868E+01;
  eg31[ 634] = 2.398833E+01;
  eg31[ 635] = 2.511886E+01;
  eg31[ 636] = 2.630268E+01;
  eg31[ 637] = 2.754229E+01;
  eg31[ 638] = 2.884032E+01;
  eg31[ 639] = 3.019952E+01;
  eg31[ 640] = 3.162278E+01;
  eg31[ 641] = 3.311311E+01;
  eg31[ 642] = 3.467369E+01;
  eg31[ 643] = 3.630781E+01;
  eg31[ 644] = 3.801894E+01;
  eg31[ 645] = 3.981072E+01;
  eg31[ 646] = 4.168694E+01;
  eg31[ 647] = 4.365158E+01;
  eg31[ 648] = 4.570882E+01;
  eg31[ 649] = 4.786301E+01;
  eg31[ 650] = 5.011872E+01;
  eg31[ 651] = 5.248075E+01;
  eg31[ 652] = 5.495409E+01;
  eg31[ 653] = 5.754399E+01;
  eg31[ 654] = 6.025596E+01;
  eg31[ 655] = 6.309573E+01;
  eg31[ 656] = 6.606934E+01;
  eg31[ 657] = 6.918310E+01;
  eg31[ 658] = 7.244360E+01;
  eg31[ 659] = 7.585776E+01;
  eg31[ 660] = 7.943282E+01;
  eg31[ 661] = 8.317638E+01;
  eg31[ 662] = 8.709636E+01;
  eg31[ 663] = 9.120108E+01;
  eg31[ 664] = 9.549926E+01;
  eg31[ 665] = 1.000000E+02;
  eg31[ 666] = 1.047129E+02;
  eg31[ 667] = 1.096478E+02;
  eg31[ 668] = 1.148154E+02;
  eg31[ 669] = 1.202264E+02;
  eg31[ 670] = 1.258925E+02;
  eg31[ 671] = 1.318257E+02;
  eg31[ 672] = 1.380384E+02;
  eg31[ 673] = 1.445440E+02;
  eg31[ 674] = 1.513561E+02;
  eg31[ 675] = 1.584893E+02;
  eg31[ 676] = 1.659587E+02;
  eg31[ 677] = 1.737801E+02;
  eg31[ 678] = 1.819701E+02;
  eg31[ 679] = 1.905461E+02;
  eg31[ 680] = 1.995262E+02;
  eg31[ 681] = 2.089296E+02;
  eg31[ 682] = 2.187762E+02;
  eg31[ 683] = 2.290868E+02;
  eg31[ 684] = 2.398833E+02;
  eg31[ 685] = 2.511886E+02;
  eg31[ 686] = 2.630268E+02;
  eg31[ 687] = 2.754229E+02;
  eg31[ 688] = 2.884032E+02;
  eg31[ 689] = 3.019952E+02;
  eg31[ 690] = 3.162278E+02;
  eg31[ 691] = 3.311311E+02;
  eg31[ 692] = 3.467369E+02;
  eg31[ 693] = 3.630781E+02;
  eg31[ 694] = 3.801894E+02;
  eg31[ 695] = 3.981072E+02;
  eg31[ 696] = 4.168694E+02;
  eg31[ 697] = 4.365158E+02;
  eg31[ 698] = 4.570882E+02;
  eg31[ 699] = 4.786301E+02;
  eg31[ 700] = 5.011872E+02;
  eg31[ 701] = 5.248075E+02;
  eg31[ 702] = 5.495409E+02;
  eg31[ 703] = 5.754399E+02;
  eg31[ 704] = 6.025596E+02;
  eg31[ 705] = 6.309573E+02;
  eg31[ 706] = 6.606934E+02;
  eg31[ 707] = 6.918310E+02;
  eg31[ 708] = 7.244360E+02;
  eg31[ 709] = 7.585776E+02;
  eg31[ 710] = 7.943282E+02;
  eg31[ 711] = 8.317638E+02;
  eg31[ 712] = 8.709636E+02;
  eg31[ 713] = 9.120108E+02;
  eg31[ 714] = 9.549926E+02;
  eg31[ 715] = 1.000000E+03;
  eg31[ 716] = 1.047129E+03;
  eg31[ 717] = 1.096478E+03;
  eg31[ 718] = 1.148154E+03;
  eg31[ 719] = 1.202264E+03;
  eg31[ 720] = 1.258925E+03;
  eg31[ 721] = 1.318257E+03;
  eg31[ 722] = 1.380384E+03;
  eg31[ 723] = 1.445440E+03;
  eg31[ 724] = 1.513561E+03;
  eg31[ 725] = 1.584893E+03;
  eg31[ 726] = 1.659587E+03;
  eg31[ 727] = 1.737801E+03;
  eg31[ 728] = 1.819701E+03;
  eg31[ 729] = 1.905461E+03;
  eg31[ 730] = 1.995262E+03;
  eg31[ 731] = 2.089296E+03;
  eg31[ 732] = 2.187762E+03;
  eg31[ 733] = 2.290868E+03;
  eg31[ 734] = 2.398833E+03;
  eg31[ 735] = 2.511886E+03;
  eg31[ 736] = 2.630268E+03;
  eg31[ 737] = 2.754229E+03;
  eg31[ 738] = 2.884032E+03;
  eg31[ 739] = 3.019952E+03;
  eg31[ 740] = 3.162278E+03;
  eg31[ 741] = 3.311311E+03;
  eg31[ 742] = 3.467369E+03;
  eg31[ 743] = 3.630781E+03;
  eg31[ 744] = 3.801894E+03;
  eg31[ 745] = 3.981072E+03;
  eg31[ 746] = 4.168694E+03;
  eg31[ 747] = 4.365158E+03;
  eg31[ 748] = 4.570882E+03;
  eg31[ 749] = 4.786301E+03;
  eg31[ 750] = 5.011872E+03;
  eg31[ 751] = 5.248075E+03;
  eg31[ 752] = 5.495409E+03;
  eg31[ 753] = 5.754399E+03;
  eg31[ 754] = 6.025596E+03;
  eg31[ 755] = 6.309573E+03;
  eg31[ 756] = 6.606934E+03;
  eg31[ 757] = 6.918310E+03;
  eg31[ 758] = 7.244360E+03;
  eg31[ 759] = 7.585776E+03;
  eg31[ 760] = 7.943282E+03;
  eg31[ 761] = 8.317638E+03;
  eg31[ 762] = 8.709636E+03;
  eg31[ 763] = 9.120108E+03;
  eg31[ 764] = 9.549926E+03;
  eg31[ 765] = 1.000000E+04;
  eg31[ 766] = 1.047129E+04;
  eg31[ 767] = 1.096478E+04;
  eg31[ 768] = 1.148154E+04;
  eg31[ 769] = 1.202264E+04;
  eg31[ 770] = 1.258925E+04;
  eg31[ 771] = 1.318257E+04;
  eg31[ 772] = 1.380384E+04;
  eg31[ 773] = 1.445440E+04;
  eg31[ 774] = 1.513561E+04;
  eg31[ 775] = 1.584893E+04;
  eg31[ 776] = 1.659587E+04;
  eg31[ 777] = 1.737801E+04;
  eg31[ 778] = 1.819701E+04;
  eg31[ 779] = 1.905461E+04;
  eg31[ 780] = 1.995262E+04;
  eg31[ 781] = 2.089296E+04;
  eg31[ 782] = 2.187762E+04;
  eg31[ 783] = 2.290868E+04;
  eg31[ 784] = 2.398833E+04;
  eg31[ 785] = 2.511886E+04;
  eg31[ 786] = 2.630268E+04;
  eg31[ 787] = 2.754229E+04;
  eg31[ 788] = 2.884032E+04;
  eg31[ 789] = 3.019952E+04;
  eg31[ 790] = 3.162278E+04;
  eg31[ 791] = 3.311311E+04;
  eg31[ 792] = 3.467369E+04;
  eg31[ 793] = 3.630781E+04;
  eg31[ 794] = 3.801894E+04;
  eg31[ 795] = 3.981072E+04;
  eg31[ 796] = 4.168694E+04;
  eg31[ 797] = 4.365158E+04;
  eg31[ 798] = 4.570882E+04;
  eg31[ 799] = 4.786301E+04;
  eg31[ 800] = 5.011872E+04;
  eg31[ 801] = 5.248075E+04;
  eg31[ 802] = 5.495409E+04;
  eg31[ 803] = 5.754399E+04;
  eg31[ 804] = 6.025596E+04;
  eg31[ 805] = 6.309573E+04;
  eg31[ 806] = 6.606934E+04;
  eg31[ 807] = 6.918310E+04;
  eg31[ 808] = 7.244360E+04;
  eg31[ 809] = 7.585776E+04;
  eg31[ 810] = 7.943282E+04;
  eg31[ 811] = 8.317638E+04;
  eg31[ 812] = 8.709636E+04;
  eg31[ 813] = 9.120108E+04;
  eg31[ 814] = 9.549926E+04;
  eg31[ 815] = 1.000000E+05;
  eg31[ 816] = 1.047129E+05;
  eg31[ 817] = 1.096478E+05;
  eg31[ 818] = 1.148154E+05;
  eg31[ 819] = 1.202264E+05;
  eg31[ 820] = 1.258925E+05;
  eg31[ 821] = 1.318257E+05;
  eg31[ 822] = 1.380384E+05;
  eg31[ 823] = 1.445440E+05;
  eg31[ 824] = 1.513561E+05;
  eg31[ 825] = 1.584893E+05;
  eg31[ 826] = 1.659587E+05;
  eg31[ 827] = 1.737801E+05;
  eg31[ 828] = 1.819701E+05;
  eg31[ 829] = 1.905461E+05;
  eg31[ 830] = 1.995262E+05;
  eg31[ 831] = 2.089296E+05;
  eg31[ 832] = 2.187762E+05;
  eg31[ 833] = 2.290868E+05;
  eg31[ 834] = 2.398833E+05;
  eg31[ 835] = 2.511886E+05;
  eg31[ 836] = 2.630268E+05;
  eg31[ 837] = 2.754229E+05;
  eg31[ 838] = 2.884032E+05;
  eg31[ 839] = 3.019952E+05;
  eg31[ 840] = 3.162278E+05;
  eg31[ 841] = 3.311311E+05;
  eg31[ 842] = 3.467369E+05;
  eg31[ 843] = 3.630781E+05;
  eg31[ 844] = 3.801894E+05;
  eg31[ 845] = 3.981072E+05;
  eg31[ 846] = 4.168694E+05;
  eg31[ 847] = 4.365158E+05;
  eg31[ 848] = 4.570882E+05;
  eg31[ 849] = 4.786301E+05;
  eg31[ 850] = 5.011872E+05;
  eg31[ 851] = 5.248075E+05;
  eg31[ 852] = 5.495409E+05;
  eg31[ 853] = 5.754399E+05;
  eg31[ 854] = 6.025596E+05;
  eg31[ 855] = 6.309573E+05;
  eg31[ 856] = 6.606934E+05;
  eg31[ 857] = 6.918310E+05;
  eg31[ 858] = 7.244360E+05;
  eg31[ 859] = 7.585776E+05;
  eg31[ 860] = 7.943282E+05;
  eg31[ 861] = 8.317638E+05;
  eg31[ 862] = 8.709636E+05;
  eg31[ 863] = 9.120108E+05;
  eg31[ 864] = 9.549926E+05;
  eg31[ 865] = 1.000000E+06;
  eg31[ 866] = 1.047129E+06;
  eg31[ 867] = 1.096478E+06;
  eg31[ 868] = 1.148154E+06;
  eg31[ 869] = 1.202264E+06;
  eg31[ 870] = 1.258925E+06;
  eg31[ 871] = 1.318257E+06;
  eg31[ 872] = 1.380384E+06;
  eg31[ 873] = 1.445440E+06;
  eg31[ 874] = 1.513561E+06;
  eg31[ 875] = 1.584893E+06;
  eg31[ 876] = 1.659587E+06;
  eg31[ 877] = 1.737801E+06;
  eg31[ 878] = 1.819701E+06;
  eg31[ 879] = 1.905461E+06;
  eg31[ 880] = 1.995262E+06;
  eg31[ 881] = 2.089296E+06;
  eg31[ 882] = 2.187762E+06;
  eg31[ 883] = 2.290868E+06;
  eg31[ 884] = 2.398833E+06;
  eg31[ 885] = 2.511886E+06;
  eg31[ 886] = 2.630268E+06;
  eg31[ 887] = 2.754229E+06;
  eg31[ 888] = 2.884032E+06;
  eg31[ 889] = 3.019952E+06;
  eg31[ 890] = 3.162278E+06;
  eg31[ 891] = 3.311311E+06;
  eg31[ 892] = 3.467369E+06;
  eg31[ 893] = 3.630781E+06;
  eg31[ 894] = 3.801894E+06;
  eg31[ 895] = 3.981072E+06;
  eg31[ 896] = 4.168694E+06;
  eg31[ 897] = 4.365158E+06;
  eg31[ 898] = 4.570882E+06;
  eg31[ 899] = 4.786301E+06;
  eg31[ 900] = 5.000000E+06;
  eg31[ 901] = 5.200000E+06;
  eg31[ 902] = 5.400000E+06;
  eg31[ 903] = 5.600000E+06;
  eg31[ 904] = 5.800000E+06;
  eg31[ 905] = 6.000000E+06;
  eg31[ 906] = 6.200000E+06;
  eg31[ 907] = 6.400000E+06;
  eg31[ 908] = 6.600000E+06;
  eg31[ 909] = 6.800000E+06;
  eg31[ 910] = 7.000000E+06;
  eg31[ 911] = 7.200000E+06;
  eg31[ 912] = 7.400000E+06;
  eg31[ 913] = 7.600000E+06;
  eg31[ 914] = 7.800000E+06;
  eg31[ 915] = 8.000000E+06;
  eg31[ 916] = 8.200000E+06;
  eg31[ 917] = 8.400000E+06;
  eg31[ 918] = 8.600000E+06;
  eg31[ 919] = 8.800000E+06;
  eg31[ 920] = 9.000000E+06;
  eg31[ 921] = 9.200000E+06;
  eg31[ 922] = 9.400000E+06;
  eg31[ 923] = 9.600000E+06;
  eg31[ 924] = 9.800000E+06;
  eg31[ 925] = 1.000000E+07;
  eg31[ 926] = 1.020000E+07;
  eg31[ 927] = 1.040000E+07;
  eg31[ 928] = 1.060000E+07;
  eg31[ 929] = 1.080000E+07;
  eg31[ 930] = 1.100000E+07;
  eg31[ 931] = 1.120000E+07;
  eg31[ 932] = 1.140000E+07;
  eg31[ 933] = 1.160000E+07;
  eg31[ 934] = 1.180000E+07;
  eg31[ 935] = 1.200000E+07;
  eg31[ 936] = 1.220000E+07;
  eg31[ 937] = 1.240000E+07;
  eg31[ 938] = 1.260000E+07;
  eg31[ 939] = 1.280000E+07;
  eg31[ 940] = 1.300000E+07;
  eg31[ 941] = 1.320000E+07;
  eg31[ 942] = 1.340000E+07;
  eg31[ 943] = 1.360000E+07;
  eg31[ 944] = 1.380000E+07;
  eg31[ 945] = 1.400000E+07;
  eg31[ 946] = 1.420000E+07;
  eg31[ 947] = 1.440000E+07;
  eg31[ 948] = 1.460000E+07;
  eg31[ 949] = 1.480000E+07;
  eg31[ 950] = 1.500000E+07;
  eg31[ 951] = 1.520000E+07;
  eg31[ 952] = 1.540000E+07;
  eg31[ 953] = 1.560000E+07;
  eg31[ 954] = 1.580000E+07;
  eg31[ 955] = 1.600000E+07;
  eg31[ 956] = 1.620000E+07;
  eg31[ 957] = 1.640000E+07;
  eg31[ 958] = 1.660000E+07;
  eg31[ 959] = 1.680000E+07;
  eg31[ 960] = 1.700000E+07;
  eg31[ 961] = 1.720000E+07;
  eg31[ 962] = 1.740000E+07;
  eg31[ 963] = 1.760000E+07;
  eg31[ 964] = 1.780000E+07;
  eg31[ 965] = 1.800000E+07;
  eg31[ 966] = 1.820000E+07;
  eg31[ 967] = 1.840000E+07;
  eg31[ 968] = 1.860000E+07;
  eg31[ 969] = 1.880000E+07;
  eg31[ 970] = 1.900000E+07;
  eg31[ 971] = 1.920000E+07;
  eg31[ 972] = 1.940000E+07;
  eg31[ 973] = 1.960000E+07;
  eg31[ 974] = 1.980000E+07;
  eg31[ 975] = 2.000000E+07;
  eg31[ 976] = 2.020000E+07;
  eg31[ 977] = 2.040000E+07;
  eg31[ 978] = 2.060000E+07;
  eg31[ 979] = 2.080000E+07;
  eg31[ 980] = 2.100000E+07;
  eg31[ 981] = 2.120000E+07;
  eg31[ 982] = 2.140000E+07;
  eg31[ 983] = 2.160000E+07;
  eg31[ 984] = 2.180000E+07;
  eg31[ 985] = 2.200000E+07;
  eg31[ 986] = 2.220000E+07;
  eg31[ 987] = 2.240000E+07;
  eg31[ 988] = 2.260000E+07;
  eg31[ 989] = 2.280000E+07;
  eg31[ 990] = 2.300000E+07;
  eg31[ 991] = 2.320000E+07;
  eg31[ 992] = 2.340000E+07;
  eg31[ 993] = 2.360000E+07;
  eg31[ 994] = 2.380000E+07;
  eg31[ 995] = 2.400000E+07;
  eg31[ 996] = 2.420000E+07;
  eg31[ 997] = 2.440000E+07;
  eg31[ 998] = 2.460000E+07;
  eg31[ 999] = 2.480000E+07;
  eg31[1000] = 2.500000E+07;
  eg31[1001] = 2.520000E+07;
  eg31[1002] = 2.540000E+07;
  eg31[1003] = 2.560000E+07;
  eg31[1004] = 2.580000E+07;
  eg31[1005] = 2.600000E+07;
  eg31[1006] = 2.620000E+07;
  eg31[1007] = 2.640000E+07;
  eg31[1008] = 2.660000E+07;
  eg31[1009] = 2.680000E+07;
  eg31[1010] = 2.700000E+07;
  eg31[1011] = 2.720000E+07;
  eg31[1012] = 2.740000E+07;
  eg31[1013] = 2.760000E+07;
  eg31[1014] = 2.780000E+07;
  eg31[1015] = 2.800000E+07;
  eg31[1016] = 2.820000E+07;
  eg31[1017] = 2.840000E+07;
  eg31[1018] = 2.860000E+07;
  eg31[1019] = 2.880000E+07;
  eg31[1020] = 2.900000E+07;
  eg31[1021] = 2.920000E+07;
  eg31[1022] = 2.940000E+07;
  eg31[1023] = 2.960000E+07;
  eg31[1024] = 2.980000E+07;
  eg31[1025] = 3.000000E+07;
  eg31[1026] = 3.019952E+07;
  eg31[1027] = 3.162278E+07;
  eg31[1028] = 3.311311E+07;
  eg31[1029] = 3.467369E+07;
  eg31[1030] = 3.630781E+07;
  eg31[1031] = 3.801894E+07;
  eg31[1032] = 3.981072E+07;
  eg31[1033] = 4.168694E+07;
  eg31[1034] = 4.365158E+07;
  eg31[1035] = 4.570882E+07;
  eg31[1036] = 4.786301E+07;
  eg31[1037] = 5.011872E+07;
  eg31[1038] = 5.248075E+07;
  eg31[1039] = 5.495409E+07;
  eg31[1040] = 5.754399E+07;
  eg31[1041] = 6.025596E+07;
  eg31[1042] = 6.309573E+07;
  eg31[1043] = 6.606934E+07;
  eg31[1044] = 6.918310E+07;
  eg31[1045] = 7.244360E+07;
  eg31[1046] = 7.585776E+07;
  eg31[1047] = 7.943282E+07;
  eg31[1048] = 8.317638E+07;
  eg31[1049] = 8.709636E+07;
  eg31[1050] = 9.120108E+07;
  eg31[1051] = 9.549926E+07;
  eg31[1052] = 1.000000E+08;
  eg31[1053] = 1.047129E+08;
  eg31[1054] = 1.096478E+08;
  eg31[1055] = 1.148154E+08;
  eg31[1056] = 1.202264E+08;
  eg31[1057] = 1.258925E+08;
  eg31[1058] = 1.318257E+08;
  eg31[1059] = 1.380384E+08;
  eg31[1060] = 1.445440E+08;
  eg31[1061] = 1.513561E+08;
  eg31[1062] = 1.584893E+08;
  eg31[1063] = 1.659587E+08;
  eg31[1064] = 1.737801E+08;
  eg31[1065] = 1.819701E+08;
  eg31[1066] = 1.905461E+08;
  eg31[1067] = 1.995262E+08;

  return eg31;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg32()
{
  vector<Real> eg32;
  eg32.resize(1103);
  eg32[   0] = 1.000000E-05;
  eg32[   1] = 1.047129E-05;
  eg32[   2] = 1.096478E-05;
  eg32[   3] = 1.148154E-05;
  eg32[   4] = 1.202264E-05;
  eg32[   5] = 1.258925E-05;
  eg32[   6] = 1.318257E-05;
  eg32[   7] = 1.380384E-05;
  eg32[   8] = 1.445440E-05;
  eg32[   9] = 1.513561E-05;
  eg32[  10] = 1.584893E-05;
  eg32[  11] = 1.659587E-05;
  eg32[  12] = 1.737801E-05;
  eg32[  13] = 1.819701E-05;
  eg32[  14] = 1.905461E-05;
  eg32[  15] = 1.995262E-05;
  eg32[  16] = 2.089296E-05;
  eg32[  17] = 2.187762E-05;
  eg32[  18] = 2.290868E-05;
  eg32[  19] = 2.398833E-05;
  eg32[  20] = 2.511886E-05;
  eg32[  21] = 2.630268E-05;
  eg32[  22] = 2.754229E-05;
  eg32[  23] = 2.884032E-05;
  eg32[  24] = 3.019952E-05;
  eg32[  25] = 3.162278E-05;
  eg32[  26] = 3.311311E-05;
  eg32[  27] = 3.467369E-05;
  eg32[  28] = 3.630781E-05;
  eg32[  29] = 3.801894E-05;
  eg32[  30] = 3.981072E-05;
  eg32[  31] = 4.168694E-05;
  eg32[  32] = 4.365158E-05;
  eg32[  33] = 4.570882E-05;
  eg32[  34] = 4.786301E-05;
  eg32[  35] = 5.011872E-05;
  eg32[  36] = 5.248075E-05;
  eg32[  37] = 5.495409E-05;
  eg32[  38] = 5.754399E-05;
  eg32[  39] = 6.025596E-05;
  eg32[  40] = 6.309573E-05;
  eg32[  41] = 6.606934E-05;
  eg32[  42] = 6.918310E-05;
  eg32[  43] = 7.244360E-05;
  eg32[  44] = 7.585776E-05;
  eg32[  45] = 7.943282E-05;
  eg32[  46] = 8.317638E-05;
  eg32[  47] = 8.709636E-05;
  eg32[  48] = 9.120108E-05;
  eg32[  49] = 9.549926E-05;
  eg32[  50] = 1.000000E-04;
  eg32[  51] = 1.047129E-04;
  eg32[  52] = 1.096478E-04;
  eg32[  53] = 1.148154E-04;
  eg32[  54] = 1.202264E-04;
  eg32[  55] = 1.258925E-04;
  eg32[  56] = 1.318257E-04;
  eg32[  57] = 1.380384E-04;
  eg32[  58] = 1.445440E-04;
  eg32[  59] = 1.513561E-04;
  eg32[  60] = 1.584893E-04;
  eg32[  61] = 1.659587E-04;
  eg32[  62] = 1.737801E-04;
  eg32[  63] = 1.819701E-04;
  eg32[  64] = 1.905461E-04;
  eg32[  65] = 1.995262E-04;
  eg32[  66] = 2.089296E-04;
  eg32[  67] = 2.187762E-04;
  eg32[  68] = 2.290868E-04;
  eg32[  69] = 2.398833E-04;
  eg32[  70] = 2.511886E-04;
  eg32[  71] = 2.630268E-04;
  eg32[  72] = 2.754229E-04;
  eg32[  73] = 2.884032E-04;
  eg32[  74] = 3.019952E-04;
  eg32[  75] = 3.162278E-04;
  eg32[  76] = 3.311311E-04;
  eg32[  77] = 3.467369E-04;
  eg32[  78] = 3.630781E-04;
  eg32[  79] = 3.801894E-04;
  eg32[  80] = 3.981072E-04;
  eg32[  81] = 4.168694E-04;
  eg32[  82] = 4.365158E-04;
  eg32[  83] = 4.570882E-04;
  eg32[  84] = 4.786301E-04;
  eg32[  85] = 5.011872E-04;
  eg32[  86] = 5.248075E-04;
  eg32[  87] = 5.495409E-04;
  eg32[  88] = 5.754399E-04;
  eg32[  89] = 6.025596E-04;
  eg32[  90] = 6.309573E-04;
  eg32[  91] = 6.606934E-04;
  eg32[  92] = 6.918310E-04;
  eg32[  93] = 7.244360E-04;
  eg32[  94] = 7.585776E-04;
  eg32[  95] = 7.943282E-04;
  eg32[  96] = 8.317638E-04;
  eg32[  97] = 8.709636E-04;
  eg32[  98] = 9.120108E-04;
  eg32[  99] = 9.549926E-04;
  eg32[ 100] = 1.000000E-03;
  eg32[ 101] = 1.047129E-03;
  eg32[ 102] = 1.096478E-03;
  eg32[ 103] = 1.148154E-03;
  eg32[ 104] = 1.202264E-03;
  eg32[ 105] = 1.258925E-03;
  eg32[ 106] = 1.318257E-03;
  eg32[ 107] = 1.380384E-03;
  eg32[ 108] = 1.445440E-03;
  eg32[ 109] = 1.513561E-03;
  eg32[ 110] = 1.584893E-03;
  eg32[ 111] = 1.659587E-03;
  eg32[ 112] = 1.737801E-03;
  eg32[ 113] = 1.819701E-03;
  eg32[ 114] = 1.905461E-03;
  eg32[ 115] = 1.995262E-03;
  eg32[ 116] = 2.089296E-03;
  eg32[ 117] = 2.187762E-03;
  eg32[ 118] = 2.290868E-03;
  eg32[ 119] = 2.398833E-03;
  eg32[ 120] = 2.511886E-03;
  eg32[ 121] = 2.630268E-03;
  eg32[ 122] = 2.754229E-03;
  eg32[ 123] = 2.884032E-03;
  eg32[ 124] = 3.019952E-03;
  eg32[ 125] = 3.162278E-03;
  eg32[ 126] = 3.311311E-03;
  eg32[ 127] = 3.467369E-03;
  eg32[ 128] = 3.630781E-03;
  eg32[ 129] = 3.801894E-03;
  eg32[ 130] = 3.981072E-03;
  eg32[ 131] = 4.168694E-03;
  eg32[ 132] = 4.365158E-03;
  eg32[ 133] = 4.570882E-03;
  eg32[ 134] = 4.786301E-03;
  eg32[ 135] = 5.011872E-03;
  eg32[ 136] = 5.248075E-03;
  eg32[ 137] = 5.495409E-03;
  eg32[ 138] = 5.754399E-03;
  eg32[ 139] = 6.025596E-03;
  eg32[ 140] = 6.309573E-03;
  eg32[ 141] = 6.606934E-03;
  eg32[ 142] = 6.918310E-03;
  eg32[ 143] = 7.244360E-03;
  eg32[ 144] = 7.585776E-03;
  eg32[ 145] = 7.943282E-03;
  eg32[ 146] = 8.317638E-03;
  eg32[ 147] = 8.709636E-03;
  eg32[ 148] = 9.120108E-03;
  eg32[ 149] = 9.549926E-03;
  eg32[ 150] = 1.000000E-02;
  eg32[ 151] = 1.047129E-02;
  eg32[ 152] = 1.096478E-02;
  eg32[ 153] = 1.148154E-02;
  eg32[ 154] = 1.202264E-02;
  eg32[ 155] = 1.258925E-02;
  eg32[ 156] = 1.318257E-02;
  eg32[ 157] = 1.380384E-02;
  eg32[ 158] = 1.445440E-02;
  eg32[ 159] = 1.513561E-02;
  eg32[ 160] = 1.584893E-02;
  eg32[ 161] = 1.659587E-02;
  eg32[ 162] = 1.737801E-02;
  eg32[ 163] = 1.819701E-02;
  eg32[ 164] = 1.905461E-02;
  eg32[ 165] = 1.995262E-02;
  eg32[ 166] = 2.089296E-02;
  eg32[ 167] = 2.187762E-02;
  eg32[ 168] = 2.290868E-02;
  eg32[ 169] = 2.398833E-02;
  eg32[ 170] = 2.511886E-02;
  eg32[ 171] = 2.630268E-02;
  eg32[ 172] = 2.754229E-02;
  eg32[ 173] = 2.884032E-02;
  eg32[ 174] = 3.019952E-02;
  eg32[ 175] = 3.162278E-02;
  eg32[ 176] = 3.311311E-02;
  eg32[ 177] = 3.467369E-02;
  eg32[ 178] = 3.630781E-02;
  eg32[ 179] = 3.801894E-02;
  eg32[ 180] = 3.981072E-02;
  eg32[ 181] = 4.168694E-02;
  eg32[ 182] = 4.365158E-02;
  eg32[ 183] = 4.570882E-02;
  eg32[ 184] = 4.786301E-02;
  eg32[ 185] = 5.011872E-02;
  eg32[ 186] = 5.248075E-02;
  eg32[ 187] = 5.495409E-02;
  eg32[ 188] = 5.754399E-02;
  eg32[ 189] = 6.025596E-02;
  eg32[ 190] = 6.309573E-02;
  eg32[ 191] = 6.606934E-02;
  eg32[ 192] = 6.918310E-02;
  eg32[ 193] = 7.244360E-02;
  eg32[ 194] = 7.585776E-02;
  eg32[ 195] = 7.943282E-02;
  eg32[ 196] = 8.317638E-02;
  eg32[ 197] = 8.709636E-02;
  eg32[ 198] = 9.120108E-02;
  eg32[ 199] = 9.549926E-02;
  eg32[ 200] = 1.000000E-01;
  eg32[ 201] = 1.047129E-01;
  eg32[ 202] = 1.096478E-01;
  eg32[ 203] = 1.148154E-01;
  eg32[ 204] = 1.202264E-01;
  eg32[ 205] = 1.258925E-01;
  eg32[ 206] = 1.318257E-01;
  eg32[ 207] = 1.380384E-01;
  eg32[ 208] = 1.445440E-01;
  eg32[ 209] = 1.513561E-01;
  eg32[ 210] = 1.584893E-01;
  eg32[ 211] = 1.659587E-01;
  eg32[ 212] = 1.737801E-01;
  eg32[ 213] = 1.819701E-01;
  eg32[ 214] = 1.905461E-01;
  eg32[ 215] = 1.995262E-01;
  eg32[ 216] = 2.089296E-01;
  eg32[ 217] = 2.187762E-01;
  eg32[ 218] = 2.290868E-01;
  eg32[ 219] = 2.398833E-01;
  eg32[ 220] = 2.511886E-01;
  eg32[ 221] = 2.630268E-01;
  eg32[ 222] = 2.754229E-01;
  eg32[ 223] = 2.884032E-01;
  eg32[ 224] = 3.019952E-01;
  eg32[ 225] = 3.162278E-01;
  eg32[ 226] = 3.311311E-01;
  eg32[ 227] = 3.467369E-01;
  eg32[ 228] = 3.630781E-01;
  eg32[ 229] = 3.801894E-01;
  eg32[ 230] = 3.981072E-01;
  eg32[ 231] = 4.168694E-01;
  eg32[ 232] = 4.365158E-01;
  eg32[ 233] = 4.570882E-01;
  eg32[ 234] = 4.786301E-01;
  eg32[ 235] = 5.011872E-01;
  eg32[ 236] = 5.248075E-01;
  eg32[ 237] = 5.500000E-01;
  eg32[ 238] = 5.750000E-01;
  eg32[ 239] = 6.000000E-01;
  eg32[ 240] = 6.250000E-01;
  eg32[ 241] = 6.500000E-01;
  eg32[ 242] = 6.750000E-01;
  eg32[ 243] = 7.000000E-01;
  eg32[ 244] = 7.250000E-01;
  eg32[ 245] = 7.500000E-01;
  eg32[ 246] = 7.750000E-01;
  eg32[ 247] = 8.000000E-01;
  eg32[ 248] = 8.250000E-01;
  eg32[ 249] = 8.500000E-01;
  eg32[ 250] = 8.750000E-01;
  eg32[ 251] = 9.000000E-01;
  eg32[ 252] = 9.250000E-01;
  eg32[ 253] = 9.500000E-01;
  eg32[ 254] = 9.750000E-01;
  eg32[ 255] = 1.000000E+00;
  eg32[ 256] = 1.025000E+00;
  eg32[ 257] = 1.050000E+00;
  eg32[ 258] = 1.075000E+00;
  eg32[ 259] = 1.100000E+00;
  eg32[ 260] = 1.125000E+00;
  eg32[ 261] = 1.150000E+00;
  eg32[ 262] = 1.175000E+00;
  eg32[ 263] = 1.200000E+00;
  eg32[ 264] = 1.225000E+00;
  eg32[ 265] = 1.250000E+00;
  eg32[ 266] = 1.275000E+00;
  eg32[ 267] = 1.300000E+00;
  eg32[ 268] = 1.325000E+00;
  eg32[ 269] = 1.350000E+00;
  eg32[ 270] = 1.375000E+00;
  eg32[ 271] = 1.400000E+00;
  eg32[ 272] = 1.425000E+00;
  eg32[ 273] = 1.450000E+00;
  eg32[ 274] = 1.475000E+00;
  eg32[ 275] = 1.500000E+00;
  eg32[ 276] = 1.525000E+00;
  eg32[ 277] = 1.550000E+00;
  eg32[ 278] = 1.575000E+00;
  eg32[ 279] = 1.600000E+00;
  eg32[ 280] = 1.625000E+00;
  eg32[ 281] = 1.650000E+00;
  eg32[ 282] = 1.675000E+00;
  eg32[ 283] = 1.700000E+00;
  eg32[ 284] = 1.725000E+00;
  eg32[ 285] = 1.750000E+00;
  eg32[ 286] = 1.775000E+00;
  eg32[ 287] = 1.800000E+00;
  eg32[ 288] = 1.825000E+00;
  eg32[ 289] = 1.850000E+00;
  eg32[ 290] = 1.875000E+00;
  eg32[ 291] = 1.900000E+00;
  eg32[ 292] = 1.925000E+00;
  eg32[ 293] = 1.950000E+00;
  eg32[ 294] = 1.975000E+00;
  eg32[ 295] = 2.000000E+00;
  eg32[ 296] = 2.025000E+00;
  eg32[ 297] = 2.050000E+00;
  eg32[ 298] = 2.075000E+00;
  eg32[ 299] = 2.100000E+00;
  eg32[ 300] = 2.125000E+00;
  eg32[ 301] = 2.150000E+00;
  eg32[ 302] = 2.175000E+00;
  eg32[ 303] = 2.200000E+00;
  eg32[ 304] = 2.225000E+00;
  eg32[ 305] = 2.250000E+00;
  eg32[ 306] = 2.275000E+00;
  eg32[ 307] = 2.300000E+00;
  eg32[ 308] = 2.325000E+00;
  eg32[ 309] = 2.350000E+00;
  eg32[ 310] = 2.375000E+00;
  eg32[ 311] = 2.400000E+00;
  eg32[ 312] = 2.425000E+00;
  eg32[ 313] = 2.450000E+00;
  eg32[ 314] = 2.475000E+00;
  eg32[ 315] = 2.500000E+00;
  eg32[ 316] = 2.525000E+00;
  eg32[ 317] = 2.550000E+00;
  eg32[ 318] = 2.575000E+00;
  eg32[ 319] = 2.600000E+00;
  eg32[ 320] = 2.625000E+00;
  eg32[ 321] = 2.650000E+00;
  eg32[ 322] = 2.675000E+00;
  eg32[ 323] = 2.700000E+00;
  eg32[ 324] = 2.725000E+00;
  eg32[ 325] = 2.750000E+00;
  eg32[ 326] = 2.775000E+00;
  eg32[ 327] = 2.800000E+00;
  eg32[ 328] = 2.825000E+00;
  eg32[ 329] = 2.850000E+00;
  eg32[ 330] = 2.875000E+00;
  eg32[ 331] = 2.900000E+00;
  eg32[ 332] = 2.925000E+00;
  eg32[ 333] = 2.950000E+00;
  eg32[ 334] = 2.975000E+00;
  eg32[ 335] = 3.000000E+00;
  eg32[ 336] = 3.025000E+00;
  eg32[ 337] = 3.050000E+00;
  eg32[ 338] = 3.075000E+00;
  eg32[ 339] = 3.100000E+00;
  eg32[ 340] = 3.125000E+00;
  eg32[ 341] = 3.150000E+00;
  eg32[ 342] = 3.175000E+00;
  eg32[ 343] = 3.200000E+00;
  eg32[ 344] = 3.225000E+00;
  eg32[ 345] = 3.250000E+00;
  eg32[ 346] = 3.275000E+00;
  eg32[ 347] = 3.300000E+00;
  eg32[ 348] = 3.325000E+00;
  eg32[ 349] = 3.350000E+00;
  eg32[ 350] = 3.375000E+00;
  eg32[ 351] = 3.400000E+00;
  eg32[ 352] = 3.425000E+00;
  eg32[ 353] = 3.450000E+00;
  eg32[ 354] = 3.475000E+00;
  eg32[ 355] = 3.500000E+00;
  eg32[ 356] = 3.525000E+00;
  eg32[ 357] = 3.550000E+00;
  eg32[ 358] = 3.575000E+00;
  eg32[ 359] = 3.600000E+00;
  eg32[ 360] = 3.625000E+00;
  eg32[ 361] = 3.650000E+00;
  eg32[ 362] = 3.675000E+00;
  eg32[ 363] = 3.700000E+00;
  eg32[ 364] = 3.725000E+00;
  eg32[ 365] = 3.750000E+00;
  eg32[ 366] = 3.775000E+00;
  eg32[ 367] = 3.800000E+00;
  eg32[ 368] = 3.825000E+00;
  eg32[ 369] = 3.850000E+00;
  eg32[ 370] = 3.875000E+00;
  eg32[ 371] = 3.900000E+00;
  eg32[ 372] = 3.925000E+00;
  eg32[ 373] = 3.950000E+00;
  eg32[ 374] = 3.975000E+00;
  eg32[ 375] = 4.000000E+00;
  eg32[ 376] = 4.025000E+00;
  eg32[ 377] = 4.050000E+00;
  eg32[ 378] = 4.075000E+00;
  eg32[ 379] = 4.100000E+00;
  eg32[ 380] = 4.125000E+00;
  eg32[ 381] = 4.150000E+00;
  eg32[ 382] = 4.175000E+00;
  eg32[ 383] = 4.200000E+00;
  eg32[ 384] = 4.225000E+00;
  eg32[ 385] = 4.250000E+00;
  eg32[ 386] = 4.275000E+00;
  eg32[ 387] = 4.300000E+00;
  eg32[ 388] = 4.325000E+00;
  eg32[ 389] = 4.350000E+00;
  eg32[ 390] = 4.375000E+00;
  eg32[ 391] = 4.400000E+00;
  eg32[ 392] = 4.425000E+00;
  eg32[ 393] = 4.450000E+00;
  eg32[ 394] = 4.475000E+00;
  eg32[ 395] = 4.500000E+00;
  eg32[ 396] = 4.525000E+00;
  eg32[ 397] = 4.550000E+00;
  eg32[ 398] = 4.575000E+00;
  eg32[ 399] = 4.600000E+00;
  eg32[ 400] = 4.625000E+00;
  eg32[ 401] = 4.650000E+00;
  eg32[ 402] = 4.675000E+00;
  eg32[ 403] = 4.700000E+00;
  eg32[ 404] = 4.725000E+00;
  eg32[ 405] = 4.750000E+00;
  eg32[ 406] = 4.775000E+00;
  eg32[ 407] = 4.800000E+00;
  eg32[ 408] = 4.825000E+00;
  eg32[ 409] = 4.850000E+00;
  eg32[ 410] = 4.875000E+00;
  eg32[ 411] = 4.900000E+00;
  eg32[ 412] = 4.925000E+00;
  eg32[ 413] = 4.950000E+00;
  eg32[ 414] = 4.975000E+00;
  eg32[ 415] = 5.000000E+00;
  eg32[ 416] = 5.025000E+00;
  eg32[ 417] = 5.050000E+00;
  eg32[ 418] = 5.075000E+00;
  eg32[ 419] = 5.100000E+00;
  eg32[ 420] = 5.125000E+00;
  eg32[ 421] = 5.150000E+00;
  eg32[ 422] = 5.175000E+00;
  eg32[ 423] = 5.200000E+00;
  eg32[ 424] = 5.225000E+00;
  eg32[ 425] = 5.250000E+00;
  eg32[ 426] = 5.275000E+00;
  eg32[ 427] = 5.300000E+00;
  eg32[ 428] = 5.325000E+00;
  eg32[ 429] = 5.350000E+00;
  eg32[ 430] = 5.375000E+00;
  eg32[ 431] = 5.400000E+00;
  eg32[ 432] = 5.425000E+00;
  eg32[ 433] = 5.450000E+00;
  eg32[ 434] = 5.475000E+00;
  eg32[ 435] = 5.500000E+00;
  eg32[ 436] = 5.525000E+00;
  eg32[ 437] = 5.550000E+00;
  eg32[ 438] = 5.575000E+00;
  eg32[ 439] = 5.600000E+00;
  eg32[ 440] = 5.625000E+00;
  eg32[ 441] = 5.650000E+00;
  eg32[ 442] = 5.675000E+00;
  eg32[ 443] = 5.700000E+00;
  eg32[ 444] = 5.725000E+00;
  eg32[ 445] = 5.750000E+00;
  eg32[ 446] = 5.775000E+00;
  eg32[ 447] = 5.800000E+00;
  eg32[ 448] = 5.825000E+00;
  eg32[ 449] = 5.850000E+00;
  eg32[ 450] = 5.875000E+00;
  eg32[ 451] = 5.900000E+00;
  eg32[ 452] = 5.925000E+00;
  eg32[ 453] = 5.950000E+00;
  eg32[ 454] = 5.975000E+00;
  eg32[ 455] = 6.000000E+00;
  eg32[ 456] = 6.025000E+00;
  eg32[ 457] = 6.050000E+00;
  eg32[ 458] = 6.075000E+00;
  eg32[ 459] = 6.100000E+00;
  eg32[ 460] = 6.125000E+00;
  eg32[ 461] = 6.150000E+00;
  eg32[ 462] = 6.175000E+00;
  eg32[ 463] = 6.200000E+00;
  eg32[ 464] = 6.225000E+00;
  eg32[ 465] = 6.250000E+00;
  eg32[ 466] = 6.275000E+00;
  eg32[ 467] = 6.300000E+00;
  eg32[ 468] = 6.325000E+00;
  eg32[ 469] = 6.350000E+00;
  eg32[ 470] = 6.375000E+00;
  eg32[ 471] = 6.400000E+00;
  eg32[ 472] = 6.425000E+00;
  eg32[ 473] = 6.450000E+00;
  eg32[ 474] = 6.475000E+00;
  eg32[ 475] = 6.500000E+00;
  eg32[ 476] = 6.525000E+00;
  eg32[ 477] = 6.550000E+00;
  eg32[ 478] = 6.575000E+00;
  eg32[ 479] = 6.600000E+00;
  eg32[ 480] = 6.625000E+00;
  eg32[ 481] = 6.650000E+00;
  eg32[ 482] = 6.675000E+00;
  eg32[ 483] = 6.700000E+00;
  eg32[ 484] = 6.725000E+00;
  eg32[ 485] = 6.750000E+00;
  eg32[ 486] = 6.775000E+00;
  eg32[ 487] = 6.800000E+00;
  eg32[ 488] = 6.825000E+00;
  eg32[ 489] = 6.850000E+00;
  eg32[ 490] = 6.875000E+00;
  eg32[ 491] = 6.900000E+00;
  eg32[ 492] = 6.925000E+00;
  eg32[ 493] = 6.950000E+00;
  eg32[ 494] = 6.975000E+00;
  eg32[ 495] = 7.000000E+00;
  eg32[ 496] = 7.025000E+00;
  eg32[ 497] = 7.050000E+00;
  eg32[ 498] = 7.075000E+00;
  eg32[ 499] = 7.100000E+00;
  eg32[ 500] = 7.125000E+00;
  eg32[ 501] = 7.150000E+00;
  eg32[ 502] = 7.175000E+00;
  eg32[ 503] = 7.200000E+00;
  eg32[ 504] = 7.225000E+00;
  eg32[ 505] = 7.250000E+00;
  eg32[ 506] = 7.275000E+00;
  eg32[ 507] = 7.300000E+00;
  eg32[ 508] = 7.325000E+00;
  eg32[ 509] = 7.350000E+00;
  eg32[ 510] = 7.375000E+00;
  eg32[ 511] = 7.400000E+00;
  eg32[ 512] = 7.425000E+00;
  eg32[ 513] = 7.450000E+00;
  eg32[ 514] = 7.475000E+00;
  eg32[ 515] = 7.500000E+00;
  eg32[ 516] = 7.525000E+00;
  eg32[ 517] = 7.550000E+00;
  eg32[ 518] = 7.575000E+00;
  eg32[ 519] = 7.600000E+00;
  eg32[ 520] = 7.625000E+00;
  eg32[ 521] = 7.650000E+00;
  eg32[ 522] = 7.675000E+00;
  eg32[ 523] = 7.700000E+00;
  eg32[ 524] = 7.725000E+00;
  eg32[ 525] = 7.750000E+00;
  eg32[ 526] = 7.775000E+00;
  eg32[ 527] = 7.800000E+00;
  eg32[ 528] = 7.825000E+00;
  eg32[ 529] = 7.850000E+00;
  eg32[ 530] = 7.875000E+00;
  eg32[ 531] = 7.900000E+00;
  eg32[ 532] = 7.925000E+00;
  eg32[ 533] = 7.950000E+00;
  eg32[ 534] = 7.975000E+00;
  eg32[ 535] = 8.000000E+00;
  eg32[ 536] = 8.025000E+00;
  eg32[ 537] = 8.050000E+00;
  eg32[ 538] = 8.075000E+00;
  eg32[ 539] = 8.100000E+00;
  eg32[ 540] = 8.125000E+00;
  eg32[ 541] = 8.150000E+00;
  eg32[ 542] = 8.175000E+00;
  eg32[ 543] = 8.200000E+00;
  eg32[ 544] = 8.225000E+00;
  eg32[ 545] = 8.250000E+00;
  eg32[ 546] = 8.275000E+00;
  eg32[ 547] = 8.300000E+00;
  eg32[ 548] = 8.325000E+00;
  eg32[ 549] = 8.350000E+00;
  eg32[ 550] = 8.375000E+00;
  eg32[ 551] = 8.400000E+00;
  eg32[ 552] = 8.425000E+00;
  eg32[ 553] = 8.450000E+00;
  eg32[ 554] = 8.475000E+00;
  eg32[ 555] = 8.500000E+00;
  eg32[ 556] = 8.525000E+00;
  eg32[ 557] = 8.550000E+00;
  eg32[ 558] = 8.575000E+00;
  eg32[ 559] = 8.600000E+00;
  eg32[ 560] = 8.625000E+00;
  eg32[ 561] = 8.650000E+00;
  eg32[ 562] = 8.675000E+00;
  eg32[ 563] = 8.700000E+00;
  eg32[ 564] = 8.725000E+00;
  eg32[ 565] = 8.750000E+00;
  eg32[ 566] = 8.775000E+00;
  eg32[ 567] = 8.800000E+00;
  eg32[ 568] = 8.825000E+00;
  eg32[ 569] = 8.850000E+00;
  eg32[ 570] = 8.875000E+00;
  eg32[ 571] = 8.900000E+00;
  eg32[ 572] = 8.925000E+00;
  eg32[ 573] = 8.950000E+00;
  eg32[ 574] = 8.975000E+00;
  eg32[ 575] = 9.000000E+00;
  eg32[ 576] = 9.025000E+00;
  eg32[ 577] = 9.050000E+00;
  eg32[ 578] = 9.075000E+00;
  eg32[ 579] = 9.100000E+00;
  eg32[ 580] = 9.125000E+00;
  eg32[ 581] = 9.150000E+00;
  eg32[ 582] = 9.175000E+00;
  eg32[ 583] = 9.200000E+00;
  eg32[ 584] = 9.225000E+00;
  eg32[ 585] = 9.250000E+00;
  eg32[ 586] = 9.275000E+00;
  eg32[ 587] = 9.300000E+00;
  eg32[ 588] = 9.325000E+00;
  eg32[ 589] = 9.350000E+00;
  eg32[ 590] = 9.375000E+00;
  eg32[ 591] = 9.400000E+00;
  eg32[ 592] = 9.425000E+00;
  eg32[ 593] = 9.450000E+00;
  eg32[ 594] = 9.475000E+00;
  eg32[ 595] = 9.500000E+00;
  eg32[ 596] = 9.525000E+00;
  eg32[ 597] = 9.550000E+00;
  eg32[ 598] = 9.575000E+00;
  eg32[ 599] = 9.600000E+00;
  eg32[ 600] = 9.625000E+00;
  eg32[ 601] = 9.650000E+00;
  eg32[ 602] = 9.675000E+00;
  eg32[ 603] = 9.700000E+00;
  eg32[ 604] = 9.725000E+00;
  eg32[ 605] = 9.750000E+00;
  eg32[ 606] = 9.775000E+00;
  eg32[ 607] = 9.800000E+00;
  eg32[ 608] = 9.825000E+00;
  eg32[ 609] = 9.850000E+00;
  eg32[ 610] = 9.875000E+00;
  eg32[ 611] = 9.900000E+00;
  eg32[ 612] = 9.925000E+00;
  eg32[ 613] = 9.950000E+00;
  eg32[ 614] = 9.975000E+00;
  eg32[ 615] = 1.000000E+01;
  eg32[ 616] = 1.047129E+01;
  eg32[ 617] = 1.096478E+01;
  eg32[ 618] = 1.148154E+01;
  eg32[ 619] = 1.202264E+01;
  eg32[ 620] = 1.258925E+01;
  eg32[ 621] = 1.318257E+01;
  eg32[ 622] = 1.380384E+01;
  eg32[ 623] = 1.445440E+01;
  eg32[ 624] = 1.513561E+01;
  eg32[ 625] = 1.584893E+01;
  eg32[ 626] = 1.659587E+01;
  eg32[ 627] = 1.737801E+01;
  eg32[ 628] = 1.819701E+01;
  eg32[ 629] = 1.905461E+01;
  eg32[ 630] = 1.995262E+01;
  eg32[ 631] = 2.089296E+01;
  eg32[ 632] = 2.187762E+01;
  eg32[ 633] = 2.290868E+01;
  eg32[ 634] = 2.398833E+01;
  eg32[ 635] = 2.511886E+01;
  eg32[ 636] = 2.630268E+01;
  eg32[ 637] = 2.754229E+01;
  eg32[ 638] = 2.884032E+01;
  eg32[ 639] = 3.019952E+01;
  eg32[ 640] = 3.162278E+01;
  eg32[ 641] = 3.311311E+01;
  eg32[ 642] = 3.467369E+01;
  eg32[ 643] = 3.630781E+01;
  eg32[ 644] = 3.801894E+01;
  eg32[ 645] = 3.981072E+01;
  eg32[ 646] = 4.168694E+01;
  eg32[ 647] = 4.365158E+01;
  eg32[ 648] = 4.570882E+01;
  eg32[ 649] = 4.786301E+01;
  eg32[ 650] = 5.011872E+01;
  eg32[ 651] = 5.248075E+01;
  eg32[ 652] = 5.495409E+01;
  eg32[ 653] = 5.754399E+01;
  eg32[ 654] = 6.025596E+01;
  eg32[ 655] = 6.309573E+01;
  eg32[ 656] = 6.606934E+01;
  eg32[ 657] = 6.918310E+01;
  eg32[ 658] = 7.244360E+01;
  eg32[ 659] = 7.585776E+01;
  eg32[ 660] = 7.943282E+01;
  eg32[ 661] = 8.317638E+01;
  eg32[ 662] = 8.709636E+01;
  eg32[ 663] = 9.120108E+01;
  eg32[ 664] = 9.549926E+01;
  eg32[ 665] = 1.000000E+02;
  eg32[ 666] = 1.047129E+02;
  eg32[ 667] = 1.096478E+02;
  eg32[ 668] = 1.148154E+02;
  eg32[ 669] = 1.202264E+02;
  eg32[ 670] = 1.258925E+02;
  eg32[ 671] = 1.318257E+02;
  eg32[ 672] = 1.380384E+02;
  eg32[ 673] = 1.445440E+02;
  eg32[ 674] = 1.513561E+02;
  eg32[ 675] = 1.584893E+02;
  eg32[ 676] = 1.659587E+02;
  eg32[ 677] = 1.737801E+02;
  eg32[ 678] = 1.819701E+02;
  eg32[ 679] = 1.905461E+02;
  eg32[ 680] = 1.995262E+02;
  eg32[ 681] = 2.089296E+02;
  eg32[ 682] = 2.187762E+02;
  eg32[ 683] = 2.290868E+02;
  eg32[ 684] = 2.398833E+02;
  eg32[ 685] = 2.511886E+02;
  eg32[ 686] = 2.630268E+02;
  eg32[ 687] = 2.754229E+02;
  eg32[ 688] = 2.884032E+02;
  eg32[ 689] = 3.019952E+02;
  eg32[ 690] = 3.162278E+02;
  eg32[ 691] = 3.311311E+02;
  eg32[ 692] = 3.467369E+02;
  eg32[ 693] = 3.630781E+02;
  eg32[ 694] = 3.801894E+02;
  eg32[ 695] = 3.981072E+02;
  eg32[ 696] = 4.168694E+02;
  eg32[ 697] = 4.365158E+02;
  eg32[ 698] = 4.570882E+02;
  eg32[ 699] = 4.786301E+02;
  eg32[ 700] = 5.011872E+02;
  eg32[ 701] = 5.248075E+02;
  eg32[ 702] = 5.495409E+02;
  eg32[ 703] = 5.754399E+02;
  eg32[ 704] = 6.025596E+02;
  eg32[ 705] = 6.309573E+02;
  eg32[ 706] = 6.606934E+02;
  eg32[ 707] = 6.918310E+02;
  eg32[ 708] = 7.244360E+02;
  eg32[ 709] = 7.585776E+02;
  eg32[ 710] = 7.943282E+02;
  eg32[ 711] = 8.317638E+02;
  eg32[ 712] = 8.709636E+02;
  eg32[ 713] = 9.120108E+02;
  eg32[ 714] = 9.549926E+02;
  eg32[ 715] = 1.000000E+03;
  eg32[ 716] = 1.047129E+03;
  eg32[ 717] = 1.096478E+03;
  eg32[ 718] = 1.148154E+03;
  eg32[ 719] = 1.202264E+03;
  eg32[ 720] = 1.258925E+03;
  eg32[ 721] = 1.318257E+03;
  eg32[ 722] = 1.380384E+03;
  eg32[ 723] = 1.445440E+03;
  eg32[ 724] = 1.513561E+03;
  eg32[ 725] = 1.584893E+03;
  eg32[ 726] = 1.659587E+03;
  eg32[ 727] = 1.737801E+03;
  eg32[ 728] = 1.819701E+03;
  eg32[ 729] = 1.905461E+03;
  eg32[ 730] = 1.995262E+03;
  eg32[ 731] = 2.089296E+03;
  eg32[ 732] = 2.187762E+03;
  eg32[ 733] = 2.290868E+03;
  eg32[ 734] = 2.398833E+03;
  eg32[ 735] = 2.511886E+03;
  eg32[ 736] = 2.630268E+03;
  eg32[ 737] = 2.754229E+03;
  eg32[ 738] = 2.884032E+03;
  eg32[ 739] = 3.019952E+03;
  eg32[ 740] = 3.162278E+03;
  eg32[ 741] = 3.311311E+03;
  eg32[ 742] = 3.467369E+03;
  eg32[ 743] = 3.630781E+03;
  eg32[ 744] = 3.801894E+03;
  eg32[ 745] = 3.981072E+03;
  eg32[ 746] = 4.168694E+03;
  eg32[ 747] = 4.365158E+03;
  eg32[ 748] = 4.570882E+03;
  eg32[ 749] = 4.786301E+03;
  eg32[ 750] = 5.011872E+03;
  eg32[ 751] = 5.248075E+03;
  eg32[ 752] = 5.495409E+03;
  eg32[ 753] = 5.754399E+03;
  eg32[ 754] = 6.025596E+03;
  eg32[ 755] = 6.309573E+03;
  eg32[ 756] = 6.606934E+03;
  eg32[ 757] = 6.918310E+03;
  eg32[ 758] = 7.244360E+03;
  eg32[ 759] = 7.585776E+03;
  eg32[ 760] = 7.943282E+03;
  eg32[ 761] = 8.317638E+03;
  eg32[ 762] = 8.709636E+03;
  eg32[ 763] = 9.120108E+03;
  eg32[ 764] = 9.549926E+03;
  eg32[ 765] = 1.000000E+04;
  eg32[ 766] = 1.047129E+04;
  eg32[ 767] = 1.096478E+04;
  eg32[ 768] = 1.148154E+04;
  eg32[ 769] = 1.202264E+04;
  eg32[ 770] = 1.258925E+04;
  eg32[ 771] = 1.318257E+04;
  eg32[ 772] = 1.380384E+04;
  eg32[ 773] = 1.445440E+04;
  eg32[ 774] = 1.513561E+04;
  eg32[ 775] = 1.584893E+04;
  eg32[ 776] = 1.659587E+04;
  eg32[ 777] = 1.737801E+04;
  eg32[ 778] = 1.819701E+04;
  eg32[ 779] = 1.905461E+04;
  eg32[ 780] = 1.995262E+04;
  eg32[ 781] = 2.089296E+04;
  eg32[ 782] = 2.187762E+04;
  eg32[ 783] = 2.290868E+04;
  eg32[ 784] = 2.398833E+04;
  eg32[ 785] = 2.511886E+04;
  eg32[ 786] = 2.630268E+04;
  eg32[ 787] = 2.754229E+04;
  eg32[ 788] = 2.884032E+04;
  eg32[ 789] = 3.019952E+04;
  eg32[ 790] = 3.162278E+04;
  eg32[ 791] = 3.311311E+04;
  eg32[ 792] = 3.467369E+04;
  eg32[ 793] = 3.630781E+04;
  eg32[ 794] = 3.801894E+04;
  eg32[ 795] = 3.981072E+04;
  eg32[ 796] = 4.168694E+04;
  eg32[ 797] = 4.365158E+04;
  eg32[ 798] = 4.570882E+04;
  eg32[ 799] = 4.786301E+04;
  eg32[ 800] = 5.011872E+04;
  eg32[ 801] = 5.248075E+04;
  eg32[ 802] = 5.495409E+04;
  eg32[ 803] = 5.754399E+04;
  eg32[ 804] = 6.025596E+04;
  eg32[ 805] = 6.309573E+04;
  eg32[ 806] = 6.606934E+04;
  eg32[ 807] = 6.918310E+04;
  eg32[ 808] = 7.244360E+04;
  eg32[ 809] = 7.585776E+04;
  eg32[ 810] = 7.943282E+04;
  eg32[ 811] = 8.317638E+04;
  eg32[ 812] = 8.709636E+04;
  eg32[ 813] = 9.120108E+04;
  eg32[ 814] = 9.549926E+04;
  eg32[ 815] = 1.000000E+05;
  eg32[ 816] = 1.047129E+05;
  eg32[ 817] = 1.096478E+05;
  eg32[ 818] = 1.148154E+05;
  eg32[ 819] = 1.202264E+05;
  eg32[ 820] = 1.258925E+05;
  eg32[ 821] = 1.318257E+05;
  eg32[ 822] = 1.380384E+05;
  eg32[ 823] = 1.445440E+05;
  eg32[ 824] = 1.513561E+05;
  eg32[ 825] = 1.584893E+05;
  eg32[ 826] = 1.659587E+05;
  eg32[ 827] = 1.737801E+05;
  eg32[ 828] = 1.819701E+05;
  eg32[ 829] = 1.905461E+05;
  eg32[ 830] = 1.995262E+05;
  eg32[ 831] = 2.089296E+05;
  eg32[ 832] = 2.187762E+05;
  eg32[ 833] = 2.290868E+05;
  eg32[ 834] = 2.398833E+05;
  eg32[ 835] = 2.511886E+05;
  eg32[ 836] = 2.630268E+05;
  eg32[ 837] = 2.754229E+05;
  eg32[ 838] = 2.884032E+05;
  eg32[ 839] = 3.019952E+05;
  eg32[ 840] = 3.162278E+05;
  eg32[ 841] = 3.311311E+05;
  eg32[ 842] = 3.467369E+05;
  eg32[ 843] = 3.630781E+05;
  eg32[ 844] = 3.801894E+05;
  eg32[ 845] = 3.981072E+05;
  eg32[ 846] = 4.168694E+05;
  eg32[ 847] = 4.365158E+05;
  eg32[ 848] = 4.570882E+05;
  eg32[ 849] = 4.786301E+05;
  eg32[ 850] = 5.011872E+05;
  eg32[ 851] = 5.248075E+05;
  eg32[ 852] = 5.495409E+05;
  eg32[ 853] = 5.754399E+05;
  eg32[ 854] = 6.025596E+05;
  eg32[ 855] = 6.309573E+05;
  eg32[ 856] = 6.606934E+05;
  eg32[ 857] = 6.918310E+05;
  eg32[ 858] = 7.244360E+05;
  eg32[ 859] = 7.585776E+05;
  eg32[ 860] = 7.943282E+05;
  eg32[ 861] = 8.317638E+05;
  eg32[ 862] = 8.709636E+05;
  eg32[ 863] = 9.120108E+05;
  eg32[ 864] = 9.549926E+05;
  eg32[ 865] = 1.000000E+06;
  eg32[ 866] = 1.047129E+06;
  eg32[ 867] = 1.096478E+06;
  eg32[ 868] = 1.148154E+06;
  eg32[ 869] = 1.202264E+06;
  eg32[ 870] = 1.258925E+06;
  eg32[ 871] = 1.318257E+06;
  eg32[ 872] = 1.380384E+06;
  eg32[ 873] = 1.445440E+06;
  eg32[ 874] = 1.513561E+06;
  eg32[ 875] = 1.584893E+06;
  eg32[ 876] = 1.659587E+06;
  eg32[ 877] = 1.737801E+06;
  eg32[ 878] = 1.819701E+06;
  eg32[ 879] = 1.905461E+06;
  eg32[ 880] = 1.995262E+06;
  eg32[ 881] = 2.089296E+06;
  eg32[ 882] = 2.187762E+06;
  eg32[ 883] = 2.290868E+06;
  eg32[ 884] = 2.398833E+06;
  eg32[ 885] = 2.511886E+06;
  eg32[ 886] = 2.630268E+06;
  eg32[ 887] = 2.754229E+06;
  eg32[ 888] = 2.884032E+06;
  eg32[ 889] = 3.019952E+06;
  eg32[ 890] = 3.162278E+06;
  eg32[ 891] = 3.311311E+06;
  eg32[ 892] = 3.467369E+06;
  eg32[ 893] = 3.630781E+06;
  eg32[ 894] = 3.801894E+06;
  eg32[ 895] = 3.981072E+06;
  eg32[ 896] = 4.168694E+06;
  eg32[ 897] = 4.365158E+06;
  eg32[ 898] = 4.570882E+06;
  eg32[ 899] = 4.786301E+06;
  eg32[ 900] = 5.000000E+06;
  eg32[ 901] = 5.200000E+06;
  eg32[ 902] = 5.400000E+06;
  eg32[ 903] = 5.600000E+06;
  eg32[ 904] = 5.800000E+06;
  eg32[ 905] = 6.000000E+06;
  eg32[ 906] = 6.200000E+06;
  eg32[ 907] = 6.400000E+06;
  eg32[ 908] = 6.600000E+06;
  eg32[ 909] = 6.800000E+06;
  eg32[ 910] = 7.000000E+06;
  eg32[ 911] = 7.200000E+06;
  eg32[ 912] = 7.400000E+06;
  eg32[ 913] = 7.600000E+06;
  eg32[ 914] = 7.800000E+06;
  eg32[ 915] = 8.000000E+06;
  eg32[ 916] = 8.200000E+06;
  eg32[ 917] = 8.400000E+06;
  eg32[ 918] = 8.600000E+06;
  eg32[ 919] = 8.800000E+06;
  eg32[ 920] = 9.000000E+06;
  eg32[ 921] = 9.200000E+06;
  eg32[ 922] = 9.400000E+06;
  eg32[ 923] = 9.600000E+06;
  eg32[ 924] = 9.800000E+06;
  eg32[ 925] = 1.000000E+07;
  eg32[ 926] = 1.020000E+07;
  eg32[ 927] = 1.040000E+07;
  eg32[ 928] = 1.060000E+07;
  eg32[ 929] = 1.080000E+07;
  eg32[ 930] = 1.100000E+07;
  eg32[ 931] = 1.120000E+07;
  eg32[ 932] = 1.140000E+07;
  eg32[ 933] = 1.160000E+07;
  eg32[ 934] = 1.180000E+07;
  eg32[ 935] = 1.200000E+07;
  eg32[ 936] = 1.220000E+07;
  eg32[ 937] = 1.240000E+07;
  eg32[ 938] = 1.260000E+07;
  eg32[ 939] = 1.280000E+07;
  eg32[ 940] = 1.300000E+07;
  eg32[ 941] = 1.320000E+07;
  eg32[ 942] = 1.340000E+07;
  eg32[ 943] = 1.360000E+07;
  eg32[ 944] = 1.380000E+07;
  eg32[ 945] = 1.400000E+07;
  eg32[ 946] = 1.420000E+07;
  eg32[ 947] = 1.440000E+07;
  eg32[ 948] = 1.460000E+07;
  eg32[ 949] = 1.480000E+07;
  eg32[ 950] = 1.500000E+07;
  eg32[ 951] = 1.520000E+07;
  eg32[ 952] = 1.540000E+07;
  eg32[ 953] = 1.560000E+07;
  eg32[ 954] = 1.580000E+07;
  eg32[ 955] = 1.600000E+07;
  eg32[ 956] = 1.620000E+07;
  eg32[ 957] = 1.640000E+07;
  eg32[ 958] = 1.660000E+07;
  eg32[ 959] = 1.680000E+07;
  eg32[ 960] = 1.700000E+07;
  eg32[ 961] = 1.720000E+07;
  eg32[ 962] = 1.740000E+07;
  eg32[ 963] = 1.760000E+07;
  eg32[ 964] = 1.780000E+07;
  eg32[ 965] = 1.800000E+07;
  eg32[ 966] = 1.820000E+07;
  eg32[ 967] = 1.840000E+07;
  eg32[ 968] = 1.860000E+07;
  eg32[ 969] = 1.880000E+07;
  eg32[ 970] = 1.900000E+07;
  eg32[ 971] = 1.920000E+07;
  eg32[ 972] = 1.940000E+07;
  eg32[ 973] = 1.960000E+07;
  eg32[ 974] = 1.980000E+07;
  eg32[ 975] = 2.000000E+07;
  eg32[ 976] = 2.020000E+07;
  eg32[ 977] = 2.040000E+07;
  eg32[ 978] = 2.060000E+07;
  eg32[ 979] = 2.080000E+07;
  eg32[ 980] = 2.100000E+07;
  eg32[ 981] = 2.120000E+07;
  eg32[ 982] = 2.140000E+07;
  eg32[ 983] = 2.160000E+07;
  eg32[ 984] = 2.180000E+07;
  eg32[ 985] = 2.200000E+07;
  eg32[ 986] = 2.220000E+07;
  eg32[ 987] = 2.240000E+07;
  eg32[ 988] = 2.260000E+07;
  eg32[ 989] = 2.280000E+07;
  eg32[ 990] = 2.300000E+07;
  eg32[ 991] = 2.320000E+07;
  eg32[ 992] = 2.340000E+07;
  eg32[ 993] = 2.360000E+07;
  eg32[ 994] = 2.380000E+07;
  eg32[ 995] = 2.400000E+07;
  eg32[ 996] = 2.420000E+07;
  eg32[ 997] = 2.440000E+07;
  eg32[ 998] = 2.460000E+07;
  eg32[ 999] = 2.480000E+07;
  eg32[1000] = 2.500000E+07;
  eg32[1001] = 2.520000E+07;
  eg32[1002] = 2.540000E+07;
  eg32[1003] = 2.560000E+07;
  eg32[1004] = 2.580000E+07;
  eg32[1005] = 2.600000E+07;
  eg32[1006] = 2.620000E+07;
  eg32[1007] = 2.640000E+07;
  eg32[1008] = 2.660000E+07;
  eg32[1009] = 2.680000E+07;
  eg32[1010] = 2.700000E+07;
  eg32[1011] = 2.720000E+07;
  eg32[1012] = 2.740000E+07;
  eg32[1013] = 2.760000E+07;
  eg32[1014] = 2.780000E+07;
  eg32[1015] = 2.800000E+07;
  eg32[1016] = 2.820000E+07;
  eg32[1017] = 2.840000E+07;
  eg32[1018] = 2.860000E+07;
  eg32[1019] = 2.880000E+07;
  eg32[1020] = 2.900000E+07;
  eg32[1021] = 2.920000E+07;
  eg32[1022] = 2.940000E+07;
  eg32[1023] = 2.960000E+07;
  eg32[1024] = 2.980000E+07;
  eg32[1025] = 3.000000E+07;
  eg32[1026] = 3.019952E+07;
  eg32[1027] = 3.162278E+07;
  eg32[1028] = 3.311311E+07;
  eg32[1029] = 3.467369E+07;
  eg32[1030] = 3.630781E+07;
  eg32[1031] = 3.801894E+07;
  eg32[1032] = 3.981072E+07;
  eg32[1033] = 4.168694E+07;
  eg32[1034] = 4.365158E+07;
  eg32[1035] = 4.570882E+07;
  eg32[1036] = 4.786301E+07;
  eg32[1037] = 5.011872E+07;
  eg32[1038] = 5.248075E+07;
  eg32[1039] = 5.495409E+07;
  eg32[1040] = 5.754399E+07;
  eg32[1041] = 6.025596E+07;
  eg32[1042] = 6.309573E+07;
  eg32[1043] = 6.606934E+07;
  eg32[1044] = 6.918310E+07;
  eg32[1045] = 7.244360E+07;
  eg32[1046] = 7.585776E+07;
  eg32[1047] = 7.943282E+07;
  eg32[1048] = 8.317638E+07;
  eg32[1049] = 8.709636E+07;
  eg32[1050] = 9.120108E+07;
  eg32[1051] = 9.549926E+07;
  eg32[1052] = 1.000000E+08;
  eg32[1053] = 1.047129E+08;
  eg32[1054] = 1.096478E+08;
  eg32[1055] = 1.148154E+08;
  eg32[1056] = 1.202264E+08;
  eg32[1057] = 1.258925E+08;
  eg32[1058] = 1.318257E+08;
  eg32[1059] = 1.380384E+08;
  eg32[1060] = 1.445440E+08;
  eg32[1061] = 1.513561E+08;
  eg32[1062] = 1.584893E+08;
  eg32[1063] = 1.659587E+08;
  eg32[1064] = 1.737801E+08;
  eg32[1065] = 1.819701E+08;
  eg32[1066] = 1.905461E+08;
  eg32[1067] = 1.995262E+08;
  eg32[1068] = 2.089296E+08;
  eg32[1069] = 2.187762E+08;
  eg32[1070] = 2.290868E+08;
  eg32[1071] = 2.398833E+08;
  eg32[1072] = 2.511886E+08;
  eg32[1073] = 2.630268E+08;
  eg32[1074] = 2.754229E+08;
  eg32[1075] = 2.884032E+08;
  eg32[1076] = 3.019952E+08;
  eg32[1077] = 3.162278E+08;
  eg32[1078] = 3.311311E+08;
  eg32[1079] = 3.467369E+08;
  eg32[1080] = 3.630781E+08;
  eg32[1081] = 3.801894E+08;
  eg32[1082] = 3.981072E+08;
  eg32[1083] = 4.168694E+08;
  eg32[1084] = 4.365158E+08;
  eg32[1085] = 4.570882E+08;
  eg32[1086] = 4.786301E+08;
  eg32[1087] = 5.011872E+08;
  eg32[1088] = 5.248075E+08;
  eg32[1089] = 5.495409E+08;
  eg32[1090] = 5.754399E+08;
  eg32[1091] = 6.025596E+08;
  eg32[1092] = 6.309573E+08;
  eg32[1093] = 6.606934E+08;
  eg32[1094] = 6.918310E+08;
  eg32[1095] = 7.244360E+08;
  eg32[1096] = 7.585776E+08;
  eg32[1097] = 7.943282E+08;
  eg32[1098] = 8.317638E+08;
  eg32[1099] = 8.709636E+08;
  eg32[1100] = 9.120108E+08;
  eg32[1101] = 9.549926E+08;
  eg32[1102] = 1.000000E+09;

  return eg32;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg33()
{
  vector<Real> eg33;
  eg33.resize(143);
  eg33[  0] = 5.000000E+03;
  eg33[  1] = 1.000000E+04;
  eg33[  2] = 1.500000E+04;
  eg33[  3] = 2.000000E+04;
  eg33[  4] = 2.500000E+04;
  eg33[  5] = 3.000000E+04;
  eg33[  6] = 3.500000E+04;
  eg33[  7] = 4.000000E+04;
  eg33[  8] = 4.500000E+04;
  eg33[  9] = 5.000000E+04;
  eg33[ 10] = 5.500000E+04;
  eg33[ 11] = 6.000000E+04;
  eg33[ 12] = 6.500000E+04;
  eg33[ 13] = 7.000000E+04;
  eg33[ 14] = 7.500000E+04;
  eg33[ 15] = 8.000000E+04;
  eg33[ 16] = 8.500000E+04;
  eg33[ 17] = 9.000000E+04;
  eg33[ 18] = 9.500000E+04;
  eg33[ 19] = 1.000000E+05;
  eg33[ 20] = 1.200000E+05;
  eg33[ 21] = 1.400000E+05;
  eg33[ 22] = 1.600000E+05;
  eg33[ 23] = 1.800000E+05;
  eg33[ 24] = 2.000000E+05;
  eg33[ 25] = 2.200000E+05;
  eg33[ 26] = 2.400000E+05;
  eg33[ 27] = 2.600000E+05;
  eg33[ 28] = 2.800000E+05;
  eg33[ 29] = 3.000000E+05;
  eg33[ 30] = 3.250000E+05;
  eg33[ 31] = 3.500000E+05;
  eg33[ 32] = 3.750000E+05;
  eg33[ 33] = 4.000000E+05;
  eg33[ 34] = 4.250000E+05;
  eg33[ 35] = 4.500000E+05;
  eg33[ 36] = 4.750000E+05;
  eg33[ 37] = 5.000000E+05;
  eg33[ 38] = 5.250000E+05;
  eg33[ 39] = 5.500000E+05;
  eg33[ 40] = 5.750000E+05;
  eg33[ 41] = 6.000000E+05;
  eg33[ 42] = 6.250000E+05;
  eg33[ 43] = 6.500000E+05;
  eg33[ 44] = 6.750000E+05;
  eg33[ 45] = 7.000000E+05;
  eg33[ 46] = 7.250000E+05;
  eg33[ 47] = 7.500000E+05;
  eg33[ 48] = 7.750000E+05;
  eg33[ 49] = 8.000000E+05;
  eg33[ 50] = 8.250000E+05;
  eg33[ 51] = 8.500000E+05;
  eg33[ 52] = 8.750000E+05;
  eg33[ 53] = 9.000000E+05;
  eg33[ 54] = 9.250000E+05;
  eg33[ 55] = 9.500000E+05;
  eg33[ 56] = 9.750000E+05;
  eg33[ 57] = 1.000000E+06;
  eg33[ 58] = 1.125000E+06;
  eg33[ 59] = 1.250000E+06;
  eg33[ 60] = 1.375000E+06;
  eg33[ 61] = 1.500000E+06;
  eg33[ 62] = 1.625000E+06;
  eg33[ 63] = 1.750000E+06;
  eg33[ 64] = 1.875000E+06;
  eg33[ 65] = 2.000000E+06;
  eg33[ 66] = 2.125000E+06;
  eg33[ 67] = 2.250000E+06;
  eg33[ 68] = 2.375000E+06;
  eg33[ 69] = 2.500000E+06;
  eg33[ 70] = 2.625000E+06;
  eg33[ 71] = 2.750000E+06;
  eg33[ 72] = 2.875000E+06;
  eg33[ 73] = 3.000000E+06;
  eg33[ 74] = 3.125000E+06;
  eg33[ 75] = 3.250000E+06;
  eg33[ 76] = 3.375000E+06;
  eg33[ 77] = 3.500000E+06;
  eg33[ 78] = 3.625000E+06;
  eg33[ 79] = 3.750000E+06;
  eg33[ 80] = 3.875000E+06;
  eg33[ 81] = 4.000000E+06;
  eg33[ 82] = 4.200000E+06;
  eg33[ 83] = 4.400000E+06;
  eg33[ 84] = 4.600000E+06;
  eg33[ 85] = 4.800000E+06;
  eg33[ 86] = 5.000000E+06;
  eg33[ 87] = 5.200000E+06;
  eg33[ 88] = 5.400000E+06;
  eg33[ 89] = 5.600000E+06;
  eg33[ 90] = 5.800000E+06;
  eg33[ 91] = 6.000000E+06;
  eg33[ 92] = 6.200000E+06;
  eg33[ 93] = 6.400000E+06;
  eg33[ 94] = 6.600000E+06;
  eg33[ 95] = 6.800000E+06;
  eg33[ 96] = 7.000000E+06;
  eg33[ 97] = 7.200000E+06;
  eg33[ 98] = 7.400000E+06;
  eg33[ 99] = 7.600000E+06;
  eg33[100] = 7.800000E+06;
  eg33[101] = 8.000000E+06;
  eg33[102] = 8.200000E+06;
  eg33[103] = 8.400000E+06;
  eg33[104] = 8.600000E+06;
  eg33[105] = 8.800000E+06;
  eg33[106] = 9.000000E+06;
  eg33[107] = 9.200000E+06;
  eg33[108] = 9.400000E+06;
  eg33[109] = 9.600000E+06;
  eg33[110] = 9.800000E+06;
  eg33[111] = 1.000000E+07;
  eg33[112] = 1.100000E+07;
  eg33[113] = 1.200000E+07;
  eg33[114] = 1.300000E+07;
  eg33[115] = 1.400000E+07;
  eg33[116] = 1.500000E+07;
  eg33[117] = 1.600000E+07;
  eg33[118] = 1.700000E+07;
  eg33[119] = 1.800000E+07;
  eg33[120] = 1.900000E+07;
  eg33[121] = 2.000000E+07;
  eg33[122] = 2.200000E+07;
  eg33[123] = 2.400000E+07;
  eg33[124] = 2.600000E+07;
  eg33[125] = 2.800000E+07;
  eg33[126] = 3.000000E+07;
  eg33[127] = 3.500000E+07;
  eg33[128] = 4.000000E+07;
  eg33[129] = 4.500000E+07;
  eg33[130] = 5.000000E+07;
  eg33[131] = 5.400000E+07;
  eg33[132] = 5.500000E+07;
  eg33[133] = 6.000000E+07;
  eg33[134] = 7.000000E+07;
  eg33[135] = 8.000000E+07;
  eg33[136] = 9.000000E+07;
  eg33[137] = 1.000000E+08;
  eg33[138] = 1.200000E+08;
  eg33[139] = 1.400000E+08;
  eg33[140] = 1.600000E+08;
  eg33[141] = 1.800000E+08;
  eg33[142] = 2.000000E+08;

  return eg33;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_eg618()
{
  vector<Real> eg618;
  eg618.resize(619);
  eg618[  0] = 1.000000E-05;
  eg618[  1] = 2.569011E-05;
  eg618[  2] = 4.235584E-05;
  eg618[  3] = 6.983297E-05;
  eg618[  4] = 1.151351E-04;
  eg618[  5] = 1.390000E-04;
  eg618[  6] = 1.898257E-04;
  eg618[  7] = 2.437410E-04;
  eg618[  8] = 3.129696E-04;
  eg618[  9] = 4.018610E-04;
  eg618[ 10] = 5.159997E-04;
  eg618[ 11] = 6.625567E-04;
  eg618[ 12] = 8.507397E-04;
  eg618[ 13] = 1.092371E-03;
  eg618[ 14] = 1.237819E-03;
  eg618[ 15] = 1.402633E-03;
  eg618[ 16] = 1.589391E-03;
  eg618[ 17] = 1.801016E-03;
  eg618[ 18] = 2.040818E-03;
  eg618[ 19] = 2.312550E-03;
  eg618[ 20] = 2.620463E-03;
  eg618[ 21] = 2.969373E-03;
  eg618[ 22] = 3.364741E-03;
  eg618[ 23] = 3.812751E-03;
  eg618[ 24] = 4.320413E-03;
  eg618[ 25] = 4.895669E-03;
  eg618[ 26] = 5.547520E-03;
  eg618[ 27] = 6.286163E-03;
  eg618[ 28] = 7.123156E-03;
  eg618[ 29] = 8.071594E-03;
  eg618[ 30] = 9.146314E-03;
  eg618[ 31] = 1.036413E-02;
  eg618[ 32] = 1.103256E-02;
  eg618[ 33] = 1.174410E-02;
  eg618[ 34] = 1.250153E-02;
  eg618[ 35] = 1.330781E-02;
  eg618[ 36] = 1.416609E-02;
  eg618[ 37] = 1.507972E-02;
  eg618[ 38] = 1.605228E-02;
  eg618[ 39] = 1.708756E-02;
  eg618[ 40] = 1.818962E-02;
  eg618[ 41] = 1.936275E-02;
  eg618[ 42] = 2.061154E-02;
  eg618[ 43] = 2.194087E-02;
  eg618[ 44] = 2.335593E-02;
  eg618[ 45] = 2.486226E-02;
  eg618[ 46] = 2.646574E-02;
  eg618[ 47] = 2.817263E-02;
  eg618[ 48] = 2.998961E-02;
  eg618[ 49] = 3.192377E-02;
  eg618[ 50] = 3.398268E-02;
  eg618[ 51] = 3.617437E-02;
  eg618[ 52] = 3.850742E-02;
  eg618[ 53] = 4.099093E-02;
  eg618[ 54] = 4.363462E-02;
  eg618[ 55] = 4.644881E-02;
  eg618[ 56] = 4.944451E-02;
  eg618[ 57] = 5.263340E-02;
  eg618[ 58] = 5.602796E-02;
  eg618[ 59] = 5.964146E-02;
  eg618[ 60] = 6.348800E-02;
  eg618[ 61] = 6.758263E-02;
  eg618[ 62] = 7.194133E-02;
  eg618[ 63] = 7.658115E-02;
  eg618[ 64] = 8.152021E-02;
  eg618[ 65] = 8.677781E-02;
  eg618[ 66] = 9.237450E-02;
  eg618[ 67] = 9.833214E-02;
  eg618[ 68] = 1.046740E-01;
  eg618[ 69] = 1.114249E-01;
  eg618[ 70] = 1.186112E-01;
  eg618[ 71] = 1.262610E-01;
  eg618[ 72] = 1.344041E-01;
  eg618[ 73] = 1.430724E-01;
  eg618[ 74] = 1.520000E-01;
  eg618[ 75] = 1.621223E-01;
  eg618[ 76] = 1.725783E-01;
  eg618[ 77] = 1.837086E-01;
  eg618[ 78] = 1.955568E-01;
  eg618[ 79] = 2.081691E-01;
  eg618[ 80] = 2.215949E-01;
  eg618[ 81] = 2.358865E-01;
  eg618[ 82] = 2.510999E-01;
  eg618[ 83] = 2.672945E-01;
  eg618[ 84] = 2.845335E-01;
  eg618[ 85] = 3.028843E-01;
  eg618[ 86] = 3.224187E-01;
  eg618[ 87] = 3.432129E-01;
  eg618[ 88] = 3.653482E-01;
  eg618[ 89] = 3.889112E-01;
  eg618[ 90] = 4.140000E-01;
  eg618[ 91] = 4.406941E-01;
  eg618[ 92] = 4.691164E-01;
  eg618[ 93] = 4.993718E-01;
  eg618[ 94] = 5.315785E-01;
  eg618[ 95] = 5.658624E-01;
  eg618[ 96] = 6.023574E-01;
  eg618[ 97] = 6.412061E-01;
  eg618[ 98] = 6.825603E-01;
  eg618[ 99] = 7.265817E-01;
  eg618[100] = 7.734422E-01;
  eg618[101] = 8.233249E-01;
  eg618[102] = 8.764248E-01;
  eg618[103] = 9.329494E-01;
  eg618[104] = 9.931194E-01;
  eg618[105] = 1.057170E+00;
  eg618[106] = 1.130000E+00;
  eg618[107] = 1.197931E+00;
  eg618[108] = 1.275191E+00;
  eg618[109] = 1.357433E+00;
  eg618[110] = 1.444980E+00;
  eg618[111] = 1.538173E+00;
  eg618[112] = 1.637377E+00;
  eg618[113] = 1.742979E+00;
  eg618[114] = 1.855391E+00;
  eg618[115] = 1.975054E+00;
  eg618[116] = 2.102434E+00;
  eg618[117] = 2.238029E+00;
  eg618[118] = 2.382370E+00;
  eg618[119] = 2.536019E+00;
  eg618[120] = 2.699579E+00;
  eg618[121] = 2.873686E+00;
  eg618[122] = 3.060000E+00;
  eg618[123] = 3.256313E+00;
  eg618[124] = 3.466327E+00;
  eg618[125] = 3.689886E+00;
  eg618[126] = 3.927864E+00;
  eg618[127] = 4.181189E+00;
  eg618[128] = 4.450853E+00;
  eg618[129] = 4.737908E+00;
  eg618[130] = 5.043477E+00;
  eg618[131] = 5.368753E+00;
  eg618[132] = 5.715008E+00;
  eg618[133] = 6.083594E+00;
  eg618[134] = 6.475952E+00;
  eg618[135] = 6.893615E+00;
  eg618[136] = 7.338215E+00;
  eg618[137] = 7.811489E+00;
  eg618[138] = 8.320000E+00;
  eg618[139] = 8.851577E+00;
  eg618[140] = 9.422455E+00;
  eg618[141] = 1.003015E+01;
  eg618[142] = 1.067704E+01;
  eg618[143] = 1.136565E+01;
  eg618[144] = 1.209867E+01;
  eg618[145] = 1.287897E+01;
  eg618[146] = 1.370959E+01;
  eg618[147] = 1.459378E+01;
  eg618[148] = 1.553500E+01;
  eg618[149] = 1.653692E+01;
  eg618[150] = 1.760346E+01;
  eg618[151] = 1.873879E+01;
  eg618[152] = 1.994734E+01;
  eg618[153] = 2.123383E+01;
  eg618[154] = 2.260000E+01;
  eg618[155] = 2.406108E+01;
  eg618[156] = 2.561289E+01;
  eg618[157] = 2.726478E+01;
  eg618[158] = 2.902320E+01;
  eg618[159] = 3.089504E+01;
  eg618[160] = 3.288760E+01;
  eg618[161] = 3.500867E+01;
  eg618[162] = 3.726653E+01;
  eg618[163] = 3.967002E+01;
  eg618[164] = 4.222851E+01;
  eg618[165] = 4.495202E+01;
  eg618[166] = 4.785117E+01;
  eg618[167] = 5.093731E+01;
  eg618[168] = 5.422248E+01;
  eg618[169] = 5.771953E+01;
  eg618[170] = 6.140000E+01;
  eg618[171] = 6.540480E+01;
  eg618[172] = 6.962305E+01;
  eg618[173] = 7.411335E+01;
  eg618[174] = 7.889325E+01;
  eg618[175] = 8.398143E+01;
  eg618[176] = 8.939776E+01;
  eg618[177] = 9.516342E+01;
  eg618[178] = 1.013009E+02;
  eg618[179] = 1.078343E+02;
  eg618[180] = 1.147890E+02;
  eg618[181] = 1.221923E+02;
  eg618[182] = 1.300730E+02;
  eg618[183] = 1.384620E+02;
  eg618[184] = 1.473920E+02;
  eg618[185] = 1.568980E+02;
  eg618[186] = 1.670000E+02;
  eg618[187] = 1.777887E+02;
  eg618[188] = 1.892551E+02;
  eg618[189] = 2.014610E+02;
  eg618[190] = 2.144541E+02;
  eg618[191] = 2.282852E+02;
  eg618[192] = 2.430083E+02;
  eg618[193] = 2.586810E+02;
  eg618[194] = 2.753645E+02;
  eg618[195] = 2.931240E+02;
  eg618[196] = 3.120288E+02;
  eg618[197] = 3.321530E+02;
  eg618[198] = 3.535750E+02;
  eg618[199] = 3.763786E+02;
  eg618[200] = 4.006530E+02;
  eg618[201] = 4.264929E+02;
  eg618[202] = 4.540000E+02;
  eg618[203] = 4.832797E+02;
  eg618[204] = 5.144486E+02;
  eg618[205] = 5.476277E+02;
  eg618[206] = 5.829466E+02;
  eg618[207] = 6.205435E+02;
  eg618[208] = 6.605651E+02;
  eg618[209] = 7.031679E+02;
  eg618[210] = 7.485183E+02;
  eg618[211] = 7.967936E+02;
  eg618[212] = 8.481824E+02;
  eg618[213] = 9.028854E+02;
  eg618[214] = 9.611165E+02;
  eg618[215] = 1.023103E+03;
  eg618[216] = 1.055580E+03;
  eg618[217] = 1.089088E+03;
  eg618[218] = 1.123659E+03;
  eg618[219] = 1.159328E+03;
  eg618[220] = 1.196129E+03;
  eg618[221] = 1.235000E+03;
  eg618[222] = 1.273273E+03;
  eg618[223] = 1.313691E+03;
  eg618[224] = 1.355392E+03;
  eg618[225] = 1.398416E+03;
  eg618[226] = 1.442807E+03;
  eg618[227] = 1.488606E+03;
  eg618[228] = 1.535860E+03;
  eg618[229] = 1.584613E+03;
  eg618[230] = 1.634914E+03;
  eg618[231] = 1.686812E+03;
  eg618[232] = 1.740357E+03;
  eg618[233] = 1.795602E+03;
  eg618[234] = 1.852601E+03;
  eg618[235] = 1.911408E+03;
  eg618[236] = 1.972083E+03;
  eg618[237] = 2.034684E+03;
  eg618[238] = 2.099271E+03;
  eg618[239] = 2.165910E+03;
  eg618[240] = 2.234663E+03;
  eg618[241] = 2.305599E+03;
  eg618[242] = 2.378786E+03;
  eg618[243] = 2.454297E+03;
  eg618[244] = 2.532205E+03;
  eg618[245] = 2.612586E+03;
  eg618[246] = 2.695518E+03;
  eg618[247] = 2.781083E+03;
  eg618[248] = 2.869364E+03;
  eg618[249] = 2.960447E+03;
  eg618[250] = 3.054422E+03;
  eg618[251] = 3.151380E+03;
  eg618[252] = 3.251415E+03;
  eg618[253] = 3.350000E+03;
  eg618[254] = 3.461114E+03;
  eg618[255] = 3.570981E+03;
  eg618[256] = 3.684336E+03;
  eg618[257] = 3.801290E+03;
  eg618[258] = 3.921955E+03;
  eg618[259] = 4.046452E+03;
  eg618[260] = 4.174900E+03;
  eg618[261] = 4.307425E+03;
  eg618[262] = 4.444158E+03;
  eg618[263] = 4.585230E+03;
  eg618[264] = 4.730781E+03;
  eg618[265] = 4.880952E+03;
  eg618[266] = 5.035890E+03;
  eg618[267] = 5.195747E+03;
  eg618[268] = 5.360678E+03;
  eg618[269] = 5.530844E+03;
  eg618[270] = 5.706412E+03;
  eg618[271] = 5.887552E+03;
  eg618[272] = 6.074443E+03;
  eg618[273] = 6.267267E+03;
  eg618[274] = 6.466211E+03;
  eg618[275] = 6.671471E+03;
  eg618[276] = 6.883246E+03;
  eg618[277] = 7.101744E+03;
  eg618[278] = 7.327177E+03;
  eg618[279] = 7.559767E+03;
  eg618[280] = 7.799740E+03;
  eg618[281] = 8.047330E+03;
  eg618[282] = 8.302780E+03;
  eg618[283] = 8.566338E+03;
  eg618[284] = 8.838263E+03;
  eg618[285] = 9.120000E+03;
  eg618[286] = 9.408282E+03;
  eg618[287] = 9.706933E+03;
  eg618[288] = 1.001506E+04;
  eg618[289] = 1.033298E+04;
  eg618[290] = 1.066098E+04;
  eg618[291] = 1.099940E+04;
  eg618[292] = 1.134855E+04;
  eg618[293] = 1.170880E+04;
  eg618[294] = 1.208047E+04;
  eg618[295] = 1.246395E+04;
  eg618[296] = 1.285960E+04;
  eg618[297] = 1.326780E+04;
  eg618[298] = 1.368897E+04;
  eg618[299] = 1.412350E+04;
  eg618[300] = 1.457183E+04;
  eg618[301] = 1.503439E+04;
  eg618[302] = 1.551163E+04;
  eg618[303] = 1.600403E+04;
  eg618[304] = 1.651205E+04;
  eg618[305] = 1.703620E+04;
  eg618[306] = 1.757698E+04;
  eg618[307] = 1.813494E+04;
  eg618[308] = 1.871060E+04;
  eg618[309] = 1.930454E+04;
  eg618[310] = 1.991733E+04;
  eg618[311] = 2.054958E+04;
  eg618[312] = 2.120189E+04;
  eg618[313] = 2.187491E+04;
  eg618[314] = 2.256930E+04;
  eg618[315] = 2.328572E+04;
  eg618[316] = 2.402489E+04;
  eg618[317] = 2.480000E+04;
  eg618[318] = 2.557436E+04;
  eg618[319] = 2.638618E+04;
  eg618[320] = 2.722377E+04;
  eg618[321] = 2.808794E+04;
  eg618[322] = 2.897955E+04;
  eg618[323] = 2.989946E+04;
  eg618[324] = 3.084857E+04;
  eg618[325] = 3.182781E+04;
  eg618[326] = 3.283813E+04;
  eg618[327] = 3.388053E+04;
  eg618[328] = 3.495601E+04;
  eg618[329] = 3.606563E+04;
  eg618[330] = 3.721048E+04;
  eg618[331] = 3.839166E+04;
  eg618[332] = 3.961035E+04;
  eg618[333] = 4.086771E+04;
  eg618[334] = 4.216499E+04;
  eg618[335] = 4.350346E+04;
  eg618[336] = 4.488440E+04;
  eg618[337] = 4.630919E+04;
  eg618[338] = 4.777920E+04;
  eg618[339] = 4.929587E+04;
  eg618[340] = 5.086069E+04;
  eg618[341] = 5.247518E+04;
  eg618[342] = 5.414093E+04;
  eg618[343] = 5.585954E+04;
  eg618[344] = 5.763271E+04;
  eg618[345] = 5.946217E+04;
  eg618[346] = 6.134971E+04;
  eg618[347] = 6.329715E+04;
  eg618[348] = 6.530642E+04;
  eg618[349] = 6.760000E+04;
  eg618[350] = 6.951832E+04;
  eg618[351] = 7.172507E+04;
  eg618[352] = 7.400187E+04;
  eg618[353] = 7.635094E+04;
  eg618[354] = 7.877458E+04;
  eg618[355] = 8.127515E+04;
  eg618[356] = 8.385511E+04;
  eg618[357] = 8.651695E+04;
  eg618[358] = 8.926329E+04;
  eg618[359] = 9.209682E+04;
  eg618[360] = 9.502028E+04;
  eg618[361] = 9.803655E+04;
  eg618[362] = 1.011486E+05;
  eg618[363] = 1.043594E+05;
  eg618[364] = 1.076721E+05;
  eg618[365] = 1.110900E+05;
  eg618[366] = 1.146163E+05;
  eg618[367] = 1.182547E+05;
  eg618[368] = 1.220085E+05;
  eg618[369] = 1.258814E+05;
  eg618[370] = 1.298773E+05;
  eg618[371] = 1.340001E+05;
  eg618[372] = 1.382537E+05;
  eg618[373] = 1.426423E+05;
  eg618[374] = 1.471703E+05;
  eg618[375] = 1.518420E+05;
  eg618[376] = 1.566620E+05;
  eg618[377] = 1.616349E+05;
  eg618[378] = 1.667658E+05;
  eg618[379] = 1.720595E+05;
  eg618[380] = 1.775213E+05;
  eg618[381] = 1.840000E+05;
  eg618[382] = 1.889704E+05;
  eg618[383] = 1.949690E+05;
  eg618[384] = 2.011579E+05;
  eg618[385] = 2.075434E+05;
  eg618[386] = 2.141315E+05;
  eg618[387] = 2.209288E+05;
  eg618[388] = 2.279418E+05;
  eg618[389] = 2.351775E+05;
  eg618[390] = 2.426428E+05;
  eg618[391] = 2.503451E+05;
  eg618[392] = 2.582919E+05;
  eg618[393] = 2.664910E+05;
  eg618[394] = 2.749503E+05;
  eg618[395] = 2.836782E+05;
  eg618[396] = 2.926831E+05;
  eg618[397] = 3.030000E+05;
  eg618[398] = 3.115595E+05;
  eg618[399] = 3.214495E+05;
  eg618[400] = 3.316534E+05;
  eg618[401] = 3.421812E+05;
  eg618[402] = 3.530432E+05;
  eg618[403] = 3.642500E+05;
  eg618[404] = 3.758125E+05;
  eg618[405] = 3.877421E+05;
  eg618[406] = 4.000503E+05;
  eg618[407] = 4.127493E+05;
  eg618[408] = 4.258514E+05;
  eg618[409] = 4.393693E+05;
  eg618[410] = 4.533164E+05;
  eg618[411] = 4.677062E+05;
  eg618[412] = 4.825528E+05;
  eg618[413] = 5.000000E+05;
  eg618[414] = 5.136748E+05;
  eg618[415] = 5.299806E+05;
  eg618[416] = 5.468040E+05;
  eg618[417] = 5.641614E+05;
  eg618[418] = 5.820698E+05;
  eg618[419] = 6.005467E+05;
  eg618[420] = 6.196101E+05;
  eg618[421] = 6.392786E+05;
  eg618[422] = 6.595715E+05;
  eg618[423] = 6.805085E+05;
  eg618[424] = 7.021102E+05;
  eg618[425] = 7.243976E+05;
  eg618[426] = 7.473924E+05;
  eg618[427] = 7.711172E+05;
  eg618[428] = 7.955951E+05;
  eg618[429] = 8.230000E+05;
  eg618[430] = 8.469066E+05;
  eg618[431] = 8.737903E+05;
  eg618[432] = 9.015273E+05;
  eg618[433] = 9.301449E+05;
  eg618[434] = 9.596709E+05;
  eg618[435] = 9.901341E+05;
  eg618[436] = 1.021564E+06;
  eg618[437] = 1.053992E+06;
  eg618[438] = 1.087450E+06;
  eg618[439] = 1.121969E+06;
  eg618[440] = 1.157584E+06;
  eg618[441] = 1.194330E+06;
  eg618[442] = 1.232242E+06;
  eg618[443] = 1.271357E+06;
  eg618[444] = 1.311715E+06;
  eg618[445] = 1.353000E+06;
  eg618[446] = 1.396313E+06;
  eg618[447] = 1.440637E+06;
  eg618[448] = 1.486367E+06;
  eg618[449] = 1.533550E+06;
  eg618[450] = 1.582230E+06;
  eg618[451] = 1.632455E+06;
  eg618[452] = 1.684275E+06;
  eg618[453] = 1.738000E+06;
  eg618[454] = 1.792901E+06;
  eg618[455] = 1.849814E+06;
  eg618[456] = 1.908533E+06;
  eg618[457] = 1.969117E+06;
  eg618[458] = 2.031623E+06;
  eg618[459] = 2.096114E+06;
  eg618[460] = 2.162652E+06;
  eg618[461] = 2.232000E+06;
  eg618[462] = 2.302131E+06;
  eg618[463] = 2.375208E+06;
  eg618[464] = 2.450605E+06;
  eg618[465] = 2.528396E+06;
  eg618[466] = 2.608656E+06;
  eg618[467] = 2.691463E+06;
  eg618[468] = 2.776900E+06;
  eg618[469] = 2.865000E+06;
  eg618[470] = 2.955994E+06;
  eg618[471] = 3.049828E+06;
  eg618[472] = 3.146640E+06;
  eg618[473] = 3.246525E+06;
  eg618[474] = 3.349580E+06;
  eg618[475] = 3.455908E+06;
  eg618[476] = 3.565610E+06;
  eg618[477] = 3.680000E+06;
  eg618[478] = 3.795572E+06;
  eg618[479] = 3.916056E+06;
  eg618[480] = 4.040365E+06;
  eg618[481] = 4.103992E+06;
  eg618[482] = 4.168620E+06;
  eg618[483] = 4.234266E+06;
  eg618[484] = 4.300946E+06;
  eg618[485] = 4.368676E+06;
  eg618[486] = 4.437473E+06;
  eg618[487] = 4.507353E+06;
  eg618[488] = 4.578334E+06;
  eg618[489] = 4.650432E+06;
  eg618[490] = 4.723666E+06;
  eg618[491] = 4.798052E+06;
  eg618[492] = 4.873611E+06;
  eg618[493] = 4.950359E+06;
  eg618[494] = 5.028316E+06;
  eg618[495] = 5.107500E+06;
  eg618[496] = 5.187932E+06;
  eg618[497] = 5.269630E+06;
  eg618[498] = 5.352614E+06;
  eg618[499] = 5.436906E+06;
  eg618[500] = 5.522525E+06;
  eg618[501] = 5.609492E+06;
  eg618[502] = 5.697828E+06;
  eg618[503] = 5.787556E+06;
  eg618[504] = 5.878697E+06;
  eg618[505] = 5.971273E+06;
  eg618[506] = 6.070000E+06;
  eg618[507] = 6.160821E+06;
  eg618[508] = 6.257840E+06;
  eg618[509] = 6.356387E+06;
  eg618[510] = 6.456485E+06;
  eg618[511] = 6.558160E+06;
  eg618[512] = 6.661436E+06;
  eg618[513] = 6.766338E+06;
  eg618[514] = 6.872893E+06;
  eg618[515] = 6.981125E+06;
  eg618[516] = 7.091062E+06;
  eg618[517] = 7.202730E+06;
  eg618[518] = 7.316156E+06;
  eg618[519] = 7.431369E+06;
  eg618[520] = 7.548396E+06;
  eg618[521] = 7.667266E+06;
  eg618[522] = 7.790000E+06;
  eg618[523] = 7.910651E+06;
  eg618[524] = 8.035226E+06;
  eg618[525] = 8.161762E+06;
  eg618[526] = 8.290291E+06;
  eg618[527] = 8.420844E+06;
  eg618[528] = 8.553453E+06;
  eg618[529] = 8.688151E+06;
  eg618[530] = 8.824969E+06;
  eg618[531] = 8.963942E+06;
  eg618[532] = 9.105104E+06;
  eg618[533] = 9.248488E+06;
  eg618[534] = 9.394131E+06;
  eg618[535] = 9.542067E+06;
  eg618[536] = 9.692332E+06;
  eg618[537] = 9.844964E+06;
  eg618[538] = 1.000000E+07;
  eg618[539] = 1.012500E+07;
  eg618[540] = 1.025000E+07;
  eg618[541] = 1.037500E+07;
  eg618[542] = 1.050000E+07;
  eg618[543] = 1.062500E+07;
  eg618[544] = 1.075000E+07;
  eg618[545] = 1.087500E+07;
  eg618[546] = 1.100000E+07;
  eg618[547] = 1.112500E+07;
  eg618[548] = 1.125000E+07;
  eg618[549] = 1.137500E+07;
  eg618[550] = 1.150000E+07;
  eg618[551] = 1.162500E+07;
  eg618[552] = 1.175000E+07;
  eg618[553] = 1.187500E+07;
  eg618[554] = 1.200000E+07;
  eg618[555] = 1.212500E+07;
  eg618[556] = 1.225000E+07;
  eg618[557] = 1.237500E+07;
  eg618[558] = 1.250000E+07;
  eg618[559] = 1.262500E+07;
  eg618[560] = 1.275000E+07;
  eg618[561] = 1.287500E+07;
  eg618[562] = 1.300000E+07;
  eg618[563] = 1.312500E+07;
  eg618[564] = 1.325000E+07;
  eg618[565] = 1.337500E+07;
  eg618[566] = 1.350000E+07;
  eg618[567] = 1.362500E+07;
  eg618[568] = 1.375000E+07;
  eg618[569] = 1.387500E+07;
  eg618[570] = 1.400000E+07;
  eg618[571] = 1.412500E+07;
  eg618[572] = 1.425000E+07;
  eg618[573] = 1.437500E+07;
  eg618[574] = 1.450000E+07;
  eg618[575] = 1.462500E+07;
  eg618[576] = 1.475000E+07;
  eg618[577] = 1.487500E+07;
  eg618[578] = 1.500000E+07;
  eg618[579] = 1.512500E+07;
  eg618[580] = 1.525000E+07;
  eg618[581] = 1.537500E+07;
  eg618[582] = 1.550000E+07;
  eg618[583] = 1.562500E+07;
  eg618[584] = 1.575000E+07;
  eg618[585] = 1.587500E+07;
  eg618[586] = 1.600000E+07;
  eg618[587] = 1.612500E+07;
  eg618[588] = 1.625000E+07;
  eg618[589] = 1.637500E+07;
  eg618[590] = 1.650000E+07;
  eg618[591] = 1.662500E+07;
  eg618[592] = 1.675000E+07;
  eg618[593] = 1.687500E+07;
  eg618[594] = 1.700000E+07;
  eg618[595] = 1.712500E+07;
  eg618[596] = 1.725000E+07;
  eg618[597] = 1.737500E+07;
  eg618[598] = 1.750000E+07;
  eg618[599] = 1.762500E+07;
  eg618[600] = 1.775000E+07;
  eg618[601] = 1.787500E+07;
  eg618[602] = 1.800000E+07;
  eg618[603] = 1.812500E+07;
  eg618[604] = 1.825000E+07;
  eg618[605] = 1.837500E+07;
  eg618[606] = 1.850000E+07;
  eg618[607] = 1.862500E+07;
  eg618[608] = 1.875000E+07;
  eg618[609] = 1.887500E+07;
  eg618[610] = 1.900000E+07;
  eg618[611] = 1.912500E+07;
  eg618[612] = 1.925000E+07;
  eg618[613] = 1.937500E+07;
  eg618[614] = 1.950000E+07;
  eg618[615] = 1.962500E+07;
  eg618[616] = 1.975000E+07;
  eg618[617] = 1.987500E+07;
  eg618[618] = 2.000000E+07;

  return eg618;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_vit199()
{
  vector<Real> vit199;
  vit199.resize(200);
  vit199[  0] = 1.00000E-05;
  vit199[  1] = 5.00000E-04;
  vit199[  2] = 2.00000E-03;
  vit199[  3] = 5.00000E-03;
  vit199[  4] = 1.00000E-02;
  vit199[  5] = 1.45000E-02;
  vit199[  6] = 2.10000E-02;
  vit199[  7] = 3.00000E-02;
  vit199[  8] = 4.00000E-02;
  vit199[  9] = 5.00000E-02;
  vit199[ 10] = 7.00000E-02;
  vit199[ 11] = 1.00000E-01;
  vit199[ 12] = 1.25000E-01;
  vit199[ 13] = 1.50000E-01;
  vit199[ 14] = 1.84000E-01;
  vit199[ 15] = 2.25000E-01;
  vit199[ 16] = 2.75000E-01;
  vit199[ 17] = 3.25000E-01;
  vit199[ 18] = 3.66800E-01;
  vit199[ 19] = 4.13990E-01;
  vit199[ 20] = 5.00000E-01;
  vit199[ 21] = 5.31580E-01;
  vit199[ 22] = 6.25060E-01;
  vit199[ 23] = 6.82560E-01;
  vit199[ 24] = 8.00000E-01;
  vit199[ 25] = 8.76430E-01;
  vit199[ 26] = 1.00000E+00;
  vit199[ 27] = 1.04000E+00;
  vit199[ 28] = 1.08000E+00;
  vit199[ 29] = 1.12530E+00;
  vit199[ 30] = 1.30000E+00;
  vit199[ 31] = 1.44500E+00;
  vit199[ 32] = 1.85540E+00;
  vit199[ 33] = 2.38240E+00;
  vit199[ 34] = 3.05900E+00;
  vit199[ 35] = 3.92790E+00;
  vit199[ 36] = 5.04350E+00;
  vit199[ 37] = 6.47600E+00;
  vit199[ 38] = 8.31530E+00;
  vit199[ 39] = 1.06770E+01;
  vit199[ 40] = 1.37100E+01;
  vit199[ 41] = 1.76040E+01;
  vit199[ 42] = 2.26030E+01;
  vit199[ 43] = 2.90230E+01;
  vit199[ 44] = 3.72660E+01;
  vit199[ 45] = 4.78510E+01;
  vit199[ 46] = 6.14420E+01;
  vit199[ 47] = 7.88930E+01;
  vit199[ 48] = 1.01300E+02;
  vit199[ 49] = 1.30070E+02;
  vit199[ 50] = 1.67020E+02;
  vit199[ 51] = 2.14450E+02;
  vit199[ 52] = 2.75360E+02;
  vit199[ 53] = 3.53570E+02;
  vit199[ 54] = 4.54000E+02;
  vit199[ 55] = 5.82950E+02;
  vit199[ 56] = 7.48520E+02;
  vit199[ 57] = 9.61120E+02;
  vit199[ 58] = 1.23410E+03;
  vit199[ 59] = 1.58460E+03;
  vit199[ 60] = 2.03470E+03;
  vit199[ 61] = 2.24870E+03;
  vit199[ 62] = 2.48520E+03;
  vit199[ 63] = 2.61260E+03;
  vit199[ 64] = 2.74650E+03;
  vit199[ 65] = 3.03540E+03;
  vit199[ 66] = 3.35460E+03;
  vit199[ 67] = 3.70740E+03;
  vit199[ 68] = 4.30740E+03;
  vit199[ 69] = 5.53080E+03;
  vit199[ 70] = 7.10170E+03;
  vit199[ 71] = 9.11880E+03;
  vit199[ 72] = 1.05950E+04;
  vit199[ 73] = 1.17090E+04;
  vit199[ 74] = 1.50340E+04;
  vit199[ 75] = 1.93050E+04;
  vit199[ 76] = 2.18750E+04;
  vit199[ 77] = 2.35790E+04;
  vit199[ 78] = 2.41760E+04;
  vit199[ 79] = 2.47880E+04;
  vit199[ 80] = 2.60580E+04;
  vit199[ 81] = 2.70000E+04;
  vit199[ 82] = 2.85010E+04;
  vit199[ 83] = 3.18280E+04;
  vit199[ 84] = 3.43070E+04;
  vit199[ 85] = 4.08680E+04;
  vit199[ 86] = 4.63090E+04;
  vit199[ 87] = 5.24750E+04;
  vit199[ 88] = 5.65620E+04;
  vit199[ 89] = 6.73790E+04;
  vit199[ 90] = 7.19980E+04;
  vit199[ 91] = 7.94990E+04;
  vit199[ 92] = 8.25030E+04;
  vit199[ 93] = 8.65170E+04;
  vit199[ 94] = 9.80370E+04;
  vit199[ 95] = 1.11090E+05;
  vit199[ 96] = 1.16790E+05;
  vit199[ 97] = 1.22770E+05;
  vit199[ 98] = 1.29070E+05;
  vit199[ 99] = 1.35690E+05;
  vit199[100] = 1.42640E+05;
  vit199[101] = 1.49960E+05;
  vit199[102] = 1.57640E+05;
  vit199[103] = 1.65730E+05;
  vit199[104] = 1.74220E+05;
  vit199[105] = 1.83160E+05;
  vit199[106] = 1.92550E+05;
  vit199[107] = 2.02420E+05;
  vit199[108] = 2.12800E+05;
  vit199[109] = 2.23710E+05;
  vit199[110] = 2.35180E+05;
  vit199[111] = 2.47240E+05;
  vit199[112] = 2.73240E+05;
  vit199[113] = 2.87250E+05;
  vit199[114] = 2.94520E+05;
  vit199[115] = 2.97210E+05;
  vit199[116] = 2.98490E+05;
  vit199[117] = 3.01970E+05;
  vit199[118] = 3.33730E+05;
  vit199[119] = 3.68830E+05;
  vit199[120] = 3.87740E+05;
  vit199[121] = 4.07620E+05;
  vit199[122] = 4.50490E+05;
  vit199[123] = 4.97870E+05;
  vit199[124] = 5.23400E+05;
  vit199[125] = 5.50230E+05;
  vit199[126] = 5.78440E+05;
  vit199[127] = 6.08100E+05;
  vit199[128] = 6.39280E+05;
  vit199[129] = 6.72060E+05;
  vit199[130] = 7.06510E+05;
  vit199[131] = 7.42740E+05;
  vit199[132] = 7.80820E+05;
  vit199[133] = 8.20850E+05;
  vit199[134] = 8.62940E+05;
  vit199[135] = 9.07180E+05;
  vit199[136] = 9.61640E+05;
  vit199[137] = 1.00260E+06;
  vit199[138] = 1.10800E+06;
  vit199[139] = 1.16480E+06;
  vit199[140] = 1.22460E+06;
  vit199[141] = 1.28740E+06;
  vit199[142] = 1.35340E+06;
  vit199[143] = 1.42270E+06;
  vit199[144] = 1.49570E+06;
  vit199[145] = 1.57240E+06;
  vit199[146] = 1.65300E+06;
  vit199[147] = 1.73770E+06;
  vit199[148] = 1.82680E+06;
  vit199[149] = 1.92050E+06;
  vit199[150] = 2.01900E+06;
  vit199[151] = 2.12250E+06;
  vit199[152] = 2.23130E+06;
  vit199[153] = 2.30690E+06;
  vit199[154] = 2.34570E+06;
  vit199[155] = 2.36530E+06;
  vit199[156] = 2.38520E+06;
  vit199[157] = 2.46600E+06;
  vit199[158] = 2.59240E+06;
  vit199[159] = 2.72530E+06;
  vit199[160] = 2.86510E+06;
  vit199[161] = 3.01190E+06;
  vit199[162] = 3.16640E+06;
  vit199[163] = 3.32870E+06;
  vit199[164] = 3.67880E+06;
  vit199[165] = 4.06570E+06;
  vit199[166] = 4.49330E+06;
  vit199[167] = 4.72370E+06;
  vit199[168] = 4.96590E+06;
  vit199[169] = 5.22050E+06;
  vit199[170] = 5.48810E+06;
  vit199[171] = 5.76950E+06;
  vit199[172] = 6.06530E+06;
  vit199[173] = 6.37630E+06;
  vit199[174] = 6.59240E+06;
  vit199[175] = 6.70320E+06;
  vit199[176] = 7.04690E+06;
  vit199[177] = 7.40820E+06;
  vit199[178] = 7.78800E+06;
  vit199[179] = 8.18730E+06;
  vit199[180] = 8.60710E+06;
  vit199[181] = 9.04840E+06;
  vit199[182] = 9.51230E+06;
  vit199[183] = 1.00000E+07;
  vit199[184] = 1.05130E+07;
  vit199[185] = 1.10520E+07;
  vit199[186] = 1.16180E+07;
  vit199[187] = 1.22140E+07;
  vit199[188] = 1.25230E+07;
  vit199[189] = 1.28400E+07;
  vit199[190] = 1.34990E+07;
  vit199[191] = 1.38400E+07;
  vit199[192] = 1.41910E+07;
  vit199[193] = 1.45500E+07;
  vit199[194] = 1.49180E+07;
  vit199[195] = 1.56830E+07;
  vit199[196] = 1.64870E+07;
  vit199[197] = 1.69050E+07;
  vit199[198] = 1.73320E+07;
  vit199[199] = 1.96400E+07;

  return vit199;
}


////////////////////////////////////////////////////////////////////////////////////////////////////

vector<Real> GroupStructureNeutron::get_vit200()
{
  vector<Real> vit200 = get_vit199();
  vit200.push_back(2.00000E+07);

  return vit200;
}
